Package: debrowser
Type: Package
Title: Interactive Differential Expresion Analysis Browser
Version: 1.38.0
Date: 2020-12-07
Author: Alper Kucukural <alper.kucukural@umassmed.edu>,
        Onur Yukselen <onur.yukselen@umassmed.edu>,
        Manuel Garber <manuel.garber@umassmed.edu>
Maintainer: Alper Kucukural <alper.kucukural@umassmed.edu>
Description: Bioinformatics platform containing interactive plots and tables
    for differential gene and region expression studies. Allows visualizing
    expression data much more deeply in an interactive and faster way. By
    changing the parameters, users can easily discover different parts of the
    data that like never have been done before. Manually creating and looking
    these plots takes time. With DEBrowser users can prepare plots without
    writing any code. Differential expression, PCA and clustering analysis are
    made on site and the results are shown in various plots such as scatter,
    bar, box, volcano, ma plots and Heatmaps.
Depends: R (>= 3.5.0),
License: GPL-3 + file LICENSE
Imports: shiny, jsonlite, shinyjs, shinydashboard, shinyBS, gplots, DT,
        ggplot2, RColorBrewer, annotate, AnnotationDbi, DESeq2, DOSE,
        igraph, grDevices, graphics, stats, utils, GenomicRanges,
        IRanges, S4Vectors, SummarizedExperiment, stringi, reshape2,
        org.Hs.eg.db, org.Mm.eg.db, limma, edgeR, clusterProfiler,
        methods, sva, RCurl, enrichplot, colourpicker, plotly,
        heatmaply, Harman, pathview, apeglm, ashr
RoxygenNote: 7.2.3
Encoding: UTF-8
Suggests: testthat, rmarkdown, knitr
VignetteBuilder: knitr, rmarkdown
URL: https://github.com/UMMS-Biocore/debrowser
BugReports: https://github.com/UMMS-Biocore/debrowser/issues/new
biocViews: Sequencing, ChIPSeq, RNASeq, DifferentialExpression,
        GeneExpression, Clustering, ImmunoOncology
git_url: https://git.bioconductor.org/packages/debrowser
git_branch: RELEASE_3_22
git_last_commit: 5dcc10b
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 03:30:06 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

export(BoxMainPlotControlsUI)
export(IQRPlotControlsUI)
export(actionButtonDE)
export(addDataCols)
export(addID)
export(all2all)
export(all2allControlsUI)
export(applyFilters)
export(applyFiltersNew)
export(applyFiltersToMergedComparison)
export(barMainPlotControlsUI)
export(batchEffectUI)
export(batchMethod)
export(changeClusterOrder)
export(checkCountData)
export(checkMetaData)
export(clustFunParamsUI)
export(clusterData)
export(compareClust)
export(condSelectUI)
export(correctCombat)
export(correctHarman)
export(customColorsUI)
export(cutOffSelectionUI)
export(dataLCFUI)
export(dataLoadUI)
export(deServer)
export(deUI)
export(debrowserIQRplot)
export(debrowserall2all)
export(debrowserbarmainplot)
export(debrowserbatcheffect)
export(debrowserboxmainplot)
export(debrowsercondselect)
export(debrowserdataload)
export(debrowserdeanalysis)
export(debrowserdensityplot)
export(debrowserheatmap)
export(debrowserhistogram)
export(debrowserlowcountfilter)
export(debrowsermainplot)
export(debrowserpcaplot)
export(dendControlsUI)
export(densityPlotControlsUI)
export(distFunParamsUI)
export(drawKEGG)
export(drawPCAExplained)
export(fileTypes)
export(fileUploadBox)
export(generateTestData)
export(getAfterLoadMsg)
export(getAll2AllPlotUI)
export(getBSTableUI)
export(getBarMainPlot)
export(getBarMainPlotUI)
export(getBoxMainPlot)
export(getBoxMainPlotUI)
export(getColorShapeSelection)
export(getColors)
export(getCompSelection)
export(getCondMsg)
export(getConditionSelector)
export(getConditionSelectorFromMeta)
export(getCovariateDetails)
export(getCutOffSelection)
export(getDEAnalysisText)
export(getDEResultsUI)
export(getDataAssesmentText)
export(getDataForTables)
export(getDataPreparationText)
export(getDensityPlot)
export(getDensityPlotUI)
export(getDomains)
export(getDown)
export(getDownloadSection)
export(getEnrichDO)
export(getEnrichGO)
export(getEnrichKEGG)
export(getEntrezIds)
export(getEntrezTable)
export(getGOLeftMenu)
export(getGOPlots)
export(getGSEA)
export(getGeneList)
export(getGeneSetData)
export(getGoPanel)
export(getGroupSelector)
export(getHeatmapUI)
export(getHelpButton)
export(getHideLegendOnOff)
export(getHistogramUI)
export(getIQRPlot)
export(getIQRPlotUI)
export(getIntroText)
export(getJSLine)
export(getKEGGModal)
export(getLeftMenu)
export(getLegendColors)
export(getLegendRadio)
export(getLegendSelect)
export(getLevelOrder)
export(getLoadingMsg)
export(getLogo)
export(getMainPanel)
export(getMainPlotUI)
export(getMainPlotsLeftMenu)
export(getMean)
export(getMergedComparison)
export(getMetaSelector)
export(getMethodDetails)
export(getMostVariedList)
export(getNormalizedMatrix)
export(getOrganism)
export(getOrganismBox)
export(getOrganismPathway)
export(getPCAPlotUI)
export(getPCAcontolUpdatesJS)
export(getPCAexplained)
export(getPCselection)
export(getPlotArea)
export(getProgramTitle)
export(getQAText)
export(getQCLeftMenu)
export(getQCPanel)
export(getSampleDetails)
export(getSampleNames)
export(getSearchData)
export(getSelHeat)
export(getSelectInputBox)
export(getSelectedCols)
export(getSelectedDatasetInput)
export(getShapeColor)
export(getStartPlotsMsg)
export(getStartupMsg)
export(getTabUpdateJS)
export(getTableDetails)
export(getTableModal)
export(getTableStyle)
export(getUp)
export(getUpDown)
export(getVariationData)
export(get_conditions_given_selection)
export(heatmapControlsUI)
export(heatmapJScode)
export(heatmapServer)
export(heatmapUI)
export(hideObj)
export(histogramControlsUI)
export(installpack)
export(kmeansControlsUI)
export(lcfMetRadio)
export(loadpack)
export(mainPlotControlsUI)
export(mainScatterNew)
export(niceKmeans)
export(normalizationMethods)
export(palUI)
export(panel.cor)
export(panel.hist)
export(pcaPlotControlsUI)
export(plotData)
export(plotMarginsUI)
export(plotSizeMarginsUI)
export(plotSizeUI)
export(plotTypeUI)
export(plot_pca)
export(prepDataContainer)
export(prepGroup)
export(prepHeatData)
export(prepPCADat)
export(push)
export(removeCols)
export(removeExtraCols)
export(round_vals)
export(runDE)
export(runDESeq2)
export(runEdgeR)
export(runHeatmap)
export(runHeatmap2)
export(runLimma)
export(run_pca)
export(selectConditions)
export(selectGroupInfo)
export(selectedInput)
export(sepRadio)
export(setBatch)
export(showObj)
export(startDEBrowser)
export(startHeatmap)
export(textareaInput)
export(togglePanels)
import(RColorBrewer)
import(apeglm)
import(ashr)
import(colourpicker)
import(heatmaply)
import(org.Hs.eg.db)
import(org.Mm.eg.db)
import(shinyBS)
importFrom(DESeq2,DESeq)
importFrom(DESeq2,DESeqDataSetFromMatrix)
importFrom(DESeq2,counts)
importFrom(DESeq2,estimateSizeFactors)
importFrom(DESeq2,lfcShrink)
importFrom(DESeq2,results)
importFrom(DOSE,enrichDO)
importFrom(DT,dataTableOutput)
importFrom(DT,datatable)
importFrom(DT,formatRound)
importFrom(DT,formatStyle)
importFrom(DT,renderDataTable)
importFrom(DT,styleInterval)
importFrom(Harman,harman)
importFrom(Harman,reconstructData)
importFrom(RCurl,getURL)
importFrom(annotate,geneSymbols)
importFrom(clusterProfiler,bitr)
importFrom(clusterProfiler,compareCluster)
importFrom(clusterProfiler,enrichGO)
importFrom(clusterProfiler,enrichKEGG)
importFrom(clusterProfiler,gseGO)
importFrom(edgeR,DGEList)
importFrom(edgeR,calcNormFactors)
importFrom(edgeR,equalizeLibSizes)
importFrom(edgeR,estimateCommonDisp)
importFrom(edgeR,exactTest)
importFrom(edgeR,glmFit)
importFrom(edgeR,glmLRT)
importFrom(edgeR,topTags)
importFrom(enrichplot,dotplot)
importFrom(enrichplot,gseaplot)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,autoplot)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,ylab)
importFrom(gplots,bluered)
importFrom(gplots,heatmap.2)
importFrom(gplots,redblue)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(graphics,barplot)
importFrom(graphics,hist)
importFrom(graphics,pairs)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,rect)
importFrom(graphics,text)
importFrom(igraph,layout.kamada.kawai)
importFrom(jsonlite,fromJSON)
importFrom(limma,eBayes)
importFrom(limma,lmFit)
importFrom(limma,topTable)
importFrom(limma,voom)
importFrom(methods,new)
importFrom(pathview,pathview)
importFrom(plotly,"%>%")
importFrom(plotly,add_bars)
importFrom(plotly,event_data)
importFrom(plotly,ggplotly)
importFrom(plotly,group_by)
importFrom(plotly,hide_legend)
importFrom(plotly,plot_ly)
importFrom(plotly,plotlyOutput)
importFrom(plotly,renderPlotly)
importFrom(reshape2,melt)
importFrom(shiny,HTML)
importFrom(shiny,NS)
importFrom(shiny,a)
importFrom(shiny,actionButton)
importFrom(shiny,actionLink)
importFrom(shiny,addResourcePath)
importFrom(shiny,br)
importFrom(shiny,callModule)
importFrom(shiny,checkboxGroupInput)
importFrom(shiny,checkboxInput)
importFrom(shiny,column)
importFrom(shiny,conditionalPanel)
importFrom(shiny,div)
importFrom(shiny,downloadButton)
importFrom(shiny,downloadHandler)
importFrom(shiny,enableBookmarking)
importFrom(shiny,eventReactive)
importFrom(shiny,fileInput)
importFrom(shiny,fluidPage)
importFrom(shiny,fluidRow)
importFrom(shiny,h2)
importFrom(shiny,h3)
importFrom(shiny,h4)
importFrom(shiny,helpText)
importFrom(shiny,htmlOutput)
importFrom(shiny,icon)
importFrom(shiny,imageOutput)
importFrom(shiny,img)
importFrom(shiny,incProgress)
importFrom(shiny,isolate)
importFrom(shiny,mainPanel)
importFrom(shiny,need)
importFrom(shiny,numericInput)
importFrom(shiny,observe)
importFrom(shiny,observeEvent)
importFrom(shiny,onBookmark)
importFrom(shiny,onBookmarked)
importFrom(shiny,onRestore)
importFrom(shiny,onRestored)
importFrom(shiny,outputOptions)
importFrom(shiny,p)
importFrom(shiny,parseQueryString)
importFrom(shiny,plotOutput)
importFrom(shiny,radioButtons)
importFrom(shiny,reactive)
importFrom(shiny,reactiveVal)
importFrom(shiny,reactiveValues)
importFrom(shiny,reactiveValuesToList)
importFrom(shiny,renderImage)
importFrom(shiny,renderPlot)
importFrom(shiny,renderPrint)
importFrom(shiny,renderTable)
importFrom(shiny,renderText)
importFrom(shiny,renderUI)
importFrom(shiny,runApp)
importFrom(shiny,selectInput)
importFrom(shiny,selectizeInput)
importFrom(shiny,shinyApp)
importFrom(shiny,shinyServer)
importFrom(shiny,shinyUI)
importFrom(shiny,showNotification)
importFrom(shiny,sidebarLayout)
importFrom(shiny,sidebarPanel)
importFrom(shiny,sliderInput)
importFrom(shiny,stopApp)
importFrom(shiny,strong)
importFrom(shiny,tabPanel)
importFrom(shiny,tableOutput)
importFrom(shiny,tabsetPanel)
importFrom(shiny,tags)
importFrom(shiny,textInput)
importFrom(shiny,textOutput)
importFrom(shiny,titlePanel)
importFrom(shiny,uiOutput)
importFrom(shiny,updateCheckboxInput)
importFrom(shiny,updateQueryString)
importFrom(shiny,updateSelectInput)
importFrom(shiny,updateTabsetPanel)
importFrom(shiny,updateTextInput)
importFrom(shiny,validate)
importFrom(shiny,verbatimTextOutput)
importFrom(shiny,wellPanel)
importFrom(shiny,withMathJax)
importFrom(shiny,withProgress)
importFrom(shinydashboard,dashboardBody)
importFrom(shinydashboard,dashboardHeader)
importFrom(shinydashboard,dashboardPage)
importFrom(shinydashboard,dashboardSidebar)
importFrom(shinydashboard,dropdownMenu)
importFrom(shinydashboard,menuItem)
importFrom(shinydashboard,menuSubItem)
importFrom(shinydashboard,messageItem)
importFrom(shinydashboard,sidebarMenu)
importFrom(shinydashboard,tabItem)
importFrom(shinydashboard,tabItems)
importFrom(shinydashboard,updateTabItems)
importFrom(shinyjs,disable)
importFrom(shinyjs,enable)
importFrom(shinyjs,extendShinyjs)
importFrom(shinyjs,hide)
importFrom(shinyjs,inlineCSS)
importFrom(shinyjs,js)
importFrom(shinyjs,onclick)
importFrom(shinyjs,show)
importFrom(shinyjs,useShinyjs)
importFrom(stats,aggregate)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,cor.test)
importFrom(stats,cov)
importFrom(stats,density)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,mahalanobis)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,prcomp)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,var)
importFrom(stringi,stri_rand_strings)
importFrom(sva,ComBat)
importFrom(utils,data)
importFrom(utils,download.file)
importFrom(utils,install.packages)
importFrom(utils,installed.packages)
importFrom(utils,packageDescription)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,update.packages)
importFrom(utils,write.table)
importMethodsFrom(AnnotationDbi,as.data.frame)
importMethodsFrom(AnnotationDbi,as.list)
importMethodsFrom(AnnotationDbi,colnames)
importMethodsFrom(AnnotationDbi,exists)
importMethodsFrom(AnnotationDbi,head)
importMethodsFrom(AnnotationDbi,keys)
importMethodsFrom(AnnotationDbi,mapIds)
importMethodsFrom(AnnotationDbi,mappedkeys)
importMethodsFrom(AnnotationDbi,ncol)
importMethodsFrom(AnnotationDbi,nrow)
importMethodsFrom(AnnotationDbi,sample)
importMethodsFrom(AnnotationDbi,select)
importMethodsFrom(AnnotationDbi,subset)
importMethodsFrom(DOSE,summary)
importMethodsFrom(GenomicRanges,as.factor)
importMethodsFrom(GenomicRanges,setdiff)
importMethodsFrom(IRanges,"colnames<-")
importMethodsFrom(IRanges,"rownames<-")
importMethodsFrom(IRanges,as.matrix)
importMethodsFrom(IRanges,gsub)
importMethodsFrom(IRanges,lapply)
importMethodsFrom(IRanges,mean)
importMethodsFrom(IRanges,nchar)
importMethodsFrom(IRanges,paste)
importMethodsFrom(IRanges,rownames)
importMethodsFrom(IRanges,toupper)
importMethodsFrom(IRanges,unique)
importMethodsFrom(IRanges,which)
importMethodsFrom(S4Vectors,eval)
importMethodsFrom(S4Vectors,grep)
importMethodsFrom(S4Vectors,grepl)
importMethodsFrom(S4Vectors,levels)
importMethodsFrom(S4Vectors,sapply)
importMethodsFrom(S4Vectors,t)
importMethodsFrom(SummarizedExperiment,cbind)
importMethodsFrom(SummarizedExperiment,order)
importMethodsFrom(SummarizedExperiment,rbind)

#' getGeneList
#'
#' Gathers the gene list to use for GOTerm analysis.
#"
#' @note \code{GOTerm}
#'
#' @export
#'
#' @note \code{getGeneList}
#' symobol to ENTREZ ID conversion
#' @param genes, gene list
#' @param org, orgranism for gene symbol entrez ID conversion
#' @param fromType, from Type
#' @param toType, to Type 
#' @return ENTREZ ID list
#'
#' @examples
#'     x <- getGeneList(c('OCLN', 'ABCC2'))
#'
getGeneList <- function(genes = NULL, org = "org.Hs.eg.db", 
    fromType= "SYMBOL", toType = c("ENTREZID")) {
    # Get the entrez gene identifiers that are mapped to a gene symbol
    if (!installpack(org)) return(NULL)

    mapped_genes <- bitr(genes, fromType = fromType,
         toType = toType,
         OrgDb = org)
    #genelist <- unique(as.vector(unlist(mapped_genes[toType])))
    genelist <- data.frame(mapped_genes)
    colnames(genelist) <- c(fromType, toType)
    genelist
}

#' getEntrezTable
#'
#' Gathers the entrezIds of the genes in given list and their data
#"
#' @note \code{GOTerm}
#'
#' @export
#'
#' @note \code{getEntrezTable}
#' symobol to ENTREZ ID conversion
#' @param genes, gene list
#' @param dat, data matrix
#' @param org, orgranism for gene symbol entrez ID conversion
#' @return table with the entrez IDs in the rownames
#'
#' @examples
#'     x <- getEntrezTable()
#'
getEntrezTable <- function(genes = NULL, dat = NULL, org = "org.Hs.eg.db") {
    if (is.null(genes)) return(NULL)
    if (!installpack(org)) return(NULL)
    allkeys <- AnnotationDbi::keys(eval(parse(text = org)),
                                   keytype="SYMBOL")
    entrezIDs <- unlist(strsplit(genes, "/"))
    
    mapped_genes <- mapIds(eval(parse(text = org)), keys = rownames(dat),
                           column="ENTREZID", keytype="SYMBOL",
                           multiVals = "first")
    mapped_genes <- mapped_genes[mapped_genes %in% entrezIDs]
    genelist <- cbind(mapped_genes, dat[names(mapped_genes), ])
    
    genelist <- data.frame(genelist)
}

#' getEntrezIds
#'
#' Gathers the gene list to use for GOTerm analysis.
#"
#' @note \code{GOTerm}
#'
#' @export
#'
#' @note \code{getEntrezIds}
#' symobol to ENTREZ ID conversion
#' @param genes, gene list with fold changes 
#' @param org, orgranism for gene symbol entrez ID conversion
#' @return ENTREZ ID list
#'
#' @examples
#'     x <- getEntrezIds()
#'
getEntrezIds <- function(genes = NULL, org = "org.Hs.eg.db") {
    if (is.null(genes)) return(NULL)
    if (!installpack(org)) return(NULL)
    allkeys <- AnnotationDbi::keys(eval(parse(text = org)),
        keytype="SYMBOL")
    
    mapped_genes <- mapIds(eval(parse(text = org)), keys = rownames(genes),
        column="ENTREZID", keytype="SYMBOL",
        multiVals = "first")
    mapped_genes <- mapped_genes[!is.na(mapped_genes)]
    genelist <- cbind(mapped_genes, names(mapped_genes), genes[names(mapped_genes), "log2FoldChange"])
    
    colnames(genelist) <- c("ENTREZID", "SYMBOL", "log2FoldChange")
    genelist <- data.frame(genelist)
    genelist$log2FoldChange <- as.numeric(as.character(genelist$log2FoldChange))
    rownames(genelist) <- genelist$ENTREZID
    genelist
}
#' getEnrichGO
#'
#' Gathers the Enriched GO Term analysis data to be used within the
#' GO Term plots.
#'
#' @note \code{getEnrichGO}
#' @param genelist, gene list
#' @param pvalueCutoff, p value cutoff
#' @param org, the organism used
#' @param ont, the ontology used
#' @return Enriched GO
#' @examples
#'     x <- getEnrichGO()
#'
#' @export
#'
getEnrichGO <- function(genelist = NULL, pvalueCutoff = 0.01,
    org = "org.Hs.eg.db", ont="CC") {
    if (is.null(genelist)) return(NULL)
    if (!installpack(org)) return(NULL)
    res <- c()
    res$enrich_p <- clusterProfiler::enrichGO(gene = genelist, OrgDb = org,
    # res$enrich_p <- enrichGO(gene = genelist, organism = "human",
        ont = ont, pvalueCutoff = pvalueCutoff)
                             
    res$p <- barplot(res$enrich_p, title = paste("Enrich GO", ont),
        font.size = 12)
    res$table <- NULL
    if (!is.null(nrow(res$enrich_p@result)) )
        res$table <- res$enrich_p@result[,c("ID", "Description", 
            "GeneRatio", "pvalue", "p.adjust", "qvalue")]
    return(res)
}

#' getEnrichKEGG
#'
#' Gathers the Enriched KEGG analysis data to be used within the
#' GO Term plots.
#'
#' @note \code{getEnrichKEGG}
#' @param genelist, gene list
#' @param org, the organism used
#' @param pvalueCutoff, the p value cutoff
#' @return Enriched KEGG
#' @examples
#'     x <- getEnrichKEGG()
#' @export
#'
getEnrichKEGG <- function(genelist = NULL, pvalueCutoff = 0.01,
    org = "org.Hs.eg.db") {
    if (is.null(genelist)) return(NULL)
    res <- c()
    res$enrich_p <- enrichKEGG(gene = genelist, organism = getOrganism(org),
        pvalueCutoff = pvalueCutoff)
    res$p <- barplot(res$enrich_p, title =
        paste("KEGG Enrichment: p-value=", pvalueCutoff))
    res$table <- NULL
    if (!is.null(nrow(res$enrich_p@result)) )
        res$table <- res$enrich_p@result[,c("ID", "Description", 
            "GeneRatio", "pvalue", "p.adjust", "qvalue")]
    return(res)
}

#' getGSEA
#'
#' Gathers the Enriched KEGG analysis data to be used within the
#' GO Term plots.
#'
#' @note \code{getGSEA}
#' @param org, the organism used
#' @param dataset, dataset
#' @param pvalueCutoff, the p value cutoff
#' @param sortfield, sort field for GSEA
#' @return GSEA
#' @examples
#'     x <- getGSEA()
#' @export
#'
getGSEA <- function(dataset=NULL, pvalueCutoff = 0.01,
                    org = "org.Hs.eg.db", sortfield = "log2FoldChange") {
    if (is.null(dataset)) return(NULL)
    
    genelist <- getGeneList(rownames(dataset), org, 
        fromType = "SYMBOL",toType = c("ENTREZID"))
    #symbol <- getGeneList(genelist, org, 
    #    fromType = "ENTREZID",toType = c("SYMBOL") )
    dataset[is.na(dataset[,sortfield]),sortfield] <- 0 
    data <- dataset[genelist$SYMBOL, c("ID", sortfield)]
    rownames(data) <- genelist$ENTREZID
    newdata <- data[order(-data[,sortfield]),] 
    newdatatmp <- newdata[,sortfield]
    names(newdatatmp) <- rownames(newdata)
    
    res <- c()
    OrgDb <- org
    if (is(OrgDb, "character")) {
        require(OrgDb, character.only = TRUE)
        OrgDb <- eval(parse(text = OrgDb))
    }
    res$enrich_p <- gseGO(geneList=newdatatmp, ont="All", OrgDb = OrgDb, verbose=F,
        pvalueCutoff=pvalueCutoff)
    
    res$table <- NULL
    if (nrow(res$enrich_p@result)>0 )
        res$table <- res$enrich_p@result[,c("ID", "Description", 
           "setSize", "enrichmentScore", "pvalue", "p.adjust", "qvalue")]

    return(res)
}

#' clusterData
#'
#' Gathers the Cluster analysis data to be used within the
#' GO Term plots.
#'
#' @note \code{clusterData}
#' @param dat, the data to cluster
#' @return clustered data
#' @examples
#'     mycluster <- clusterData()
#'
#' @export
#'
clusterData <- function(dat = NULL) {
    if (is.null(dat)) return(NULL)
    ret <- list()
    itemlabels <- rownames(dat)
    norm_data <- getNormalizedMatrix(dat)
    mydata <- na.omit(norm_data)  # listwise deletion of missing
    mydata <- scale(mydata)  # standardize variables

    wss <- (nrow(mydata) - 1) * sum(apply(mydata, 2, var))
    for (i in 2:15) wss[i] <- sum(kmeans(mydata, centers = i)$withinss)
    plot(1:15, wss, type = "b",
        xlab = "Number of Clusters",
        ylab = "Within groups sum of squares")
    k <- 0
    for (i in 1:14) {
        if ( ( wss[i] / wss[i + 1] ) > 1.2 ) {
            k <- k + 1
        }
    }
    # K-Means Cluster Analysis
    fit <- kmeans(mydata, k)  # 5 cluster solution
    # get cluster means
    aggregate(mydata, by = list(fit$cluster), FUN = mean)
    # append cluster assignment
    mydata_cluster <- data.frame(mydata, fit$cluster)

    # distance <- dist(mydata, method = 'euclidean')
    # distance matrix fit <- hclust(distance,
    # method='ward.D2') plot(fit, cex = 0.1)
    # display dendogram groups <- cutree(fit, k=k) rect.hclust(fit,
    # k=k, border='red')
    return(mydata_cluster)
}

#' compareClust
#'
#' Compares the clustered data to be displayed within the GO Term
#' plots.
#'
#' @note \code{compareClust}
#' @param dat, data to compare clusters
#' @param ont, the ontology to use
#' @param org, the organism used
#' @param fun, fun
#' @param title, title of the comparison
#' @param pvalueCutoff, pvalueCutoff
#' @return compared cluster
#' @examples
#'     x <- compareClust()
#'   
#' @export
#' 
compareClust <- function(dat = NULL, ont = "CC", org = "org.Hs.eg.db",
    fun = "enrichGO", title = "Ontology Distribution Comparison",
    pvalueCutoff = 0.01) {
        if (is.null(dat)) return(NULL)
        if (!installpack(org)) return(NULL)
        res <- c()
        genecluster <- list()
        k <- max(dat$fit.cluster)
        for (i in 1:k) {
            clgenes <- rownames(dat[dat$fit.cluster == i, ])
            genelist <- getGeneList(clgenes, org)
            genecl <- list()
            genecl <- push(genecl, genelist)
            genecluster[c(paste("X", i, sep = ""))] <- genecl
        }
        res$table <- NULL
        p <- tryCatch({
            title <- paste(fun, title)
            xx <- c()
            if (fun == "enrichKEGG"){
                xx <- compareCluster(genecluster, fun = fun, 
                    organism = getOrganism(org),
                    pvalueCutoff = pvalueCutoff)
            } else if (fun == "enrichDO") {
                 if (!installpack("DOSE")) return(NULL)
                 xx <- compareCluster(genecluster, fun = fun,
                     pvalueCutoff = pvalueCutoff) 
            } else {
                title <- paste(ont, title)
                xx <- compareCluster(genecluster, fun = fun,
                    ont = ont, OrgDb = org, pvalueCutoff = pvalueCutoff)
                #ont = ont, organism = "human", pvalueCutoff = pvalueCutoff)
            }
            if (!is.null(xx@compareClusterResult) )
                res$table <- xx@compareClusterResult[,
                c("Cluster", "ID", "Description", "GeneRatio", "BgRatio", 
                  "pvalue", "p.adjust", "qvalue")]
            res$p <- dotplot(xx, title = title)
        })
        res
}
#' getEnrichDO
#'
#' Gathers the Enriched DO Term analysis data to be used within the
#' GO Term plots.
#'
#' @note \code{getEnrichDO}
#' @param genelist, gene list
#' @param pvalueCutoff, the p value cutoff
#' @return enriched DO
#' @examples
#'     x <- getEnrichDO()
#' @export
#'
getEnrichDO <- function(genelist = NULL, pvalueCutoff = 0.01) {
    if (is.null(genelist)) return(NULL)
    res <- c()
    res$enrich_p <- enrichDO(gene = genelist, ont = "DO",
        pvalueCutoff = pvalueCutoff)

    res$p <- barplot(res$enrich_p, title = "Enrich DO", font.size = 12)
    res$table <- NULL
    if (!is.null(nrow(res$enrich_p@result)) )
         res$table <- res$enrich_p@result[,c("ID", "Description", 
            "GeneRatio", "pvalue", "p.adjust", "qvalue")]
    res
}

#' drawKEGG
#'
#' draw KEGG patwhay with expression values
#'
#' @note \code{drawKEGG}
#' @param input, input
#' @param dat, expression matrix
#' @param pid, pathway id
#' @return enriched DO
#' @examples
#'     x <- drawKEGG()
#' @importFrom pathview pathview 
#' @export
#'
drawKEGG <- function(input = NULL, dat = NULL, pid = NULL) {
    if (is.null(dat) && is.null(pid)) return(NULL)
    tryCatch({
        if (installpack("pathview")){
            org <- input$organism
            genedata <- getEntrezIds(dat[[1]], org)
            foldChangeData <- data.frame(genedata$log2FoldChange)
            rownames(foldChangeData) <- rownames(genedata)
            pathview(gene.data = foldChangeData,
               pathway.id = pid,
               species = substr(pid,0,3),
               gene.idtype="entrez",
               out.suffix = "b.2layer", kegg.native = TRUE)
    
            unlink(paste0(pid,".png"))
            unlink(paste0(pid,".xml"))
        }
    })
}

require(shiny)
library(debrowser)
library(testthat)

test_that("passing no data returns NULL", {
    expect_null(compareClust() )
    expect_null(getGOPlots(NULL, NULL))
    null_deseq <- runDE(NULL)
    expect_null(null_deseq)
    expect_null(plot_pca(NULL))
})

#' deServer
#'
#' Sets up shinyServer to be able to run DEBrowser interactively.
#'
#' @note \code{deServer}
#' @param input, input params from UI
#' @param output, output params to UI
#' @param session, session variable
#' @return the panel for main plots;
#'
#' @examples
#'     deServer
#'
#' @export
#' @importFrom shiny actionButton actionLink addResourcePath column 
#'             conditionalPanel downloadButton downloadHandler 
#'             eventReactive fileInput fluidPage helpText isolate 
#'             mainPanel need numericInput observe observeEvent 
#'             outputOptions parseQueryString plotOutput radioButtons 
#'             reactive reactiveValues renderPlot renderUI runApp 
#'             selectInput shinyApp  shinyServer  shinyUI sidebarLayout 
#'             sidebarPanel sliderInput  stopApp  tabPanel tabsetPanel 
#'             textInput textOutput titlePanel uiOutput tags HTML
#'             h4 img icon updateTabsetPanel updateTextInput  validate 
#'             wellPanel checkboxInput br p checkboxGroupInput onRestore
#'             reactiveValuesToList renderText onBookmark onBookmarked 
#'             updateQueryString callModule enableBookmarking htmlOutput
#'             onRestored NS reactiveVal withProgress tableOutput
#'             selectizeInput fluidRow div renderPrint renderImage
#'             verbatimTextOutput imageOutput renderTable incProgress
#'             a h3 strong h2 withMathJax updateCheckboxInput
#'             showNotification updateSelectInput
#' @importFrom shinyjs show hide enable disable useShinyjs extendShinyjs
#'             js inlineCSS onclick
#' @importFrom DT datatable dataTableOutput renderDataTable formatStyle
#'             styleInterval formatRound
#' @importFrom ggplot2 aes aes_string geom_bar geom_point ggplot
#'             labs scale_x_discrete scale_y_discrete ylab
#'             autoplot theme_minimal theme geom_density
#'             geom_text element_blank margin
#' @importFrom plotly renderPlotly plotlyOutput plot_ly add_bars event_data
#'             hide_legend %>% group_by ggplotly
#' @importFrom gplots heatmap.2 redblue bluered
#' @importFrom igraph layout.kamada.kawai  
#' @importFrom grDevices dev.off pdf colorRampPalette 
#' @importFrom graphics barplot hist pairs par rect text plot
#' @importFrom stats aggregate as.dist cor cor.test dist
#'             hclust kmeans na.omit prcomp var sd model.matrix
#'             p.adjust runif cov mahalanobis quantile as.dendrogram
#'             density as.formula coef
#' @importFrom utils read.csv read.table write.table update.packages
#'             download.file read.delim data install.packages
#'             packageDescription installed.packages
#' @importFrom DOSE enrichDO
#' @importFrom enrichplot gseaplot dotplot
#' @importMethodsFrom DOSE summary
#' @importMethodsFrom AnnotationDbi as.data.frame as.list colnames
#'             exists sample subset head mappedkeys ncol nrow subset 
#'             keys mapIds select
#' @importMethodsFrom GenomicRanges as.factor setdiff
#' @importMethodsFrom IRanges as.matrix "colnames<-" mean
#'             nchar paste rownames toupper unique which
#'             as.matrix lapply "rownames<-" gsub
#' @importMethodsFrom S4Vectors eval grep grepl levels sapply t 
#' @importMethodsFrom SummarizedExperiment cbind order rbind
#' @importFrom jsonlite fromJSON
#' @importFrom methods new
#' @importFrom stringi stri_rand_strings
#' @importFrom annotate geneSymbols
#' @importFrom reshape2 melt
#' @importFrom Harman harman reconstructData
#' @importFrom clusterProfiler compareCluster enrichKEGG enrichGO gseGO bitr
#' @importFrom DESeq2 DESeq DESeqDataSetFromMatrix results estimateSizeFactors
#'             counts lfcShrink
#' @importFrom edgeR calcNormFactors equalizeLibSizes DGEList glmLRT
#'             exactTest estimateCommonDisp glmFit topTags
#' @importFrom shinydashboard dashboardHeader dropdownMenu messageItem
#'             dashboardPage dashboardSidebar sidebarMenu dashboardBody
#'             updateTabItems menuItem tabItems tabItem menuSubItem
#' @importFrom limma lmFit voom eBayes topTable
#' @importFrom sva ComBat
#' @importFrom RCurl getURL
#' @import org.Hs.eg.db
#' @import org.Mm.eg.db
#' @import shinyBS
#' @import colourpicker
#' @import RColorBrewer
#' @import heatmaply
#' @import apeglm
#' @import ashr

deServer <- function(input, output, session) {
    options(warn = -1)
    tryCatch(
    {
        if (!interactive()) {
            options( shiny.maxRequestSize = 30 * 1024 ^ 2,
                    shiny.fullstacktrace = FALSE, shiny.trace=FALSE, 
                    shiny.autoreload=TRUE, warn =-1)
        }
        # To hide the panels from 1 to 4 and only show Data Prep
        togglePanels(0, c(0), session)

        choicecounter <- reactiveValues(nc = 0)
        
        output$programtitle <- renderUI({
            togglePanels(0, c(0), session)
            getProgramTitle(session)
        })
        
        updata <- reactiveVal()
        filtd <- reactiveVal()
        batch <- reactiveVal()
        sel <- reactiveVal()
        dc <- reactiveVal()
        compsel <- reactive({
            cp <- 1
            if (!is.null(input$compselect_dataprep))
                cp <- input$compselect_dataprep
            cp
        })

        observe({
            updata(callModule(debrowserdataload, "load", "Filter"))
            updateTabItems(session, "DataPrep", "Upload")

            observeEvent (input$Filter, {
                if(!is.null(updata()$load())){ 
                    updateTabItems(session, "DataPrep", "Filter")
                    filtd(callModule(debrowserlowcountfilter, "lcf", updata()$load()))
                }
            })
            observeEvent (input$Batch, {
                if(!is.null(filtd()$filter())){ 
                    updateTabItems(session, "DataPrep", "BatchEffect")
                    batch(callModule(debrowserbatcheffect, "batcheffect", filtd()$filter()))
                }
            })

            observeEvent (input$goDEFromFilter, {
                if(is.null(batch())) batch(setBatch(filtd()))
                updateTabItems(session, "DataPrep", "CondSelect")
                sel(debrowsercondselect(input, output, session,
                    batch()$BatchEffect()$count, batch()$BatchEffect()$meta))
                choicecounter$nc <- sel()$cc()
            })
            observeEvent (input$goDE, {
                updateTabItems(session, "DataPrep", "CondSelect")
                sel(debrowsercondselect(input, output, session,
                    batch()$BatchEffect()$count, batch()$BatchEffect()$meta))
                choicecounter$nc <- sel()$cc()
            })
            observeEvent (input$startDE, {
                if(!is.null(batch()$BatchEffect()$count)){
                    togglePanels(0, c(0), session)
                    res <- prepDataContainer(batch()$BatchEffect()$count, sel()$cc(), input, batch()$BatchEffect()$meta)
                    if(is.null(res)) return(NULL)
                    dc(res)
                    updateTabItems(session, "DataPrep", "DEAnalysis")
                    buttonValues$startDE <- TRUE
                    buttonValues$goQCplots <- FALSE
                    hideObj(c("load-uploadFile","load-demo", 
                        "load-demo2", "goQCplots", "goQCplotsFromFilter"))
                }
            })

            observeEvent (input$goMain, {
                updateTabItems(session, "methodtabs", "panel1")
                updateTabItems(session, "menutabs", "discover")
                togglePanels(0, c( 0, 1, 2, 3, 4), session)
            })
            
            output$compselectUI <- renderUI({
                if (!is.null(sel()) && !is.null(sel()$cc()))
                    getCompSelection("compselect_dataprep",sel()$cc())
            })

            output$cutOffUI <- renderUI({
                cutOffSelectionUI(paste0("DEResults", compsel()))
            })  
            output$deresUI <- renderUI({
                column(12, getDEResultsUI(paste0("DEResults",compsel())))
            })
        })
        output$mainpanel <- renderUI({
            getMainPanel()
        })
        output$qcpanel <- renderUI({
            getQCPanel(input)
        })
        output$gopanel <- renderUI({
            getGoPanel()
        })
        output$cutoffSelection <- renderUI({
            nc <- 1
            if (!is.null(choicecounter$nc)) nc <- choicecounter$nc
            getCutOffSelection(nc)
        })
        output$downloadSection <- renderUI({
            choices <- c("most-varied", "alldetected")
            if (buttonValues$startDE)
                choices <- c("up+down", "up", "down",
                             "comparisons", "alldetected",
                             "most-varied", "selected")
            choices <- c(choices, "searched")
            getDownloadSection(choices)
        })
       
        output$leftMenu  <- renderUI({
            getLeftMenu(input)
        })
        output$loading <- renderUI({
            getLoadingMsg()
        })
        output$logo <- renderUI({
            getLogo()
        })
        output$startup <- renderUI({
            getStartupMsg()
        })
        output$afterload <- renderUI({
            getAfterLoadMsg()
        })
        output$mainmsgs <- renderUI({
            if (is.null(condmsg()))
                getStartPlotsMsg()
            else
                condmsg()
        })
        buttonValues <- reactiveValues(goQCplots = FALSE, goDE = FALSE,
            startDE = FALSE)
        output$dataready <- reactive({
            query <- parseQueryString(session$clientData$url_search)
            jsonobj<-query$jsonobject
            if (!is.null(jsonobj) && (is.null(updata()) || is.null(updata()$load() ))) return(NULL)
            hide(id = "loading-debrowser", anim = TRUE, animType = "fade")  
            return(!is.null(init_data()))
        })
        outputOptions(output, "dataready", 
                      suspendWhenHidden = FALSE)

        observeEvent(input$resetsamples, {
            buttonValues$startDE <- FALSE
            showObj(c("goQCplots", "goDE"))
            hideObj(c("add_btn","rm_btn","startDE"))
            choicecounter$nc <- 0
        })

        output$condReady <- reactive({
            if (!is.null(sel()))
                choicecounter$nc <- sel()$cc()
            choicecounter$nc
        })
        outputOptions(output, 'condReady', suspendWhenHidden = FALSE)
        observeEvent(input$goQCplotsFromFilter, {
            if(is.null(batch())) batch(setBatch(filtd()))
            buttonValues$startDE <- FALSE
            buttonValues$goQCplots <- TRUE
            updateTabItems(session, "menutabs", "discover")
            togglePanels(2, c( 0, 2, 4), session)
        })
        observeEvent(input$goQCplots, {
            buttonValues$startDE <- FALSE
            buttonValues$goQCplots <- TRUE
            updateTabItems(session, "menutabs", "discover")
            togglePanels(2, c( 0, 2, 4), session)
        })
        comparison <- reactive({
            compselect <- 1
            if (!is.null(input$compselect))
                compselect <- as.integer(input$compselect)
            dc()[[compselect]]
        })
        conds <- reactive({ comparison()$conds })
        cols <- reactive({ comparison()$cols })
        cond_names <- reactive({ comparison()$cond_names })
        
        init_data <- reactive({ 
            if (buttonValues$startDE && !is.null(comparison()$init_data))
                comparison()$init_data 
            else if (!is.null(batch()))
                batch()$BatchEffect()$count
        })
        filt_data <- reactive({
            if (!is.null(init_data()) && !is.null(comparison()) && !is.null(input$padj))
                applyFilters(init_data(), cols(), conds(), input)
        })

        selectedQCHeat <- reactiveVal()
        observe({
            if ((!is.null(input$genenames) && input$interactive == TRUE) || 
                (!is.null(input$genesetarea) && input$genesetarea != "")){
                tmpDat <- init_data()
                if (!is.null(filt_data()))
                    tmpDat <- filt_data()
                genenames <- ""
                if (!is.null(input$genenames)){
                    genenames <- input$genenames
                } else {
                   tmpDat <- getSearchData(tmpDat, input)
                   genenames <- paste(rownames(tmpDat), collapse = ",")
                }
            }
            if(!is.null(input$qcplot) && !is.null(normdat())){
                if (input$qcplot == "all2all") {
                    callModule(debrowserall2all, "all2all", normdat(), input$cex)
                } else if (input$qcplot == "pca") {
                    callModule(debrowserpcaplot, "qcpca", normdat(), batch()$BatchEffect()$meta)
                } else if (input$qcplot == "heatmap") {
                    selectedQCHeat(callModule(debrowserheatmap, "heatmapQC", normdat()))
                } else if (input$qcplot == "IQR") {
                    callModule(debrowserIQRplot, "IQR", df_select())
                    callModule(debrowserIQRplot, "normIQR", normdat())
                } else if (input$qcplot == "Density"){
                    callModule(debrowserdensityplot, "density", df_select())
                    callModule(debrowserdensityplot, "normdensity", normdat())
                }
            }
        })
        condmsg <- reactiveVal()
        selectedMain <- reactiveVal()
        observe({
            if (!is.null(filt_data())) {
            condmsg(getCondMsg(dc(), input,
                cols(), conds()))
            selectedMain(callModule(debrowsermainplot, "main", filt_data(), cond_names()))
            }
        })
        selectedHeat <- reactiveVal()
        observe({
            if (!is.null(selectedMain()) && !is.null(selectedMain()$selGenes())) {
                withProgress(message = 'Creating plot', style = "notification", value = 0.1, {
                    selectedHeat(callModule(debrowserheatmap, "heatmap", filt_data()[selectedMain()$selGenes(), cols()]))
                })
            }
        })
        
        selgenename <- reactiveVal()
        observe({
            if (!is.null(selectedMain()) && !is.null(selectedMain()$shgClicked()) 
                && selectedMain()$shgClicked()!=""){
                selgenename(selectedMain()$shgClicked())
                if (!is.null(selectedHeat()) && !is.null(selectedHeat()$shgClicked()) && 
                    selectedHeat()$shgClicked() != ""){
                    js$resetInputParam("heatmap-hoveredgenenameclick")
                }
            }
        })
        observe({
            if (!is.null(selectedHeat()) && !is.null(selectedHeat()$shgClicked()) && 
                selectedHeat()$shgClicked() != ""){
                selgenename(selectedHeat()$shgClicked())
            }
        })

        observe({
            if (!is.null(selgenename()) && selgenename()!=""){
                withProgress(message = 'Creating Bar/Box plots', style = "notification", value = 0.1, {
                    callModule(debrowserbarmainplot, "barmain", filt_data(), 
                               cols(), conds(), cond_names(), selgenename())
                    callModule(debrowserboxmainplot, "boxmain", filt_data(), 
                               cols(), conds(), cond_names(), selgenename())
                })
            }
        })
        
        normdat <-  reactive({
            if (!is.null(init_data()) && !is.null(datasetInput())){
                dat <- init_data()
                norm <- c()
                if(!is.null(cols())){
                    norm <- removeExtraCols(datasetInput())
                }else{
                    norm <- getNormalizedMatrix(dat, input$norm_method)
                }
                getSelectedCols(norm, datasetInput(), input)
            }
        })

        df_select <- reactive({
            if (!is.null(init_data()) && !is.null(datasetInput()) )
                getSelectedCols(init_data(), datasetInput(), input)
        })
        
        output$columnSelForQC <- renderUI({
            existing_cols <- colnames(removeExtraCols(datasetInput()))
            wellPanel(id = "tPanel",
                style = "overflow-y:scroll; max-height: 300px",
                checkboxGroupInput("col_list", "Select col to include:",
                existing_cols, 
                selected=existing_cols)
            )
        })

        selectedData <- reactive({
            dat <- isolate(filt_data())
            ret <- c()
            if (input$selectedplot == "Main Plot"  && !is.null(selectedMain())){
                ret <- dat[selectedMain()$selGenes(), ]
            }
            else if (input$selectedplot == "Main Heatmap" &&  !is.null(selectedHeat())){
                ret <- dat[selectedHeat()$selGenes(), ]
            }
            else if (input$selectedplot == "QC Heatmap" && !is.null(selectedQCHeat())){
                ret <- dat[selectedQCHeat()$selGenes(), ]
            }
            ret
        })
        
        datForTables <- reactive({
            getDataForTables(input, normdat(),
                filt_data(), selectedData(),
                getMostVaried(), mergedComp())
        })

        inputGOstart <- reactive({
            if (input$startGO){
                withProgress(message = 'GO Started', detail = "interactive", value = 0, {
                    dat <- datForTables()
                    getGOPlots(dat[[1]], isolate(getGSEARes()), input)
                })
            }
        })
        
        getGSEARes <- reactive({
            if (input$goplot == "GSEA"){
                dat <- datForTables()
                gopval <- as.numeric(input$gopvalue)
                getGSEA(dat[[1]], pvalueCutoff = gopval, 
                    org=input$organism, sortfield = input$sortfield)
            }
        })
        
        observeEvent(input$startGO, {
            inputGOstart()
        })

        output$GOPlots1 <- renderPlot({
            if (!is.null(inputGOstart()$p) && input$startGO){
                if (input$goplot == "GSEA" && !is.null(input$gotable_rows_selected)){
                    pid <- input$gotable_rows_selected
                    p <- enrichplot::gseaplot(inputGOstart()$enrich_p, by = "all", 
                    title = inputGOstart()$enrich_p$Description[pid[1]], 
                    geneSetID = pid[1])
                    return(p)
                }
                return(inputGOstart()$p)
            }
        })
        output$KEGGPlot <- renderImage({
            shiny::validate(need(!is.null(input$gotable_rows_selected),
                "Please select a category in the GO/KEGG table tab to be able
                to see the pathway diagram")
            )
            
            withProgress(message = 'KEGG Started', detail = "interactive", value = 0, {

            i <- input$gotable_rows_selected
           
            pid <- inputGOstart()$table$ID[i]

            drawKEGG(input, datForTables(), pid)
            list(src = paste0(pid,".b.2layer.png"),
                 contentType = 'image/png')
            })

        }, deleteFile = TRUE)

        getGOCatGenes <- reactive({
            if(is.null(input$gotable_rows_selected)) return (NULL)
            org <- input$organism
            dat <- tabledat()
            i <- input$gotable_rows_selected
            if (input$goplot == "GSEA"){
                genes <- inputGOstart()$enrich_p$core_enrichment[i]
            } else{
                genes <- inputGOstart()$enrich_p$geneID[i]
            }
            
            genedata <- getEntrezTable(genes,
                dat[[1]], org)
            dat[[1]] <- genedata
            dat
        })
        output$GOGeneTable <- DT::renderDataTable({
            shiny::validate(need(!is.null(input$gotable_rows_selected),
                "Please select a category in the GO/KEGG table to be able
                to see the gene list"))
            dat <- getGOCatGenes()
            if (!is.null(dat)){
                DT::datatable(dat[[1]], extensions = 'Buttons',
                      options = list( server = TRUE,
                                      dom = "Blfrtip",
                                      buttons = 
                                          list("copy", list(
                                              extend = "collection"
                                              , buttons = c("csv", "excel", "pdf")
                                              , text = "Download"
                                          ) ), # end of buttons customization
                        lengthMenu = list(c(10, 25, 50, 100),
                        c("10", "25", "50", "100")),
                        pageLength = 25, paging = TRUE, searching = TRUE)) %>%
                    getTableStyle(input, dat[[2]], dat[[3]], buttonValues$startDE)
            }
        })
        
        output$getColumnsForTables <-  renderUI({
            if (is.null(table_col_names())) return (NULL)
            selected_list <- table_col_names()
            if (!is.null(input$table_col_list) 
                && all(input$table_col_list %in% colnames(tabledat()[[1]])))
                selected_list <- input$table_col_list
            colsForTable <- list(
                wellPanel(id = "tPanel",
                    style = "overflow-y:scroll; max-height: 200px",
                    checkboxGroupInput("table_col_list", "Select col to include:",
                    table_col_names(), 
                    selected=selected_list)
                )
            )
            return(colsForTable)
        })
        table_col_names <- reactive({
            if (is.null(tabledat())) return (NULL)
            colnames(tabledat()[[1]])
        })
        tabledat <- reactive({
            dat <- datForTables()
            if (is.null(dat)) return (NULL)
            if (nrow(dat[[1]])<1) return(NULL)
            dat2 <- removeCols(c("ID", "x", "y","Legend", "Size"), dat[[1]])
            
            pcols <- c(names(dat2)[grep("^padj", names(dat2))], 
                       names(dat2)[grep("pvalue", names(dat2))])
            if (!is.null(pcols) && length(pcols) > 1)
                dat2[,  pcols] <- apply(dat2[,  pcols], 2,
                    function(x) format( as.numeric(x), scientific = TRUE, digits = 3 ))
            else
                dat2[,  pcols] <- format( as.numeric( dat2[,  pcols] ), 
                    scientific = TRUE, digits = 3 )
            rcols <- names(dat2)[!(names(dat2) %in% pcols)]
            if (!is.null(rcols) && length(rcols) > 1)
                dat2[,  rcols] <- apply(dat2[,  rcols], 2,
                                    function(x) round( as.numeric(x), digits = 2))
            else
                dat2[,  rcols] <-  round( as.numeric(dat2[,  rcols]), digits = 2)
                
            dat[[1]] <- dat2
            return(dat)
        })
        output$tables <- DT::renderDataTable({
            dat <- tabledat()
            if (is.null(dat) || is.null(table_col_names())
                || is.null(input$table_col_list) || length(input$table_col_list)<1) 
                return (NULL)
            if (!all(input$table_col_list %in% colnames(dat[[1]]), na.rm = FALSE)) 
                return(NULL)
            #if (!dat[[2]] %in% input$table_col_list)
            #    dat[[2]] <- ""
            #if (!dat[[3]] %in% input$table_col_list)
            #    dat[[3]] <- ""
            
            datDT <- DT::datatable(dat[[1]][, input$table_col_list],
                extensions = 'Buttons',
                options = list( server = TRUE,
                                    dom = "Blfrtip",
                                    buttons = 
                                        list("copy", list(
                                            extend = "collection"
                                            , buttons = c("csv", "excel", "pdf")
                                            , text = "Download"
                                        ) ), # end of buttons customization
                    lengthMenu = list(c(10, 25, 50, 100),
                c("10", "25", "50", "100")),
                pageLength = 25, paging = TRUE, searching = TRUE)) %>%
                getTableStyle(input, dat[[2]], dat[[3]], buttonValues$startDE)
            return(datDT)
        })
        getMostVaried <- reactive({
            dat <- init_data()
            if (!is.null(cols()))
                dat <- init_data()[,cols()]
            getMostVariedList(dat, colnames(dat), input)
        })
        output$gotable <- DT::renderDataTable({
            if (!is.null(inputGOstart()$table)){
                DT::datatable(inputGOstart()$table, rownames = FALSE,
                              extensions = 'Buttons',
                              options = list( server = TRUE,
                                              dom = "Blfrtip",
                                              buttons = 
                                                  list("copy", list(
                                                      extend = "collection"
                                                      , buttons = c("csv", "excel", "pdf")
                                                      , text = "Download"
                                                  ) ), # end of buttons customization
                    lengthMenu = list(c(10, 25, 50, 100),
                    c("10", "25", "50", "100")),
                    pageLength = 25, paging = TRUE, searching = TRUE))
            }
        })
        mergedComp <- reactive({
            dat <- applyFiltersToMergedComparison(isolate(dc()), choicecounter$nc, input)
            dat[dat$Legend == "Sig", ]
        })
        
        datasetInput <- function(addIdFlag = FALSE){
            tmpDat <- NULL
            sdata <- NULL
            if (input$selectedplot != "QC Heatmap"){
                sdata <- selectedData()
            }else{
                sdata <- isolate(selectedData())
            }
            if (buttonValues$startDE) {
                mergedCompDat <- NULL
                if (input$dataset == "comparisons"){
                    mergedCompDat <- mergedComp()
                }
                tmpDat <- getSelectedDatasetInput(rdata = filt_data(), 
                     getSelected = sdata, getMostVaried = getMostVaried(),
                     mergedCompDat, input = input)
            }
            else{
                tmpDat <- getSelectedDatasetInput(rdata = init_data(), 
                     getSelected = sdata,
                     getMostVaried = getMostVaried(),
                     input = input)
            }
            if(addIdFlag)
                tmpDat <- addID(tmpDat)
            return(tmpDat)
        }
        output$metaFile <-  renderTable({
            read.delim(system.file("extdata", "www", "metaFile.txt",
                package = "debrowser"), header=TRUE, skipNul = TRUE)
        })
        output$countFile <-  renderTable({
            read.delim(system.file("extdata", "www", "countFile.txt",
                package = "debrowser"), header=TRUE, skipNul = TRUE)
        })
        
        output$downloadData <- downloadHandler(filename = function() {
            paste(input$dataset, "csv", sep = ".")
        }, content = function(file) {
            dat <- datForTables()
            dat2 <- removeCols(c("x", "y","Legend", "Size"), dat[[1]])
            if(!("ID" %in% names(dat2)))
                dat2 <- addID(dat2)
            write.table(dat2, file, sep = ",", row.names = FALSE)
        })

        output$downloadGOPlot <- downloadHandler(filename = function() {
            paste(input$goplot, ".pdf", sep = "")
        }, content = function(file) {
            pdf(file)
            print( inputGOstart()$p )
            dev.off()
        })
    },
    err=function(errorCondition) {
        cat("in err handler")
        message(errorCondition)
    },
    warn=function(warningCondition) {
        cat("in warn handler")
        message(warningCondition)
    })
}

library(debrowser)

header <- dashboardHeader(
  title = "DEBrowser IQR Plots"
)
sidebar <- dashboardSidebar(  sidebarMenu(id="DataAssessment",
    menuItem("IQR", tabName = "IQR"),
    textInput("maxCutoff", "Max Cutoff", value = "10" ),
    plotSizeMarginsUI("IQR", w=400, h=400),
    plotSizeMarginsUI("afterFiltering", w=400, h=400)
))

body <- dashboardBody(
  tabItems(
    tabItem(tabName="IQR", 
    fluidRow(
        column(5,
               getIQRPlotUI("IQR")),
        column(5,
               getIQRPlotUI("afterFiltering"))
    ))
))

ui <- dashboardPage(header, sidebar, body, skin = "blue")

server <- function(input, output, session) {
  load(system.file("extdata", "demo", "demodata.Rda",
                   package = "debrowser"))
  filtd <- reactive({
      # Filter out the rows that has maximum 100 reads in a sample
      subset(demodata, apply(demodata, 1, max, na.rm = TRUE)  >=  
                 as.numeric(input$maxCutoff))
  })
  observe({
      if(!is.null(filtd())){
          callModule(debrowserIQRplot, "IQR", demodata)
          callModule(debrowserIQRplot, "afterFiltering", filtd())
      }
  })
}

shinyApp(ui, server)

#' getPCAPlotUI
#'
#' PCA plots UI.  
#' @param id, namespace id
#' @note \code{getPCAPlotUI}
#' @return the panel for PCA plots;
#'
#' @examples
#'     x <- getPCAPlotUI("pca")
#'
#' @export
#'
getPCAPlotUI <- function(id) {
    ns <- NS(id)
    uiOutput(ns("pcaplot"))
}

#' debrowserpcaplot
#'
#' Module for a pca plot with its loadings 
#' as a mainplot in debrowser
#' 
#' @param input, input variables
#' @param output, output objects
#' @param session, session 
#' @param pcadata, a matrix that includes expression values
#' @param metadata, metadata to color the plots
#' @return main plot
#'
#' @return panel
#' @export
#'
#' @examples
#'     x <- debrowserpcaplot()
#'
debrowserpcaplot <- function(input = NULL, output = NULL, session = NULL, pcadata = NULL, metadata = NULL) {
    if(is.null(pcadata)) return(NULL)
    qcplots <-  reactive({ 
        if(is.null(pcadata)) return(NULL)
        sc <- getShapeColor(input)
        plot_pca(pcadata, input$pcselx, input$pcsely,
            metadata = metadata, color = sc$color,
            size = 5, shape = sc$shape,
            textonoff = "On",
            legendSelect = sc$legendSelect, input = input )
    })
    output$pcaplot <- renderUI({
        list(fluidRow(
        column(12,
        shinydashboard::box(
        collapsible = TRUE, title = "PCA Plot", status = "primary", 
        solidHeader = TRUE, width = NULL,
        div(style = 'overflow-x: scroll'),
        draggable = TRUE, plotlyOutput(session$ns("pca1"), 
        height= input$height, width=input$width, inline=FALSE) 
        ),
        shinydashboard::box(
        collapsible = TRUE, title = "Loadings", status = "primary", 
        solidHeader = TRUE, width = NULL,
        draggable = TRUE,  plotlyOutput(session$ns("pca2"), 
        height= input$height, width=input$width, inline=FALSE) )) ) 
        )
    })
    output$pca1 <- renderPlotly({
        p <- qcplots()$plot1
        p$elementId <- NULL
        p
    })
    output$pca2 <- renderPlotly({
        p <- qcplots()$plot2
        p$elementId <- NULL
        p
    })
    output$colorShapeSelect <- renderUI({
        getColorShapeSelection(metadata, input, session)
    })
}

#' pcaPlotControlsUI
#'
#' Generates the PCA PLots Left menu to be displayed within the DEBrowser.
#'
#' @param id, namespace id
#' @note \code{pcaPlotControlsUI}
#' @return returns the left menu according to the selected tab;
#' @examples
#'     x <- pcaPlotControlsUI("pca")
#' @export
#'
pcaPlotControlsUI <- function(id  = "pca") {
    ns <- NS(id)
    list(fluidRow(column(12, getPCselection(id, 1, "x")), 
         column(12, getPCselection(id, 2, "y"))),
            fluidRow(
            column(12, getHideLegendOnOff(id)),
            column(12, getLegendSelect(id))),
    uiOutput(ns("colorShapeSelect")))
}

#' run_pca
#'
#' Runs PCA on the selected dataset.
#'
#' @param x, dataframe with experiment data
#' @param retx, specifies if the data should be returned
#' @param center, center the PCA (Boolean)
#' @param scale, scale the PCA (Boolean)
#' @return pca list
#' @examples
#'     load(system.file("extdata", "demo", "demodata.Rda", 
#'         package="debrowser"))
#'     pca_data<-run_pca(getNormalizedMatrix(
#'         demodata[rowSums(demodata[,1:6])>10,1:6]))
#'
#' @export
#'
run_pca <- function(x=NULL, retx = TRUE,
    center = TRUE, scale = TRUE) {
    if ( is.null(x) || ncol(x) < 2 || nrow(x) < 1) return (NULL)
    x <- subset(x, apply(x, 1, var, na.rm = TRUE) >  0)
    pca <- prcomp(t(x), retx = retx,
         center = center, scale. = scale)
    variances <- pca$sdev ^ 2
    explained <- variances / sum(variances)
    
    return(list(PCs = pca$x, explained = explained, pca = pca))
}

#' plot_pca
#'
#' Plots the PCA results for the selected dataset.
#'
#' @param dat, data
#' @param pcx, x axis label
#' @param pcy, y axis label
#' @param metadata, additional data
#' @param color, color for plot
#' @param shape, shape for plot
#' @param size, size of the plot
#' @param textonoff, text on off
#' @param legendSelect, select legend
#' @param input, input param
#' @return pca list
#' @examples
#'     load(system.file("extdata", "demo", "demodata.Rda",
#'             package="debrowser"))
#'     metadata<-cbind(colnames(demodata[,1:6]), 
#'             colnames(demodata[,1:6]),
#'             c(rep("Cond1",3), rep("Cond2",3)))
#'     colnames(metadata)<-c("samples", "color", "shape")
#'     
#'     a <- plot_pca(getNormalizedMatrix(
#'             demodata[rowSums(demodata[,1:6])>10,1:6]),
#'             metadata = metadata, color = "samples",
#'             size = 5, shape = "shape")
#'
#' @export
#'
plot_pca <- function(dat = NULL, pcx = 1, pcy = 2,
    metadata = NULL, color = NULL, shape = NULL,
    size = NULL, textonoff = "On", legendSelect = "samples", input = NULL) {
    if ( is.null(dat) || is.null(ncol(dat)) || ncol(dat) < 2 || nrow(dat)<1) return(NULL)

    pca_data <- run_pca(dat)
    p_data <- prepPCADat(pca_data, metadata, input, pcx, pcy)
    
    # Prepare axis labels
    xaxis <- sprintf("PC%d (%.2f%%)", pcx,
                     round(pca_data$explained[pcx] * 100, 2))
    yaxis <- sprintf("PC%d (%.2f%%)", pcy,
                     round(pca_data$explained[pcy] * 100, 2))

    plot1 <- ggplot(data=p_data, aes(x=x, y=y))
    
    if (legendSelect == "color") {
        plot1 <-  plot1 + geom_point(mapping=aes(shape=shape, color=color), size=3 )
    }else{
        plot1 <-  plot1 + geom_point(mapping=aes(shape=shape, color=shape), size=3 )
    }
    if (textonoff == "On")
        plot1 <- plot1 + geom_text(aes(label=textName), vjust = 0, nudge_y = 1)
    plot1 <- plot1 + theme(legend.title = element_blank())
    plot1 <- plot1 +  labs(x = xaxis, y = yaxis)
    if (!is.null(input$top))
        plot1 <- plot1 + theme( plot.margin = margin(t = input$top, r =input$right, b =input$bottom, l = input$left, "pt"))
    plot1 <- ggplotly(plot1, width = input$width, height = input$height)
    if (!is.null(input$legendonoff) && input$legendonoff=="Off") 
        plot1 <- plotly::hide_legend(plot1)

    plot1$elementId <- NULL
    if (!is.null(input$svg) && input$svg == TRUE)
      plot1 <- plot1 %>% config(toImageButtonOptions = list(format = "svg"))
    pcaExp <- getPCAexplained(dat, pca_data, input)
    plot2 <- drawPCAExplained(pcaExp$plotdata)
    plot2$elementId <- NULL
    if (!is.null(input$svg) && input$svg == TRUE)
      plot2 <- plot2 %>% config(toImageButtonOptions = list(format = "svg"))
    return (list(plot1 =  plot1, plot2 =  plot2, pcaset = pcaExp$pcaset))
}

#' prepPCADat
#'
#' prepares pca data with metadata. If metadata doesn't exists
#' it puts all the sampels into a signlge group; "Conds".
#' 
#' @param pca_data, pca run results
#' @param metadata, additional meta data
#' @param input, input
#' @param pcx, x axis label
#' @param pcy, y axis label
#' @return Color and shape from selection boxes or defaults
#' @examples
#'     x <- prepPCADat()
#' @export
#'
prepPCADat <- function(pca_data = NULL, metadata = NULL, input = NULL, pcx = 1, pcy = 2){
    if (is.null(pca_data)) return (NULL)
    rownames(metadata) <- metadata[,1]
    
    x <- pca_data$PCs
    plot_data <- data.frame(x)
    # Prepare data frame to pass to ggplot
    xaxis <- paste0("PC", pcx)
    yaxis <- paste0("PC", pcy)
    if (!is.null(metadata)) {
        samples <- rownames(plot_data)
        color  <- rownames(plot_data)
        shape <- "Conds"
        textName <- ""
        if (!is.null(input$text_pca) && input$text_pca != "None")
            textName <- as.character(metadata[samples, input$text_pca])
        if (!is.null(input$color_pca) && input$color_pca != "None")
            color <- as.character(metadata[samples, input$color_pca])
        if (!is.null(input$shape_pca) && input$shape_pca != "None")
            shape <- as.character(metadata[samples, input$shape_pca])
        
        metadata <- cbind(textName, color, shape)
        plot_data <- cbind(plot_data, metadata)
        p_data <- plot_data[,c(xaxis, yaxis, "textName", "color", "shape")]
    } else {
        samples <- rownames(plot_data)
        color  <- rownames(plot_data)
        shape <- "Conds"
        p_data <- cbind( plot_data[,c(xaxis, yaxis)], textName, color, shape)
    }
    colnames(p_data) <- c("x", "y", "textName", "color", "shape")
    p_data
}
#' getPCAexplained
#'
#' Creates a more detailed plot using the PCA results from
#' the selected dataset.
#'
#' @param datasetInput, selected data
#' @param pca_data, from user
#' @param input, input params
#' @return explained plot
#' @examples
#' load(system.file("extdata", "demo", "demodata.Rda", package="debrowser"))
#' input<-c()
#' input$qcplot<-"pca"
#' input$col_list<-colnames(demodata[,1:6])
#' dat <- getNormalizedMatrix(demodata[,1:6])
#' pca_data <- run_pca(dat)
#' x <- getPCAexplained(dat, pca_data, input)
#'
#' @export
#'
getPCAexplained <- function(datasetInput = NULL, 
    pca_data = NULL, input = NULL) {
    if (is.null(datasetInput)) return(NULL)
    datexp <- NULL
    pcaset <- NULL
    size <- length(pca_data$explained)
    if (size>9)
        size <- 9
    datexp <- data.frame(cbind(unlist(lapply(
        c(1:size), 
        function(x){paste0("PC", x)})), 
        round(pca_data$explained * 100, 2)))
    colnames(datexp) <- c("PCs", "explained")
    datexp$explained <- as.numeric( as.character(datexp$explained) )
    datexp <- datexp[1:size,]
    var <- pca_data$pca$sdev^2/sum(pca_data$pca$sdev^2)
    
    ## Select the genes for PCA, removing the least variable 
    
    dThresh.pctile <- 1 - as.numeric(input$pctile)     # distance threshold
    gList.dThresh <- c()
    
    d <- pca_data$pca$rotation[,c(input$pcselx)]
    dThresh<-quantile(d, dThresh.pctile)
    gList.dThresh <- names(which(d>dThresh))
    pcaset <-  datasetInput[gList.dThresh, ]
    return (list(plotdata =  datexp, pcaset = pcaset))
}

#' getShapeColor
#'
#' Generates the fill and shape selection boxes for PCA plots.
#' metadata file has to be loaded in this case
#'
#' @param input, input values
#' @return Color and shape from selection boxes or defaults
#' @examples
#'     x <- getShapeColor()
#' @export
#'
getShapeColor <- function(input = NULL) {
    if (is.null(input)) return (NULL)
    sc <-  c()
    if (!is.null(input$text_pca))
        sc$textField <- input$text_pca
    if (!is.null(input$color_pca))
        sc$color <- input$color_pca
    if (!is.null(input$shape_pca))
        sc$shape <- input$shape_pca
    
    sc$legendSelect <- input$legendSelect
    return(sc)
}

#' Creates a more detailed plot using the PCA results from
#' the selected dataset.
#'
#' @param explainedData, selected data
#' @return explained plot
#' @examples
#'     x <- drawPCAExplained()
#'
#' @export
#'
drawPCAExplained <- function(explainedData = NULL){
    p <- NULL
    if (is.null(explainedData)) return(NULL)
    
    p<- plot_ly(data=explainedData, x=~PCs, y=~explained,
                type = 'bar')
  
    p$elementId <- NULL
    p
}


#' getPCselection
#'
#' Generates the PC selection number to be used within DEBrowser.
#' @param id, namespace id
#' @param num, PC selection number
#' @param xy, x or y coordinate
#' @note \code{getPCselection}
#' @return PC selection for PCA analysis
#' @examples
#'     x <- getPCselection("pca")
#' @export
#'
getPCselection <- function(id, num = 1, xy = "x" ) {
    ns <- NS(id)
    numericInput(ns(paste0("pcsel", xy)),
        paste0("PC selection[", xy, "]"), num, 1, 6)
}

#' getColorShapeSelection
#'
#' Generates the fill and shape selection boxes for PCA plots.
#' metadata file has to be loaded in this case
#'
#' @param metadata, metadata table
#' @param input, input
#' @param session, session
#' @return Color and shape selection boxes
#' @examples
#'     x <- getColorShapeSelection()
#' @export
#'
getColorShapeSelection <- function(metadata = NULL, input = NULL, session = NULL) {
    if (is.null(metadata) ||  is.null(session)) return (NULL)
    list(fluidRow(column(12, selectGroupInfo(metadata, input, session$ns("text_pca"), "Text field")),
    column(12, selectGroupInfo(metadata, input, session$ns("color_pca"), "Color field")),
    column(12, selectGroupInfo(metadata, input, session$ns("shape_pca"), "Shape field"))))
}

#' getLegendSelect
#'
#' select legend
#' @param id, namespace id
#' @note \code{getLegendSelect}
#' @examples
#'     x <- getLegendSelect("pca")
#' @export
#'
getLegendSelect <- function(id = "pca") {
    ns <- NS(id)
    lst.choices <- as.list(c("color", "shape"))
    selectInput(ns("legendSelect"), label = "Select legend",
                choices = lst.choices,
                selected = "color")
}

#' getHideLegendOnOff
#'
#' hide legend
#' @param id, namespace id
#' @examples
#'     x <- getHideLegendOnOff("pca")
#' @export
#'
getHideLegendOnOff <- function(id = "pca") {
    ns <- NS(id)
    lst.choices <- as.list(c("On", "Off"))
    selectInput(ns("legendonoff"), label = "Legend On/Off",
                choices = lst.choices,
                selected = "On")
}

library(debrowser)
library(DESeq2)

header <- dashboardHeader(
  title = "DEBrowser DE Analysis"
)
sidebar <- dashboardSidebar(  sidebarMenu(id="DataPrep",
    menuItem("CondSelect", tabName = "CondSelect"),
    menuItem("DEAnalysis", tabName = "DEAnalysis"),
    shinydashboard::menuItem("Filter", tabName = "DEAnalysis", startExpanded = TRUE,
    uiOutput("cutOffUI"),
    uiOutput("compselectUI"))
))

body <- dashboardBody(
  tabItems(
    tabItem(tabName="CondSelect",
        condSelectUI()),
    tabItem(tabName="DEAnalysis",
    uiOutput("deresUI"),
    column(12,
           verbatimTextOutput("dcres")
    ))
))

ui <- dashboardPage(header, sidebar, body, skin = "blue")

server <- function(input, output, session) {
  load(system.file("extdata", "demo", "demodata.Rda",
                   package = "debrowser"))
  # Filter out the rows that has maximum 10 reads in a sample
  filtd <-
        subset(demodata, apply(demodata, 1, max, na.rm = TRUE)  >=  10)
    
  sel <- debrowsercondselect(input, output, session, demodata, metadatatable)
  dc <- reactiveVal()
  observeEvent(input$startDE, {
      updateTabItems(session, "DataPrep", "DEAnalysis")
      dc(prepDataContainer(filtd, sel$cc(), input))
  })
  output$condReady <- reactive({
      sel$cc()
  })
  outputOptions(output, 'condReady', suspendWhenHidden = FALSE)
  
  output$compselectUI <- renderUI({
      if (!is.null(sel$cc())) return(NULL)
        getCompSelection(sel$cc())
  })

  compsel <- reactive({
      cp <- 1
      if (!is.null(input$compselect))
          cp <- input$compselect
      cp
  })
  output$cutOffUI <- renderUI({
      if (is.null(dc())) return(NULL)
      cutOffSelectionUI(paste0("DEResults", compsel()))
  })  
  output$deresUI <- renderUI({
      if (is.null(dc())) return(NULL)
      column(12, getDEResultsUI(paste0("DEResults",compsel())))
  })
  output$dcres <- renderPrint({
      if (is.null(dc())) return("")
          print(head(sel$cc()))
  })
}

shinyApp(ui, server)

#' debrowsermainplot
#'
#' Module for a scatter, volcano and ma plots that are going to be used 
#' as a mainplot in debrowser
#' 
#' @param input, input variables
#' @param output, output objects
#' @param session, session 
#' @param data, a matrix that includes expression values
#' @param cond_names, condition names
#' @return main plot
#'
#' @return panel
#' @export
#'
#' @examples
#'     x <- debrowsermainplot()
#'
debrowsermainplot <- function(input = NULL, output = NULL, session = NULL, data = NULL, cond_names = NULL) {
    if (is.null(data)) return(NULL)
    
    plotdata <-  reactive({
        plotData(data, input)
    })
    output$mainplot <- renderUI({
        list(fluidRow(
            column(12,
            shinydashboard::box(
                collapsible = TRUE, title = "Main Plots", status = "primary", 
                solidHeader = TRUE,width = NULL,
                draggable = TRUE, plotlyOutput(session$ns("main"), 
                height=input$plotheight, width=input$plotwidth)
            ))))
    })
    
    xylabels <- reactive({
        if (input$mainplot == "scatter"){
            x <- paste0('log10 Norm. Mean(Read Counts) in ', cond_names[1])
            y <- paste0('log10 Norm. Mean(Read Counts) in ', cond_names[2])
            
        }else if  (input$mainplot == "volcano"){
            x <- "log2FC"
            y <- "-log10padj"
        }else {
            x <- "A"
            y <- "M"
        }
        dat <- c(x,y)
    })

    output$mainPlotControlsUI <- renderUI({
        labs <- xylabels()
        x <- labs[1]
        y <- labs[2]
        list(
            textInput(session$ns('xlab'),'x label', x),
            textInput(session$ns('ylab'),'y label', y),
            checkboxInput(session$ns('labelsearched'), 'Label searched points', value = FALSE),
            conditionalPanel(paste0("input['",session$ns("labelsearched"), "']"),
            colourpicker::colourInput(session$ns("labelcolor"), "Label colour", "black"),
            selectInput(session$ns("labelsize"), "Label Size", choices=c(6:30), selected=14))
        )
    })
    
    output$volcanoControlsUI <- renderUI({
        if(input$mainplot != 'volcano') return(NULL)
        list(
         checkboxInput(session$ns("limitPadj"), "Limit -log10 padj", FALSE),
         conditionalPanel(condition <- paste0("input['", session$ns("limitPadj"),"']"),
                          sliderInput(session$ns("log10padjCutoff"), "Log10 padj value cutoff:",
                                      min=2, max=100, value=60, sep = "",
                                      animate = FALSE))
        )
    })
    selectedPoint <- reactive({
        eventdata <- event_data("plotly_click", source = session$ns("source"))
        if (is.null(eventdata)){
            eventdata <- event_data("plotly_hover", source = session$ns("source"))
        }
        key <- ""
        if (!is.null(eventdata$key))
            key <- as.vector(unlist(eventdata$key))
        
        return(key)
    })
    
    getSelected  <- reactive({
        keys <- NULL
        selGeneList <- event_data("plotly_selected", source = session$ns("source"))
        if (is.null(selGeneList$key)) return (NULL)
        keys <- as.vector(unlist(selGeneList$key))
        return(keys)
    })
    
    output$main <- renderPlotly({
        data <- plotdata()$data
        mainScatterNew(input, data, cond_names, session$ns("source"))
    })
    
    list( shg = (selectedPoint), shgClicked=(selectedPoint), selGenes=(getSelected))
}

#' getMainPlotUI
#'
#' main plot for volcano, scatter and maplot.  
#' @param id, namespace id
#' @note \code{getMainPlotUI}
#' @return the panel for main plots;
#'
#' @examples
#'     x <- getMainPlotUI("main")
#'
#' @export
#'
getMainPlotUI <- function(id) {
    ns <- NS(id)
    uiOutput(ns("mainplot"))
}

#' mainScatterNew
#'
#' Creates the main scatter, volcano or MA plot to be displayed within the main
#' panel.
#' @param input, input params
#' @param data, dataframe that has log2FoldChange and log10padj values
#' @param cond_names condition names
#' @param source, for event triggering to select genes
#' @return scatter, volcano or MA plot
#'
#' @examples
#'     
#'     x <- mainScatterNew()
#'
#' @export
#'
mainScatterNew <- function(input = NULL, data = NULL, cond_names=NULL, source = NULL) {
    if ( is.null(data) ) return(NULL)

    p <- plot_ly(source = source, data=data, x=~x, y=~y, key=~key, alpha = 0.8,
                 color=~Legend, colors=getLegendColors(getLevelOrder(unique(data$Legend))), 
                 type="scatter", mode = "markers",
                 width=input$width - 100, height=input$height,
                 text=~paste("<b>", ID, "</b><br>",
                             "<br>", "padj=", format.pval(padj, digits = 2), " ",
                             "-log10padj=", round(log10padj, digits = 2),
                             "<br>", "log2FC=", round(log2FoldChange, digits = 2), " ",
                             "foldChange=", round(foldChange, digits = 2),
                             "<br>", sep = " ")) %>%
        plotly::layout(xaxis = list(title = input$xlab),
               yaxis = list(title = input$ylab)) %>% 
        plotly::layout(
            margin = list(l = input$left,
                          b = input$bottom,
                          t = input$top,
                          r = input$right
            ))
    
    if (!is.null(input$labelsearched) && input$labelsearched == TRUE){
        searched_genes <- data[(data$Legend == "GS"),]
        a <- list()
        for (i in seq_len(nrow(searched_genes))) {
            m <- searched_genes[i, ]
            a[[i]] <- list(
                x = m$x,
                y = m$y,
                text = rownames(m),
                color = 'blue',
                xref = "x",
                yref = "y",
                showarrow = TRUE,
                arrowhead = 0.5,
                ax = 20,
                ay = -40,
                font = list(color = input$labelcolor,
                            face = 2,
                            size = input$labelsize)
            )
        }
        
        p <- p %>%  plotly::layout(annotations = a)
    }
    if (!is.null(input$svg) && input$svg == TRUE)
        p <- p %>% config(toImageButtonOptions = list(format = "svg"))
    p$elementId <- NULL
    return(p)
}

#' plotData
#'
#' prepare plot data for mainplots 
#'
#' @note \code{plotData}
#' @param pdata, data
#' @param input, input
#' @return prepdata
#' @examples
#'     x <- plotData()
#' @export
#'
plotData <- function(pdata = NULL, input = NULL){
    if (is.null(pdata)) return(NULL)
    pdata$key <- pdata$ID
    data_rest <- pdata[ pdata$Legend!="NS",]
    data_NS <- pdata[ pdata$Legend=="NS",]
    backperc <- 10
    if (!is.null(input$backperc))  backperc <- input$backperc
    mainplot <- "scatter"
    if (!is.null(input$mainplot))  mainplot <- input$mainplot
    
    datapoints <- as.integer(nrow(data_NS) * backperc/ 100)
    if (nrow(data_NS) > datapoints){
        data_rand <- data_NS[sample(1:nrow(data_NS), datapoints,
            replace=FALSE),]
    }else{
        data_rand  <- data_NS
    }
    plot_init_data <- rbind(data_rand, data_rest)
    plot_init_data$Legend  <- factor(plot_init_data$Legend, 
         levels = getLevelOrder(unique(plot_init_data$Legend)))
    
    plot_data <- plot_init_data
    if (mainplot == "volcano") {
        plot_data <- plot_init_data[which(!is.na(plot_init_data$log2FoldChange)
                                          & !is.na(plot_init_data$log10padj)
                                          & !is.na(plot_init_data$Legend)),]
        plot_data$x <- plot_data$log2FoldChange
        plot_data$log10padjOrg <- plot_data$log10padj
        if (!is.null(input$limitPadj) && input$limitPadj){
            plot_data$log10padj[plot_data$log10padj>input$log10padjCutoff] <- input$log10padjCutoff
        }else{
            plot_data$log10padj <- plot_data$log10padjOrg
        }
        plot_data$y <- plot_data$log10padj
    } else if (mainplot == "maplot") {
        plot_data$x <- (plot_init_data$x + plot_init_data$y) / 2
        plot_data$y <- plot_init_data$y - plot_init_data$x
    }
    list( data = (plot_data))
}

#' mainPlotControlsUI
#'
#' Generates the left menu to be used for main plots
#'
#' @note \code{mainPlotControlsUI}
#' @param id, module ID
#' @return mainPlotControls
#' @examples
#'     x <- mainPlotControlsUI("main")
#' @export
#'
mainPlotControlsUI <- function(id) {
    ns <- NS(id)
    list(shinydashboard::menuItem(" Plot Type",
        startExpanded=TRUE,
        radioButtons(ns("mainplot"), "Main Plots:",
        c(Scatter = "scatter", VolcanoPlot = "volcano",
        MAPlot = "maplot"))
    ),
    shinydashboard::menuItem("Main Options",
        startExpanded=TRUE,
        sliderInput(ns("backperc"), "Background Data(%):",
        min=10, max=100, value=10, sep = "", animate = FALSE),
        uiOutput(ns("volcanoControlsUI")),
        uiOutput(ns("mainPlotControlsUI"))
    ))
    
}

#' getLegendColors
#'
#' Generates colors according to the data
#'
#' @note \code{getLegendColors}
#' @param Legend, unique Legends
#' @return mainPlotControls
#' @examples
#'     x <- getLegendColors(c("up", "down", "GS", "NS"))
#' @export
#'

getLegendColors<-function(Legend=c("up", "down", "NS"))
{
    colors <- c()
    for(i in seq(1:length(Legend))){
        if (Legend[i]=="Up"){
            colors <- c(colors, "red")
        }
        else if (Legend[i]=="Down"){
            colors <- c(colors, "blue")
        }
        else if (Legend[i]=="NS"){
            colors <- c(colors, "grey")
        }
        else if (Legend[i]=="GS"){
            colors <- c(colors, "green")
        }
    }
    colors
}
#' getLevelOrder
#'
#' Generates the order of the overlapping points 
#'
#' @note \code{getLevelOrder}
#' @param Level, factor levels shown in the legend
#' @return order
#' @examples
#'     x <- getLevelOrder(c("up", "down", "GS", "NS"))
#' @export
#'

getLevelOrder<-function(Level=c("up", "down", "NS"))
{
    levels <- c( "NS", "Up", "Down", "GS")
    for(i in seq(1:length(levels)))
    {
        if(!levels[i] %in% Level){
            levels <- levels[-(i)]
        }
    }
    levels
}

#' generateTestData
#'
#' This generates a test data that is suitable to main plots in debrowser
#' @param dat, DESeq results will be generated for loaded data
#' @return testData
#'
#' @examples
#'     x <- generateTestData()
#'
#' @export
#'
generateTestData <- function(dat = NULL) {
    if (is.null(dat)) return (NULL)
    ##################################################
    columns <- dat$columns
    conds <- dat$conds
    data <- dat$data
    params <-
        #Run DESeq2 with the following parameters
        c("DESeq2","NoCovariate", "parametric", F, "Wald", "None")
    non_expressed_cutoff <- 10
    data <- subset(data, rowSums(data) > 10)
    deseqrun <- runDE(data, metadata, columns, conds, params)
    
    met <- as.data.frame(cbind(as.vector(conds), columns))
    colnames(met) <- c("conds", "columns")
    cols1 <- as.vector(met[met$conds==as.character(unique(met$conds)[1]), "columns"])
    cols2 <- as.vector(met[met$conds==as.character(unique(met$conds)[2]), "columns"])
    
    de_res <- data.frame(deseqrun)
    norm_data <- getNormalizedMatrix(data[, columns])
    rdata <- cbind(rownames(de_res), norm_data[rownames(de_res), columns],
        log10(rowMeans(norm_data[rownames(de_res),cols1])
        + 0.1), log10( rowMeans( norm_data[ rownames( de_res ), cols2])
        + 0.1), de_res[rownames(de_res),
        c("padj", "log2FoldChange")], 2 ^ de_res[rownames(de_res),                                                                                            "log2FoldChange"], -1 *
        log10(de_res[rownames(de_res), "padj"]))
    colnames(rdata) <- c("ID", columns, "x", "y", "padj",
        "log2FoldChange", "foldChange", "log10padj")
    rdata <- as.data.frame(rdata)
    rdata$padj[is.na(rdata$padj)] <- 1
    
    padj_cutoff <- 0.01
    foldChange_cutoff <- 2
    
    
    rdata$Legend <- "NS"
    rdata$Legend[rdata$log2FoldChange > log2(foldChange_cutoff) &
                     rdata$padj < padj_cutoff] <- "Up"
    rdata$Legend[rdata$log2FoldChange <= log2(1 / foldChange_cutoff) &
                     rdata$padj < padj_cutoff] <- "Down"
    
    dat <- rdata
    dat$M <- rdata$x - rdata$y
    dat$A <- (rdata$x + rdata$y) / 2
    dat
}

#' debrowserbatcheffect
#'
#' Module to correct batch effect
#' 
#' @param input, input variables
#' @param output, output objects
#' @param session, session 
#' @param ldata, loaded data
#' @return main plot
#'
#' @return panel
#' @export
#'
#' @examples
#'     x <- debrowserbatcheffect()
#'
debrowserbatcheffect <- function(input, output, session, ldata = NULL) {
    if(is.null(ldata)) return(NULL)
    batchdata <- reactiveValues(count=NULL, meta = NULL)
    observeEvent(input$submitBatchEffect, {
    if (is.null(ldata$count)) return (NULL)

    countData <- ldata$count
    withProgress(message = 'Normalization', detail = "Normalization", value = NULL, {
        if (input$norm_method != "none"){
            countData <- getNormalizedMatrix(ldata$count, method=input$norm_method)
        }
    })
    withProgress(message = 'Batch Effect Correction', detail = "Adjusting the Data", value = NULL, {
    if (input$batchmethod == "CombatSeq" |  input$batchmethod == "Combat"){
        batchdata$count <- correctCombat(input, countData, ldata$meta, method = input$batchmethod)
    }
    else if (input$batchmethod == "Harman"){
        batchdata$count <- correctHarman(input, countData, ldata$meta)
    }
    else{
        batchdata$count <-  countData
    }
    })
    if (is.null(batchdata$count)) return(NULL)
    batchdata$meta <- ldata$meta
  })
  
  output$batchfields <- renderUI({
    if (!is.null(ldata$meta))
        list( conditionalPanel(condition = paste0("input['", session$ns("batchmethod"),"']!='none'"),
             selectGroupInfo( ldata$meta, input, session$ns("treatment"), "Treatment"),
             selectGroupInfo( ldata$meta, input, session$ns("batch"), "Batch")))
  })
  
  batcheffectdata <- reactive({
    ret <- NULL
    if(!is.null(batchdata$count)){
      ret <- batchdata
    }
    return(ret)
  })
  
  observe({
    getSampleDetails(output, "uploadSummary", "sampleDetails", ldata)
    getSampleDetails(output, "filteredSummary", "filteredDetails", batcheffectdata())
    getTableDetails(output, session, "beforebatchtable", ldata$count, modal = TRUE)
    callModule(debrowserpcaplot, "beforeCorrectionPCA", ldata$count, ldata$meta)
    callModule(debrowserIQRplot, "beforeCorrectionIQR",  ldata$count)
    callModule(debrowserdensityplot, "beforeCorrectionDensity", ldata$count)
    if ( !is.null(batcheffectdata()$count ) && nrow(batcheffectdata()$count)>2 ){
      withProgress(message = 'Drawing the plot', detail = "Preparing!", value = NULL, {
       getTableDetails(output, session, "afterbatchtable", batcheffectdata()$count, modal = TRUE)
       callModule(debrowserpcaplot, "afterCorrectionPCA",  batcheffectdata()$count, batcheffectdata()$meta)
       callModule(debrowserIQRplot, "afterCorrectionIQR",  batcheffectdata()$count)
       callModule(debrowserdensityplot, "afterCorrectionDensity", batcheffectdata()$count)
      })
    }
  })
  
  list(BatchEffect=batcheffectdata)
}


#' batchEffectUI
#' Creates a panel to coorect batch effect
#'
#' @param id, namespace id
#' @return panel
#' @examples
#'     x <- batchEffectUI("batcheffect")
#'
#' @export
#'
batchEffectUI <- function (id) {
  ns <- NS(id)

  list(
    fluidRow(
        shinydashboard::box(title = "Batch Effect Correction and Normalization",
        solidHeader = TRUE, status = "info",  width = 12, 
        fluidRow(
            column(5,div(style = 'overflow: scroll',
                tableOutput(ns("uploadSummary")),
                DT::dataTableOutput(ns("sampleDetails"))),
                uiOutput(ns("beforebatchtable"))
            ),
            column(2,
            shinydashboard::box(title = "Options",
                solidHeader = TRUE, status = "info",
                width = 12, 
                normalizationMethods(id),
                batchMethod(id),
                uiOutput(ns("batchfields")),
                actionButtonDE(ns("submitBatchEffect"), label = "Submit", styleclass = "primary")
           )
          ),
          column(5,div(style = 'overflow: scroll', 
                tableOutput(ns("filteredSummary")),
                DT::dataTableOutput(ns("filteredDetails"))),
                uiOutput(ns("afterbatchtable"))
          )
        ),
        conditionalPanel(condition = paste0("input['", ns("submitBatchEffect"),"']"),
        actionButtonDE("goDE", "Go to DE Analysis", styleclass = "primary"),
        actionButtonDE("goQCplots", "Go to QC plots", styleclass = "primary"))),
    shinydashboard::box(title = "Plots",
        solidHeader = TRUE, status = "info",  width = 12, 
        fluidRow(column(1, div()),
            tabsetPanel( id = ns("batchTabs"),
                tabPanel(id = ns("PCA"), "PCA",
                    column(5,
                        getPCAPlotUI(ns("beforeCorrectionPCA"))),
                    column(2,  
                        shinydashboard::box(title = "PCA Controls",
                        solidHeader = T, status = "info",  width = 12, 
                        tabsetPanel( id = ns("pcacontrols"),
                        tabPanel ("Before",
                        pcaPlotControlsUI(ns("beforeCorrectionPCA"))),
                        tabPanel ( "After",
                        pcaPlotControlsUI(ns("afterCorrectionPCA")))))),
                    column(5,
                        getPCAPlotUI(ns("afterCorrectionPCA")))
                ),
                tabPanel(id = ns("IQR"), "IQR",
                    column(5,
                        getIQRPlotUI(ns("beforeCorrectionIQR"))),
                    column(2, div()),
                    column(5,
                        getIQRPlotUI(ns("afterCorrectionIQR")))
                ),
                tabPanel(id = ns("Density"), "Density",
                    column(5,
                        getDensityPlotUI(ns("beforeCorrectionDensity"))),
                    column(2, div()),
                    column(5,
                        getDensityPlotUI(ns("afterCorrectionDensity")))
                )
            )
        )
      )
    ), getPCAcontolUpdatesJS())
}
#' normalizationMethods
#'
#' Select box to select normalization method prior to batch effect correction
#'
#' @note \code{normalizationMethods}
#' @param id, namespace id
#' @return radio control
#'
#' @examples
#'    
#'     x <- normalizationMethods("batch")
#'
#' @export
#'
normalizationMethods <- function(id) {
    ns <- NS(id)
    selectInput(ns("norm_method"), "Normalization Method:",
        choices = c("none", "MRN", "TMM", "RLE", "upperquartile"))
}

#' batchMethod
#'
#' select batch effect method
#' @param id, namespace id
#' @note \code{batchMethod}
#' @return radio control
#'
#' @examples
#'    
#'     x <- batchMethod("batch")
#'
#' @export
#'
batchMethod <- function(id) {
  ns <- NS(id)
  selectInput(ns("batchmethod"), "Correction Method:",
              choices = c("none", "Combat", "CombatSeq", "Harman"),
               selected='none'
  )
}

#' Correct Batch Effect using Combat in sva package
#'
#' Batch effect correction
#' @param input, input values
#' @param idata, data
#' @param metadata, metadata
#' @param method, method: either Combat or CombatSeq
#' @return data
#' @export
#'
#' @examples
#'     x<-correctCombat ()
correctCombat <- function (input = NULL, idata = NULL, metadata = NULL, method = NULL) {
  if (is.null(idata)) return(NULL)
  
  if (input$batch == "None") {
      showNotification("Please select the batch field to use Combat!", type = "error")
      return(NULL)
  }
  
  batch <- metadata[, input$batch]
  
  columns <- colnames(idata)
  datacor <- data.frame(idata[, columns])
  datacor[, columns] <- apply(datacor[, columns], 2,
      function(x) as.integer(x) + runif(1, 0, 0.01))
  
  if (input$treatment != "None") {
      treatment <- metadata[, input$treatment]
      meta <- data.frame(cbind(columns, treatment, batch))
      modcombat = model.matrix(~as.factor(treatment), data = meta)
      if(method == "Combat"){
        combat_res = sva::ComBat(dat=as.matrix(datacor), mod=modcombat, batch=batch)
      } else {
        combat_res = sva::ComBat_seq(counts=as.matrix(datacor), covar_mod = modcombat, batch=batch)
      }
  } else {
      if(method == "Combat"){
        combat_res = sva::ComBat(dat=as.matrix(datacor), batch=batch)
      } else {
        combat_res = sva::ComBat_seq(counts=as.matrix(datacor), batch=batch)
      }
  }
  
  a <- cbind(idata[rownames(combat_res), 2], combat_res)
  a[, columns] <- apply(a[, columns], 2, function(x) ifelse(x<0, 0, x))
  a[, columns] <- apply(a[, columns], 2, function(x) as.integer(x))
  colnames(a[, 1]) <- colnames(idata[, 1])
  a[,columns]
}

#' Correct Batch Effect using Harman
#'
#' Batch effect correction
#' @param input, input values
#' @param idata, data
#' @param metadata, metadata
#' @return data
#' @export
#'
#' @examples
#'     x<-correctHarman ()
correctHarman <- function (input = NULL, idata = NULL, metadata = NULL) {
  if (is.null(idata)) return(NULL)
  if (input$treatment == "None" || input$batch == "None") {
      showNotification("Please select the batch and treatment fields to use Harman!", type = "error")
      return(NULL)
  }

  batch.info <- data.frame(metadata[, c(input$treatment, input$batch)])
  rownames(batch.info) <- rownames(metadata)
  colnames(batch.info) <- c("treatment", "batch") 
  
  harman.res <- harman(idata, expt= batch.info$treatment, batch= batch.info$batch, limit=0.95)
  harman.corrected <- reconstructData(harman.res)
  harman.corrected[harman.corrected<0] <- 0
  harman.corrected
}

#' getSampleDetails
#' 
#' get sample details
#'
#' @param output, output
#' @param summary, summary output name
#' @param details, details ouput name
#' @param data, data 
#' @return panel
#' @examples
#'     x <- getSampleDetails()
#'
#' @export
#'
getSampleDetails<- function (output = NULL, summary = NULL, details = NULL, data = NULL) {
    if (is.null(data)) return(NULL)
    
    output[[summary]]<- renderTable({ 
        countdata <-  data$count
        samplenums <- length(colnames(countdata))
        rownums <- dim(countdata)[1]
        result <- rbind(samplenums, rownums)
        rownames(result) <- c("# of samples", "# of rows (genes/regions)")
        colnames(result) <- "Value"
        result
    },digits=0, rownames = TRUE, align="lc")
    
    output[[details]] <- DT::renderDataTable({ 
        dat <- colSums(data$count)
        dat <- cbind(names(dat), dat)
        dat[, c("dat")] <-  format(
            round( as.numeric( dat[,  c("dat")], digits = 2)),
            big.mark=",",scientific=FALSE)
        
        if (!is.null(data$meta)){
            met <- data$meta
            dat <- cbind(met, dat[,"dat"])
            rownames(dat) <- NULL
            colnames(dat)[ncol(dat)] <- "read counts"
        }else{
            rownames(dat) <- NULL
            colnames(dat) <- c("samples", "read counts")
        }
        dat
    })
}

#' selectGroupInfo
#'
#' Group info column selection. This can be used in batch effect
#' or coloring the groups in the plots.
#' @param metadata, metadata
#' @param input, input values
#' @param selectname, name of the select box
#' @param label, label of the select box
#' @note \code{selectGroupInfo}
#' @examples
#'     x <- selectGroupInfo()
#' @export
#'
selectGroupInfo <- function(metadata = NULL, input = NULL,
                              selectname = "groupselect",
                              label = "Group info") {
    if (is.null(metadata)) return (NULL)
    lst.choices <- as.list(c("None", colnames(metadata)))
    selectInput(selectname, label = label,
                choices = lst.choices,
                selected = 1)
}


#' addID
#'
#' Adds an id to the data frame being used.
#'
#' @param data, loaded dataset
#' @return data
#' @export
#'
#' @examples
#'     x <- addID()
#'
addID <- function(data = NULL) {
    if (is.null(data)) return (NULL)
    dat1 <- data.frame(data)
    dat1 <- cbind(rownames(data), data)
    colnames(dat1) <- c("ID", colnames(data))
    dat1
}

#' getVariationData
#'
#' Adds an id to the data frame being used.
#'
#' @param inputdata, dataset 
#' @param cols, columns
#' @param conds, conditions
#' @param key, gene or region name
#' @return plotdata
#' @export
#'
#' @examples
#'     x <- getVariationData()
#'
getVariationData <- function(inputdata = NULL, 
    cols = NULL, conds = NULL, key = NULL) {
    if (is.null(inputdata)) return (NULL)
    # Pick out the gene with this ID
    vardata <- inputdata[key, ]
    bardata <- as.data.frame(cbind(key, cols,
        t(vardata[, cols]), as.character(conds)) )
    colnames(bardata) <- c("genename", "libs", "count", "conds")
    bardata$count <- as.numeric(as.character(bardata$count))
    bardata$conds <- factor(bardata$conds)
    data <- rbind(bardata[bardata$conds == levels(bardata$conds)[1], ],
        bardata[bardata$conds == levels(bardata$conds)[2], ])
    data$conds  <- factor(data$conds, levels = unique(data$conds))
    data
}

#' getBSTableUI
#' prepares a Modal to put a table
#' @param name, name 
#' @param label, label
#' @param trigger, trigger button for the modal
#' @param size, size of the modal
#' @param modal, modal yes/no
#' @return the modal
#'
#' @examples
#'     x<- getBSTableUI()
#'
#' @export
getBSTableUI<-function(name = NULL,  label = NULL, trigger = NULL, size="large", modal = NULL){
    if (is.null(name)) return (NULL)
    ret <- div(style = "display:block;overflow-y:auto; overflow-x:auto;",
               wellPanel( DT::dataTableOutput(name)))
    if (!is.null(modal) && modal)
        ret <- shinyBS::bsModal(name, label, trigger, size = size, ret)
    ret
}

#' getTableDetails
#' 
#' get table details
#' To be able to put a table into two lines are necessary;
#' into the server part;
#' getTableDetails(output, session, "dataname", data, modal=TRUE)
#' into the ui part;
#' uiOutput(ns("dataname"))
#'   
#' @param output, output
#' @param session, session
#' @param tablename, table name
#' @param data, matrix data
#' @param modal, if it is true, the matrix is going to be in a modal
#' @return panel
#' @examples
#'     x <- getTableDetails()
#'
#' @export
#'
getTableDetails <- function(output  = NULL, session  = NULL, tablename  = NULL, data = NULL, modal = NULL){
    if (is.null(data)) return(NULL)
    tablenameUI <-  paste0(tablename,"Table")
    output[[paste(tablename, "Download")]] <- downloadHandler(
        filename = function() {
            paste0(tablename,".tsv")
        },
        content = function(file) {
            if(!("ID" %in% names(data)))
                data <- addID(data)
            write.table(data, file, sep = "\t", row.names = FALSE)
        }
    )
    
    output[[tablename]] <- renderUI({
        ret <- getBSTableUI( session$ns(tablenameUI), "Show Data", paste0("show",tablename), modal = modal) 
        if (!is.null(modal) && modal)
           ret <- list( downloadButton(session$ns(paste(tablename, "Download")), "Download"),
               actionButtonDE(paste0("show",tablename), "Show Data", styleclass = "primary", icon="show"),
               ret)
        ret    
    })
    
    output[[tablenameUI]] <- DT::renderDataTable({
        if (!is.null(data)){
            DT::datatable(data, extensions = 'Buttons',
            options = list( server = TRUE,
            dom = "Blfrtip",
            buttons = 
              list("copy", list(
                  extend = "collection"
                  , buttons = c("csv", "excel", "pdf")
                  , text = "Download"
              ) ), # end of buttons customization
            
            # customize the length menu
            lengthMenu = list( c(10, 20,  50, -1) # declare values
                               , c(10, 20, 50, "All") # declare titles
            ), # end of lengthMenu customization
            pageLength = 10))
        }
    })
}

#' push
#'
#' Push an object to the list.
#'
#' @param l, that are going to push to the list
#' @param ..., list object
#' @return combined list
#'
#' @export
#'
#' @examples
#'     mylist <- list()
#'     newlist <- push ( 1, mylist )
push <- function(l, ...) c(l, list(...))

#' round_vals
#'
#' Plot PCA results.
#'
#' @param l, the value
#' @return round value
#' @export
#'
#' @examples
#'     x<-round_vals(5.1323223)
round_vals <- function(l) {
    l <- round(as.numeric(l), digits = 2)
    parse(text = l)
}

#' Buttons including Action Buttons and Event Buttons
#'
#' Creates an action button whose value is initially zero, and increments by one
#' each time it is pressed.
#'
#' @param inputId Specifies the input slot that will be used to access the
#'   value.
#' @param label The contents of the button--usually a text label, but you could
#'   also use any other HTML, like an image.
#' @param styleclass The Bootstrap styling class of the button--options are
#'   primary, info, success, warning, danger, inverse, link or blank
#' @param size The size of the button--options are large, small, mini
#' @param block Whehter the button should fill the block
#' @param icon Display an icon for the button
#' @param css.class Any additional CSS class one wishes to add to the action
#'   button
#' @param ... Other argument to feed into shiny::actionButton
#'
#' @export
#'
#' @examples
#'     actionButtonDE("goDE", "Go to DE Analysis")
#'
actionButtonDE <- function(inputId, label, styleclass = "", size = "",
        block = FALSE, icon = NULL, css.class = "", ...) {
    if (styleclass %in% c("primary", "info", "success", "warning",
        "danger", "inverse", "link")) {
        btn.css.class <- paste("btn", styleclass, sep = "-")
    } else btn.css.class = ""
    
    if (size %in% c("large", "small", "mini")) {
        btn.size.class <- paste("btn", size, sep = "-")
    } else btn.size.class = ""
    
    if (block) {
        btn.block = "btn-block"
    } else btn.block = ""
    
    if (!is.null(icon)) {
        icon.code <- HTML(paste0("<i class='fa fa-", icon, "'></i>"))
    } else icon.code = ""
    tags$button(id = inputId, type = "button", class = paste("btn action-button",
        btn.css.class, btn.size.class, btn.block, css.class, collapse = " "),
        icon.code, label, ...)
}


#' getNormalizedMatrix
#'
#' Normalizes the matrix passed to be used within various methods
#' within DEBrowser.  Requires edgeR package
#'
#' @note \code{getNormalizedMatrix}
#' @param M, numeric matrix
#' @param method, normalization method for edgeR. default is TMM
#' @return normalized matrix
#'
#' @examples
#'     x <- getNormalizedMatrix(mtcars)
#'
#' @export
#'
getNormalizedMatrix <- function(M = NULL, method = "TMM") {
    if (is.null(M) ) return (NULL)
    M[is.na(M)] <- 0
    norm <- M
    if (!(method == "none" || method == "MRN")){
        norm.factors <- edgeR::calcNormFactors(M, method = method)
        norm <- edgeR::equalizeLibSizes(edgeR::DGEList(M,
            norm.factors = norm.factors))$pseudo.counts
    }else if(method == "MRN"){
        columns <- colnames(M)
        conds <- columns
        coldata <- prepGroup(conds, columns)
        M[, columns] <- apply(M[, columns], 2,
            function(x) as.integer(x))
        dds <- DESeqDataSetFromMatrix(countData = as.matrix(M),
            colData = coldata, design = ~group)
        dds <- estimateSizeFactors(dds)
        norm <- counts(dds, normalized=TRUE)
    }
    return(norm)
}

#' getCompSelection
#'
#' Gathers the user selected comparison set to be used within the
#' DEBrowser.
#' @param name, the name of the selectInput
#' @param count, comparison count
#' @note \code{getCompSelection}
#' @examples
#'     x <- getCompSelection(name="comp", count = 2)
#' @export
#'
getCompSelection <- function(name = NULL, count = NULL) {
    a <- NULL
    if (count>1){
        a <- list(selectInput(name,
            label = "Choose a comparison:",
            choices = c(1:count)))
    }
    a
}
#' getHelpButton
#' prepares a helpbutton for to go to a specific site in the documentation
#'
#' @param name, name that are going to come after info
#' @param link, link of the help
#' @return the info button
#'
#' @examples
#'     x<- getHelpButton()
#'
#' @export
getHelpButton<-function(name = NULL, link = NULL){
    if (is.null(name)) return(NULL)
    btn <- actionButtonDE(paste0("info_",name),"",icon="info",
        styleclass="info", size="small")
    
    HTML(paste0("<a id=\"info_",name,"\" href=\"",link,"\" target=\"_blank\">",
       btn,"</a>"))
    
}

#' getDomains
#'
#' Get domains for the main plots.
#'
#' @param filt_data, data to get the domains
#' @return domains
#' @export
#'
#' @examples
#'     x<-getDomains()
getDomains <- function(filt_data = NULL){
    if (is.null(filt_data)) return (NULL)
    a <- unique(filt_data$Legend)
    a <- a[a != ""]
    if (length(a) == 1)
        a <- c(a, "NA")
    a
}

#' getColors
#'
#' get colors for the domains 
#'
#' @param domains, domains to be colored
#' @return colors
#' @export
#'
#' @examples
#'     x<-getColors()
#'
getColors <- function(domains = NULL){
    if (is.null(domains)) return (NULL)
    colors <- c()
    for ( dn in seq(1:length(domains)) ){
        if (domains[dn] == "NS" || domains[dn] == "NA")
            colors <- c(colors, "#aaa")
        else if (domains[dn] == "Up")
            colors <- c(colors, "green")
        else if (domains[dn] == "Down")
            colors <- c(colors, "red")
        else if (domains[dn] == "MV")
            colors <- c(colors, "orange")
        else if (domains[dn] == "GS")
            colors <- c(colors, "blue")
    } 
    colors
}

#' getKEGGModal
#' prepares a modal for KEGG plots
#'
#' @return the info button
#'
#' @examples
#'     x<- getKEGGModal()
#'
#' @export
getKEGGModal<-function(){
    bsModal("modalExample", "KEGG Pathway", "KeggPathway", size = "large",
            div(style = "display:block;overflow-y:auto; overflow-x:auto;",imageOutput("KEGGPlot")))
}

#' getTableModal
#' prepares table modal for KEGG
#'
#' @return the info button
#'
#' @examples
#'     x<- getTableModal()
#'
#' @export
getTableModal<-function(){
    bsModal("modalTable", "Genes in the category", "GeneTableButton", size = "large",
            div(style = "display:block;overflow-y:auto; overflow-x:auto;",
                wellPanel( DT::dataTableOutput("GOGeneTable"))))
}

#' setBatch
#' to skip batch effect correction batch variable set with the filter results
#'
#' @param fd, filtered data
#' @return fd data
#'
#' @examples
#'    
#'     x <- setBatch()
#'
#' @export
#'
setBatch <- function(fd = NULL){
    if(!is.null(fd)){ 
        batchdata <- reactiveValues(count=NULL, meta = NULL)
        batchdata$count <-  fd$filter()$count
        batchdata$meta <-  fd$filter()$meta
        batcheffectdata <- reactive({
            ret <- NULL
            if(!is.null(batchdata$count)){
                ret <- batchdata
            }
            return(ret)
        })
        list(BatchEffect=batcheffectdata)
    }
}
#' getTabUpdateJS
#' prepmenu tab and discovery menu tab updates
#'
#' @return the JS for tab updates
#'
#' @examples
#'     x<- getTabUpdateJS()
#'
#' @export
getTabUpdateJS<-function(){
    tags$script(HTML( "
                      $(function() {
                      $('#methodtabs').attr('selectedtab', '2')
                      $($('#methodtabs >')[0]).attr('id', 'dataprepMethod')
                      $($('#menutabs >')[0]).attr('id', 'dataprepMenu')
                      for(var i=1;i<=5;i++){
                      $($('#methodtabs >')[i]).attr('id', 'discoveryMethod')
                      }
                      $($('#menutabs >')[1]).attr('id', 'discoveryMenu')
                      $(document).on('click', '#dataprepMethod', function () {
                      if($('#dataprepMenu').attr('class')!='active'){   
                      $('#dataprepMenu').find('a').click()
                      }
                      });
                      $(document).on('click', '#dataprepMenu', function () {
                      if($('#dataprepMethod').attr('class')!='active'){   
                      $('#dataprepMethod').find('a').click()
                      }
                      });
                      $(document).on('click', '#discoveryMethod', function () {
                      $('#methodtabs').attr('selectedtab', $(this).index())
                      if($('#discoveryMenu').attr('class')!='active'){   
                      $('#discoveryMenu').find('a').click()
                      }
                      });
                      $('#discoveryMenu > ').css('display', 'none');
                      $(document).on('click', '#goMain', function () {
                      $('#discoveryMenu > ').css('display', 'block');
                      });
                      $(document).on('click', '#discoveryMenu', function () {
                      $($('#methodtabs >')[ $('#methodtabs').attr('selectedtab')]).find('a').click()
                      });
                      //hide buttons on entrance
                      $('.sidebar-menu > ').css('display', 'none');
                      $('.sidebar-menu > :nth-child(1)').css('display', 'inline');
                      $('.sidebar-menu > :nth-child(2)').css('display', 'inline');
                      $(document).on('click', '#Filter', function () {
                      $('.sidebar-menu > :nth-child(2)').css('display', 'inline');
                      $('.sidebar-menu > :nth-child(3)').css('display', 'inline');
                      $('.sidebar-menu > :nth-child(4)').css('display', 'none');
                      $('.sidebar-menu > :nth-child(5)').css('display', 'none');
                      $('.sidebar-menu > :nth-child(6)').css('display', 'none');
                      $('.sidebar-menu > :nth-child(7)').css('display', 'none');
                      });
                      $(document).on('click', '#Batch', function () {
                      $('.sidebar-menu > :nth-child(4)').css('display', 'inline');
                      $('.sidebar-menu > :nth-child(5)').css('display', 'none');
                      $('.sidebar-menu > :nth-child(6)').css('display', 'none');
                      $('.sidebar-menu > :nth-child(7)').css('display', 'none');
                      });
                      $(document).on('click', '#goDEFromFilter', function () {
                      $('.sidebar-menu > :nth-child(5)').css('display', 'inline');
                      $('.sidebar-menu > :nth-child(6)').css('display', 'none');
                      $('.sidebar-menu > :nth-child(7)').css('display', 'none');
                      });
                      $(document).on('click', '#goDE', function () {
                      $('.sidebar-menu > :nth-child(5)').css('display', 'inline');
                      $('.sidebar-menu > :nth-child(6)').css('display', 'none');
                      $('.sidebar-menu > :nth-child(7)').css('display', 'none');
                      });
                      $(document).on('click', '#startDE', function () {
                      $('.sidebar-menu > :nth-child(6)').css('display', 'inline');
                      $('.sidebar-menu > :nth-child(7)').css('display', 'inline');
                      $('.sidebar-menu > :nth-child(2)').css('display', 'none');
                      });
                      })
                      "))
}
#' getPCAcontolUpdatesJS
#' in the prep menu we have two PCA plots to show how batch effect correction worked. 
#' One set of PCA input controls updates two PCA plots with this JS.
#' @return the JS for tab updates
#'
#' @examples
#'     x<- getTabUpdateJS()
#'
#' @export
getPCAcontolUpdatesJS<-function(){
    tags$script(HTML("  
                        var nameInputs = ['pcselx', 'pcsely'];
                        $.each(nameInputs, function (el) {
                                              $(function () {
                                              $(document).on('change keyup', '#batcheffect-beforeCorrectionPCA-'+nameInputs[el], function () {
                                              var value = $(this).val();
                                              $('#batcheffect-afterCorrectionPCA-'+nameInputs[el]).val(value)
                                              $('#batcheffect-afterCorrectionPCA-'+nameInputs[el]).trigger('change');
                                              });
                                              });
                        });
                        
                        var nameDropdowns = [ 'legendonoff', 'legendSelect', 'text_pca', 'color_pca','shape_pca'];
                        $.each(nameDropdowns, function (el) {
                            $(function () {
                                $(document).on('change', '#batcheffect-beforeCorrectionPCA-'+nameDropdowns[el], function () {
                                    var value = $(this).val();
                                    $('#batcheffect-afterCorrectionPCA-'+nameDropdowns[el])[0].selectize.setValue(value)
                                    $('#batcheffect-afterCorrectionPCA-'+nameDropdowns[el]).trigger('change');
                                });
                            });
                        });
                     $($('#batcheffect-pcacontrols')[0]).css('display', 'none');                     
                     $(document).on('click','#batcheffect-submitBatchEffect',function () {
                     setTimeout(function () { $($($('#batcheffect-pcacontrols')[0]).children()[1]).children().trigger('click')
                     setTimeout(function () { $($($('#batcheffect-pcacontrols')[0]).children()[0]).children().trigger('click')}, 1000);
                     }, 1000); })
                     "))
}

.initial <- function() {
    req <- function(...){
    reqFun <- function(pack) {
        if(!suppressWarnings(suppressMessages(require(pack, character.only = TRUE)))) {
            message(paste0("unable to load package ", pack))
            require(pack, character.only = TRUE)
        }
    }
    lapply(..., reqFun)
    }
    packs <- c("debrowser", "plotly", "shiny", "jsonlite", "shinyjs", "shinydashboard", "shinyBS")
    req(packs)
}

.onAttach <- function(libname, pkgname) {
    pkgVersion <- packageDescription("debrowser", fields="Version")
    msg <- paste0("DEBrowser v", pkgVersion, "  ",
                  "For help: https://debrowser.readthedocs.org/", "\n\n")
    
    citation <- paste0("If you use DEBrowser in published research, please cite:\n\n",
                       "Alper Kucukural, Onur Yukselen, Deniz M. Ozata, Melissa J. Moore, Manuel Garber\n", 
                       "DEBrowser: Interactive Differential Expression Analysis and Visualization Tool for Count Data\n",
                       "BMC Genomics 2019 20:6\n\ndoi:0.1186/s12864-018-5362-x\n")
    
    packageStartupMessage(paste0(msg, citation))
    .initial()

}

library(debrowser)
library(testthat)

load(system.file("extdata", "demo", "demodata.Rda",
    package = "debrowser"))
metadata <- metadatatable 
columns <- c("exper_rep1", "exper_rep2", "exper_rep3",
    "control_rep1", "control_rep2", "control_rep3")
conds <- factor( c("Control", "Control", "Control",
    "Treat", "Treat", "Treat") )
data <- data.frame(demodata[, columns])

#Run DESeq2 with the following parameters
params <- c("DESeq2", "NoCovariate", "parametric", F, "Wald", "None") 
non_expressed_cutoff <- 10
data <- subset(data, rowSums(data) > 10)
test_that("Able to run DESeq2", {
    deseqrun <- runDE(data, metadata, columns, conds, params)
    expect_true(exists("deseqrun"))
})

##################################################
deseqrun <- runDE(data, metadata, columns, conds, params)

de_res <- data.frame(deseqrun)
norm_data <- getNormalizedMatrix(data[, columns])
rdata <- cbind(rownames(de_res), norm_data[rownames(de_res), columns],
                log10(rowMeans(norm_data[rownames(de_res),
                paste(c("exper_rep1", "exper_rep2", "exper_rep3"))])
                + 0.1), log10( rowMeans( norm_data[ rownames( de_res ),
                paste(c("control_rep1", "control_rep2", "control_rep3"))])
                + 0.1), de_res[rownames(de_res),
                c("padj", "log2FoldChange")], 2 ^ de_res[rownames(de_res),
                "log2FoldChange"], -1 *
                log10(de_res[rownames(de_res), "padj"]))
colnames(rdata) <- c("ID", columns, "Cond1", "Cond2", "padj",
                "log2FoldChange", "foldChange", "log10padj")
rdata <- as.data.frame(rdata)
rdata$padj[is.na(rdata$padj)] <- 1

padj_cutoff <- 0.01
foldChange_cutoff <- 2

rdata$Legend <- character(nrow(rdata))
rdata$Legend[rdata$log2FoldChange > log2(foldChange_cutoff) &
        rdata$padj < padj_cutoff] <- "Up"
rdata$Legend[rdata$log2FoldChange < log2(1 / foldChange_cutoff) &
        rdata$padj < padj_cutoff] <- "Down"
rdata$Legend[abs(rdata$log2FoldChange) <= 
        log2(foldChange_cutoff)] <- "NS"
rdata$Legend[is.null(rdata$log10padj)] <- "NA"
rdata$Size <- character(nrow(rdata))
rdata[, "Size"] <- "40"

dat <- rdata
dat$M <- rdata$Cond1 - rdata$Cond2
dat$A <- (rdata$Cond1 + rdata$Cond2) / 2

updown <- rdata[rdata$Legend=="Up" | rdata$Legend=="Down",columns]
##################################################

library(debrowser)
library(plotly)
source("../../R/barmain.R")

options(warn =-1)

header <- dashboardHeader(
  title = "DEBrowser Bar Plots"
)
sidebar <- dashboardSidebar(  sidebarMenu(id="DEAnlysis",
      menuItem("BarMain", tabName = "BarMain"),
      textInput("genename", "Gene/Region Name", value = "Foxa3" ),
      plotSizeMarginsUI("barmain", h=200, t = 30)
))

body <- dashboardBody(
  tabItems(
    tabItem(tabName="BarMain", 
        fluidRow(
        column(12,
            getBarMainPlotUI("barmain")))
    )
  ))

ui <- dashboardPage(header, sidebar, body, skin = "blue")

server <- function(input, output, session) {
    load(system.file("extdata", "demo", "demodata.Rda",
                   package = "debrowser"))
    
    cond_names <- c("Control", "Treat")
    observe({
        if (!is.null(input$genename))
            callModule(debrowserbarmainplot, "barmain", demodata, 
                metadatatable$sample,  
                metadatatable$treatment, cond_names, input$genename)
    })
}

shinyApp(ui, server)

#' getGoPanel
#'
#' Creates go term analysis panel within the shiny
#' display.
#' 
#' @note \code{getGoPanel}
#' @return the panel for go term analysis;
#'
#' @examples  
#'     x <- getGoPanel()
#'
#' @export
#' 
getGoPanel <- function(){
       gopanel <- list(
           wellPanel(helpText( "Please select parameters and press the 
                    submit button in the left menu for the plots"),
            getHelpButton("method", 
            "http://debrowser.readthedocs.io/en/master/examples/examples.html#go-term-plots")),
            tabsetPanel(id = "gotabs", type = "tabs",
                tabPanel(title = "Plot", value = "gopanel1", id = "gopanel1",
                     column(12, wellPanel( plotOutput("GOPlots1")))),
                tabPanel(title = "Table", value = "gopanel2", id = "gopanel2",
                     column(12, wellPanel( DT::dataTableOutput("gotable"))))
            ),
            getKEGGModal(),
            getTableModal()
        )
       return(gopanel)
}

#' getGOPlots
#'
#' Go term analysis panel.  Generates appropriate GO plot
#' based on user selection.
#'
#' @param dataset, the dataset used
#' @param GSEARes, GSEA results
#' @param input, input params
#' @note \code{getGOPlots}
#' @return the panel for go plots;
#'
#' @examples
#'     x<- getGOPlots()
#' @export
#' 
getGOPlots <- function(dataset = NULL, GSEARes = NULL, input = NULL){
    if (is.null(dataset)) return(NULL)
    goplots <- NULL
    org <- input$organism
    if (input$goplot == "disease")
        org <- "org.Hs.eg.db"
    genelist <- getGeneList(rownames(dataset), org)
    genelist <- genelist$ENTREZID
    gopval <- as.numeric(input$gopvalue)
    if (input$goplot == "enrichGO"){
        res <- getEnrichGO(genelist, ont = input$ontology,
            pvalueCutoff = gopval, org = input$organism)
        goplots<-res
        if (input$goextplot == "Dotplot")
            goplots$p <- dotplot(res$enrich_p, showCategory=30)
    }
    else if (input$goplot == "enrichKEGG"){
        res <- getEnrichKEGG(genelist, pvalueCutoff=
            gopval, org = input$organism)
        goplots<-res
        if (input$goextplot == "Dotplot")
            goplots$p <- dotplot(res$enrich_p, showCategory=30)
    }
    else if (input$goplot == "compare"){
        cl <- clusterData(dataset)
        res <- compareClust(cl, fun=input$gofunc, input$ontology,
            org = input$organism)
        goplots<-res
    }
    else if (input$goplot == "disease"){
        res <- getEnrichDO(genelist, pvalueCutoff = gopval )
        goplots<-res
        if (input$goextplot == "Dotplot")
            goplots$p <- dotplot(res$enrich_p, showCategory=30)
    }
    else if (input$goplot == "GSEA") {
        res <- GSEARes
        if (nrow(res$enrich_p@result)>0) {
            res$p <-  gseaplot(res$enrich_p, by = "all", 
                title = res$enrich_p$Description[1], geneSetID = 1)
            goplots <- res
            
            if (input$goextplot == "Dotplot")
                goplots$p <- dotplot(res$enrich_p, showCategory=10, 
                    split=".sign") + facet_grid(.~.sign)
        } else {
            showNotification("no term enriched under specific pvalueCutoff...", 
                type = "error")
        }
    }
    return(goplots)
}



#' getOrganismBox
#'
#' Get the organism Box.
#"
#' @note \code{getOrganismBox}
#'
#' @export
#'
#' @note \code{getOrganismBox}
#' makes the organism box
#' @return selectInput
#'
#' @examples
#'     x <- getOrganismBox()
#'
getOrganismBox <- function(){
    organismBox <- list(
        conditionalPanel( ( condition <- "input.goplot!='disease' &&
                            input.gofunc != 'enrichDO'"),
        selectInput("organism", "Choose an organism:",
        choices =  c( "Human" = "org.Hs.eg.db", 
            "Mouse" = "org.Mm.eg.db", 
            "Rat" = "org.Rn.eg.db", 
            "Zebrafish" = "org.Dr.eg.db",
            "Fly" = "org.Dm.eg.db",
            "Worm" = "org.Ce.eg.db",
            "Yeast" = "org.Sc.sgd.db",
            "Arabidopsis" = "org.At.tair.db"
        ))))
    return(organismBox)
}

#' getOrganism
#'
#' @param org, organism
#' @note \code{getOrganism}
#'
#' @export
#' @return organism name for keg
#'
#' @examples
#'     x <- getOrganism()
#'
getOrganism <- function(org){
    organisms <-  list("hsa", "mmu", "rno", 
                      "dre", "dme", "cel", "sce", "At")
    names(organisms) <- c("org.Hs.eg.db", 
                          "org.Mm.eg.db", 
                          "org.Rn.eg.db", 
                          "org.Dr.eg.db",
                          "org.Dm.eg.db",
                          "org.Ce.eg.db",
                          "org.Sc.sgd.db",
                          "org.At.tair.db")
    organisms[org][[1]]
}

#' getOrganismPathway
#'
#' @param org, organism
#' @note \code{getOrganismPathway}
#'
#' @export
#' @return organism name for pathway
#'
#' @examples
#'     x <- getOrganismPathway()
#'
getOrganismPathway <- function(org){
    organisms <- list("human", "mouse", "rat", 
                      "zebrafish", "fly", "celegans", "yeast", "arabidopsis")
    names(organisms) <- c("org.Hs.eg.db", 
                          "org.Mm.eg.db", 
                          "org.Rn.eg.db", 
                          "org.Dr.eg.db",
                          "org.Dm.eg.db",
                          "org.Ce.eg.db",
                          "org.Sc.sgd.db",
                          "org.At.tair.db")
    organisms[org][[1]]
}

library(debrowser)
library(DESeq2)
library(heatmaply)
library(RColorBrewer)
library(gplots)
source("../../R/heatmap.R")
source("../../R/plotSize.R")

options(warn=-1)
header <- dashboardHeader(
    title = "DEBrowser Heatmap"
)
sidebar <- dashboardSidebar(  getJSLine(), sidebarMenu(id="DataAssessment",
           menuItem("Heatmap", tabName = "Heatmap"),
           plotSizeMarginsUI("heatmap"),
           heatmapControlsUI("heatmap")))

body <- dashboardBody(
    tabItems(
        tabItem(tabName="Heatmap",  getHeatmapUI("heatmap"),
                column(4,
                       verbatimTextOutput("heatmap_hover"),
                       verbatimTextOutput("heatmap_selected")
                )
        )
    ))

ui <- dashboardPage(header, sidebar, body, skin = "blue")

server <- function(input, output, session) {
    load(system.file("extdata", "demo", "demodata.Rda",
                     package = "debrowser"))
    insulinSignalingGenes <- reactive({
        genes <- c("Prkar2a", "Tsc1", "Mapk8", "Sos1", "Pik3r1", "Srebf1",
                   "Insr", "Fasn", "Ppp1r3b", "Pik3r3", "Ptprf", "Pklr",
                   "Irs2", "Socs4", "Eif4ebp1", "Ppp1r3c", "Pygl", "Socs2",
                   "Cbl","Acaca", "Crkl")
        normDat <- getNormalizedMatrix(demodata, method = "MRN")
        normDat[genes, ]
    })
    selected <- reactiveVal()
    observe({
        withProgress(message = 'Creating plot', style = "notification", value = 0.1, {
            selected(callModule(debrowserheatmap, "heatmap", insulinSignalingGenes()))
        })
    })
    output$heatmap_hover <- renderPrint({
        if (!is.null(selected()) && !is.null(selected()$shgClicked()) && 
            selected()$shgClicked() != "")
            return(paste0("Clicked: ",selected()$shgClicked()))
        else
            return(paste0("Hovered:", selected()$shg()))
    })
    output$heatmap_selected <- renderPrint({
        if (!is.null(selected()))
            selected()$selGenes()
    })
}

shinyApp(ui, server)

#' plotSizeMarginsUI
#'
#' Size and margins module for plotly plots
#'
#' @note \code{plotSizeMarginsUI}
#' @param id, id
#' @param h, height
#' @param w, width
#' @param t, top margin	
#' @param b, bottom margin	
#' @param l, left margin	
#' @param r, right margin
#' @return size and margins controls
#' @examples
#'     x <- plotSizeMarginsUI("heatmap")
#' @export
#'
plotSizeMarginsUI <- function(id, w=800, h=640, t=20, b=100, l=100, r=20) {
    shinydashboard::menuItem(paste0(id, " - Size & Margins"),
    plotSizeUI(id, w, h),
    plotMarginsUI(id, t, b, l, r),
    plotTypeUI(id)
    )
}

#' plotSizeUI
#'
#' Size module for plotly plots
#'
#' @note \code{plotSizeUI}
#' @param id, id
#' @param h, height
#' @param w, width
#' @return size and margins controls
#' @examples
#'     x <- plotSizeUI("heatmap")
#' @export
#'
plotSizeUI <- function(id, w=800, h=600){
    ns <- NS(id)
    list(
    checkboxInput(r(ns('size'), "-"), paste0('Plot Size'), value = FALSE),
    conditionalPanel(paste0('input.', r(ns('size'), "-")),
    sliderInput(ns("width"), "width",
    min = 100, max = 2000, step = 10, value = w),
    sliderInput(ns("height"), "height",
    min = 100, max = 2000, step = 10, value = h)
    )
    )
}

r <- function(str, chr)
{
    gsub(chr, '', str)
}

#' plotMarginsUI
#'
#' Margins module for plotly plots
#'
#' @note \code{plotMarginsUI}
#' @param id, id
#' @param t, top margin
#' @param b, bottom margin
#' @param l, left margin
#' @param r, right margin
#' @return size and margins controls
#' @examples
#'     x <- plotMarginsUI("heatmap")
#' @export
#'
plotMarginsUI <- function(id, t=20, b=100, l=100, r=20){
    ns <- NS(id)
    list(
        checkboxInput(r(ns('margins'), "-"), 'Margins', value = FALSE),
        conditionalPanel(paste0('input.', r(ns('margins'), "-")),
        sliderInput(ns("top"), "Margin Top", min = 0, max = 200, value = t),
        sliderInput(ns("bottom"), "Margin Bottom", min = 0, max = 200, value = b),
        sliderInput(ns("left"), "Margin Left", min = 0, max = 200, value = l),
        sliderInput(ns("right"), "Margin Right", min = 0, max = 200, value = r))
    )
}

#' plotTypeUI
#'
#' Plot download type
#'
#' @note \code{plotTypeUI}
#' @param id, id
#' @return size and margins controls
#' @examples
#'     x <- plotTypeUI("heatmap")
#' @export
#'
plotTypeUI <- function(id){
    ns <- NS(id)
    list(
        checkboxInput(ns('svg'), paste0('Download SVG'), value = FALSE)
    )
}

library(debrowser)

options(warn =-1)

header <- dashboardHeader(
  title = "DEBrowser All2All Plots"
)
sidebar <- dashboardSidebar(  sidebarMenu(id="DEAnlysis",
      menuItem("All2All", tabName = "All2All"),
      plotSizeMarginsUI("all2all", h=800, w=800),
      all2allControlsUI("all2all")
))

body <- dashboardBody(
    tabItems(
        tabItem(tabName="All2All", 
        fluidRow(
            column(12,
            getAll2AllPlotUI("all2all")))
    )
))

ui <- dashboardPage(header, sidebar, body, skin = "blue")

server <- function(input, output, session) {
    load(system.file("extdata", "demo", "demodata.Rda",
                   package = "debrowser"))
    observe({
        callModule(debrowserall2all, "all2all", demodata, input$cex)
    })
}

shinyApp(ui, server)

library(debrowser)
library(DESeq2)

header <- dashboardHeader(
  title = "DEBrowser DE Analysis"
)
sidebar <- dashboardSidebar(  sidebarMenu(id="DEAnalysis",
    menuItem("DEAnalysis", tabName = "DEAnalysis"),
    shinydashboard::menuItem("Filter", tabName = "DEAnalysis",
    cutOffSelectionUI("DEResults"))
))

body <- dashboardBody(
  tabItems(
    tabItem(tabName="DEAnalysis", 
    fluidRow(
        # If these getDEResultsUI line put into the UI. The results shown in a table.
        column(12, getDEResultsUI("DEResults")),
        column(12,
            verbatimTextOutput("DERes")
        )
    ))
))

ui <- dashboardPage(header, sidebar, body, skin = "blue")

server <- function(input, output, session) {
  ######## Prepare demo data to use the module ########
  load(system.file("extdata", "demo", "demodata.Rda",
                   package = "debrowser"))
  filtd <-
      # Filter out the rows that has maximum 100 reads in a sample
      subset(demodata, apply(demodata, 1, max, na.rm = TRUE)  >=  10)
 
  params <-
      #Run DESeq2 with the following parameters
      c("DESeq2", "parametric", FALSE, "Wald", "None")
 
  cols <- 
      #For demo purpose use the coumns below;
      c("exper_rep1", "exper_rep2", "exper_rep3", "control_rep1", "control_rep2", "control_rep3")
  
  conds <- 
      #For each column, there is matching condition defined
      c("exper", "exper", "exper", "control", "control", "control")
  
  ######################################################
  deres<-callModule(debrowserdeanalysis, "DEResults", data = filtd, 
            columns = cols, conds = conds, params = params)
  observe({
      output$DERes <- renderPrint({
          head( deres$dat() )
      })
  })
}

shinyApp(ui, server)

#' getIntroText
#' Intro text
#'
#' @return the JS for tab updates
#'
#' @examples
#'     x<- getIntroText()
#'
#' @export
getIntroText<-function(){

list(
h2("Quick Start Guide"),
p("Differential expression (DE) analysis has become an increasingly popular tool
in determining and viewing up and/or down experssed genes between two sets of
samples."),
p("The goal of DEBrowser is to provide an easy way to perform and visualize DE analysis."),
#####################################################
##Data input section
h3("1. Data input"),
p("The input data for DEBrowser is 2 files in '.txt', '.csv' or '.tsv' format, 
  they are named as 'Count Data File' and 'Metadata File'."),
h4("1.1 Count Data File"),            
p("This input file should contain summarized count results of all samples in the experiment, 
  an example of the expected input data format is presented as below:"),
tableOutput("countFile"),
p("Where columns are samples, rows are the mapped genomic features 
  (e.g. genes, isoforms, miRNAs, ATAC or Chip regions etc.)."),
p("If you do not have a dataset to upload, you can use the built in demo data file by 
  clicking on the 'Load Demo' buttons from two different publications. To view the entire demo data file, you can 
  download the ", a("demo set 1 (Vernia et. al).", href="https://bioinfo.umassmed.edu/pub/debrowser/simple_demo.tsv"),
  "For another example, try our ", a("full dataset for demo set 1 (Vernia et. al)", href="https://bioinfo.umassmed.edu/pub/debrowser/advanced_demo.tsv")),
h4("1.2 Metadata File"),
p("In addition to the count data file; you may also upload metadata file to correct for 
batch effects or any other normalizing conditions you might want to address that might be
within your results. To handle for these conditions, simply create a metadata file by 
using the example table at below "),
tableOutput("metaFile"),
p("or download sample metadata file used for:",
a("metadata file for set 1 (Vernia et. al).", href="https://bioinfo.umassmed.edu/pub/debrowser/simple_demo_meta.txt")),
p("To be able to upload the data please press",  
actionButton("UploadBut", label = "Upload", styleclass = "primary"), "button in the data upload page."),
p("After sucessfull upload, you should see the summary of your data in the 'Upload Summary' section. 
  To move the filtering section please click",  
actionButton("FilterBut", label = "Filter", styleclass = "primary"), "button in the upload page."),
p("If you are ready to use DEBrowser, please click 'Upload' menu on the left to start using DEBrowser"))
}

#' getDataAssesmentText
#' DataAssesment text
#'
#' @return help text for data assesment 
#'
#' @examples
#'     x<- getDataAssesmentText()
#'
#' @export
getDataAssesmentText<-function(){
list(
h3("2. Data Assesment"),
h4("2.1 Low Count Filtering"),    
p("In this section, you can simultaneously visualise the changes of your dataset 
    while filtering out the low count genes. Choose your filtration criteria from 
    Filtering Methods box which is located just center of the screen. 
    Three methods are available to be used:"),
p(strong("Max:"), "Filters out genes where maximum count for each gene across all samples are less 
   than defined threshold."),
p(strong("Mean:"), "Filters out genes where mean count for each gene are less than defined threshold."),
p(strong("CPM:"), "First, counts per million (CPM) is calculated as the raw counts divided by the 
    library sizes and multiplied by one million. Then it filters out genes where at least 
    defined number of samples is less than defined CPM threshold."),
withMathJax(),
p("The expression cutoff value is determined according to the library size 
    and normalization factors with formula $$\\text{CPM} = \\frac{\\text{raw counts}}{\\text{library size} * \\text{normalization factors}} * 10^{-6}$$ 
    For example, if the cutoff CPM value is 10,
    the library size and normalization factors are estimated approximately equal to \\(\\ 3 \\text{ x} 10 ^ 6\\) and 1 for at least 4 samples, 
    then 10 CPM expression cutoff corresponds to about 30 read counts. 
    Therefore, in this example features in more than 4 samples have less than 
    30 read counts (10 CPM) is going to be low expression features
    and will be removed for batch effect correction and DE analysis."),
p("To be able to filter out the low expression counts please press",  
    actionButton("FilterBut", label = "Filter", styleclass = "primary"), "button in data filtering page."),
h4("2.2 Quality control(QC)"),
p("After filtering low count features, you may continue your analysis with Batch Effect 
    Detection & Correction or directly jump to differential expression analysis or view 
    quality control (QC) information of your dataset."),
p("If specified metadata file containing your treatment and batch fields, by clicking ", 
    actionButton("BatchBut", label = "Batch effect correction", styleclass = "primary"), 
    "button, you have the option to conduct PCA, interquartile range (IQR) and density 
    plots to asses if the data requires batch effect correction or not."),
p("If user wants to skip batch effect assesment and correction step, they can either click",
   actionButton("goDEFromFilterBut", "Go to DE Analysis", styleclass = "primary"),  
   " button to perform DE Analysis or ",
   actionButton("goQCplotsFromFilterBut", "Go to QC plots", styleclass = "primary"), 
   " button for QC plots to draw PCA, all2all scatter, heatmaps, IQR and density plots.")
)
}

#' getDataPreparationText
#' DataPreparation text
#'
#' @return help text for data preparation 
#'
#' @examples
#'     x<- getDataPreparationText()
#'
#' @export
getDataPreparationText<-function(){
list(
    h3("3. Data Preparation"),
    p("With metadata file containing your batch correction fields 
    then you have the option to conduct ",strong("batch effect correction"), " prior to 
    your analysis. By adjusting parameters of Options box, you can investigate 
    your character of your dataset. These parameters of the options box are 
    explained as following:"),
    p(strong("Normalization Method:"), "DEBrowser allows performing normalization 
    prior the batch effect correction. You may choose your normalization method 
    (among MRE, TMM, RLE, upperquartile), or if you don't want to normalize your 
    data you can select none for this item."),
    p(strong("Correction Method:"), "DEBrowser uses ComBat (part of the SVA 
    bioconductor package) or Harman to adjust for possible batch effect or conditional 
    biases. For more information, you can visit following links for 
    documentation: ComBat, Harman"),
    p(strong("Treatment:"), "Please select the column that is specified in 
    metadata file for comparision, such as cancer vs control. It is named 
    condition for our sample metadata."),
    p(strong("Batch:"), "Please select the column name in metadata file 
    which differentiate the batches. For example in our metadata, it is called batch.
    Upon clicking submit button, comparison tables and plots will be created on the right 
    part of the screen as shown below."),
    p("You can investigate the changes on the data by comparing following features:"),
    p("1. Read counts for each sample.",br(),
    "2. PCA, IQR and Density plot of the dataset.", br(),
    "3. Gene/region vs samples data"),
    p("After batch effect correction, user can click ",
    actionButton("goDEFromFilterBut", "Go to DE Analysis", styleclass = "primary"),  
    " button to perform DE Analysis or ",
    actionButton("goQCplotsFromFilterBut", "Go to QC plots", styleclass = "primary"), 
    " button for QC plots to draw PCA, all2all scatter, heatmaps, IQR and density plots.")
)
}
#' getDEAnalysisText
#' DEAnalysis text
#'
#' @return help text for DE Analysis 
#'
#' @examples
#'     x<- getDEAnalysisText()
#'
#' @export
getDEAnalysisText<-function(){
list(
h3("4. DE analysis"),
p("The goal of differential gene expression analysis is to find genes
or transcripts whose difference in expression, when accounting for the
variance within condition, is higher than expected by chance."),
p(a("DESeq2", href="https://bioconductor.org/packages/release/bioc/html/DESeq2.html"),
"is an R package available via Bioconductor and is designed to normalize count
data from high-throughput sequencing assays such as RNA-Seq and test for
differential expression (Love et al. 2014).  With multiple parameters such as
padjust values, log fold changes, plot styles, and so on, altering plots
created with your DE data can be a hassle as well as time consuming. The
Differential Expression Browser uses DESeq2 (Love et al., 2014)",
a("EdgeR",href="https://bioconductor.org/packages/release/bioc/html/edgeR.html"),
"(Robinson et al., 2010), and ",
a("Limma", href="https://bioconductor.org/packages/release/bioc/html/limma.html"),
"(Ritchie et al., 2015) coupled with shiny (Chang, W. et al., 2016)
to produce real-time changes within your
plot queries and allows for interactive browsing of your DE results.
In addition to DE analysis, DEBrowser also offers a variety of other plots
and analysis tools to help visualize your data even further."),
p("If you are ready to discover and visualize your data, please click ",
  actionButton("mainPlotsBut", "Go to Main Plots", styleclass = "primary"), 
  "button in DE Results section."),
h4("4.1 Used parameters for DESeq2"),
p(strong("fitType:")),
p("Either 'parametric', 'local', or 'mean' for the type of fitting of 
dispersions to the mean intensity. See estimateDispersions for description."),
p(strong("betaPrior:")),
p("Whether or not to put a zero-mean normal prior on the non-intercept 
coefficients See nbinomWaldTest for description of the calculation of 
the beta prior. By default, the beta prior is used only for the Wald test, 
but can also be specified for the likelihood ratio test."),
p(strong("testType:")),
p("Either 'Wald' or 'LRT', which will then use either Wald significance tests 
(defined by nbinomWaldTest), or the likelihood ratio test on the difference in 
deviance between a full and reduced model formula (defined by nbinomLRT)"),
h4("4.2 Used parameters for EdgeR"),
p(strong("Normalization:")),
p("Calculate normalization factors to scale the raw library sizes. Values 
can be 'TMM','RLE','upperquartile','none'."),
p(strong("Dispersion:")),
p("Either a numeric vector of dispersions or a character string indicating 
that dispersions should be taken from the data object."),
p(strong("testType:")),
p("ExactTest or glmLRT. ",strong("exactTest:")," Computes p-values for differential 
abundance for each gene between two samples, conditioning on the total 
count for each gene. The counts in each group are assumed to follow a 
binomial distribution. ",strong("glmLRT:")," Fits a negative binomial generalized 
log-linear model to the read counts for each gene and conducts 
genewise statistical tests."),
h4("4.3 Used parameters for Limma"),
p(strong("Normalization:")),
p("Calculate normalization factors to scale the raw library sizes. 
Values can be 'TMM','RLE','upperquartile','none'."),
p(strong("Fit Type:")),
p("fitting method; 'ls' for least squares or 'robust' for robust regression"),
p(strong("Norm. Bet. Arrays:")),
p("Normalization Between Arrays; Normalizes expression intensities so that the 
intensities or log-ratios have similar distributions across a set of arrays.")
)
}
#' getQAText
#' Some questions and answers
#'
#' @return help text for QA
#'
#' @examples
#'     x<- getQAText()
#'
#' @export
getQAText<-function(){
    list(
        h3("5. Frequently asked questions (FAQ)"),
        h4("5.1 Why un-normalized counts?"),
        p("DESeq2 requires count data as input obtained from 
          RNA-Seq or another high-thorughput sequencing experiment 
          in the form of matrix values. Here we convert un-integer 
          values to integer to be able to run DESeq2. The matrix values 
          should be un-normalized, since DESeq2 model internally corrects for 
          library size. So, transformed or normalized values such as counts 
          scaled by library size should not be used as input. Please use edgeR 
          or limma for normalized counts."),
        h4("5.2 Why am I getting error while uploading files?"),
        p("* DEBrowser supports tab, comma or semi-colon separated files. However spaces or characters in numeric regions not supported and causes an error while uploading files. It is crutial to remove these kind of instances from the files before uploading files."),
        p("* Another reason of getting an error is using same gene name multiple times. This may occurs after opening files in programs such as Excel, which tends to automatically convert some gene names to dates (eg. SEP9 to SEP.09.2018). This leads numerous problems therefore you need to disable these kind of automatic conversion before opening files in these kind of programs."),
        p("* Some files contain both tab and space as an delimiter which lead to error. It is required to be cleaned from these kind of files before loading."),
        h4("5.3 Why some columns not showed up after upload?"),
        p("If a character in numeric area or space is exist in one of your column, either column will be eliminated or you will get an error. Therefore it is crutial to remove for these kind of instances from your files before uploading."),
        h4("5.4 Why am I getting error while uploading CSV/TSV files exported from Excel?"),
        p("* You might getting an error, because of using same gene name multiple times. This may occurs after opening files in programs such as Excel, which tends to automatically convert some gene names to dates (eg. SEP9 to SEP.09.2018). Therefore you need to disable these kind of automatic conversion before opening files in these kind of programs."),
        h4("5.5 Why can't I see all the background data in Main Plots?"),
        p("In order to increase the performance, by default 10% of non-significant(NS) genes are used to generate plots. We strongly suggest you to use all of the NS genes in your plots while publishing your results. You can easily change this parameter by clicking **Main Options** button and change Background Data(%) to 100% on the left sidebar."),
        h4("5.6 Why am I getting error when I click on DE Genes in Go Term Analysis?"),
        p("To start ", strong("Go Term"), " analysis, it is important to select correct organism from ", strong("Choose an organism"), " field. After selecting other desired parameters, you can click ", strong("Submit")," button to run Go Term analysis. After this stage, you will able to see", strong(" categories")," regarding to your selected gene list in the ", strong("Table")," Tab. Once you select this category, you can click DE Genes button to see gene list regarding to selected category."),
        h4("5.7 How to download selected data from Main plots/QC Plots/Heatmaps?"),
        p("First, you need to choose ", strong("Choose dataset"), " field as ",strong("selected")," under ",strong("Data Options")," in the left sidebar. When you select this option, new field: ",strong("The plot used in selection")," will appear under ", strong("Choose dataset")," field. You need to specify the plot you are interested from following options: Main plot, Main Heatmap, QC Heatmap. Finally you can click ", strong("Download Data"), " button to download data, or if you wish to see the selected data, you can click ",strong("Tables")," tab.")
    )
}

library(debrowser)

header <- dashboardHeader(
  title = "DEBrowser Condition Selector"
)
sidebar <- dashboardSidebar(  sidebarMenu(id="DataPrep",
    menuItem("CondSelect", tabName = "CondSelect")
))

body <- dashboardBody(
  tabItems(
    tabItem(tabName="CondSelect", 
    condSelectUI(),
    column(12,
           verbatimTextOutput("denum")
    ))
))

ui <- dashboardPage(header, sidebar, body, skin = "blue")

server <- function(input, output, session) {
  load(system.file("extdata", "demo", "demodata.Rda",
                   package = "debrowser"))
  observe({
     sel <- debrowsercondselect(input, output, session, demodata, metadatatable)
     output$denum <- renderPrint({
         head( sel$cc())
     })
  })
}

shinyApp(ui, server)

library(debrowser)
library(DESeq2)
library(Harman)
options(warn =-1)
source("../../R/batcheffect.R")
source("../../R/funcs.R")
source("../../R/pca.R")
header <- dashboardHeader(
    title = "DEBrowser Batch Effect"
)
sidebar <- dashboardSidebar(  sidebarMenu(id="DataPrep",
       menuItem("BatchEffect", tabName = "BatchEffect")))

body <- dashboardBody(
    tabItems(
        tabItem(tabName="BatchEffect", batchEffectUI("batcheffect"),
                column(4,
                       verbatimTextOutput("batcheffecttable")
                )
        )
    ))

ui <- dashboardPage(header, sidebar, body, skin = "blue")

server <- function(input, output, session) {
    load(system.file("extdata", "demo", "demodata.Rda",
                     package = "debrowser"))
    
    ldata <- reactiveValues(count=NULL, meta=NULL)
    ldata$count <- demodata
    ldata$meta <- metadatatable
    data <- callModule(debrowserbatcheffect, "batcheffect", ldata)
    observe({
        output$batcheffecttable <- renderPrint({
            head( data$BatchEffect()$count )
        })
    })
}

shinyApp(ui, server)

#' debrowserlowcountfilter
#'
#' Module to filter low count genes/regions
#' 
#' @param input, input variables
#' @param output, output objects
#' @param session, session 
#' @param ldata, loaded data
#' @return main plot
#'
#' @return panel
#' @export
#'
#' @examples
#'     x <- debrowserlowcountfilter()
#'
debrowserlowcountfilter <- function(input = NULL, output = NULL, session = NULL, ldata = NULL) {
    if (is.null(ldata)) return(NULL)
    fdata <- reactiveValues(count=NULL, meta = NULL)
    observeEvent(input$submitLCF, {
        if (is.null(ldata$count)) return (NULL)
        filtd <- ldata$count
        filtd[, colnames(filtd)] <- apply(filtd[, colnames(filtd)], 2, function(x) as.integer(x))
    
        if (input$lcfmethod == "Max"){
          filtd <- subset(filtd, apply(filtd, 1, max, na.rm = TRUE)  >=  as.numeric(input$maxCutoff))
        } else if (input$lcfmethod == "Mean") {
          filtd <- subset(filtd, rowMeans(filtd, na.rm = TRUE) >= as.numeric(input$meanCutoff))
        }
        else if (input$lcfmethod == "CPM") {
            cpmcount <- edgeR::cpm(filtd)
            filtd <- subset(filtd, rowSums(cpmcount > as.numeric(input$CPMCutoff), 
            na.rm = TRUE) >= as.numeric(input$numSample))
        }
        fdata$count <- filtd
        fdata$meta <- ldata$meta
    })
  
  output$cutoffLCFMet <- renderUI({
    ret<-textInput(session$ns("maxCutoff"), "Filter features where Max Value <", value = "10" )
    if (input$lcfmethod == "Mean"){
      ret<-textInput(session$ns("meanCutoff"), "Filter features where Row Means <", value = "10" )
    }
    else if (input$lcfmethod == "CPM"){
      ret <- list(textInput(session$ns("CPMCutoff"), "Filter features where CPM <", value = "1" ),
         textInput(session$ns("numSample"), "at least # of samples", value = toString(ncol(ldata$count)-1) ))
    }
    ret
  })

  filtereddata <- reactive({
    ret <- NULL
    if(!is.null(fdata$count)){
      ret <- fdata
    }
    return(ret)
  })
 
  observe({
    getSampleDetails(output, "uploadSummary", "sampleDetails", ldata)
    getSampleDetails(output, "filteredSummary", "filteredDetails", filtereddata())
    getTableDetails(output, session, "loadedtable",  data = ldata$count,  modal = TRUE)
    callModule(debrowserhistogram, "beforeFiltering", ldata$count)
    
    if ( !is.null(filtereddata()$count ) && nrow(filtereddata()$count)>2 ) {
        getTableDetails(output, session, "filteredtable",  data = filtereddata()$count, modal = TRUE)
        callModule(debrowserhistogram, "afterFiltering", filtereddata()$count)
    }
  })
  
  list(filter=filtereddata)
}

#' dataLCFUI
#' Creates a panel to filter low count genes and regions
#'
#' @param id, namespace id
#' @return panel
#' @examples
#'     x <- dataLCFUI("lcf")
#'
#' @export
#'
dataLCFUI<- function (id) {
  ns <- NS(id)
  list(
    fluidRow(
      shinydashboard::box(title = "Low Count Filtering",
          solidHeader = TRUE, status = "info",  width = 12, 
          fluidRow(
            column(5,div(style = 'overflow: scroll',
                tableOutput(ns("uploadSummary")),
                DT::dataTableOutput(ns("sampleDetails"))),
                uiOutput(ns("loadedtable"))
            ),
            column(2,
                shinydashboard::box(title = "Filtering Methods",
                    solidHeader = TRUE, status = "info",
                    width = 12, 
                    lcfMetRadio(id),
                    uiOutput(ns("cutoffLCFMet")),
                    actionButtonDE(ns("submitLCF"), label = "Filter", styleclass = "primary")
                )
            ),
            column(5,div(style = 'overflow: scroll',
                tableOutput(ns("filteredSummary")),
                DT::dataTableOutput(ns("filteredDetails"))),
                uiOutput(ns("filteredtable"))
            )
          ),
          conditionalPanel(condition = paste0("input['", ns("submitLCF"),"']"),
          actionButtonDE("Batch", label = "Batch Effect Correction", styleclass = "primary"),
          conditionalPanel(condition = "!(input.Batch)",
          actionButtonDE("goDEFromFilter", "Go to DE Analysis", styleclass = "primary"),
          actionButtonDE("goQCplotsFromFilter", "Go to QC plots", styleclass = "primary")))
      ),
      shinydashboard::box(title = "Histograms",
                          solidHeader = TRUE, status = "info",  width = 12, 
      fluidRow(
          column(6,histogramControlsUI(ns("beforeFiltering")),
                 getHistogramUI(ns("beforeFiltering"))),
          column(6,histogramControlsUI(ns("afterFiltering")),
                 getHistogramUI(ns("afterFiltering")))
      ))
    ))
}

#' lcfMetRadio
#'
#' Radio buttons for low count removal methods
#'
#' @param id, namespace id
#' @note \code{lcfMetRadio}
#' @return radio control
#'
#' @examples
#'    
#'     x <- lcfMetRadio("lcf")
#'
#' @export
#'
lcfMetRadio <- function(id) {
    ns <- NS(id)
    radioButtons(inputId=ns("lcfmethod"), 
    label="Low count filtering method:",
    choices=c(Max='Max',
        Mean='Mean',
        CPM='CPM'
    ),
    selected='Max'
    )
}

library(debrowser)
library(reshape2)

options(warn =-1)

header <- dashboardHeader(
  title = "DEBrowser Density Plots"
)
sidebar <- dashboardSidebar(  sidebarMenu(id="DataAssessment",
      menuItem("Density", tabName = "Density"),
      textInput("maxCutoff", "Max Cutoff", value = "10" ),
      plotSizeMarginsUI("density", h=400, t=0, b=0,r=0,l=0),
      plotSizeMarginsUI("afterFiltering", h=400, t=0, b=0,r=0,l=0)
))

body <- dashboardBody(
  tabItems(
    tabItem(tabName="Density", 
        fluidRow(
            column(12,
                getDensityPlotUI("density"))),
        fluidRow(
            column(12,
                getDensityPlotUI("afterFiltering"))
        )
    )
  ))

ui <- dashboardPage(header, sidebar, body, skin = "blue")

server <- function(input, output, session) {
    load(system.file("extdata", "demo", "demodata.Rda",
                   package = "debrowser"))
    filtd <- reactive({
        # Filter out the rows that has maximum 100 reads in a sample
        subset(demodata, apply(demodata, 1, max, na.rm = TRUE)  >=  
                   as.numeric(input$maxCutoff))
    })
    observe({
        if(!is.null(filtd())){
            callModule(debrowserdensityplot, "density", demodata)
            callModule(debrowserdensityplot, "afterFiltering", filtd())
        }
    })
}

shinyApp(ui, server)

#' debrowserdataload
#'
#' Module to load count data and metadata
#' 
#' @param input, input variables
#' @param output, output objects
#' @param session, session 
#' @param nextpagebutton, the name of the next page button after loading the data
#' @return main plot
#'
#' @return panel
#' @export
#'
#' @examples
#'     x <- debrowserdataload()
#'
debrowserdataload <- function(input = NULL, output = NULL, session = NULL, nextpagebutton = NULL) {
    if (is.null(input)) return(NULL)
    ldata <- reactiveValues(count=NULL, meta=NULL)
    loadeddata <- reactive({
        ret <- NULL
        if(!is.null(ldata$count)){
            ldata$count <- ldata$count[,sapply(ldata$count, is.numeric)]
            ret <- list(count = ldata$count, meta = ldata$meta)
        }
        return(ret)
    })
    output$dataloaded <- reactive({
        return(!is.null(loadeddata()))
    })
    outputOptions(output, "dataloaded", 
        suspendWhenHidden = FALSE)
    observe({
    query <- parseQueryString(session$clientData$url_search)
    jsonobj<-query$jsonobject
    type = ""
    if (!is.null(query$type)){
        type <- query$type
    }
        
    # To test json load;
    # It accepts two parameters:
    # 1. jsonobject=https://debrowser.umassmed.edu/?jsonobject=https://umms.dolphinnext.com/pub/debrowser/advanced_demo_org.json
    # 2. meta=https://umms.dolphinnext.com/pub/debrowser/advanced_meta.json
    # The finished product of the link will look like this without metadata:
    # 
    # https://127.0.0.1:3427/debrowser/R/?jsonobject=https://debrowser.umassmed.edu/?jsonobject=https://umms.dolphinnext.com/pub/debrowser/advanced_demo_org.json
    #        
    #  With metadata
    #
    #http://127.0.0.1:3427/?jsonobject=https://debrowser.umassmed.edu/?jsonobject=https://umms.dolphinnext.com/pub/debrowser/advanced_demo_org.json&meta=https://umms.dolphinnext.com/pub/debrowser/advanced_meta.json
    #
    if (!is.null(jsonobj))
    {
        if (type == "nojson"){
            ex <- strsplit(basename(jsonobj), split="\\.")[[1]]
            if (ex[-1] == "tsv"){
                data <- read.delim(jsonobj)
            } else {
                data <- read.csv(jsonobj) 
            }
        } else {
            raw <- RCurl::getURL(jsonobj, .opts = list(ssl.verifypeer = FALSE),
                 crlf = TRUE)
            data <- fromJSON(raw, simplifyDataFrame = TRUE)
        }
        colnames(data) <- gsub("\\s+|\\.|\\-", "_", colnames(data))
        jsondata<-data.frame(data,stringsAsFactors = TRUE)
        
        rownames(jsondata)<-jsondata[, 1]
        jsondata<-jsondata[,c(3:ncol(jsondata))]
        jsondata[,c(1:ncol(jsondata))] <- sapply(
            jsondata[,c(1:ncol(jsondata))], as.numeric)
        jsondata <- jsondata[,sapply(jsondata, is.numeric)]
        
        metadatatable <- NULL
        jsonmet <-query$meta

        if(!is.null(jsonmet)){
            if (type == "nojson"){
                ex <- strsplit(basename(jsonmet), split="\\.")[[1]]
                if (ex[-1] == "tsv"){
                    data <- read.delim(jsonmet)
                } else {
                    data <- read.csv(jsonmet)
                }
            } else {
                raw <- RCurl::getURL(jsonmet, .opts = list(ssl.verifypeer = FALSE),
                    crlf = TRUE)
                data <- fromJSON(raw, simplifyDataFrame = TRUE)
            }
            data[,1] <- gsub("\\s+|\\.|\\-", "_", data[,1])
            
            metadatatable<-data.frame(data,
                stringsAsFactors = TRUE)
            cnames <-  names(jsondata)
            selectcols <-  cnames[cnames %in%  metadatatable[,1]] 
            ldata$count <- jsondata[, selectcols]
            print(dim(ldata$count))
        }else{
            ldata$count <- jsondata
            metadatatable <- cbind(colnames(ldata$count), 1)
            colnames(metadatatable) <- c("Sample", "Batch")
        }
        ldata$meta <- metadatatable
        input$Filter
    }
    })
    observeEvent(input$demo, {
        load(system.file("extdata", "demo", "demodata.Rda",
                         package = "debrowser"))

        ldata$count <- demodata
        ldata$meta <- metadatatable
    })
    observeEvent(input$demo2, {
        load(system.file("extdata", "demo", "demodata2.Rda",
                         package = "debrowser"))
        ldata$count <- demodata
        ldata$meta <- metadatatable
    })
    
    observeEvent(input$uploadFile, {
        if (is.null(input$countdata)) return (NULL)
        checkRes <- checkCountData(input)
        
        if (checkRes != "success"){
            showNotification(checkRes, type = "error")
            return(NULL)
        }
        counttable <-as.data.frame(
            try(
                read.delim(input$countdata$datapath, 
                header=T, sep=input$countdataSep, 
            row.names=1, strip.white=TRUE ), TRUE))
        colnames(counttable) <- gsub("\\s+|\\.|\\-", "_", colnames(counttable))
        counttable <- counttable[,sapply(counttable, is.numeric)]
        metadatatable <- c()
        if (!is.null(input$metadata$datapath)){
            metadatatable <- as.data.frame(
            try(
                read.delim(input$metadata$datapath, 
                header=TRUE, sep=input$metadataSep, strip.white=TRUE), TRUE))

            metadatatable[,1] <- gsub("\\s+|\\.|\\-", "_", metadatatable[,1])
            checkRes <- checkMetaData(input, counttable)
            if (checkRes != "success"){
                  showNotification(checkRes, type = "error")
                  return(NULL)
            }
            counttable <- counttable[, metadatatable[,1]]
        }
        else{
            metadatatable <- cbind(colnames(counttable), 1)
            colnames(metadatatable) <- c("Sample", "Batch")
        }
        if (is.null(counttable)) 
            {stop("Please upload the count file")}
        ldata$count <- counttable
        ldata$meta <- metadatatable
    })
    output$nextButton <- renderUI({
        actionButtonDE(nextpagebutton, label = nextpagebutton, styleclass = "primary")
    })
    observe({
        getSampleDetails(output, "uploadSummary", "sampleDetails", loadeddata())
    })
  list(load=loadeddata)
}

#' dataLoadUI
#' 
#' Creates a panel to upload the data
#'
#' @param id, namespace id
#' @return panel
#' @examples
#'     x <- dataLoadUI("load")
#'
#' @export
#'
dataLoadUI<- function (id) {
  ns <- NS(id)
  list(conditionalPanel(condition =  paste0("!output['", ns("dataloaded"),"']"), fluidRow(
             fileUploadBox(id, "countdata", "Count Data"),
             fileUploadBox(id, "metadata", "Metadata")
        ),
        fluidRow(column(12,
        actionButtonDE(ns("uploadFile"), label = "Upload", styleclass = "primary"), 
        actionButtonDE(ns("demo"),  label = "Load Demo (Vernia et. al)", styleclass = "primary"),
        actionButtonDE(ns("demo2"),  label = "Load Demo (Donnard et. al)", styleclass = "primary")))),
        fluidRow(column(12,
        conditionalPanel(condition = paste0("output['", ns("dataloaded"),"']"),
        uiOutput(ns("nextButton"))
        ))
        ), br(),
  fluidRow(
    shinydashboard::box(title = "Upload Summary",
        solidHeader = T, status = "info",
        width = 12, 
        fluidRow(
          column(12, 
              tableOutput(ns("uploadSummary"))
          )),
        fluidRow(
          column(12,div(style = 'overflow: scroll', 
              DT::dataTableOutput(ns("sampleDetails")))
          )
        )
    )
  ))
}

#' fileUploadBox
#'
#' File upload module
#' @param id, namespace id
#' @param inputId, input file ID
#' @param label, label
#' @note \code{fileUploadBox}
#' @return radio control
#'
#' @examples
#'    
#'     x <- fileUploadBox("meta", "metadata", "Metadata")
#'
#' @export
#'
fileUploadBox <- function(id = NULL, inputId = NULL, label = NULL) {
ns <- NS(id)
shinydashboard::box(title = paste0(label, " File"),
    solidHeader = TRUE, status = "info",
    width = 6,
    helpText(paste0("Upload your '", label," File'")),
    fileInput(inputId=ns(inputId), 
        label=NULL, 
        accept=fileTypes()
    ),
    sepRadio(id, paste0(inputId, "Sep")))
}

#' sepRadio
#'
#' Radio button for separators
#'
#' @param id, module id
#' @param name, name
#' @note \code{sepRadio}
#' @return radio control
#'
#' @examples
#'    
#'     x <- sepRadio("meta", "metadata")
#'
#' @export
#'
sepRadio <- function(id, name) {
  ns <- NS(id)
  radioButtons(inputId=ns(name), 
               label="Separator",
               choices=c(Comma=',',
                         Semicolon=';',
                         Tab='\t'
               ),
               selected='\t'
  )
}

#' fileTypes
#'
#' Returns fileTypes that are going to be used in creating fileUpload UI
#'
#' @note \code{fileTypes}
#' @return file types
#'
#' @examples
#'     x <- fileTypes()
#'
#' @export
#'
fileTypes <- function() {
  c('text/tab-separated-values',
    'text/csv',
    'text/comma-separated-values',
    'text/tab-separated-values',
    '.txt',
    '.csv',
    '.tsv')
}

#' checkCountData
#'
#' Returns if there is a problem in the count data.
#'
#' @note \code{checkCountData}
#' @param input, inputs
#' @return error if there is a problem about the loaded data
#
#' @examples
#'     x <- checkCountData()
#'
#' @export
#'
checkCountData <- function(input = NULL){
    if (is.null(input$countdata$datapath)) return(NULL)
    tryCatch({
        data <- read.table(input$countdata$datapath, sep=input$countdataSep)
        if (ncol(data) < 3) return ("Error: Please check if you chose the right separator!")
        dups <- data[duplicated(data[,1], fromLast = TRUE),1]
        if (length(dups)>1) return (paste0("Error: There are duplicate entried in  the rownames. (", 
            paste0(dups, collapse=","),")"))

        return("success")
    }, error = function(err) {
        return (paste0("Error(Count file):",toString(err)))
    }, warning = function(war) {
        return(paste0("Warning(Count file):",toString(err)))
    })
}


#' checkMetaData
#'
#' Returns if there is a problem in the count data.
#'
#' @note \code{checkMetaData}
#' @param input, input
#' @param counttable, counttable
#' @return error if there is a problem about the loaded data
#
#' @examples
#'     x <- checkMetaData()
#'
#' @export
#'
checkMetaData <- function(input = NULL, counttable = NULL){
    if (is.null(counttable) || is.null(input$metadata$datapath)) return(NULL)
     tryCatch({
        metadatatable <- read.table(input$metadata$datapath, sep=input$metadataSep, header=T)
        if (ncol(metadatatable) < 2) return ("Error: Please check if you chose the right separator!")
        met <- as.vector(metadatatable[order(as.vector(metadatatable[,1])), 1])
        met <- gsub("\\s+|\\.|\\-", "_", met)
        count <- as.vector(colnames(counttable)[order(as.vector(colnames(counttable)))])
        difference <- base::setdiff(met, count)
        if (length(difference)>0){
            return(paste0("Colnames doesn't match with the metada table(", paste0(difference,sep=",", collapse=" "), ")"))
        }
        return("success")
    }, error = function(err) {
        return (paste0("Error(Matadata file):",toString(err)))
    }, warning = function(war) {
        return (paste0("Warning(Matadata file):",toString(war)))
    })
}

#' debrowserheatmap
#'
#' Heatmap module to create interactive heatmaps and get selected list from
#' a heatmap
#' @param input, input variables
#' @param output, output objects
#' @param session, session 
#' @param expdata, a matrix that includes expression values
#' @return heatmapply plot
#'
#' @examples
#'     x <- debrowserheatmap()
#'
#' @export
#'
#'
debrowserheatmap <- function( input, output, session, expdata = NULL){
    if(is.null(expdata)) return(NULL)
    output$heatmap <- renderPlotly({
        shinyjs::onevent("mousemove", "heatmap", js$getHoverName(session$ns("hoveredgenename")))
        shinyjs::onevent("click", "heatmap", js$getHoverName(session$ns("hoveredgenenameclick")))

        withProgress(message = 'Drawing Heatmap', detail = "interactive", value = 0, {
            runHeatmap(input, session, orderData())
        })
    })
    output$heatmap2 <- renderPlot({
        withProgress(message = 'Drawing Heatmap', detail = "non-interactive", value = 0, {
            runHeatmap2(input, session, orderData())
        })
    })
    heatdata <- reactive({
        cld <- prepHeatData(expdata, input)
        if (input$kmeansControl)
        {
            res <- niceKmeans(cld, input)
            cld <- res$clustered
        }
        cld
    })
    
    button <- reactiveVal(FALSE)
    orderData <- reactive({
        newclus <- heatdata()
        if (input$changeOrder && isolate(button()) && !is.null(input$clusterorder)){
            newclus <- changeClusterOrder(isolate(input$clusterorder), newclus)
        }
        button(FALSE)
        newclus
    })
    observeEvent(input$changeOrder,{
        button(TRUE)
    })
    output$heatmapUI <- renderUI({
        if (is.null(input$interactive)) return(NULL)
        shinydashboard::box(
            collapsible = TRUE, title = session$ns("Heatmap"), status = "primary", 
            solidHeader = TRUE, width = NULL,
            draggable = TRUE,   getPlotArea(input, session))
    })
    
    hselGenes <- reactive({
        if (is.null(input$selgenenames)) return("")
        unlist(strsplit(input$selgenenames, split=","))
    })
    shg <- reactive({
        if (is.null(input$hoveredgenename)) return("")
        js$getSelectedGenes(session$ns("heatmap"), session$ns("selgenenames"))
        input$hoveredgenename
    })
    observe({
        if(!input$changeOrder)
            updateTextInput(session, "clusterorder", value = paste(seq(1:input$knum), collapse=","))
        
        if (is.null(shg()))
            js$getSelectedGenes()
    })
    shgClicked <- reactive({
        if (is.null(input$hoveredgenenameclick) || input$hoveredgenenameclick == "") return(input$hoveredgenename)
        input$hoveredgenenameclick
    })
    
    list( shg = (shg), shgClicked=(shgClicked), selGenes=(hselGenes), getSelected = (orderData))
}
#' getPlotArea
#'
#' returns plot area either for heatmaply or heatmap.2
#' @param input, input variables
#' @param session, session 
#' @return heatmapply/heatmap.2 plot area
#'
#' @examples
#'     x <- getPlotArea()
#'
#' @export
#'
#'
getPlotArea <- function(input = NULL, session = NULL){
    if (is.null(input)) return(NULL)
    ret <- c()
    
    if (input$interactive){
        ret <- plotlyOutput(session$ns("heatmap"),
            height=input$height, width=input$width)
    }
    else{
        ret <- plotOutput(session$ns("heatmap2"),
            height = input$height, input$width)
    }
    ret
}

#' runHeatmap
#'
#' Creates a heatmap based on the user selected parameters within shiny
#' @param input, input variables
#' @param session, session 
#' @param expdata, a matrix that includes expression values
#' @return heatmapply plot
#'
#' @examples
#'     x <- runHeatmap()
#'
#' @export
#'
#'
runHeatmap <- function(input = NULL, session = NULL, expdata = NULL){
    if (is.null(expdata)) return(NULL)
    cld <-expdata
    hclustfun_row <- function(x, ...) hclust(x, method = input$hclustFun_Row)
    hclustfun_col <- function(x, ...) hclust(x, method = input$hclustFun_Col)
    distfun_row <- function(x, ...) {
        if (input$distFun_Row != "cor") {
            return(dist(x, method = input$distFun_Row))
        } else {
            return(as.dist(1 - cor(t(x))))
        }
    }
    distfun_col <- function(x, ...) {
        if (input$distFun_Col != "cor") {
            return(dist(x, method = input$distFun_Col))
        } else {
            return(as.dist(1 - cor(t(x))))
        }
    }
    
    if (!input$customColors ) {
        heatmapColors <- eval(parse(text=paste0(input$pal,
                                                '(',input$ncol,')')))
    }
    else{
        if (!is.null(input$color1))
            heatmapColors <- colorRampPalette(c(input$color1, 
               input$color2, input$color3))(n = 1000)
    }
    
    if (!input$kmeansControl){
        p <- heatmaply(cld,
                       main = input$main,
                       xlab = input$xlab,
                       ylab = input$ylab,
                       row_text_angle = input$row_text_angle,
                       column_text_angle = input$column_text_angle,
                       dendrogram = input$dendrogram,
                       branches_lwd = input$branches_lwd,
                       seriate = input$seriation,
                       colors = heatmapColors,
                       distfun_row =  distfun_row,
                       hclustfun_row = hclustfun_row,
                       distfun_col = distfun_col,
                       hclustfun_col = hclustfun_col,
                       showticklabels = c(input$labCol, input$labRow),
                       k_col = input$k_Col, 
                       k_row = input$k_Row
        ) 
    }else {
        if (!input$showClasses){
            cld <- data.frame(cld)
            cld <- as.matrix(cld [, -match("class",names(cld))])
        }
        rhcr <- hclust(dist(cld))
        chrc <- hclust(dist(t(cld)))
        p <- heatmaply(cld,
                       main = input$main,
                       xlab = input$xlab,
                       ylab = input$ylab,
                       row_text_angle = input$row_text_angle,
                       column_text_angle = input$column_text_angle,
                       #dendrogram = input$dendrogram,
                       dendrogram = "none",
                       branches_lwd = input$branches_lwd,
                       seriate = input$seriation,
                       colors = heatmapColors,
                       showticklabels = c(input$labCol, input$labRow),
                       Rowv = as.dendrogram(rhcr),
                       Colv = as.dendrogram(chrc),
                       k_col = input$k_Col,
                       k_row = input$knum
        )
    }
    p <- p %>% 
        plotly::layout(
            height=input$height, width=input$width,
            margin = list(l = input$left,
                          b = input$bottom,
                          t = input$top,
                          r = input$right
            ))
    if (!is.null(input$svg) && input$svg == TRUE)
        p <- p %>% config(toImageButtonOptions = list(format = "svg"))
    p$elementId <- NULL
    p
}

#' runHeatmap2
#'
#' Creates a heatmap based on the user selected parameters within shiny
#' @param input, input variables
#' @param session, session 
#' @param expdata, a matrix that includes expression values
#' @return heatmap.2
#'
#' @examples
#'     x <- runHeatmap2()
#'
#' @export
#'
#'
runHeatmap2 <- function(input = NULL, session = NULL, expdata = NULL){
    if(is.null(expdata)) return(NULL)
    if (nrow(expdata)>5000)
        expdata <- expdata[1:5000, ]
    
    if (!input$customColors ) {
        heatmapColors <- eval(parse(text=paste0(input$pal,
                                                '(',input$ncol,')')))
    }
    else{
        if (!is.null(input$color1))
            heatmapColors <- colorRampPalette(c(input$color1, 
                                                input$color2, input$color3))(n = 1000)
        #heatmapColors <- colorRampPalette(c("red", "white", "blue"))(n = 1000)
    }
    
    hclustfun_row <- function(x, ...) hclust(x, method = input$hclustFun_Row)
    distfun_row <- function(x, ...) {
        if (input$distFun_Row != "cor") {
            return(dist(x, method = input$distFun_Row))
        } else {
            return(as.dist(1 - cor(t(x))))
        }
    }
    if (!input$showClasses && "class" %in% names(expdata) ){
        expdata <- data.frame(expdata)
        expdata <- as.matrix(expdata [, -match("class",names(expdata))])
    }
    if (input$kmeansControl){
        m <- heatmap.2(as.matrix(expdata), Rowv = FALSE, main = input$main, dendrogram = input$dendrogram,
                       Colv = FALSE, col = heatmapColors, labRow = input$labRow,
                       distfun = distfun_row, hclustfun = hclustfun_row, density.info = "none",
                       trace = "none", margins = c(input$bottom/10, input$right/10))
    }else{
        m <- heatmap.2(as.matrix(expdata), main = input$main, dendrogram = input$dendrogram,
                       col = heatmapColors, labRow = input$labRow,
                       distfun = distfun_row, hclustfun = hclustfun_row, density.info = "none",
                       trace = "none", margins = c(input$bottom/10, input$right/10))
    }
    m
}


#' changeClusterOrder
#'
#' change order of K-means clusters
#'
#' @note \code{changeClusterOrder}
#' @param order, order
#' @param cld, data
#' @return heatmap plot area
#' @examples
#'     x <- changeClusterOrder()
#' @export
#'
changeClusterOrder <- function(order = NULL, cld = NULL){
    if (is.null(order) || is.null(cld) ) return(NULL)
    newcluster <- c()
    idx <- as.integer(as.vector(unlist(strsplit(order, ","))))
    da <- data.frame(cld)
    for (i in 1:length(idx)) {
        newcluster <- rbind(newcluster, da[da$class == idx[i], ])
    }
    newcluster
}


#' niceKmeans
#'
#' Generates hierarchially clustered K-means clusters
#'
#' @note \code{niceKmeans}
#' @param df, data
#' @param input, user inputs
#' @param iter.max, max iteration for kmeans clustering
#' @param nstart, n for kmeans clustering
#' @return heatmap plot area
#' @examples
#'     x <- niceKmeans()
#' @export
#'
niceKmeans <-function (df = NULL, input = NULL, iter.max = 1000, nstart=100) {
    if(is.null(df)) return(NULL)
    source <-df
    kmeans <- kmeans(source, centers = input$knum, iter.max = iter.max, algorithm=input$kmeansalgo, nstart=nstart)
    clustered <- data.frame()
    distfun_row <- function(x, ...) {
        if (input$distFun_Row != "cor") {
            return(dist(x, method = input$distFun_Row))
        } else {
            return(as.dist(1 - cor(t(x))))
        }
    }
    breaks <- c();
    for (i in 1:input$knum) {
        cluster <- source[kmeans$cluster==i,]
        rows <- row.names(cluster)
        clust <- hclust(distfun_row(as.matrix(cluster)), method = input$hclustFun_Row)
        clust$rowInd <- clust[[3]]
        cluster.ordered <- cluster[clust$rowInd,]
        cluster.ordered.genes <- rows[clust$rowInd]
        row.names(cluster.ordered) <- cluster.ordered.genes
        class <- data.frame(row.names = cluster.ordered.genes)
        class[,"class"] <- i 
        cluster.ordered <- cbind(cluster.ordered, class)
        clustered <- rbind(clustered, cluster.ordered)
        if(i > 1 & i < input$knum) {
            breaks[i] <- as.numeric(breaks[i-1]) + length(rows)
        } else if(i==1) {
            breaks[i] <- length(rows);
        }
    }
    
    result <- list();
    result$clustered <- clustered;
    result$breaks <- breaks;
    return(result);
}


#' getHeatmapUI
#'
#' Generates the left menu to be used for heatmap plots
#'
#' @note \code{getHeatmapUI}
#' @param id, module ID
#' @return heatmap plot area
#' @examples
#'     x <- getHeatmapUI("heatmap")
#' @export
#'
getHeatmapUI <- function(id) {
    ns <- NS(id)
    uiOutput(ns("heatmapUI"))
}

#' heatmapControlsUI
#'
#' Generates the left menu to be used for heatmap plots
#'
#' @note \code{heatmapControlsUI}
#' @param id, module ID
#' @return HeatmapControls
#' @examples
#'     x <- heatmapControlsUI("heatmap")
#' @export
#'
heatmapControlsUI <- function(id) {
    ns <- NS(id)
    list(
        checkboxInput(ns('interactive'), 'Interactive', value = FALSE),
        kmeansControlsUI(id),
        shinydashboard::menuItem("Scale Options",
            checkboxInput(ns('scale'), 'Scale', value = TRUE),
            checkboxInput(ns('center'), 'Center', value = TRUE),
            checkboxInput(ns('log'), 'Log', value = TRUE),
            textInput(ns('pseudo'),'Pseudo Count','0.1')
        ),
        dendControlsUI(id, "Row"),
        dendControlsUI(id, "Col"),
        shinydashboard::menuItem("Heatmap Colors",
            conditionalPanel(paste0("!input['", ns("customColors"), "']"),
            palUI(id),
            sliderInput(ns("ncol"), "# of Colors", 
            min = 1, max = 256, value = 256)),
            customColorsUI(id)
        ),
        shinydashboard::menuItem("Heatmap Dendrogram",
            selectInput(ns('dendrogram'),'Type',
            choices = c("both", "row", "column", "none"),selected = 'both'),
            selectizeInput(ns("seriation"), "Seriation", 
            c(OLO="OLO", GW="GW", Mean="mean", None="none"),selected = 'OLO'),
            sliderInput(ns('branches_lwd'),'Branch Width',
            value = 0.6,min=0,max=5,step = 0.1)
        ),
        shinydashboard::menuItem("Heatmap Layout",
            textInput(ns('main'),'Title',''),
            textInput(ns('xlab'),'Sample label',''),
            sliderInput(ns('row_text_angle'),'Sample Text Angle',
            value = 0,min=0,max=180),
            textInput(ns('ylab'), 'Gene/Region label',''),
            sliderInput(ns('column_text_angle'),'Gene/Region Text Angle',
            value = 45,min=0,max=180)
        ))
}
#' kmeansControlsUI
#'
#' get kmeans controls
#'
#' @note \code{kmeansControlsUI}
#' @param id, module ID
#' @return controls
#' @examples
#'     x <- kmeansControlsUI("heatmap")
#' @export
#'
kmeansControlsUI <- function(id) {
    ns <- NS(id)
    shinydashboard::menuItem("kmeans",
        checkboxInput(ns('kmeansControl'), 'kmeans clustering', value = FALSE),
        conditionalPanel(paste0("input['", ns("kmeansControl"), "']"),
            sliderInput(ns("knum"), "k: # of Clusters", 
                min = 2, max = 20, value = 2),
            selectizeInput(ns("kmeansalgo"), "kmeans.algorithm",
                c("Hartigan-Wong", "Lloyd", "Forgy",
                "MacQueen"), selected = 'Lloyd'),
            textInput(ns('clusterorder'), 
                'The order of the clusters', ""),
            actionButtonDE(ns("changeOrder"), label = "Change Order", styleclass = "primary"),
            checkboxInput(ns('showClasses'), 'Show Classes', value = FALSE)))
}
#' dendControlsUI
#'
#' get distance metric parameters 
#'
#' @note \code{dendControlsUI}
#' @param id, module ID
#' @param dendtype, Row or Col
#' @return controls
#' @examples
#'     x <- dendControlsUI("heatmap")
#' @export
#'
dendControlsUI <- function(id, dendtype = "Row") {
    ns <- NS(id)
    shinydashboard::menuItem(paste0(dendtype, " dendrogram"),
        selectizeInput(ns(paste0("distFun_", dendtype)), "Dist. method", 
            distFunParamsUI(),
            selected = 'euclidean'),
        selectizeInput(ns(paste0("hclustFun_", dendtype)), "Clustering linkage",
            clustFunParamsUI(), 
            selected = 'complete'),
        sliderInput(ns(paste0("k_", dendtype)), "# of Clusters",
            min = 1, max = 10, value = 2),
        checkboxInput(ns(paste0('lab',dendtype)), paste0(dendtype, ' Labels'), value = TRUE))
}

#' clustFunParamsUI
#'
#' get cluster function parameter control
#'
#' @note \code{clustFunParamsUI}
#' @return cluster params
#' @examples
#'     x <- clustFunParamsUI()
#' @export
#'
clustFunParamsUI <- function() {
    c(Complete= "complete",Single= "single",Average= "average",
    Mcquitty= "mcquitty",Median= "median",Centroid= "centroid",
    Ward.D= "ward.D",Ward.D2= "ward.D2")
}

#' distFunParamsUI
#'
#' get distance metric parameters 
#'
#' @note \code{distFunParamsUI}
#' @return funParams
#' @examples
#'     x <- distFunParamsUI()
#' @export
#'
distFunParamsUI <- function() {
    c(Cor="cor", Euclidean="euclidean",Maximum='maximum',
    Manhattan='manhattan',Canberra='canberra',
    Binary='binary',Minkowski='minkowski')
}

#' palUI
#'
#' get pallete 
#'
#' @note \code{palUI}
#' @param id, namespace ID
#' @return pals
#' @examples
#'     x <- palUI("heatmap")
#' @export
#'
palUI <- function(id) {
    ns <- NS(id)
    # colSel='RdBu'
    colSel='BlueRed'
    selectizeInput(inputId = ns("pal"), 
    label ="Select Color Palette",
    choices = c('BlueRed' = 'bluered',
        'RdBu' = 'RdBu',
        'RedBlue' = 'redblue',
        'RdYlBu' = 'RdYlBu',
        'RdYlGn' = 'RdYlGn',
        'BrBG' = 'BrBG',
        'Spectral' = 'Spectral',
        'BuGn' = 'BuGn',
        'PuBuGn' = 'PuBuGn',
        'YlOrRd' = 'YlOrRd',
        'Heat' = 'heat.colors',
        'Grey' = 'grey.colors'),
    selected=colSel)
}

#' customColorsUI
#'
#' get Custom Color controls
#'
#' @note \code{getColRng}
#' @param id, namespace ID
#' @return color range
#' @examples
#'     x <- customColorsUI("heatmap")
#' @export
#'
customColorsUI <- function(id) {
    ns <- NS(id)
    list(
        checkboxInput(ns('customColors'), 'Custom Colors', value = FALSE),
        conditionalPanel(paste0("input['", ns("customColors"), "']"),
            colourpicker::colourInput(ns("color1"), "Choose min colour", "blue"),
            colourpicker::colourInput(ns("color2"), "Choose median colour", "white"),
            colourpicker::colourInput(ns("color3"), "Choose max colour", "red")))
}

#' prepHeatData
#'
#' scales the data
#'
#' @param expdata, a matrixthat includes expression values
#' @param input, input variables
#' @return heatdata
#'
#' @examples
#'     x <- prepHeatData()
#'
#' @export
#'
prepHeatData <- function(expdata = NULL, input = NULL) 
{
    if(is.null(expdata)) return(NULL)
    ld <- expdata
    if (!is.null(input$pseudo))
        ld <- ld + as.numeric(input$pseudo)
    if (!is.null(input$log) && input$log)
        ld <- log2(ld)
    cldt <- scale(t(ld), center = input$center, scale = input$scale)
    cld <- t(cldt)
    return(cld)
}

#' getSelHeat
#'
#' heatmap selection functionality
#'
#' @param expdata, selected genes
#' @param input, input params
#' @return plot
#' @export
#'
#' @examples
#'     x <- getSelHeat()
#'
getSelHeat <- function(expdata = NULL, input = NULL) {
    if (is.null(input)) return(NULL)
    getSelected <- reactive({
        expdata[unlist(strsplit(input, ",")), ]
    })
    list( getSelected = isolate(getSelected) )
}


#' heatmapJScode
#'
#' heatmap JS code for selection functionality
#'
#' @return JS Code
#' @export
#'
#' @examples
#'     x <- heatmapJScode()
#'
heatmapJScode <- function() {        
    'shinyjs.getHoverName = function(params){
    
    var defaultParams = {
    controlname : "hoveredgenename"
    };
    params = shinyjs.getParams(params, defaultParams);
    var out = ""
    
    if (typeof  document.getElementsByClassName("nums")[0] != "undefined"){
    if (typeof  document.getElementsByClassName("nums")[0].querySelectorAll("tspan.line")[0] != "undefined"){
    out = document.getElementsByClassName("nums")[0].querySelectorAll("tspan.line")[0].innerHTML.match("row: (.*)")[1]
    $("#heatmap-heatmap").attr("gname", out)
    }
    }
    Shiny.onInputChange(params.controlname, $("#heatmap-heatmap").attr("gname"));
    }
    shinyjs.resetInputParam = function(params){
        var defaultParams = {
                controlname : "hoveredgenename"
        };
        params = shinyjs.getParams(params, defaultParams);
        console.log(params.controlname)
        Shiny.onInputChange(params.controlname, "");
    }

    shinyjs.getSelectedGenes = function(params){
    var defaultParams = {
    plotId : "heatmap",
    controlname : "selgenenames"
    };
    params = shinyjs.getParams(params, defaultParams);
    var count = document.getElementById(params.plotId).querySelectorAll("g.y2tick").length
    var start = 0
    var out = ""
    
    for (i = start; i < count; i++)
    {
        if (typeof document.getElementById(params.plotId).querySelectorAll("g.y2tick")[i] != "undefined"){
        out += document.getElementById(params.plotId).querySelectorAll("g.y2tick")[i].innerHTML.match(">(.*)</text>")[1]  + ","
        }
    }
    Shiny.onInputChange(params.controlname, out);
    }'
}

#' getJSLine
#'
#' heatmap JS code for selection functionality
#'
#' @return JS Code
#' @export
#'
#' @examples
#'     x <- getJSLine()
#'
getJSLine <-function()
{        
    list(shinyjs::useShinyjs(),
         shinyjs::extendShinyjs(text = heatmapJScode(), functions = c("getHoverName", "getSelectedGenes", "resetInputParam")))
}


#' heatmapServer
#'
#' Sets up shinyServer to be able to run heatmapServer interactively.
#'
#' @note \code{heatmapServer}
#' @param input, input params from UI
#' @param output, output params to UI
#' @param session, session variable
#' @return the panel for main plots;
#'
#' @examples
#'     heatmapServer
#'
#' @export


heatmapServer <- function(input, output, session) {
    updata <- reactiveVal()
    selected <- reactiveVal()
    expdata <- reactiveVal()
    observe({
        updata(callModule(debrowserdataload, "load", "Submit"))
    })
    observe({
        if(!is.null(updata()$load()$count))
        if (nrow(updata()$load()$count) > 1000){
            updateCheckboxInput(session, "mostvaried", value = TRUE)
            expdata(getMostVariedList(updata()$load()$count, 
            colnames(updata()$load()$count), input))
        }
        else
            expdata(updata()$load()$count)
    })
    
    observeEvent (input$Submit, {
        updateTabItems(session, "DEBrowserHeatmap", "Heatmap")
    })
    observe({
        if (!is.null(expdata())){
            withProgress(message = 'Creating plot', style = "notification", value = 0.1, {
                selected(callModule(debrowserheatmap, "heatmap", expdata()))
            })
        }
    })
    output$heatmap_hover <- renderPrint({
        if (!is.null(selected()) && !is.null(selected()$shgClicked()) && 
            selected()$shgClicked() != "")
            return(paste0("Clicked: ",selected()$shgClicked()))
        else
            return(paste0("Hovered:", selected()$shg()))
    })
    output$heatmap_selected <- renderPrint({
        if (!is.null(selected()))
            selected()$selGenes()
    })
    output$topn <- renderPrint({
        if (!is.null(input$topn))
            input$topn
    })
    output$mincount <- renderPrint({
        if (!is.null(input$mincount))
            input$mincount
    })
}

#' heatmapUI
#'
#' Creates a shinyUI to be able to run DEBrowser interactively.
#'
#' @param input, input variables
#' @param output, output objects
#' @param session, session
#'
#' @note \code{heatmapUI}
#' @return the panel for heatmapUI;
#'
#' @examples
#'     x<-heatmapUI()
#'
#' @export
#'

heatmapUI <- function(input, output, session) {
    header <- dashboardHeader(
        title = "DEBrowser Heatmap"
    )
    sidebar <- dashboardSidebar(  getJSLine(),  
        sidebarMenu(id="DEBrowserHeatmap",
        menuItem("Upload", tabName = "Upload"),
        menuItem("Heatmap", tabName = "Heatmap"),
        menuItem("Options", tabName = "Heatmap",
        checkboxInput('mostvaried', 'Most Varied Set', value = FALSE),
        conditionalPanel( (condition <- "input.mostvaried"),
        textInput("topn", "top-n", value = "500" ), 
        textInput("mincount", "total min count", value = "10" )),
        plotSizeMarginsUI("heatmap"),
        heatmapControlsUI("heatmap"))))
    
    body <- dashboardBody(
        tabItems(
            tabItem(tabName="Upload", dataLoadUI("load")),
            tabItem(tabName="Heatmap",  getHeatmapUI("heatmap"),
                    column(4,
                        verbatimTextOutput("heatmap_hover"),
                        verbatimTextOutput("heatmap_selected"),
                        verbatimTextOutput("topn"),
                        verbatimTextOutput("mincount")
                    ))
        ))
    
    dashboardPage(header, sidebar, body, skin = "blue")
}

#' startDEBrowser
#'
#' Starts the DEBrowser to be able to run interactively.
#'
#' @note \code{startDEBrowser}
#' @return the app
#'
#' @examples
#'     startDEBrowser()
#'
#' @export
#'
startDEBrowser <- function(){
    if (interactive()) {
        #the upload file size limit is 30MB
        options( shiny.maxRequestSize = 30 * 1024 ^ 2, warn = -1,
                 shiny.sanitize.errors = TRUE)
        addResourcePath(prefix = "demo", directoryPath =
                        system.file("extdata", "demo", 
                        package = "debrowser"))
        addResourcePath(prefix = "www", directoryPath =
                        system.file("extdata", "www", 
                        package = "debrowser"))
        environment(deServer) <- environment()

        app <- shinyApp( ui = shinyUI(deUI),
                    server = shinyServer(deServer))
        runApp(app)
    }
}

#' startHeatmap
#'
#' Starts the DEBrowser heatmap
#'
#' @note \code{startHeatmap}
#' @return the app
#'
#' @examples
#'     startHeatmap()
#'
#' @export
#'
startHeatmap <- function(){
    if (interactive()) {
        #the upload file size limit is 30MB
        options( shiny.maxRequestSize = 30 * 1024 ^ 2, warn = -1,
                 shiny.sanitize.errors = TRUE)
        addResourcePath(prefix = "demo", directoryPath =
                            system.file("extdata", "demo", 
                                        package = "debrowser"))
        addResourcePath(prefix = "www", directoryPath =
                            system.file("extdata", "www", 
                                        package = "debrowser"))
        environment(heatmapServer) <- environment()

        app <- shinyApp( ui = shinyUI(heatmapUI),
                         server = shinyServer(heatmapServer))
        runApp(app)
    }
}

#' deUI
#'
#' Creates a shinyUI to be able to run DEBrowser interactively.
#'
#' @note \code{deUI}
#' @return the panel for main plots;
#'
#' @examples
#'     x<-deUI()
#'
#' @export
#'

deUI <- function() {
    dbHeader <- shinydashboard::dashboardHeader(titleWidth = 250)
    dbHeader$children[[2]]$children <- tags$a(style='color: white;',
         id="top_logo" , paste0("DEBrowser v",getNamespaceVersion("debrowser")))
    addResourcePath(prefix = "www", directoryPath = system.file("extdata",
        "www", package = "debrowser"))
    library("debrowser")
    debrowser <- (fluidPage(
        shinyjs::useShinyjs(),
        shinyjs::inlineCSS("
        #loading-debrowser {
        position: absolute;
        background: #000000;
        opacity: 0.9;
        z-index: 100;
        left: 0;
        right: 0;
        height: 100%;
        text-align: center;
        color: #EFEFEF;
    }"),
    # Loading message
    tags$div(h4(paste0("Loading DEBrowser v",getNamespaceVersion("debrowser"))), id = "loading-debrowser",
        tags$img(src = "www/images/initial_loading.gif")),
    tags$head(tags$title(paste0("DEBrowser v",getNamespaceVersion("debrowser"))),
        tags$link(rel = "stylesheet", type = "text/css",
        href = "www/shinydashboard_additional.css")
    ),
    dashboardPage(
        dbHeader,
        dashboardSidebar(
            width = 250,
            debrowser::getJSLine(),
                uiOutput("loading"),
                tabsetPanel(id = "menutabs", type = "tabs",
                tabPanel(title = "Data Prep", value = "dataprep", id="dataprep",
                sidebarMenu(id="DataPrep",
                    menuItem("Quick Start Guide", icon = icon("user"),
                             menuSubItem("Introduction", tabName = "Intro"),
                             menuSubItem("Data Assesment", tabName = "assesment"),
                             menuSubItem("Data Preparation", tabName = "preparation"),
                             menuSubItem("DE Anaylsis", tabName = "deanalysis"),
                             menuSubItem("FAQ", tabName ="FAQ")
                    ),
                    menuItem("Upload", icon = icon("upload"), tabName = "Upload"),
                    menuItem("Filter", icon = icon("filter"), tabName = "Filter"),
                    menuItem("BatchEffect",  icon = icon("align-left"), tabName = "BatchEffect"),
                    menuItem("CondSelect",  icon = icon("bars"), tabName = "CondSelect"),
                    menuItem("DEAnalysis", icon = icon("adjust"), tabName = "DEAnalysis"),
                    menuItem("DEFilter",  icon = icon("code"), tabName = "DEAnalysis",  startExpanded = TRUE,
                             uiOutput("cutOffUI"),
                             uiOutput("compselectUI"))
                ),helpText("Developed by ", a("UMMS Biocore.", 
                href="https://www.umassmed.edu/biocore/", target = "_blank"))),
                tabPanel(title = "Discover", value = "discover", id="discover",
                conditionalPanel(condition = "(output.dataready)",
                    conditionalPanel( (condition <- "input.methodtabs=='panel1'"),
                    debrowser::mainPlotControlsUI("main")),
                    uiOutput("downloadSection"),
                    uiOutput('cutoffSelection'),
                    uiOutput("leftMenu"))
                 ))
        ),
    dashboardBody(
        mainPanel(
            width = 12,
            tags$head(
                tags$style(type = "text/css",
                        "#methodtabs.nav-tabs {font-size: 14px} ")),
                tabsetPanel(id = "methodtabs", type = "tabs",
                    tabPanel(title = "Data Prep", value = "panel0", id="panel0",
                             tabItems(
                                 tabItem(tabName="Intro", debrowser::getIntroText()),
                                 tabItem(tabName="assesment", debrowser::getDataAssesmentText()),
                                 tabItem(tabName="preparation", debrowser::getDataPreparationText()),
                                 tabItem(tabName="deanalysis", debrowser::getDEAnalysisText()),
                                 tabItem(tabName="FAQ",  debrowser::getQAText()),
                                 tabItem(tabName="Upload", debrowser::dataLoadUI("load")),
                                 tabItem(tabName="Filter",
                                         conditionalPanel(
                                             (condition <- "input.Filter"),
                                         debrowser::dataLCFUI("lcf"))),
                                 tabItem(tabName="BatchEffect", 
                                         conditionalPanel(
                                             (condition <- "input.Batch"),
                                         debrowser::batchEffectUI("batcheffect"))),
                                 tabItem(tabName="CondSelect", 
                                         conditionalPanel(
                                             (condition <- "input.goDE || input.goDEFromFilter"),
                                        debrowser::condSelectUI())),
                                 tabItem(tabName="DEAnalysis", 
                                         conditionalPanel(
                                             (condition <- "input.goDE || input.goDEFromFilter"),
                                         uiOutput("deresUI")))
                             )),
                    tabPanel(title = "Main Plots", value = "panel1", id="panel1",
                            uiOutput("mainmsgs"),
                            uiOutput("mainpanel")),
                    tabPanel(title = "QC Plots", value = "panel2", id="panel2",
                            uiOutput("qcpanel")),
                    tabPanel(title = "GO Term", value = "panel3", id="panel3",
                            uiOutput("gopanel")),
                    tabPanel(title = "Tables", value = "panel4", id="panel4",
                            DT::dataTableOutput("tables")))
        ),
        debrowser::getTabUpdateJS()
        ))
    )
    )
    debrowser
}

library(debrowser)

options(shiny.maxRequestSize = 30*1024^2)

header <- dashboardHeader(
    title = "DEBrowser Upload"
)
sidebar <- dashboardSidebar(  sidebarMenu(id="DataPrep",
   menuItem("Upload", tabName = "Upload")))

body <- dashboardBody(
    tabItems(
        tabItem(tabName="Upload", dataLoadUI("load"),
                column(4,
                       verbatimTextOutput("counttable"),
                       verbatimTextOutput("metadatatable")
                )
        )
    ))

ui <- dashboardPage(header, sidebar, body, skin = "blue")

server <- function(input, output, session) {
    data <- callModule(debrowserdataload, "load", "Filter")

    output$counttable <- renderPrint({
        head( data$load()$count )
    })
    output$metadatatable <- renderPrint({
        head( data$load()$meta)
    })
    
}

shinyApp(ui, server)

#' debrowserbarmainplot
#'
#' Module for a bar plot that can be used in data prep, main plots 
#' low count removal modules or any desired module
#' 
#' @param input, input variables
#' @param output, output objects
#' @param session, session 
#' @param data, a matrix that includes expression values
#' @param cols, columns
#' @param conds, conditions
#' @param cond_names, condition names
#' @param key, the gene or region name
#' @return density plot 
#' @export
#'
#' @examples
#'     x <- debrowserbarmainplot()
#'
debrowserbarmainplot <- function(input, output, session, data = NULL,
                                 cols = NULL, conds=NULL, cond_names=NULL, key=NULL) {
    if(is.null(data)) return(NULL)
    output$BarMainUI <- renderUI({
        shinydashboard::box(
            collapsible = TRUE, title = session$ns("plot"), status = "primary", 
            solidHeader = TRUE, width = NULL,
            draggable = TRUE,  plotlyOutput(session$ns("BarMain"),
                height=input$height, width=input$width))
    })
    output$BarMain <- renderPlotly({
        getBarMainPlot(data, cols, conds, cond_names, key, title = "", input =input)
    })
}

#' getBarMainPlotUI
#'
#' main bar plots UI.  
#'
#' @note \code{getBarMainPlotUI}
#' @param id, namespace id
#' @return the panel for Density plots;
#'
#' @examples
#'     x <- getBarMainPlotUI("bar")
#'
#' @export
#'
getBarMainPlotUI <- function(id) {
    ns <- NS(id)
    uiOutput(ns("BarMainUI"))
}


#' barMainPlotControlsUI
#'
#' Generates the controls in the left menu for a bar main plot
#'
#' @note \code{barMainPlotControlsUI}
#' @param id, namespace id
#' @return returns the controls for left menu
#' @examples
#'     x <- barMainPlotControlsUI("bar")
#' @export
#'
barMainPlotControlsUI <- function(id) {
  ns <- NS(id)
  shinydashboard::menuItem(paste0(id, " - Options"),
      textInput(ns("genename"), "Gene/Region Name", value = "Foxa3" )
  )
}

#' getBarMainPlot
#'
#' Makes Density plots
#'
#' @param data, count or normalized data
#' @param cols, cols
#' @param conds, conds
#' @param cond_names, condition names
#' @param key, key
#' @param title, title
#' @param input, input
#' @export
#'
#' @examples
#'     getBarMainPlot()
#'
getBarMainPlot <- function(data=NULL, cols = NULL, conds=NULL, cond_names=NULL, key=NULL, title = "", input = NULL){
    if (is.null(data)) return(NULL)
    
    cn <- unique(conds)
    conds[conds==cn[1]] <- cond_names[1]
    conds[conds==cn[2]] <- cond_names[2]
    vardata <- getVariationData(data, cols, conds, key)
 
    title <- paste(key, "variation")
    
    p <- plot_ly(vardata, x = ~libs, y = ~count, 
                 color=~conds, colors=c("Red", "Blue"),
                 type = "bar", height=input$height, width=input$width)
    p <- p %>% 
        plotly::layout(title = title,
            xaxis = list(categoryorder = "array",
                        categoryarray = cols,
                        title = "Conditions"
                        ),
            yaxis = list(title = "Read Count"),
            margin = list(l = input$left,
                          b = input$bottom,
                          t = input$top,
                          r = input$right
            )) 
    if (!is.null(input$svg) && input$svg == TRUE)
      p <- p %>% config(toImageButtonOptions = list(format = "svg"))
    p$elementId <- NULL
    p
}

#' getHistogramUI
#'
#' Histogram plots UI.  
#'
#' @note \code{getHistogramUI}
#' @param id, namespace id
#' @return the panel for PCA plots;
#'
#' @examples
#'     x <- getHistogramUI("histogram")
#'
#' @export
#'
getHistogramUI <- function(id) {
  ns <- NS(id)
  uiOutput(ns("histogramUI"))
}

#' debrowserhistogram
#'
#' Module for a histogram that can be used in data prep and 
#' low count removal modules
#' 
#' @param input, input variables
#' @param output, output objects
#' @param session, session 
#' @param data, a matrix that includes expression values
#' @return histogram 
#' @export
#'
#' @examples
#'     x <- debrowserhistogram()
#'
debrowserhistogram <- function(input = NULL, output = NULL, session = NULL, data = NULL) {
    if(is.null(data)) return(NULL)
    output$histogram <- renderPlotly({

      h <- hist(log10(rowSums(data)), breaks = as.numeric(input$breaks), plot = FALSE)
      
      p <- plot_ly(x = h$mids, y = h$counts, 
          width = input$width, height=input$height) %>% 
      add_bars() %>%
      plotly::layout(
        margin = list(l = input$left,
                      b = input$bottom,
                      t = input$top,
                      r = input$right
        ))
      p$elementId <- NULL
      if (!is.null(input$svg) && input$svg == TRUE)
          p <- p %>% config(toImageButtonOptions = list(format = "svg"))
      p
    })
    output$histogramUI <- renderUI({
    shinydashboard::box(
        collapsible = TRUE, title = session$ns("plot"), status = "primary", 
        solidHeader = TRUE, width = NULL,
        draggable = TRUE,  plotlyOutput(session$ns("histogram"),
             width = input$width, height=input$height))
    })
}

#' histogramControlsUI
#'
#' Generates the controls in the left menu for a histogram
#'
#' @note \code{histogramControlsUI}
#' @param id, namespace id
#' @return returns the left menu
#' @examples
#'     x <- histogramControlsUI("histogram")
#' @export
#'
histogramControlsUI <- function(id) {
  ns <- NS(id)
  textInput(ns("breaks"), "Breaks", value = "100" )
}

#' getBoxMainPlotUI
#'
#' main Box plots UI.  
#'
#' @note \code{getBoxMainPlotUI}
#' @param id, namespace id
#' @return the panel for Density plots;
#'
#' @examples
#'     x <- getBoxMainPlotUI("box")
#'
#' @export
#'
getBoxMainPlotUI <- function(id) {
  ns <- NS(id)
  uiOutput(ns("BoxMainUI"))
}

#' debrowserboxmainplot
#'
#' Module for a box plot that can be used in DEanalysis main part and 
#' used heatmaps
#' 
#' @param input, input variables
#' @param output, output objects
#' @param session, session 
#' @param data, a matrix that includes expression values
#' @param cols, columns
#' @param conds, conditions
#' @param cond_names, condition names
#' @param key, the gene or region name
#' @return density plot 
#' @export
#'
#' @examples
#'     x <- debrowserboxmainplot()
#'
debrowserboxmainplot <- function(input = NULL, output = NULL, session = NULL, data = NULL,
                                 cols = NULL, conds = NULL, cond_names = NULL, key=NULL) {
    if(is.null(data)) return(NULL)
    output$BoxMain <- renderPlotly({
        getBoxMainPlot(data, cols, conds, cond_names, key, title="", input)
    })
    
    output$BoxMainUI <- renderUI({
    shinydashboard::box(
        collapsible = TRUE, title = session$ns("plot"), status = "primary", 
        solidHeader = TRUE, width = NULL,
        draggable = TRUE,  plotlyOutput(session$ns("BoxMain"),
            height=input$height, width=input$width))
    })
}

#' BoxMainPlotControlsUI
#'
#' Generates the controls in the left menu for a Box main plot
#'
#' @note \code{BoxMainPlotControlsUI}
#' @param id, namespace id
#' @return returns the controls for left menu
#' @examples
#'     x <- BoxMainPlotControlsUI("box")
#' @export
#'
BoxMainPlotControlsUI <- function(id) {
  ns <- NS(id)
  shinydashboard::menuItem(paste0(id, " - Options"),
      textInput(ns("breaks"), "Breaks", value = "100" )
  )
}

#' getBoxMainPlot
#'
#' Makes Density plots
#'
#' @param data, count or normalized data
#' @param cols, cols
#' @param conds, conds
#' @param cond_names, condition names
#' @param key, key
#' @param title, title
#' @param input, input
#' @export
#'
#' @examples
#'     getBoxMainPlot()
#'
getBoxMainPlot <- function(data=NULL, cols = NULL, conds=NULL, cond_names=NULL, key=NULL, title = "", input = NULL){
  if (is.null(data)) return(NULL) 

  cn <- unique(conds)
  conds[conds==cn[1]] <- cond_names[1]
  conds[conds==cn[2]] <- cond_names[2]
  vardata <- getVariationData(data, cols, conds, key)

  title <- paste(key, "variation")
  p <- plot_ly(vardata, x = ~conds, y = ~count, 
               color=~conds, colors=c("Red", "Blue"),
               boxpoints = "all", type = "box", height=input$height, width=input$width) %>%
       plotly::layout(title = title,
                  xaxis = list(categoryorder = "array",
                               categoryarray = cols,
                               title = "Conditions"),
                   yaxis = list(title = "Read Count"),
                   margin = list(l = input$left,
                                 b = input$bottom,
                                 t = input$top,
                                 r = input$right
                   ))
  if (!is.null(input$svg) && input$svg == TRUE)
    p <- p %>% config(toImageButtonOptions = list(format = "svg"))
  p$elementId <- NULL
  p
}

library(debrowser)

header <- shinydashboard::dashboardHeader(
    title = "DEBrowser PCA Plots"
)
sidebar <- shinydashboard::dashboardSidebar(  
    shinydashboard::sidebarMenu(id="DataAssessment",
        shinydashboard::menuItem("PCA", tabName = "PCA"),
        shinydashboard::menuItem("PCA Options",
    pcaPlotControlsUI("pca")),
    plotSizeMarginsUI("pca", w=600, h=400, t=0, b=0, l=0, r=0)
    ))

body <- shinydashboard::dashboardBody(
    shinydashboard::tabItems(
        shinydashboard::tabItem(tabName="PCA", getPCAPlotUI("pca"),
                column(4,
                       verbatimTextOutput("pca_hover"),
                       verbatimTextOutput("pca_selected")
                )
        )
    ))

ui <- shinydashboard::dashboardPage(header, sidebar, body, skin = "blue")


server <- function(input, output, session) {
    load(system.file("extdata", "demo", "demodata.Rda",
                     package = "debrowser"))
    selected <- callModule(debrowserpcaplot, "pca", demodata, metadatatable)
}

shinyApp(ui, server)

library(debrowser)

header <- dashboardHeader(
  title = "DEBrowser Histogram Plots"
)
sidebar <- dashboardSidebar(  sidebarMenu(id="DataAssessment",
    menuItem("Histogram", tabName = "Histogram"),
    textInput("maxCutoff", "Max Cutoff", value = "10" ),
    shinydashboard::menuItem("histogram - Options",
    histogramControlsUI("histogram")),
    plotSizeMarginsUI("histogram", w=400, h=300),
    shinydashboard::menuItem("afterFiltering - Options",
    histogramControlsUI("afterFiltering")),
    plotSizeMarginsUI("afterFiltering", w=400, h=300)
))

body <- dashboardBody(
  tabItems(
    #########################################
    ## Introduction tab panel
    tabItem(tabName="Histogram", 
        fluidRow(
        column(5,
        getHistogramUI("histogram")),
        column(5,
        getHistogramUI("afterFiltering"))
        )
    )
  ))

ui <- dashboardPage(header, sidebar, body, skin = "blue")

server <- function(input, output, session) {
    #load(system.file("extdata", "demo", "demodata.Rda",
     #                package = "debrowser"))
    
    filtd <- reactive({
        # Filter out the rows that has maximum 100 reads in a sample
        #subset(demodata, apply(demodata, 1, max, na.rm = TRUE)  >=  
        #    as.numeric(input$maxCutoff))
        subset(cpmnorm, apply(cpmnorm, 1, max, na.rm = TRUE)  >=  
                      as.numeric(input$maxCutoff))
    })
    observe({
      if(!is.null(filtd())){
          callModule(debrowserhistogram, "histogram", cpmnorm)
          callModule(debrowserhistogram, "afterFiltering", filtd())
      }
    })
}

shinyApp(ui, server)

#' debrowserdeanalysis
#'
#' Module to perform and visualize DE results.
#' 
#' @param input, input variables
#' @param output, output objects
#' @param session, session 
#' @param data, a matrix that includes expression values
#' @param metadata, metadata
#' @param columns, columns
#' @param conds, conditions
#' @param params, de parameters
#' @return DE panel 
#' @export
#'
#' @examples
#'     x <- debrowserdeanalysis()
#'
debrowserdeanalysis <- function(input = NULL, output = NULL, session = NULL, 
    data = NULL, metadata = NULL, columns = NULL, conds = NULL, params = NULL) {
    if(is.null(data)) return(NULL)
    deres <- reactive({
        runDE(data, metadata, columns, conds, params)
    })
    prepDat <- reactive({
        applyFiltersNew(addDataCols(data, deres(), columns, conds), input)
    })
    observe({
        if(!is.null(input$legendradio)){
            if(input$legendradio == "All"){
                dat <- prepDat()
            } else {
                dat <- prepDat()[prepDat()$Legend == input$legendradio,]
            }   
        } else {
            dat <- NULL
        }
        dat2 <- removeCols(c("ID", "x", "y","Legend", "Size"), dat)
        getTableDetails(output, session, "DEResults", dat2, modal=FALSE)
    })
    list(dat = prepDat)
}
#' getDEResultsUI
#' Creates a panel to visualize DE results
#'
#' @param id, namespace id
#' @return panel
#' @examples
#'     x <- getDEResultsUI("batcheffect")
#'
#' @export
#'
getDEResultsUI<- function (id) {
    ns <- NS(id)
    list(
        fluidRow(
            shinydashboard::box(title = "DE Results",
            solidHeader = T, status = "info",  width = 12, 
            fluidRow(
                column(12,
                uiOutput(ns("DEResults"))
                ),
                    actionButtonDE("goMain", "Go to Main Plots", styleclass = "primary")
                )
            )
        )
        )
}

#' cutOffSelectionUI
#'
#' Gathers the cut off selection for DE analysis
#'
#' @param id, namespace id
#' @note \code{cutOffSelectionUI}
#' @return returns the left menu according to the selected tab;
#' @examples
#'     x <- cutOffSelectionUI("cutoff")
#' @export
#'
cutOffSelectionUI <- function(id){
    ns <- NS(id)
    list(
        getLegendRadio(id),
        textInput(ns("padj"), "padj value cut off", value = "0.01" ),
        textInput(ns("foldChange"), "or foldChange", value = "2" )
    )
}

#' applyFiltersNew
#'
#' Apply filters based on foldChange cutoff and padj value.
#' This function adds a "Legend" column with "Up", "Down" or
#' "NS" values for visualization.
#'
#' @param data, loaded dataset
#' @param input, input parameters
#' @return data
#' @export
#'
#' @examples
#'     x <- applyFiltersNew()
#'
applyFiltersNew <- function(data = NULL, input = NULL) {
    if (is.null(data)) return(NULL)
    padj_cutoff <- as.numeric(input$padj)
    foldChange_cutoff <- as.numeric(input$foldChange)
    m <- data
    if (!("Legend" %in% names(m))) {
        m$Legend <- character(nrow(m))
        m$Legend <- "NS"
    }
    m$Legend[m$foldChange >= foldChange_cutoff &
        m$padj <= padj_cutoff] <- "Up"
    m$Legend[m$foldChange <= (1 / foldChange_cutoff) &
        m$padj <= padj_cutoff] <- "Down"
    return(m)
}

    
#' runDE
#'
#' Run DE algorithms on the selected parameters.  Output is
#' to be used for the interactive display.
#'
#' @param data, A matrix that includes all the expression raw counts,
#'     rownames has to be the gene, isoform or region names/IDs
#' @param metadata, metadata of the matrix of expression raw counts
#' @param columns, is a vector that includes the columns that are going
#'     to be analyzed. These columns has to match with the given data.
#' @param conds, experimental conditions. The order has to match
#'     with the column order
#' @param params, all params for the DE methods
#' @return de results
#'
#' @export
#'
#' @examples
#'     x <- runDE()
#'
runDE <- function(data = NULL, metadata = NULL, columns = NULL, conds = NULL, params = NULL) {
    if (is.null(data)) return(NULL)
    de_res <- NULL
    if (startsWith(params[1], "DESeq2"))
        de_res <- runDESeq2(data, metadata, columns, conds, params)
    else if (startsWith(params[1], "EdgeR"))    
        de_res <- runEdgeR(data, metadata, columns, conds, params)
    else if (startsWith(params[1], "Limma"))
        de_res <- runLimma(data, metadata, columns, conds, params)
    data.frame(de_res)
}

#' runDESeq2
#'
#' Run DESeq2 algorithm on the selected conditions.  Output is
#' to be used for the interactive display.
#'
#' @param data, A matrix that includes all the expression raw counts,
#'     rownames has to be the gene, isoform or region names/IDs
#' @param metadata, metadata of the matrix of expression raw counts
#' @param columns, is a vector that includes the columns that are going
#'     to be analyzed. These columns has to match with the given data.
#' @param conds, experimental conditions. The order has to match
#'     with the column order
#' @param params, fitType: either "parametric", "local", or "mean" for the type 
#'     of fitting of dispersions to the mean intensity. 
#'     See estimateDispersions for description.
#'  betaPrior: whether or not to put a zero-mean normal prior
#'     on the non-intercept coefficients See nbinomWaldTest for 
#'     description of the calculation of the beta prior. By default, 
#'     the beta prior is used only for the Wald test, but can also be 
#'     specified for the likelihood ratio test.
#' testType: either "Wald" or "LRT", which will then use either 
#'     Wald significance tests (defined by nbinomWaldTest), or the 
#'     likelihood ratio test on the difference in deviance between a 
#'     full and reduced model formula (defined by nbinomLRT)
#' shrinkage: Adds shrunken log2 fold changes (LFC) and SE to a results 
#'     table from DESeq run without LFC shrinkage. For consistency with 
#'     results, the column name lfcSE is used here although what is 
#'     returned is a posterior SD. Three shrinkage estimators for 
#'     LFC are available via type (see the vignette for more details 
#'     on the estimators). The apeglm publication demonstrates that 
#'     'apeglm' and 'ashr' outperform the original 'normal' shrinkage 
#'     estimator.
#' @return deseq2 results
#'
#' @export
#'
#' @examples
#'     x <- runDESeq2()
#'
runDESeq2 <- function(data = NULL, metadata = NULL, columns = NULL, conds = NULL, params = NULL) {
    if (is.null(data)) return(NULL)
    if (length(params)<3)
        params <- strsplit(params, ",")[[1]]
    covariates <- if (!is.null(params[2])) params[2]
    covariates <- strsplit(covariates, split = "\\|")[[1]]
    fitType <- if (!is.null(params[3])) params[3]
    betaPrior <-  if (!is.null(params[4])) params[4]
    testType <- if (!is.null(params[5])) params[5]
    shrinkage <-  if (!is.null(params[6])) params[6]

    if(length(columns)<3){
        showNotification("You cannot use DESeq2 if you don't have multiple samples per condition", type = "error")
        return(NULL)
    }
    
    data <- data[, columns]

    data[, columns] <- apply(data[, columns], 2,
        function(x) as.integer(x))

    coldata <- prepGroup(conds, columns, metadata, covariates)
    # Filtering non expressed genes
    filtd <- data
    
    # DESeq data structure is going to be prepared
    if(covariates != "NoCovariate"){
        dds_formula <- as.formula(paste0("~ group", paste0(" + covariate", 1:length(covariates))))
        dds <- DESeqDataSetFromMatrix(countData = as.matrix(filtd),
                                      colData = coldata, design = dds_formula)
    } else {
        dds <- DESeqDataSetFromMatrix(countData = as.matrix(filtd),
                                      colData = coldata, design = ~group)
    }
    # Running DESeq
    if (testType == "LRT")
        dds <- DESeq(dds, fitType = fitType, betaPrior = as.logical(betaPrior), test=testType, reduced= ~ 1)
    else
        dds <- DESeq(dds, fitType = fitType, betaPrior = as.logical(betaPrior), test=testType)
        
    coef_names <- colnames(coef(dds))
    group_name <- coef_names[grepl("group",coef_names)][1]
    res <- results(dds, name = group_name)
    if (shrinkage != "None"){
        res <- lfcShrink(dds, coef=2, res=res, type = shrinkage)
        if (testType == "Wald"){
           colname <- names(dds@rowRanges@elementMetadata)[grepl(paste0(testType, "Statistic_group"), names(dds@rowRanges@elementMetadata))]
        }else{
           colname <- paste0(testType, "Statistic")
        }
        stat <- dds@rowRanges@elementMetadata[colname]
        res <- cbind(res, stat)
        colnames(res)[colnames(res) == colname] <- "stat"
    }
    return(res)
}

#' runEdgeR
#'
#' Run EdgeR algorithm on the selected conditions.  Output is
#' to be used for the interactive display.
#'
#' @param data, A matrix that includes all the expression raw counts,
#'     rownames has to be the gene, isoform or region names/IDs
#' @param metadata, metadata of the matrix of expression raw counts
#' @param columns, is a vector that includes the columns that are going
#'     to be analyzed. These columns has to match with the given data.
#' @param conds, experimental conditions. The order has to match
#'     with the column order
#' @param params, normfact: Calculate normalization factors to scale the raw 
#'     library sizes. Values can be "TMM","RLE","upperquartile","none".
#' dispersion: either a numeric vector of dispersions or a character 
#'     string indicating that dispersions should be taken from the data 
#'     object. If a numeric vector, then can be either of length one or 
#'     of length equal to the number of genes. Allowable character 
#'     values are "common", "trended", "tagwise" or "auto". 
#'     Default behavior ("auto" is to use most complex dispersions 
#'     found in data object.
#' testType: exactTest or glmLRT. exactTest: Computes p-values for differential 
#'     abundance for each gene between two digital libraries, conditioning 
#'     on the total count for each gene. The counts in each group as a 
#'     proportion of the whole are assumed to follow a binomial distribution. 
#'     glmLRT: Fit a negative binomial generalized log-linear model to the read 
#'     counts for each gene. Conduct genewise statistical tests for a given 
#'     coefficient or coefficient contrast.
#' @return edgeR results
#'
#' @export
#'
#' @examples
#'     x <- runEdgeR()
#'
runEdgeR<- function(data = NULL, metadata = NULL, columns = NULL, conds = NULL, params = NULL){
    if (is.null(data)) return(NULL)
    if (length(params)<3)
        params <- strsplit(params, ",")[[1]]
    covariates <- if (!is.null(params[2])) params[2]
    covariates <- strsplit(covariates, split = "\\|")[[1]]
    normfact <- if (!is.null(params[3])) params[3]
    dispersion <- if (!is.null(params[4])) params[4]
    testType <- if (!is.null(params[5])) params[5]

    data <- data[, columns]
    data[, columns] <- apply(data[, columns], 2,
        function(x) as.integer(x))

    if (!is.na(dispersion) && !(dispersion %in% c("common", "trended", "tagwise", "auto")))
        dispersion <- as.numeric(dispersion)

    conds <- factor(conds)
    filtd <- data
    
    d<- edgeR::DGEList(counts = filtd, group = conds)
    d <- edgeR::calcNormFactors(d, method = normfact)
    # If dispersion is 0, it will estimate the dispersions.
    de.com <- c() 
    cnum = summary(conds)[levels(conds)[1]]
    tnum = summary(conds)[levels(conds)[2]]
    des <- c(rep(1, cnum),rep(2, tnum))
    if(cnum == 1 && tnum == 1 && 
         (dispersion %in% c("common", "trended", "tagwise", "auto") ||
            dispersion == 0)){
        showNotification("You cannot use this dispersion with common, trended, tagwise, 
            auto, when there are 1 replicates for each condition. 
            Please use a numeric value if that's the case.", type = "error")
        return(NULL)
    }
    
    if(covariates != "NoCovariate"){
        des_formula <- as.formula(paste0("~ des", paste0(" + covariate", 1:length(covariates))))
        model.matrix_data <- data.frame(des = des)
        sample_column_ind <- which(apply(metadata, 2, function(x) sum(x %in% columns) == length(columns))) 
        sample_column <- colnames(metadata)[sample_column_ind]
        cov_metadata <- metadata[match(columns,metadata[,sample_column]),covariates, drop = FALSE]
        for(i in 1:length(covariates)){
            cur_covariate <- cov_metadata[,i]
            cur_covariate <- factor(cur_covariate)
            model.matrix_data[[paste0("covariate",i)]] <- cur_covariate 
        }
        design <- model.matrix(des_formula, data = model.matrix_data)
    } else {
        design <- model.matrix(~des)
    }

    d <- edgeR::estimateDisp(d, design)
    if (testType == "exactTest"){
        if (dispersion == 0){
            de.com <- edgeR::exactTest(d)
        }else{
            de.com <- edgeR::exactTest(d, dispersion=dispersion)
            
        }
        de.com$table<-topTags(de.com, n = nrow(de.com$table))$table
        colnames(de.com$table)[colnames(de.com$table) == "FDR"] <- "stat"
    }else if (testType == "glmLRT"){
        if (dispersion == 0){
            fit <- edgeR::glmFit(d, design)
        }else{
            fit <- edgeR::glmFit(d, design, dispersion=dispersion)
        }   
        de.com <- edgeR::glmLRT(fit,coef=2)
        colnames(de.com$table)[colnames(de.com$table) == "LR"] <- "stat"
    }

    options(digits=4)

    padj<- p.adjust(de.com$table$PValue, method="BH")
    res <-data.frame(cbind(de.com$table$logFC/log(2),de.com$table$PValue, padj, de.com$table$stat))
    colnames(res) <- c("log2FoldChange", "pvalue", "padj", "stat")
    rownames(res) <- rownames(filtd)
    return(res)
}

#' runLimma
#'
#' Run Limma algorithm on the selected conditions.  Output is
#' to be used for the interactive display.
#'
#' @param data, A matrix that includes all the expression raw counts,
#'     rownames has to be the gene, isoform or region names/IDs
#' @param metadata, metadata of the matrix of expression raw counts
#' @param columns, is a vector that includes the columns that are going
#'     to be analyzed. These columns has to match with the given data.
#' @param conds, experimental conditions. The order has to match
#'     with the column order
#' @param params, normfact: Calculate normalization factors to scale the raw 
#'     library sizes. Values can be "TMM","RLE","upperquartile","none".
#' fitType, fitting method; "ls" for least squares or "robust" 
#'     for robust regression
#' normBet: Normalizes expression intensities so that the 
#'     intensities or log-ratios have similar distributions across a set of arrays.
#' @return Limma results
#'
#' @export
#'
#' @examples
#'     x <- runLimma()
#'
runLimma<- function(data = NULL, metadata = NULL, columns = NULL, conds = NULL, params = NULL){
    if (is.null(data)) return(NULL)
    if (length(params)<3)
        params <- strsplit(params, ",")[[1]]
    covariates <- if (!is.null(params[2])) params[2]
    covariates <- strsplit(covariates, split = "\\|")[[1]]
    normfact = if (!is.null(params[3])) params[3]
    fitType = if (!is.null(params[4])) params[4]
    normBet = if (!is.null(params[5])) params[5]

    data <- data[, columns]
    data[, columns] <- apply(data[, columns], 2,
        function(x) as.integer(x))
    conds <- factor(conds)
 
    cnum = summary(conds)[levels(conds)[1]]
    tnum = summary(conds)[levels(conds)[2]]
    filtd <- data
    
    des <- factor(c(rep(levels(conds)[1], cnum),rep(levels(conds)[2], tnum)))
    names(filtd) <- des
    
    if(covariates != "NoCovariate"){
        design <- cbind(Grp1=1,Grp2vs1=des)
        sample_column_ind <- which(apply(metadata, 2, function(x) sum(x %in% columns) == length(columns))) 
        sample_column <- colnames(metadata)[sample_column_ind]
        cov_metadata <- metadata[match(columns,metadata[,sample_column]),covariates, drop = FALSE]
        for(i in 1:length(covariates)){
            cur_covariate <- cov_metadata[,i]
            cur_covariate <- factor(cur_covariate)
            design <- cbind(design, cur_covariate)
            colnames(design)[length(colnames(design))] <-  paste0("covariate",i)
        }
    } else {
        design <- cbind(Grp1=1,Grp2vs1=des)
    }
    
    dge <- DGEList(counts=filtd, group = des)
    dge <- calcNormFactors(dge, method=normfact, samples=columns)
    
    v <- voom(dge, design=design, normalize.method = normBet, plot=FALSE)
    
    fit <- lmFit(v, design=design)
    fit <- eBayes(fit)
    
    options(digits=4)
    tab <- topTable(fit,coef=2, number=dim(fit)[1],genelist=fit$genes$NAME)
    res <-data.frame(cbind(tab$logFC, tab$P.Value, tab$adj.P.Val, tab$t))
    
    colnames(res) <- c("log2FoldChange", "pvalue", "padj", "stat")
    rownames(res) <- rownames(tab)
    return(res)
}

#' prepGroup
#'
#' prepare group table
#'
#' @param cols, columns
#' @param conds, inputconds
#' @param metadata, metadata
#' @param covariates, covariates
#' @return data
#' @export
#'
#' @examples
#'     x <- prepGroup()
#'
prepGroup <- function(conds = NULL, cols = NULL, metadata = NULL, covariates = NULL) {
    if (is.null(conds) || is.null(cols)) return (NULL)
    coldata <- data.frame(cbind(cols, conds))
    coldata$conds <- factor(coldata$conds)
    colnames_coldata <- c("libname", "group")
    if(!is.null(covariates)){
        if(covariates!="NoCovariate"){
            sample_column_ind <- which(apply(metadata, 2, function(x) sum(x %in% cols) == length(cols))) 
            sample_column <- colnames(metadata)[sample_column_ind]
            covariates <- metadata[match(cols,metadata[,sample_column]), covariates, drop = FALSE]
            for(i in 1:ncol(covariates)){
                cur_covariate <- covariates[,i]
                cur_covariate <- factor(cur_covariate)
                coldata <- data.frame(cbind(coldata, cur_covariate))
                colnames_coldata <- c(colnames_coldata, paste0("covariate",i))   
            }
        }
    }
    colnames(coldata) <- colnames_coldata
    coldata
}

#' addDataCols
#'
#' add aditional data columns to de results
#'
#' @param data, loaded dataset
#' @param de_res, de results
#' @param cols, columns
#' @param conds, inputconds
#' @return data
#' @export
#'
#' @examples
#'     x <- addDataCols()
#'
addDataCols <- function(data = NULL, de_res = NULL, cols = NULL, conds = NULL) {
    if (is.null(data) || (nrow(de_res) == 0 && ncol(de_res) == 0)) return (NULL)
    norm_data <- data[, cols]
    
    coldata <- prepGroup(conds, cols)
    
    mean_cond_first <- getMean(norm_data, as.vector(coldata[coldata$group==levels(coldata$group)[1], "libname"]))
    mean_cond_second <- getMean(norm_data, as.vector(coldata[coldata$group==levels(coldata$group)[2], "libname"]))
    
    m <- cbind(rownames(de_res), norm_data[rownames(de_res), cols],
               log10(unlist(mean_cond_second) + 1),
               log10(unlist(mean_cond_first) + 1),
               de_res[rownames(de_res),
                      c("padj", "log2FoldChange", "pvalue", "stat")], 
               2 ^ de_res[rownames(de_res),
                          "log2FoldChange"],
               -1 * log10(de_res[rownames(de_res), "padj"]))
    colnames(m) <- c("ID", cols, "x", "y",
                     "padj", "log2FoldChange", "pvalue", "stat",
                     "foldChange", "log10padj")
    m <- as.data.frame(m)
    m$padj[is.na(m[paste0("padj")])] <- 1
    m$pvalue[is.na(m[paste0("pvalue")])] <- 1
    m
}

#' getMean
#'
#' Gathers the mean for selected condition.
#'
#' @param data, dataset
#' @param selcols, input cols
#' @return data
#' @export
#'
#' @examples
#'     x <- getMean()
#'
getMean<-function(data = NULL, selcols=NULL) {
    if (is.null(data)) return (NULL)
    mean_cond<-NULL
    if (length(selcols) > 1)
        mean_cond <-list(rowMeans( data[, selcols]))
    else
        mean_cond <-list(data[selcols])
    mean_cond
}

#' getLegendRadio
#'
#' Radio buttons for the types in the legend
#' @param id, namespace id
#' @note \code{getLegendRadio}
#' @return radio control
#'
#' @examples
#'    
#'     x <- getLegendRadio("deprog")
#'
#' @export
#'
getLegendRadio <- function(id) {
    ns <- NS(id)
    types <- c("Up", "Down", "NS", "All")
    radioButtons(inputId=ns("legendradio"), 
                 label="Data Type:",
                 choices=types
                 )
}

#' getLeftMenu
#'
#' Generates the left menu for for plots within the DEBrowser.
#'
#' @param input, input values
#' @note \code{getLeftMenu}
#' @return returns the left menu according to the selected tab;
#' @examples
#'     x <- getLeftMenu()
#' @export
#'
getLeftMenu <- function(input = NULL) {
if (is.null(input)) return(NULL)
   leftMenu <- list(
        conditionalPanel( (condition <- "input.methodtabs=='panel1'"),
            getMainPlotsLeftMenu()),
        conditionalPanel( (condition <- "input.methodtabs=='panel2'"),
        shinydashboard::menuItem(" Plot Type", startExpanded = TRUE,
        wellPanel(radioButtons("qcplot",
                paste("QC Plots:", sep = ""),
                c(PCA = "pca", All2All = "all2all", Heatmap = "heatmap", IQR = "IQR",
                  Density = "Density")))),
            getQCLeftMenu(input)),
        conditionalPanel( (condition <- "input.methodtabs=='panel3'"),
            actionButton("startGO", "Submit"),
        shinydashboard::menuItem(" Plot Type", startExpanded = TRUE,
            wellPanel(radioButtons("goplot", paste("Go Plots:", sep = ""),
                c(enrichGO = "enrichGO", enrichKEGG = "enrichKEGG",
                Disease = "disease", compareClusters = "compare", GSEA = "GSEA")))),
                getGOLeftMenu()
                ),
        conditionalPanel( (condition <- "input.methodtabs=='panel4'"),
        shinydashboard::menuItem(" Select Columns", startExpanded=TRUE,
             uiOutput("getColumnsForTables")
        ))
    )
   return(leftMenu)
}
#' getMainPlotsLeftMenu
#'
#' Generates the Main PLots Left menu to be displayed within the DEBrowser.
#'
#' @note \code{getMainPlotsLeftMenu}
#' @return returns the left menu according to the selected tab;
#' @examples
#'     x <- getMainPlotsLeftMenu()
#' @export
#'
getMainPlotsLeftMenu <- function() {
    mainPlotsLeftMenu <- list(
        plotSizeMarginsUI("main",  w=600, h=400),
        shinydashboard::menuItem("Heatmap Options", startExpanded=FALSE,
        heatmapControlsUI("heatmap"),
        plotSizeMarginsUI("heatmap", w=550, h=400)),
        plotSizeMarginsUI("barmain", w=550,h=400, t=90),
        plotSizeMarginsUI("boxmain", w=550, h=400, t=90)
        )
    return(mainPlotsLeftMenu)
}

#' getGOLeftMenu
#'
#' Generates the GO Left menu to be displayed within the DEBrowser.
#'
#' @note \code{getGOLeftMenu}
#' @return returns the left menu according to the selected tab;
#' @examples
#'     x <- getGOLeftMenu()
#' @export
#'
getGOLeftMenu <- function() {
    list(
    shinydashboard::menuItem(" Go Term Options", startExpanded=TRUE, 
    textInput("gopvalue", "p.adjust", value = "0.01" ),
        getOrganismBox(),
            actionButton("GeneTableButton", "DE Genes"),
            conditionalPanel( (condition <- "input.goplot=='enrichKEGG'"),
                      actionButton("KeggPathway", "KeggPathway")),
            conditionalPanel( ( condition <- "(input.goplot=='enrichGO' ||
            (input.goplot=='compare' && input.gofunc!='enrichDO' &&
            input.gofunc!='enrichKEGG'))" ),
            selectInput("ontology", "Choose an ontology:",
                choices =  c( "CC", "MF", "BP"))
            ),
            conditionalPanel( ( condition <- "input.goplot!='compare'"),
                selectInput("goextplot", "Plot Type:",
                choices =  c("Summary", "Dotplot"))
            ),
            conditionalPanel( ( condition <- "input.goplot=='compare'"),
                selectInput("gofunc", "Plot Function:",
                choices =  c( "enrichGO", "enrichDO", "enrichKEGG"))
            ),
            conditionalPanel( ( condition <- "input.goplot=='GSEA'"),
                selectInput("sortfield", "Sort field:",
                choices =  c( "stat", "log2FoldChange"))
            ),
            downloadButton("downloadGOPlot", "Download Plots"))
    )

}

#' getQCLeftMenu
#'
#' Generates the left menu to be used for QC plots within the
#' DEBrowser.
#'
#' @param input, input values
#' @note \code{getQCLeftMenu}
#' @return QC left menu
#' @examples
#'     x <- getQCLeftMenu()
#' @export
#'
getQCLeftMenu <- function( input = NULL) {
    if (is.null(input)) return(NULL)
        list(
        shinydashboard::menuItem(" Select Columns", startExpanded=TRUE, 
            uiOutput("columnSelForQC")),
            shinydashboard::menuItem(" QC Options", startExpanded=FALSE,
            conditionalPanel( (condition <- "input.qcplot=='heatmap'"),
                plotSizeMarginsUI("heatmapQC"),
                heatmapControlsUI("heatmapQC")),
            conditionalPanel( condition <- "(input.qcplot=='all2all')",
                plotSizeMarginsUI("all2all"),
                all2allControlsUI("all2all")
            ),
            conditionalPanel( condition <- "(input.qcplot=='Density')",
                              plotSizeMarginsUI("density"),
                              plotSizeMarginsUI("normdensity")
            ),
            conditionalPanel( condition <- "(input.qcplot=='IQR')",
                              plotSizeMarginsUI("IQR"),
                              plotSizeMarginsUI("normIQR")
            ),
        getHelpButton("method",
        "http://debrowser.readthedocs.io/en/master/heatmap/heatmap.html"),
        conditionalPanel( (condition <- "input.qcplot=='pca'"),
            shinydashboard::menuItem("PCA Options",
            pcaPlotControlsUI("qcpca")),
            plotSizeMarginsUI("qcpca", w=600, h=400, t=0, b=0, l=0, r=0)
        ))
    )
}

#' getCutOffSelection
#'
#' Gathers the cut off selection for DE analysis
#'
#' @param nc, total number of comparisons
#' @note \code{getCutOffSelection}
#' @return returns the left menu according to the selected tab;
#' @examples
#'     x <- getCutOffSelection()
#' @export
#'
getCutOffSelection <- function(nc = 1){
    compselect <- getCompSelection("compselect", nc)
    list( conditionalPanel( (condition = "input.dataset!='most-varied' &&
        input.methodtabs!='panel0'"),
        shinydashboard::menuItem(" Filter",
        #h4("Filter"),
        textInput("padj", "padj", value = "0.01" ),
        textInput("foldChange", "foldChange", value = "2" ),
        compselect
        )
    ) )
}

#' getMainPanel
#'
#' main panel for volcano, scatter and maplot.  
#' Barplot and box plots are in this page as well.
#'
#' @note \code{getMainPanel}
#' @return the panel for main plots;
#'
#' @examples
#'     x <- getMainPanel()
#'
#' @export
#'
getMainPanel <- function() {
    list(
        fluidRow(column(6,
            getMainPlotUI("main")
        ),
        column(6,
            getHeatmapUI("heatmap")
        )),
        fluidRow(column(6,
            getBarMainPlotUI("barmain")),
        column(6,
            getBoxMainPlotUI("boxmain"))))
}

#' getProgramTitle
#'
#' Generates the title of the program to be displayed within DEBrowser.
#' If it is called in a program, the program title will be hidden
#'
#' @param session, session var
#' @note \code{getProgramTitle}
#' @return program title
#' @examples
#'     title<-getProgramTitle()
#' @export
#'
getProgramTitle <- function(session = NULL) {
    if (is.null(session)) return (NULL)
    DEBrowser <- NULL
    title<-parseQueryString(session$clientData$url_search)$title
    if (is.null(title) || title != "no" )
        DEBrowser <- list(titlePanel("DEBrowser"))
    else
        DEBrowser <- list(titlePanel(" "))
    return(DEBrowser)
}

#' getLoadingMsg
#'
#' Creates and displays the loading message/gif to be displayed
#' within the DEBrowser.
#'
#' @param output, output message
#' @note \code{getLoadingMsg}
#' @return loading msg
#' @examples
#'     x <- getLoadingMsg()
#' @export
#'
getLoadingMsg <- function(output = NULL) {
    addResourcePath(prefix = "www", directoryPath =
        system.file("extdata", "www",
        package = "debrowser"))
    imgsrc_full <- "www/images/loading_start.gif"
    imgsrc_small <- "www/images/loading.gif"
    a <- list(
        tags$head(tags$style(type = "text/css", "
            #loadmessage {
            position: fixed;
            top: 0px;
            left: 0px;
            width: 100%;
            height: 100%;
            padding: 5px 0px 5px 0px;
            text-align: center;
            font-weight: bold;
            font-size: 100%;
            color: #000000;
            opacity: 0.8;
            z-index: 100;
            }
            #loadmessage_small {
            position: fixed;
            left: 50%;
            transform: translateX(-50%);
            top: 50px;
            text-align: center;
            opacity: 0.8;
            z-index: 999999;
            }
                             ")),
        conditionalPanel(condition = paste0("$('html').hasClass('shiny-busy')",
            "& input.startDE & input.methodtabs=='panel0'"),
            tags$div(id = "loadmessage",
            tags$img(src = imgsrc_full
            ))),
        conditionalPanel(condition =  paste0("$('html').hasClass('shiny-busy')",
                "& !(input.startDE & input.methodtabs=='panel0')"),
            tags$div(id = "loadmessage_small",
            tags$img(src = imgsrc_small
            )))
        )
}

#' getLogo
#'
#' Generates and displays the logo to be shown within DEBrowser.
#'
#' @note \code{getLogo}
#' @return return logo
#' @examples
#'     x <- getLogo()
#' @export
#'
getLogo <- function(){
    addResourcePath(prefix = "www", directoryPath =
        system.file("extdata", "www",
        package = "debrowser"))
    imgsrc <- "www/images/logo.png"
    a<-list(img(src=imgsrc, align = "right"))
}

#' getStartupMsg
#'
#' Generates and displays the starting message within DEBrowser.
#'
#' @note \code{getStartupMsg}
#' @return return startup msg
#' @examples
#'     x <- getStartupMsg()
#' @export
#'
getStartupMsg <- function() {
a <- list( column( 12, 
helpText("Please select a file or load the demo data."),
helpText( "For more information;" ),
helpText(   a("Quick Start Guide",
href = "http://debrowser.readthedocs.org",
target = "_blank"),
getHelpButton("method", "http://debrowser.readthedocs.org")) ))
}

#' getAfterLoadMsg
#'
#' Generates and displays the message to be shown after loading data
#' within the DEBrowser.
#'
#' @note \code{getAfterLoadMsg}
#' @return return After Load Msg
#' @examples
#'     x <- getAfterLoadMsg()
#' @export
#'
getAfterLoadMsg <- function() {
a <- list( column( 12, wellPanel(
helpText( "Please choose the appropriate conditions for DESeq analysis
            and press 'Run DESeq' button in the left menu" ),
helpText( "To be able to select conditions please click
            'Condition1' or 'Condition2' boxes.
            You can also use delete button to remove the
            samples from the list."))))
}

#' getStartPlotsMsg
#'
#' Generates and displays the starting messgae to be shown once
#' the user has first seen the main plots page within DEBrowser.
#'
#' @note \code{getStartPlotsMsg}
#' @return return start plot msg
#' @examples
#'     x <- getStartPlotsMsg()
#' @export
#'
getStartPlotsMsg <- function() {
a <- list( conditionalPanel(condition <- "!input.goMain",
    column( 12, 
    helpText( "Please choose the appropriate parameters to discover
               more in DE Results" ),
    getHelpButton("method", "http://debrowser.readthedocs.io/en/master/quickstart/quickstart.html"))))
}

#' getCondMsg
#'
#' Generates and displays the current conditions and their samples
#' within the DEBrowser.
#'
#' @param dc, columns
#' @param input, selected comparison
#' @param cols, columns
#' @param conds, selected conditions
#' @note \code{getCondMsg}
#' @return return conditions
#' @examples
#'     x <- getCondMsg()
#' @export
#'
getCondMsg <- function(dc = NULL, input = NULL, cols = NULL, conds = NULL) {
    if (is.null(cols) || is.null(conds)) return (NULL)
    num <- input$compselect
    if (is.null(num)) num <- 1
    cnd <- data.frame(cbind(conds, cols))
    cond_names <- dc[[as.numeric(num)]]$cond_names
    
    params_str <- paste(dc[[as.numeric(num)]]$demethod_params, collapse = ',')
    heatmap_str <-  paste0( "<b>Heatmap Params: Scaled:</b> ", input[['heatmap-scale']],
        " <b>Centered:</b> ", input[['heatmap-center']],
        " <b>Log:</b> ", input[['heatmap-log']],
        " <b>Pseudo-count:</b> ", input[['heatmap-pseudo']])
    a <-list( conditionalPanel(condition <- "input.goMain",
            shinydashboard::box(
            collapsible = TRUE, title = "Plot Information", status = "primary", 
            solidHeader = TRUE, width = NULL,
            draggable = TRUE,
            style = "overflow-x:scroll",
            HTML( paste0( "<b>DE Params:</b> ", params_str,
            " - <b>Dataset:</b> ", input$dataset," <b>Normalization:</b> ",input$norm_method,
            " - ", heatmap_str,
            "</br><b>", cond_names[1], ":</b> "),
            paste(cnd[cnd$conds ==  unique(conds)[1], "cols"],
            collapse =","),
            paste0(" vs. ","<b>", cond_names[2], ":", "</b> "),
            paste(cnd[cnd$conds == unique(conds)[2], "cols"],
            collapse =",")),
        getHelpButton("method",
"http://debrowser.readthedocs.io/en/master/quickstart/quickstart.html#the-main-plots-of-de-analysis")
)))
}

#' togglePanels
#'
#' User defined toggle to display which panels are to be shown within
#' DEBrowser.
#'
#' @param num, selected panel
#' @param nums, all panels
#' @param session, session info
#' @note \code{togglePanels}
#' @examples
#'     x <- togglePanels()
#' @export
#'
togglePanels <- function(num = NULL, nums = NULL, session = NULL){
    if (is.null(num)) return (NULL)
    for(i in 0:4){
        if (i %in% nums)
            shinyjs::show(selector =
                paste0("#methodtabs li a[data-value=panel",i,"]"))
        else
            shinyjs::hide(selector =
                paste0("#methodtabs li a[data-value=panel",i,"]"))
    }
    if(num)
        updateTabsetPanel(session, "methodtabs",
            selected = paste0("panel", num))
}


#' getTableStyle
#'
#' User defined selection that selects the style of table to display
#' within the DEBrowser.
#'
#' @param dat, dataset
#' @param input, input params
#' @param padj, the name of the padj value column in the dataset
#' @param foldChange, the name of the foldChange column in the dataset
#' @param DEsection, if it is in DESection or not
#' @note \code{getTableStyle}
#' @examples
#'     x <- getTableStyle()
#' @export
#'
getTableStyle <- function(dat = NULL, input = NULL,
    padj = c("padj"), foldChange=c("foldChange"), DEsection = TRUE){
    if (is.null(dat)) return (NULL)
    
    a <- dat
    if(!is.null(padj) && DEsection && all(padj %in% names(dat$x$data) ))
        a <- a %>% formatStyle(
            padj,
            color = styleInterval(c(0, input$padj),
            c('black', "white", "black")),
            backgroundColor = styleInterval(
            input$padj, c('green', 'white'))
        )
    if(!is.null(foldChange) && DEsection && all( foldChange %in% names(dat$x$data) ) )
        a <- a %>%
            formatStyle(
            foldChange,
            color = styleInterval(c(1/as.numeric(input$foldChange),
            as.numeric(input$foldChange)), c('white', 'black', 'white')),
            backgroundColor = styleInterval(
            c(1/as.numeric(input$foldChange),
            as.numeric(input$foldChange)),
            c('blue', 'white', 'red'))
    )
    a
}

#' textareaInput
#'
#' Generates a text area input to be used for gene selection within
#' the DEBrowser.
#'
#' @param id, id of the control
#' @param label, label of the control
#' @param value, initial value
#' @param rows, the # of rows
#' @param cols, the # of  cols
#' @param class, css class
#' @examples
#'     x <- textareaInput("genesetarea", "Gene Set",
#'         "Fgf21", rows = 5, cols = 35)
#' @export
#'
textareaInput <- function(id, label, value, rows=20, cols=35,
    class="form-control"){
    tags$div(
    class="form-group shiny-input-container",
    tags$label('for'=id,label),
    tags$textarea(id=id,class=class,rows=rows,cols=cols,value))
}

#' showObj
#'
#' Displays a shiny object.
#'
#' @param btns, show group of objects with shinyjs
#' @examples
#'     x <- showObj()
#' @export
#'
showObj <- function(btns = NULL) {
    if (is.null(btns)) return (NULL)
    for (btn in seq(1:length(btns)))
        shinyjs::show(btns[btn])
}

#' hideObj
#'
#' Hides a shiny object.
#'
#' @param btns, hide group of objects with shinyjs
#' @examples
#'     x <- hideObj()
#' @export
#'
hideObj <- function(btns = NULL) {
    if (is.null(btns)) return (NULL)
    for (btn in seq(1:length(btns)))
        shinyjs::hide(btns[btn])
}

#' getKEGGModal
#' prepares a helpbutton for to go to a specific site in the documentation
#'
#' @return the info button
#'
#' @examples
#'     x<- getKEGGModal()
#'
#' @export
getKEGGModal<-function(){
    bsModal("modalExample", "KEGG Pathway", "KeggPathway", size = "large",
    div(style = "display:block;overflow-y:auto; overflow-x:auto;",imageOutput("KEGGPlot")))
}

#' getDownloadSection
#'
#' download section button and dataset selection box in the
#' menu for user to download selected data.
#'
#' @param choices, main vs. QC section
#'
#' @note \code{getDownloadSection}
#' @return the panel for download section in the menu;
#'
#' @examples
#'     x<- getDownloadSection()
#'
#' @export
#'
getDownloadSection <- function(choices=NULL) {
    list(conditionalPanel( (condition = "input.methodtabs!='panel0'"),
        shinydashboard::menuItem(" Data Options",                
        selectInput("dataset", "Choose a dataset:",
        choices = choices),
        conditionalPanel( (condition = "input.dataset=='selected'"),
        selectInput("selectedplot", "The plot used in selection:",
        choices = c("Main Plot", "Main Heatmap", "QC Heatmap"))),
        selectInput("norm_method", "Normalization Method:",
        c("none", "MRN", "TMM", "RLE", "upperquartile"), selected = "MRN"),
        downloadButton("downloadData", "Download Data"),
        conditionalPanel(condition = "input.dataset=='most-varied'",
        textInput("topn", "top-n", value = "500" ), 
        textInput("mincount", "total min count", value = "10" )),
        textareaInput("genesetarea","Search", 
        "", rows = 5, cols = 35),
        helpText("Regular expressions can be used\n
        Ex: ^Al => Al.., Al$ => ...al")
    )))
}

#' getQCPanel
#'
#' Gathers the conditional panel for QC plots
#'
#' @param input, user input
#' @note \code{getQCSection}
#' @return the panel for QC plots
#'
#' @examples
#'     x <- getQCPanel()
#'
#' @export
#'
getQCPanel <- function(input = NULL) {
    height = "700"
    width = "500"
    if (!is.null(input)) {
        height = input$height
        width = input$width
    }
    qcPanel <- list(
        wellPanel(helpText( HTML("Please select the parameters and press the 
                            submit button in the left menu for the plots. 
                            The default data set is <b>'most-varied'</b> 500 genes 
                            and total min count is 10 in QC plots. Make sure to
                            change the parameters, if you need to look another part of th data. 
                            For example if you need to draw plots for all detected genes after
                            filtering, select <b>'alldetected'</b> in 
                            'Data Options' -> 'Choose Dataset' on the left menu.") ),
                  getHelpButton("method", 
                                "http://debrowser.readthedocs.io/en/master/quickstart/quickstart.html#quality-control-plots")),
        conditionalPanel(condition = "input.qcplot == 'pca'",
                         getPCAPlotUI("qcpca")),    
        conditionalPanel(condition = "(input.qcplot == 'heatmap')",
                         getHeatmapUI("heatmapQC")),
        conditionalPanel(condition = "(input.qcplot == 'IQR')",
                         getIQRPlotUI("IQR"),
                         getIQRPlotUI("normIQR")),
        conditionalPanel(condition = "(input.qcplot == 'Density')",
                         getDensityPlotUI("density"),
                         getDensityPlotUI("normdensity")),
        conditionalPanel(condition = "(input.qcplot == 'all2all')",
                         getAll2AllPlotUI("all2all"))
        )
    return(qcPanel)
}

#' getSelectedCols
#'
#' gets selected columns
#'
#' @param data, all loaded data
#' @param datasetInput, selected dataset
#' @param input, user input params 
#'
#' @export
#'
#' @examples
#'     getSelectedCols()
#'
#'
getSelectedCols <- function(data = NULL, datasetInput = NULL, input=NULL){
    if(is.null(data) || is.null(datasetInput)) return(NULL)
    selCols <- NULL
    if (!is.null(input$dataset)){
        selection <- colnames(data)
        if (!is.null(input$col_list))
            selection <- input$col_list

        selection <- selection[selection %in% colnames(data)]
        
        if (!is.null(selection))
            selCols <- data[rownames(datasetInput), selection]
    }
    return(selCols)
}


#' removeExtraCols
#'
#' remove extra columns for QC plots
#'
#' @param dat, selected data
#'
#' @export
#'
#' @examples
#'     removeExtraCols()
#'
#'
removeExtraCols <- function(dat = NULL){
    rcols <- c(names(dat)[grep("^padj", names(dat))], 
               names(dat)[grep("^foldChange", names(dat))],
               names(dat)[grep("^log2FoldChange$", names(dat))],
               names(dat)[grep("^pvalue$", names(dat))],
               names(dat)[grep("^Legend$", names(dat))],
               names(dat)[grep("^Size$", names(dat))],
               names(dat)[grep("^log10padj$", names(dat))],
               names(dat)[grep("^x$", names(dat))],
               names(dat)[grep("^y$", names(dat))],
               names(dat)[grep("^M$", names(dat))],
               names(dat)[grep("^A$", names(dat))],
               names(dat)[grep("^ID$", names(dat))],
               names(dat)[grep("^stat$", names(dat))]
    )
    if (!is.null(rcols))
        dat <- dat[, !(names(dat) %in% rcols)]
    else
        dat
}

require(shiny)
library(debrowser)
library(testthat)

test_that("able to create panel UI", {
    expect_silent( QCPanel <- getQCPanel() )
    expect_true(exists("QCPanel"))
    expect_equal(QCPanel[[1]][[1]], "div")

    expect_silent( downloads <- getDownloadSection() )
    expect_true(exists("downloads"))
    expect_equal(downloads[[1]][[1]], "div")

    expect_silent( getMain <- getMainPanel() )
    expect_true(exists("getMain"))
    expect_equal(getMain[[1]][[1]], "div")

    expect_silent( getStart <- getStartupMsg() )
    expect_true(exists("getStart"))
    expect_equal(getStart[[1]][[1]], "div")

    expect_silent( getAfter <- getAfterLoadMsg() )
    expect_true(exists("getAfter"))
    expect_equal(getAfter[[1]][[1]], "div")

    expect_silent(getGO <- getGoPanel() )
    expect_true(exists("getGO"))
    expect_equal(getGO[[1]][[1]], "div")
})

library(debrowser)
source("../../R/dataLoad.R")

options(shiny.maxRequestSize = 30*1024^2)

header <- dashboardHeader(
  title = "DEBrowser DataPrep"
)
sidebar <- dashboardSidebar(  sidebarMenu(id="DataPrep",
      menuItem("Upload", tabName = "Upload"),
      menuItem("Filter", tabName = "Filter"),
      menuItem("BatchEffect", tabName = "BatchEffect")
      ))

body <- dashboardBody(
  tabItems(
   tabItem(tabName="Upload", dataLoadUI("load"),
           column(4, verbatimTextOutput("loadedtable")
   )),
   tabItem(tabName="Filter",dataLCFUI("lcf"),                
           column(4, verbatimTextOutput("filtertable")
   )),
   tabItem(tabName="BatchEffect", batchEffectUI("batcheffect"),
           column(4, verbatimTextOutput("batcheffecttable")
   ))
  )
)
ui <- dashboardPage(header, sidebar, body, skin = "blue")

server <- function(input, output, session) {
  filtd <- reactiveVal()
  batch <- reactiveVal()
  observe({
    updata <- reactive({ 
        ret <- callModule(debrowserdataload, "load", "Filter")
        ret
    })
    observeEvent (input$Filter, {
        if(!is.null(updata()$load())){ 
            updateTabItems(session, "DataPrep", "Filter")
            filtd(callModule(debrowserlowcountfilter, "lcf", updata()$load()))
        }
    })
    observeEvent (input$Batch, {
        if(!is.null(filtd()$filter())){ 
            updateTabItems(session, "DataPrep", "BatchEffect")
            batch(callModule(debrowserbatcheffect, "batcheffect", filtd()$filter()))
        }
    })
    
    output$loadedtable <- renderPrint({
         head( updata()$load()$count )
    })
    output$filtertable <- renderPrint({
         head( filtd()$filter()$count  )
    })
    output$batcheffecttable <- renderPrint({
        head( batch()$BatchEffect()$count )
    })
  })
}

shinyApp(ui, server)

library(debrowser)
library(plotly)
source("../../R/boxmain.R")

options(warn =-1)

header <- dashboardHeader(
    title = "DEBrowser Box Plots"
)
sidebar <- dashboardSidebar(  sidebarMenu(id="DEAnlysis",
    menuItem("BoxMain", tabName = "BoxMain"),
    textInput("genename", "Gene/Region Name", value = "Foxa3" ),
    plotSizeMarginsUI("boxmain", h=400, t = 30)
))

body <- dashboardBody(
    tabItems(
        tabItem(tabName="BoxMain", 
            fluidRow(
                column(12,
                getBoxMainPlotUI("boxmain")))
        )
))

ui <- dashboardPage(header, sidebar, body, skin = "blue")

server <- function(input, output, session) {
    load(system.file("extdata", "demo", "demodata.Rda",
                     package = "debrowser"))
    cond_names <- c("Control", "Treat")
    
    observe({
        if (!is.null(input$genename))
            callModule(debrowserboxmainplot, "boxmain", demodata, 
            metadatatable$sample,  
            metadatatable$treatment, cond_names, input$genename)
})
}

shinyApp(ui, server)

#' debrowserall2all
#'
#' Module for a bar plot that can be used in data prep, main plots 
#' low count removal modules or any desired module
#' 
#' @param input, input variables
#' @param output, output objects
#' @param session, session 
#' @param data, a matrix that includes expression values
#' @param cex, the size of the dots
#' @return all2all plot 
#' @export
#'
#' @examples
#'     x <- debrowserall2all()
#'
debrowserall2all <- function(input, output, session, data = NULL,
                                 cex=2) {
    if(is.null(data)) return(NULL)
    output$all2allplot <- renderPlot({
        all2all(data, cex)
    })
    output$all2allUI <- renderUI({
        shinydashboard::box(
            collapsible = TRUE, title = "All2all plot", status = "primary", 
            solidHeader = TRUE, width = NULL,
            draggable = TRUE,  plotOutput(session$ns("all2allplot"),
                width = input$width, height=input$height))
    })
}

#' getAll2AllPlotUI
#'
#' all2all plots UI.  
#'
#' @note \code{getAll2AllPlotUI}
#' @param id, namespace id
#' @return the panel for all2all plots;
#'
#' @examples
#'     x <- getAll2AllPlotUI("bar")
#'
#' @export
#'
getAll2AllPlotUI <- function(id) {
    ns <- NS(id)
    uiOutput(ns("all2allUI"))
}

#' all2allControlsUI
#'
#' Generates the controls in the left menu for an all2all plot
#'
#' @note \code{all2allControlsUI}
#' @param id, namespace id
#' @return returns the controls for left menu
#' @examples
#'     x <- all2allControlsUI("bar")
#' @export
#'
all2allControlsUI <- function(id) {
    ns <- NS(id)
    shinydashboard::menuItem(paste0(id, " - Options"),
         sliderInput("cex", "corr font size",
                min = 0.1, max = 10,
                step = 0.1, value = 2)
    )
}

#' all2all
#'
#' Prepares all2all scatter plots for given datasets. 
#'
#' @param data, data that have the sample names in the header.
#' @param cex text size
#' @return all2all scatter plots
#' @examples
#'     plot<-all2all(mtcars)
#'
#' @export
#'
all2all <- function(data, cex=2) {
    pcor <- function(x, y, ...) panel.cor(x, y, cex.cor = cex)
    nr <- nrow(data)
    if (nr > 1000)
        nr <- 1000
    pairs(log10(data[1:nr, ]), cex = 0.25,
            diag.panel = panel.hist, lower.panel = pcor)
}

#' panel.hist
#'
#' Prepares the historgram for the all2all plot. 
#'
#' @param x, a vector of values for which the histogram is desired
#' @param ..., any additional params
#' @return all2all histogram plots
#' @examples
#'     panel.hist(1)
#'
#' @export
#'
panel.hist <- function(x, ...) {
    usr <- par("usr")
    on.exit(par(usr))
    par(usr = c(usr[1:2], 0, 1.5))
    h <- hist(x, plot = FALSE)
    breaks <- h$breaks
    nb <- length(breaks)
    y <- h$counts
    y <- y / max(y)
    rect(breaks[-nb], 0, breaks[-1], y, col = "red", ...)
}

#' panel.cor
#'
#' Prepares the correlations for the all2all plot. 
#'
#' @param x, numeric vector x
#' @param y, numeric vector y
#' @param prefix, prefix for the text
#' @param cex.cor, correlation font size
#' @param ..., additional parameters
#' @return all2all correlation plots
#' @examples
#'     panel.cor(c(1,2,3), c(4,5,6))
#'
#' @export
#'
panel.cor <- function(x, y, prefix = "rho=", cex.cor=2, ...){
    usr <- par("usr")
    on.exit(par(usr))
    par(usr = c(0, 1, 0, 1))
    r <- cor.test(x, y, method = "spearman",
        na.rm = TRUE, exact = FALSE)$estimate
    txt <- round(r, digits = 2)
    txt <- paste0(prefix, txt)
    text(0.5, 0.5, txt, cex = cex.cor)
}

library(debrowser)
library(plotly)
library(heatmaply)
library(gplots)
library(DESeq2)
source("../../R/mainScatter.R")
source("../../R/boxmain.R")
source("../../R/barmain.R")

header <- dashboardHeader(
    title = "DEBrowser Main Plots"
)
sidebar <- dashboardSidebar(  sidebarMenu(id="DEAnalysis",
    menuItem("Main", tabName = "Main"),
    mainPlotControlsUI("main"),
    plotSizeMarginsUI("main",  w=600, h=400),
    textareaInput("genesetarea","Search", 
                  "", rows = 5, cols = 35),
    helpText(HTML("&nbsp;&nbsp;Regular expressions can be used<br>",
    "&nbsp;&nbsp;Ex: ^Al => Al.., Al$ => ...al")),
    heatmapControlsUI("heatmap"),
    plotSizeMarginsUI("heatmap", w=600, h=360),
    plotSizeMarginsUI("barmain", w=600,h=400, t=60),
    plotSizeMarginsUI("boxmain", w=600, h=400, t=60)))

body <- dashboardBody(
    tabItems(
        tabItem(tabName="Main",
                fluidRow(column(6,
                getMainPlotUI("main")
                ),
                column(6,
                getHeatmapUI("heatmap")
                )),
                fluidRow(column(6,
                getBarMainPlotUI("barmain")),
                column(6,
                getBoxMainPlotUI("boxmain")))
        )
    ))

ui <- dashboardPage(header, sidebar, body, skin = "blue")

server <- function(input, output, session) {
    #Example usage with demodata
    load(system.file("extdata", "demo", "demodata.Rda",
                     package = "debrowser"))
    dat <-c()
    dat$columns <- c("exper_rep1", "exper_rep2", "exper_rep3",
                 "control_rep1", "control_rep2", "control_rep3")
    dat$conds <- factor( c("Control", "Control", "Control",
                       "Treat", "Treat", "Treat") )
    dat$data <- data.frame(demodata[, dat$columns])
    
    dat$cond_names <- data.frame(cbind(dat$columns, as.character(dat$conds), metadatatable$treatment))
    colnames(dat$cond_names) <- c("sample", "cond_name", "conds")
    
    # You can also use your dataset by reading your data from a file like below;
    # The data in this commented out exampele is not supplied but these lines 
    # can give you an idea about how to read the data from a file;
    #
    # data  <- read.table("~/Downloads/shKRAS.tsv", header=T, row.names=1, sep="\t")
    # dat$columns <- c("CNT.2", "CNT.3", "CNT.4", 
    #               "shKRAS_T1", "shKRAS_T2", "shKRAS_T3")
    # dat$conds <- factor( c("Control", "Control", "Control",
    #                        "shKRAS", "shKRAS", "shKRAS") )
    # dat$data <- data.frame(data[, dat$columns])
    #
    xdata <- generateTestData(dat)

    selected <- reactiveVal()
    observe({
        if(input$genesetarea != ""){
            xdata[rownames(getSearchData(xdata, input)), "Legend"] <- "GS"
        }
        selected(callModule(debrowsermainplot, "main", xdata, dat$cond_names))
    })
    selectedHeat <- reactiveVal()
    observe({
        if (!is.null(selected()) && !is.null(selected()$selGenes())) {
        withProgress(message = 'Creating plot', style = "notification", value = 0.1, {
            selectedHeat(callModule(debrowserheatmap, "heatmap", xdata[selected()$selGenes(), dat$columns]))
        })
        }
    })
    
    getGeneName <- reactive({
        gname <- NULL
        if (!is.null(selected()$shgClicked()))
            gname <- selected()$shgClicked()
        
        if (!is.null(selectedHeat()) && !is.null(selectedHeat()$shgClicked()))
            gname <- selectedHeat()$shgClicked()
        return(gname)
    })
    observe({
        if (!is.null(selected()) && !is.null(selected()$shgClicked()) && selected()$shgClicked()!=""){
            withProgress(message = 'Creating Bar/Box plots', style = "notification", value = 0.1, {
            callModule(debrowserbarmainplot, "barmain", xdata, 
                       dat$columns,
                       dat$conds, dat$cond_names, getGeneName())
            callModule(debrowserboxmainplot, "boxmain", xdata, 
                       dat$columns,
                       dat$conds, dat$cond_names, getGeneName())
            })
        }
    })
}
shinyApp(ui, server)

#' getDensityPlotUI
#'
#' Density plot UI.  
#'
#' @param id, namespace id
#' @note \code{getDensityPlotUI}
#' @return the panel for Density plots;
#'
#' @examples
#'     x <- getDensityPlotUI("density")
#'
#' @export
#'
getDensityPlotUI <- function(id) {
  ns <- NS(id)
  uiOutput(ns("DensityUI"))
}

#' debrowserdensityplot
#'
#' Module for a density plot that can be used in data prep and 
#' low count removal modules
#' 
#' @param input, input variables
#' @param output, output objects
#' @param session, session 
#' @param data, a matrix that includes expression values
#' @return density plot 
#' @export
#'
#' @examples
#'     x <- debrowserdensityplot()
#'
debrowserdensityplot <- function(input = NULL, output = NULL, session = NULL, data = NULL) {
    if(is.null(data)) return(NULL)
    output$Density <- renderPlotly({
        getDensityPlot(data, input)
    })
    output$DensityUI <- renderUI({
    shinydashboard::box(
        collapsible = TRUE, title = session$ns("plot"), status = "primary", 
        solidHeader = TRUE, width = NULL,
        draggable = TRUE,  plotlyOutput(session$ns("Density"),
             width = input$width, height=input$height))
    })
}

#' densityPlotControlsUI
#'
#' Generates the controls in the left menu for a densityPlot
#'
#' @note \code{densityPlotControlsUI}
#' @param id, namespace id
#' @return returns the left menu
#' @examples
#'     x <- densityPlotControlsUI("density")
#' @export
#'
densityPlotControlsUI <- function(id) {
  ns <- NS(id)
  shinydashboard::menuItem(paste0(id, " - Options"),
      textInput(ns("breaks"), "Breaks", value = "100" )
  )
}

#' getDensityPlot
#'
#' Makes Density plots
#'
#' @param data, count or normalized data
#' @param input, input
#' @param title, title
#'
#' @export
#'
#' @examples
#'     getDensityPlot()
#'
getDensityPlot <- function(data=NULL, input = NULL, title = ""){
  if (is.null(data)) return(NULL)
  data <- as.data.frame(data)
  cols <- colnames(data)
  data[, cols] <- apply(data[, cols], 2,
                        function(x) log10(as.integer(x) + 1))
  
  data <- addID(data)
  mdata <- melt(as.data.frame(data[,c("ID", cols)]),"ID")
  colnames(mdata)<-c("ID", "samples", "density")
  
  p <- ggplot(data=mdata, aes(x=density)) +
    geom_density(aes(fill = samples), alpha = 0.5) +
    labs(x = "logcount", y = "Density") +
    theme_minimal()
  if (!is.null(input$top))
      p <- p + theme( plot.margin = margin(t = input$top, r =input$right, b =input$bottom, l = input$left, "pt"))
  p <- ggplotly(p, width = input$width, height = input$height)
  if (!is.null(input$svg) && input$svg == TRUE)
    p <- p %>% config(toImageButtonOptions = list(format = "svg"))
  p$elementId <- NULL
  p
}

#' installpack
#'
#' install packages if they don't exist
#' display.
#'
#' @param package_name, package name to be installed
#' @note \code{installpack}
#'
#' @examples
#'     x <- installpack()
#'
#' @export
#'
installpack <- function(package_name = NULL) {
    if (is.null(package_name)) return(NULL)
   
    if (!loadpack(package_name))
    {
       txt <- paste0("Please install ",package_name, " to use this function.")
       print(txt)
       showNotification(txt, type="error")
       return(FALSE)
    }else{
        loadpack(package_name)
    }
    return(TRUE)
}
#' loadpack
#'
#' load packages 
#'
#' @param package_name, package name to be loaded
#' @note \code{loadpack}
#'
#' @examples
#'     x <- loadpack()
#'
#' @export
#'
loadpack <- function (package_name = NULL){
    if(isTRUE(package_name %in% .packages(all.available=TRUE))) {
        eval(parse(text = sprintf("require(\"%s\")", package_name)))
        return (TRUE)
    }
    return(FALSE)
}

## ----echo=FALSE, results="hide", warning=FALSE--------------------------------
suppressPackageStartupMessages({
    library(gplots)
    library(ggplot2)
    library(shiny)
    library(DT)
    library(clusterProfiler)
    library(DESeq2)
    library(org.Hs.eg.db)
    library(edgeR)
    library(debrowser)
    library(RColorBrewer)
    library(annotate)
})

#' debrowsercondselect
#'
#' Condition selection
#' This is not a module. Module construction didn't used here, just use it 
#' as functions not in a module.
#' 
#' @param input, input variables
#' @param output, output objects
#' @param session, session 
#' @param data, count data
#' @param metadata, metadata
#' @return main plot
#'
#' @return panel
#' @export
#'
#' @examples
#'     x <- debrowsercondselect()
#'
debrowsercondselect <- function(input = NULL, output = NULL, session = NULL, data = NULL, metadata = NULL) {
    if (is.null(data)) return(NULL)
    choicecounter <- reactiveVal(0)
    output$conditionSelector <- renderUI({
        selectConditions(data, metadata, choicecounter(), session, input)
    })
    
    observeEvent(input$add_btn, {
        choicecounter(choicecounter() + 1)
    })
    observeEvent(input$rm_btn, {
        if (choicecounter() > 0) 
            choicecounter(choicecounter() - 1)
    })
    list(cc = choicecounter)
}

#' condSelectUI
#' Creates a panel to select samples for each condition
#'
#' @return panel
#' @examples
#'     x <- condSelectUI()
#'
#' @export
#'
condSelectUI<- function () {
    list(
        shinydashboard::box(title = "Comparison Selection",
            solidHeader = TRUE, status = "info",  width = NULL, height = NULL, collapsible = TRUE,
        fluidRow(
            uiOutput("conditionSelector"),
            column(12,actionButtonDE("add_btn", "Add New Comparison",styleclass = "primary"),
                actionButtonDE("rm_btn", "Remove", styleclass = "primary"),
                getHelpButton("method", "http://debrowser.readthedocs.io/en/master/deseq/deseq.html"),
                conditionalPanel(condition = ("output.condReady>0"),
                actionButtonDE("startDE", "Start DE", styleclass = "primary")))
        ))
    )
}
#getMethodDetails
#'
#' get the detail boxes after DE method selected 
#'
#' @param num, panel that is going to be shown
#' @param input, user input
#' @examples
#'     x <- getMethodDetails()
#'
#' @export
#'
#'
getMethodDetails <- function(num = NULL, input = NULL) {
    if (is.null(num)) return(NULL)
    if (num > 0)
        list(
            conditionalPanel(
                (condition <- paste0("input.demethod",num," == 'DESeq2'")),
                getSelectInputBox("fitType", "Fit Type", num, 
                    c("parametric", "local", "mean"), 
                    selectedInput("testType", num, "parametric",
                    input), 3),
                getSelectInputBox("betaPrior", "betaPrior", num, 
                    c(FALSE, TRUE), 
                    selectedInput("betaPrior", num,
                    FALSE, input),2),
                getSelectInputBox("testType", "Test Type", num, 
                    c("LRT", "Wald"),  
                    selectedInput("testType", num, "LRT", input)),
                getSelectInputBox("shrinkage", "Shrinkage", num, 
                    c("None", "apeglm", "ashr", "normal"),
                    selectedInput("shrinkage", num, "None", input))),
            conditionalPanel(
                (condition <- paste0("input.demethod",num," == 'EdgeR'")),
                getSelectInputBox("edgeR_normfact", "Normalization", num, 
                    c("TMM","RLE","upperquartile","none"), 
                    selectedInput("edgeR_normfact", num, "TMM", input), 3),
                column(2,textInput(paste0("dispersion", num), "Dispersion", 
                    value = isolate(selectedInput("dispersion", 
                    num, "0", input) ))),
                getSelectInputBox("edgeR_testType", "Test Type", num, 
                    c("exactTest", "glmLRT"), 
                    selectedInput("edgeR_testType", num,
                    "exactTest", input))),
            conditionalPanel(
                (condition <- paste0("input.demethod",num," ==  'Limma'")),
                getSelectInputBox("limma_normfact", "Normalization", num, 
                    c("TMM","RLE","upperquartile","none"), 
                    selectedInput("limma_normfact", num, "TMM", input), 3),
                getSelectInputBox("limma_fitType", "Fit Type", num,
                    c("ls", "robust"), 
                    selectedInput("limma_fitType", num, "ls", input)),
                getSelectInputBox("normBetween", "Norm. Bet. Arrays", num,
                    c("none", "scale", "quantile", "cyclicloess",
                    "Aquantile", "Gquantile", "Rquantile","Tquantile"),
                    selectedInput("normBetween", num, "none", input))),
            br())
}

#' getCovariateDetails
#'
#' get the covariate detail box after DE method selected 
#'
#' @param num, panel that is going to be shown
#' @param input, user input
#' @param metadata, metadata
#' 
#' @examples
#'     x <- getCovariateDetails()
#'
#' @export
#'
getCovariateDetails <- function(num = NULL, input = NULL, metadata = NULL) {
    if (is.null(num)) return(NULL)
    if (num > 0) {
        choices <- as.list(c(colnames(metadata)[2:ncol(metadata)]))
        list(
            getSelectInputBox("covariate", "Covariate", num, choices,
                              selected = selectedInput("covariate", num, NULL, input), 
                              2, multiple = TRUE)
        )
    }
}

#' getConditionSelector
#'
#' Selects user input conditions to run in DESeq.
#'
#' @param num, panel that is going to be shown
#' @param choices, sample list
#' @param selected, selected smaple list
#' @examples
#'     x <- getConditionSelector()
#'
#' @export
#'
getConditionSelector<- function(num=NULL, choices = NULL, selected = NULL) {
    if (is.null(num)) return(NULL)
    if (!is.null(choices))
        list(column(3, selectInput(paste0("condition", num),
            label = paste0("Condition ", num, ifelse(num %% 2 == 0, "(Numerator)", "(Denominator)")),
            choices = choices, multiple = TRUE,
            selected = selected)))
}

#' getConditionSelectorFromMeta
#'
#' Selects user input conditions to run in DESeq from metadata
#'
#' @param metadata, meta data table
#' @param input, input
#' @param index, index
#' @param num, num
#' @param choices, choices
#' @param selected, selected
#' 
#' @examples
#'     x <- getConditionSelectorFromMeta()
#'
#' @export
#'
getConditionSelectorFromMeta <- function(metadata = NULL, input = NULL, index = 1, num=0, 
    choices = NULL, selected = NULL) {
    if (is.null(metadata)) return(NULL)
     a <- list(column(6, selectInput(paste0("condition", num),
            label = paste0("Condition ", num, ifelse(num %% 2 == 0, "(Numerator)", "(Denominator)")),
            choices = choices, multiple = TRUE,
            selected = selected)))

     if (!is.null(metadata)){
        selected_meta <- selectedInput("conditions_from_meta", 
            index, NULL, input)
            
        if (is.null(selected_meta)) selected_meta <- "No Selection"
    
        if (selected_meta != "No Selection"){
            old_selection <- ""
            
        if(!is.null(input[[paste0("condition", num)]])){
            selected <- input[[paste0("condition", num)]]
        } 
        grps <- unique(metadata[selected_meta])
        grps <- grps[grps!="NA"]
        grps<- grps[!is.na(grps) ]

        if (length(grps) == -1) {
            meta_choices_all <- NULL
            if (!is.null(selected_meta))
                meta_choices_all <- get_conditions_given_selection(metadata,
                                            selected_meta)
            if(old_selection != selected_meta){
                if(typeof(meta_choices_all) == "character"){
                    meta_choices <- list("There must be exactly 2 groups.")
                } else {
                    meta1 <- meta_choices_all[[2 - (num %% 2)]]
                    meta_choices <- unlist(meta1, recursive=FALSE)
                }
                selected <- meta_choices
            }
        }else{
            if(!is.null(input[[paste0("group", num)]])){
                selected <- metadata[metadata[,selected_meta] == input[[paste0("group", num)]], 1]
            }
        }
    
        a <- list(column(6, selectInput(paste0("condition", num),
            label = paste0("Condition ", num, ifelse(num %% 2 == 0, "(Numerator)", "(Denominator)")),
            choices = choices, multiple = TRUE,
            selected = selected)))
        }
    }
    return(a)
}

#' selectedInput
#'
#' Selects user input conditions to run in DESeq.
#'
#' @param id, input id
#' @param num, panel that is going to be shown
#' @param default, default text
#' @param input, input params
#' @examples
#'     x <- selectedInput()
#'
#' @export
#'
selectedInput <- function(id = NULL, num = 0, default = NULL, 
    input = NULL) {
    if (is.null(id)) return(NULL)
    m <- NULL
    if (is.null(input[[paste0(id, num)]]))
        m <- default
    else
        m <- input[[paste0(id, num)]]
    m
}

#' getSelectInputBox
#'
#' Selects user input conditions to run in DESeq.
#'
#' @param id, input id
#' @param name, label of the box
#' @param num, panel that is going to be shown
#' @param choices, sample list
#' @param selected, selected smaple list
#' @param cw, column width
#' @param multiple, if multiple choices are available
#' @examples
#'     x <- getSelectInputBox()
#'
#' @export
#'
getSelectInputBox <- function(id = NULL, name = NULL, 
                              num = 0, choices = NULL, selected = NULL, 
                              cw = 2, multiple = FALSE) {
    if (is.null(id)) return(NULL)
    if (!is.null(choices))
        list(column(cw, selectInput(paste0(id, num),
            label = name,
            choices = choices, multiple = multiple,
            selected = selected)))
}


#' selectConditions
#'
#' Selects user input conditions, multiple if present, to be
#' used in DESeq.
#'
#' @param Dataset, used dataset 
#' @param metadata, metadatatable to select from metadata
#' @param choicecounter, choicecounter to add multiple comparisons
#' @param session, session
#' @param input, input params
#' @note \code{selectConditions}
#' @return the panel for go plots;
#'
#' @examples
#'     x<- selectConditions()
#'
#' @export
#'
selectConditions<-function(Dataset = NULL,
                           metadata = NULL,
                           choicecounter = NULL,
                           session = NULL,
                           input = NULL) {
    if (is.null(Dataset)) return(NULL)
    
    selectedSamples <- function(num){
        if (is.null(input[[paste0("condition", num)]]))
            getSampleNames(colnames(Dataset), num %% 2 )
        else
            input[[paste0("condition", num)]]
    }
    nc <- choicecounter
    
    if (nc >= 0) {
        allsamples <- getSampleNames( colnames(Dataset), "all" )

        lapply(seq_len(nc), function(i) {

            selected1 <- selectedSamples(2 * i - 1)
            selected2 <- selectedSamples( 2 * i )
            to_return <- list(column(12, getMetaSelector(metadata = metadata, input=input, n = i),
                    isolate(getGroupSelector(metadata, input, i, (2*i-1))),
                    isolate(getGroupSelector(metadata, input, i, (2*i))),
                    getConditionSelectorFromMeta(metadata, input, i,
                        (2 * i - 1), allsamples, selected1),
                    getConditionSelectorFromMeta(metadata, input, i,
                        (2 * i), allsamples, selected2)
            ),
            column(12, 
                   # column(1, helpText(" ")),
                   getSelectInputBox("demethod", "DE Method", i, 
                        c("DESeq2", "EdgeR", "Limma"),
                        selectedInput("demethod", i, "DESeq2", input)),
                   getMethodDetails(i, input)),
            column(12,
                   getCovariateDetails(i, input, metadata = metadata))
            )
            
            # check DE conditions
            if (!is.null(selectedInput("conditions_from_meta", 
                i, NULL, input)) && selectedInput("conditions_from_meta", 
                i, NULL, input) != "No Selection"){
                facts <- levels(factor(metadata[,selectedInput("conditions_from_meta", 
                     i, NULL, input)]))
                facts <- facts[facts != "" & facts != "NA"]
                if (length(facts) < 2) {
                    showNotification("There must be more than 2 groups in the selected condition.", 
                         type = "error")
                    updateSelectInput(session, paste0("conditions_from_meta", i), selected="No Selection" )
                }
            }
            
            # update selected of the covariate
            # if condition is selected, dont let the same column selected as covariate, then remove
            metadata_columns <- colnames(metadata)[2:ncol(metadata)]
            metadata_columns <- metadata_columns[!metadata_columns %in% selectedInput("conditions_from_meta", i, NULL, input)]
            if(!is.null(selectedInput("conditions_from_meta", i, NULL, input)) && !is.null(selectedInput("covariate", i, NULL, input)) &&
               selectedInput("conditions_from_meta", i, NULL, input) != "No Selection"){
                if(selectedInput("conditions_from_meta", i, NULL, input) %in% selectedInput("covariate", i, NULL, input)){
                    selected_meta <- selectedInput("covariate", i, NULL, input)
                    selected_meta <- selected_meta[!selected_meta %in% selectedInput("conditions_from_meta", i, NULL, input)]
                    updateSelectInput(session, paste0("covariate", i), choices = c(metadata_columns), 
                                      selected = c(selected_meta))
                    showNotification("Condition column is included in covariate list, removing!", 
                                     type = "error")
                    return(to_return)
                } 
            }
    
            # check appropriateness of covariates
            if (!is.null(selectedInput("covariate", i, NULL, input))){
                
                # establish metadata with selected samples, conditions and covariates
                selected_samples <- c(selected1,selected2)
                match_selected_samples <- match(selected_samples, colnames(Dataset))
                selected_covariate_names <- selectedInput("covariate",i, NULL, input)
                selected_covariates_metadata <- metadata[match_selected_samples,selected_covariate_names, drop = FALSE]
                selected_treatment <- rep(c("Cond1","Cond2"), c(length(selected1), length(selected2)))

                # loop over all covariates, flag covariates to be removed
                flag_selected <- rep(F,length(selected_covariate_names))
                for(kk in 1:ncol(selected_covariates_metadata)){
                    covariate <- selected_covariates_metadata[,kk]

                    # check if there are at least two groups in metadata
                    if (sum(is.na(covariate)) > 0) {
                        showNotification("Covariate shouldnt have an NA or empty values in any selected sample.",
                                         type = "error")
                        flag_selected[kk] <- T
                    }
                    
                    # check if there are at least two groups in metadata
                    if (length(unique(covariate)) < 2) {
                        showNotification("There must be at least 2 groups in the selected covariate.",
                                         type = "error")
                        flag_selected[kk] <- T
                    }
                    
                    # check if there are confounding covariates with treatment
                    covariate_vs_treatment <- table(covariate, selected_treatment)
                    if (any(covariate_vs_treatment == 0)) {
                        showNotification("Each condition should have at least one of all covariate groups.",
                                         type = "error")
                        flag_selected[kk] <- T
                    }
                }
                
                # remove covariates if an inappropriate flag is found
                if(any(flag_selected)){
                    new_covariate_names <- selected_covariate_names[!flag_selected]
                    selected_covariate_names <- selectedInput("covariate",i, NULL, input)
                    selected_covariate_names <- selected_covariate_names[selected_covariate_names %in% new_covariate_names]
                    updateSelectInput(session, paste0("covariate", i), selected= c(selected_covariate_names))   
                }
            }
            
            return(to_return)
        })
    }
}

#' getGroupSelector
#' Return the groups 
#'
#' @param metadata, meta data table
#' @param input, input params
#' @param index, index
#' @param num, num
#' @return meta select box
#'
#' @examples
#'     x<-getGroupSelector()
#' @export
#'
getGroupSelector <- function(metadata = NULL, input = NULL, index = 1, num=0) {
    a <- NULL
    selected_meta <- selectedInput("conditions_from_meta", 
        index, NULL, input)
    if (is.null(selected_meta) || selected_meta == "No Selection") return(NULL) 
    grps <- unique(metadata[selected_meta])
    grps <- grps[grps!="NA"]
    grps<- grps[!is.na(grps) ]

    sel <- NULL
    if ( is.null(input[[paste0("group", num)]]) || input[[paste0("group", num)]] %in% grps ){
        sel <- selectedInput("group",
            num, grps[num], input)
    }
    if (length(grps) > 1) {
        grps_choices <- c("No Selection", grps)
        a <- list(column(6, selectInput(paste0("group", num),
            label = paste0("Group ", num),
            choices = grps_choices, 
            selected =  sel,
            multiple = FALSE)))
    } 

    return(a)
}

#' getMetaSelector
#'
#' Return the sample selection box using meta data table
#'
#' @param metadata, meta data table
#' @param input, input params
#' @param n, the box number
#' @return meta select box
#'
#' @examples
#'     x<-getMetaSelector()
#' @export
#'
getMetaSelector <- function(metadata = NULL, input = NULL, n = 0){		
    if(!is.null(metadata)){		
        df <- metadata
        col_count <- length(colnames(df))		

        list(HTML('<hr style="color: white; border:solid 1px white;">'),
             br(), column(10, selectInput(paste0("conditions_from_meta", 
                 n), label = "Select Meta",
                 choices = as.list(c("No Selection", 
                 colnames(df)[2:col_count])),
                 multiple = FALSE,
                 selected =  selectedInput("conditions_from_meta",
                 n, "Selection 2", input))))
    }
}

#' get_conditions_given_selection
#'
#' Return the two set of conditions given the selection of meta select box
#'
#' @param metadata, meta data table
#' @param selection, selection
#' @return meta select box
#'
#' @examples
#'     x<-get_conditions_given_selection()
#' @export
#'
get_conditions_given_selection <- function(metadata = NULL, selection = NULL){		
    if(is.null(metadata)) return(NULL)		
    df <- metadata
    if(selection == "No Selection"){		
        return(NULL)	
    }
    facts <- levels(factor(df[,selection]))
    facts <- facts[facts != "" & facts != "NA"]
    if(length(facts) != 2){		
        return(NULL)	
    } else {
        # Assuming the first column has samples		
        sample_col_name <- colnames(df)[1]		
        
        condition1 <- facts[1]		
        condition2 <- facts[2]		
        
        # In case the conditions are integers		
        if(is.null(condition2)){		
            condition1 <- factor(condition1)		
            condition2 <- factor(condition2)	
        }

        condition1_filtered <- df[df[,selection] == condition1, ]		
        a <- condition1_filtered[,sample_col_name]		

        condition2_filtered <- df[df[,selection] == condition2, ]		
        b <- condition2_filtered[,sample_col_name]	

        both_groups <- list(a, b)
        return(both_groups)		
    }
}

#' getSampleNames
#'
#' Prepares initial samples to fill condition boxes.  
#' it reads the sample names from the data and splits into two. 
#'
#' @param cnames, sample names in the header of a dataset
#' @param part, c(1,2). 1=first half and 2= second half
#' @return sample names.
#'
#' @examples
#'     x<-getSampleNames()
#' @export
#'
getSampleNames <- function(cnames = NULL, part = 1) {
    if (is.null(cnames)) return(NULL)
    
    startpos <- 1
    endpos <- length(cnames)
    if (part == 1)
        endpos <- floor(endpos / 2) 
    else if (part == 0)
        startpos <- floor(endpos / 2) + 1
    
    cn <- cnames[startpos:endpos]
    m <- as.list(NULL)
    for (i in seq(cn)) {
        m[[i]] <- cn[i]
    }
    m
}

#' prepDataContainer
#'
#' Prepares the data container that stores values used within DESeq.
#'
#' @param data, loaded dataset
#' @param counter, the number of comparisons
#' @param input, input parameters
#' @param meta, loaded metadata
#' @return data
#' @export
#'
#' @examples
#'     x <- prepDataContainer()
#'
prepDataContainer <- function(data = NULL, counter=NULL, 
                              input = NULL, meta = NULL) {
    if (is.null(data)) return(NULL)
    
    inputconds <- reactiveValues(demethod_params = list(), conds = list(), dclist = list())
    inputconds$conds <- list()
    for (cnt in seq(1:(2*counter))){
        inputconds$conds[cnt] <- list(isolate(input[[paste0("condition",cnt)]]))
    }
    #Get parameters for each method
    inputconds$demethod_params <- NULL
    for (cnt in seq(1:counter)){
        covariate <- isolate(paste(input[[paste0("covariate",cnt)]], collapse = "|"))
        covariate <- ifelse(covariate == "", "NoCovariate", covariate)
        if (isolate(input[[paste0("demethod",cnt)]]) == "DESeq2"){
            inputconds$demethod_params[cnt] <- paste(
                isolate(input[[paste0("demethod",cnt)]]),
                covariate,
                isolate(input[[paste0("fitType",cnt)]]),
                isolate(input[[paste0("betaPrior",cnt)]]),
                isolate(input[[paste0("testType",cnt)]]),
                isolate(input[[paste0("shrinkage",cnt)]]), sep=",")
        }
        else if (isolate(input[[paste0("demethod",cnt)]]) == "EdgeR"){
            inputconds$demethod_params[cnt]<- paste(
                isolate(input[[paste0("demethod",cnt)]]),
                covariate,
                isolate(input[[paste0("edgeR_normfact",cnt)]]),
                isolate(input[[paste0("dispersion",cnt)]]),
                isolate(input[[paste0("edgeR_testType",cnt)]]), sep=",")
        }
        else if (isolate(input[[paste0("demethod",cnt)]]) == "Limma"){
            inputconds$demethod_params[cnt] <- paste(
                isolate(input[[paste0("demethod",cnt)]]),
                covariate,
                isolate(input[[paste0("limma_normfact",cnt)]]),
                isolate(input[[paste0("limma_fitType",cnt)]]),
                isolate(input[[paste0("normBetween",cnt)]]), sep=",")
        }
    }
    
    for (i in seq(1:counter))
    {
        conds <- c(rep(paste0("Cond", 2*i-1), 
                       length(inputconds$conds[[2*i-1]])), 
                   rep(paste0("Cond", 2*i), length(inputconds$conds[[2*i]])))
        cols <- c(paste(inputconds$conds[[2*i-1]]), 
                  paste(inputconds$conds[[2*i]]))
        params <- unlist(strsplit(inputconds$demethod_params[i], ","))
        withProgress(message = 'Running DE Algorithms', detail = inputconds$demethod_params[i], value = 0, {
            initd <- callModule(debrowserdeanalysis, paste0("DEResults",i), data = data, metadata = meta, 
                  columns = cols, conds = conds, params = params)
            if (!is.null(initd$dat()) && nrow(initd$dat()) > 1){
                meta_selection <- selectedInput("conditions_from_meta", i, NULL, input)
                cond_names <- c(paste0("Cond", 2*i-1), paste0("Cond", 2*i))
                if (meta_selection != "No Selection"){
                    rownames(meta) <- meta[,1]
                    ctable <- data.frame(cbind(meta[cols,c(colnames(meta[1]), meta_selection)], conds))
                    colnames(ctable) <- c("sample", "cond_name", "conds")
                    cond_names <- c(unique(ctable$cond_name[ctable$conds == paste0("Cond", 2*i-1)]),
                                    unique(ctable$cond_name[ctable$conds == paste0("Cond", 2*i)]))
                }
                inputconds$dclist[[i]] <- list(conds = conds, cols = cols, cond_names=cond_names, init_data=initd$dat(), 
                    demethod_params = inputconds$demethod_params[i])
            }else{
                return(NULL)
            }
            incProgress(1/counter)
        })
    }

    if(length(inputconds$dclist) < 1) return(NULL)

    inputconds$dclist
}

library(debrowser)

header <- dashboardHeader(
    title = "DEBrowser Filter"
)
sidebar <- dashboardSidebar(  sidebarMenu(id="DataPrep",
       menuItem("Filter", tabName = "Filter")))

body <- dashboardBody(
    tabItems(
        tabItem(tabName="Filter", dataLCFUI("lcf"),
                column(4,
                       verbatimTextOutput("filtertable")
                )
        )
    ))

ui <- dashboardPage(header, sidebar, body, skin = "blue")

server <- function(input, output, session) {
    load(system.file("extdata", "demo", "demodata.Rda",
                     package = "debrowser"))
    ldata <- reactiveValues(count=NULL, meta=NULL)
    ldata$count <- demodata
    ldata$meta <- metadatatable
    data <- callModule(debrowserlowcountfilter, "lcf", ldata)
    observe({
        output$filtertable <- renderPrint({
            head( data$filter()$count )
        })
    })
}

shinyApp(ui, server)

library(plotly)
library(debrowser)
library(DESeq2)

header <- dashboardHeader(
    title = "DEBrowser Main Plots"
)
sidebar <- dashboardSidebar(  sidebarMenu(id="DEAnalysis",
    menuItem("Main", tabName = "Main"),
    mainPlotControlsUI("main"),
    plotSizeMarginsUI("main",  w=600, h=400),
    heatmapControlsUI("heatmap"),
    plotSizeMarginsUI("heatmap", w=600, h=400)))

body <- dashboardBody(
    tabItems(
        tabItem(tabName="Main",
                fluidRow(column(6,
                getMainPlotUI("main"),
                verbatimTextOutput("main_hover"),
                verbatimTextOutput("main_selected")
                ),
                column(6,
                getHeatmapUI("heatmap"),
                verbatimTextOutput("heatmap_hover"),
                verbatimTextOutput("heatmap_selected")
                ))
        )
    ))

ui <- dashboardPage(header, sidebar, body, skin = "blue")

server <- function(input, output, session) {
    #Example usage with demodata
    load(system.file("extdata", "demo", "demodata.Rda",
                     package = "debrowser"))
    dat <-c()
    dat$columns <- c("exper_rep1", "exper_rep2", "exper_rep3",
                 "control_rep1", "control_rep2", "control_rep3")
    dat$conds <- factor( c("Control", "Control", "Control",
                       "Treat", "Treat", "Treat") )
    dat$data <- data.frame(demodata[, dat$columns])
    
    dat$cond_names <- c("Control", "Treat")
    
    # You can also use your dataset by reading your data from a file like below;
    # The data in this commented out exampele is not supplied but these lines 
    # can give you an idea about how to read the data from a file;
    #
    # data  <- read.table("~/Downloads/shKRAS.tsv", header=T, row.names=1, sep="\t")
    # dat$columns <- c("CNT.2", "CNT.3", "CNT.4", 
    #               "shKRAS_T1", "shKRAS_T2", "shKRAS_T3")
    # dat$conds <- factor( c("Control", "Control", "Control",
    #                        "shKRAS", "shKRAS", "shKRAS") )
    # dat$data <- data.frame(data[, dat$columns])
    #
    xdata <- generateTestData(dat)
    selected <- callModule(debrowsermainplot, "main", xdata, dat$cond_names)
    
    output$main_hover <- renderPrint({
        selected$shgClicked()
    })
    output$main_selected <- renderPrint({
        selected$selGenes()
    })
    selectedHeat <- NULL
    observe({
        if (!is.null(selected$selGenes())) {
        withProgress(message = 'Creating plot', style = "notification", value = 0.1, {
            selectedHeat <- callModule(debrowserheatmap, "heatmap", 
                xdata[selected$selGenes(), dat$columns])
        })
        }
    })
    output$heatmap_hover <- renderPrint({
       if( !is.null(selectedHeat))
            selectedHeat$shgClicked()
    })
    output$heatmap_selected <- renderPrint({
        if( !is.null(selectedHeat))
        selectedHeat$selGenes()
    })
}
shinyApp(ui, server)

library(plotly)
library(debrowser)
library(DESeq2)
source("../../R/mainScatter.R")
source("../../R/deprogs.R")

header <- dashboardHeader(
    title = "DEBrowser Main Plots"
)
sidebar <- dashboardSidebar(  sidebarMenu(id="DEAnalysis",
    menuItem("Main", tabName = "Main"),
    mainPlotControlsUI("main"),
    plotSizeMarginsUI("main")))

body <- dashboardBody(
    tabItems(
        tabItem(tabName="Main", getMainPlotUI("main"),
                column(4,
                       verbatimTextOutput("main_hover"),
                       verbatimTextOutput("main_selected")
                )
        )
    ))

ui <- dashboardPage(header, sidebar, body, skin = "blue")

server <- function(input, output, session) {
    #Example usage with demodata
    load(system.file("extdata", "demo", "demodata.Rda",
                     package = "debrowser"))
    dat <-c()
    dat$columns <- c( "control_rep1", "control_rep2", "control_rep3",
                      "exper_rep1", "exper_rep2", "exper_rep3")
    dat$conds <- factor( c("Control", "Control", "Control",
                       "Treat", "Treat", "Treat") )
    
    cond_names <- c("Control", "Treat")
    
    
    dat$data <- data.frame(demodata[, dat$columns])
    
    # You can also use your dataset by reading your data from a file like below;
    # The data in this commented out exampele is not supplied but these lines 
    # can give you an idea about how to read the data from a file;
    #
    # data  <- read.table("~/Downloads/shKRAS.tsv", header=T, row.names=1, sep="\t")
    # dat$columns <- c("CNT.2", "CNT.3", "CNT.4", 
    #               "shKRAS_T1", "shKRAS_T2", "shKRAS_T3")
    # dat$conds <- factor( c("Control", "Control", "Control",
    #                        "shKRAS", "shKRAS", "shKRAS") )
    # dat$data <- data.frame(data[, dat$columns])
    #
    xdata <- generateTestData(dat)
    selected <- callModule(debrowsermainplot, "main", xdata, cond_names)
    
    output$main_hover <- renderPrint({
        selected$shgClicked()
    })
    output$main_selected <- renderPrint({
        selected$selGenes()
    })
}
shinyApp(ui, server)

#' debrowserIQRplot
#'
#' Module for an IQR plot that can be used in data prep and 
#' low count removal modules
#' 
#' @param input, input variables
#' @param output, output objects
#' @param session, session 
#' @param data, a matrix that includes expression values
#' @return IQR 
#' @export
#'
#' @examples
#'     x <- debrowserIQRplot()
#'
debrowserIQRplot <- function(input = NULL, output = NULL, session = NULL, data = NULL) {
    if (is.null(data)) return(NULL)
    output$IQR <- renderPlotly({
      getIQRPlot(data, input)
    })
    output$IQRUI <- renderUI({
    shinydashboard::box(
        collapsible = TRUE, title = session$ns("plot"), status = "primary", 
        solidHeader = TRUE, width = NULL,
        draggable = TRUE,  plotlyOutput(session$ns("IQR"),
             width = input$width, height=input$height))
    })
}

#' getIQRPlotUI
#'
#' IQR plot UI.  
#' @param id, namespace id
#' @note \code{getIQRPlotUI}
#' @return the panel for IQR plots;
#'
#' @examples
#'     x <- getIQRPlotUI("IQR")
#'
#' @export
#'
getIQRPlotUI <- function(id) {
    ns <- NS(id)
    uiOutput(ns("IQRUI"))
}

#' IQRPlotControlsUI
#'
#' Generates the controls in the left menu for an IQR plot#' 
#' @param id, namespace id
#' @note \code{IQRPlotControlsUI}
#' @return returns the left menu
#' @examples
#'     x <- IQRPlotControlsUI("IQR")
#' @export
#'
IQRPlotControlsUI <- function(id) {
  ns <- NS(id)
  shinydashboard::menuItem(paste0(id, " - Options"),
      textInput(ns("breaks"), "Breaks", value = "100" )
  )
}


#' getIQRPlot
#'
#' Makes IQR boxplot plot
#'
#' @param data, count or normalized data
#' @param input, input
#' @param title, title
#'
#' @export
#'
#' @examples
#'     getIQRPlot()
#'
getIQRPlot <- function(data=NULL, input=NULL, title = ""){
  if (is.null(data)) return(NULL)
  data <- as.data.frame(data)
  cols <- colnames(data)
  data[, cols] <- apply(data[, cols], 2,
      function(x) log10(as.integer(x) + 1))
  
  data <- addID(data)
  mdata <- melt(as.data.frame(data[,c("ID", cols)]),"ID")
  colnames(mdata)<-c("ID", "samples", "logcount")

  p <- plot_ly(mdata, x = ~samples, y = ~logcount, type = "box",
               width = input$width, height=input$height,
               marker = list(color = 'rgb(8,81,156)',
                             outliercolor = 'rgba(219, 64, 82, 0.6)',
                             line = list(outliercolor = 'rgba(219, 64, 82, 1.0)',
                                         outlierwidth = 2))) %>%
      plotly::layout(title = title,
           xaxis = list(title = "samples"),
           yaxis = list(title = "logcount")) 
    if (!is.null(input$left))
        p <- p %>% plotly::layout(margin = list(l = input$left,
                                 b = input$bottom,
                                 t = input$top,
                                 r = input$right
    ))
  p$elementId <- NULL
  if (!is.null(input$svg) && input$svg == TRUE)
      p <- p %>% config(toImageButtonOptions = list(format = "svg"))
  p
}

library(debrowser)
library(plotly)
library(gplots)
library(heatmaply)
library(RColorBrewer)

if (interactive()) {
    options(shiny.maxRequestSize = 30*1024^2, shiny.sanitize.errors = TRUE)
    environment(heatmapServer) <- environment()
    app <- shinyApp( ui = shinyUI(heatmapUI),
        server = shinyServer(heatmapServer))
    runApp(app)
}

#' applyFilters
#'
#' Applies filters based on user selected parameters to be
#' displayed within the DEBrowser.
#'
#' @param filt_data, loaded dataset
#' @param cols, selected samples
#' @param conds, seleced conditions
#' @param input, input parameters
#' @return data
#' @export
#'
#' @examples
#'     x <- applyFilters()
#'
applyFilters <- function(filt_data = NULL, cols = NULL, conds=NULL,
    input = NULL){
    if (is.null(input$padj) || is.null(input$foldChange) 
        || is.null(filt_data)) return(NULL)
    compselect <- 1
    if (!is.null(input$compselect) ) 
        compselect <- as.integer(input$compselect)
    x <- paste0("Cond", 2*compselect - 1) 
    y <- paste0("Cond", 2*compselect)
    norm_data <- getNormalizedMatrix(filt_data[, cols], 
        input$norm_method)
    g <- data.frame(cbind(cols, conds))
    if (length(as.vector(g[g$conds == x, "cols"])) > 1 )
        filt_data$x <- log10(rowMeans(norm_data[, 
            as.vector(g[g$conds == x, "cols"])]) + 0.1)
    else
        filt_data$x <- log10(norm_data[, 
             as.vector(g[g$conds == x, "cols"])] + 0.1)
    if (length(as.vector(g[g$conds == y, "cols"])) > 1 )
        filt_data$y <- log10(rowMeans(norm_data[, 
             as.vector(g[g$conds == y, "cols"])]) + 0.1)
    else
        filt_data$y <- log10(norm_data[, 
             as.vector(g[g$conds == y, "cols"])] + 0.1)
    filt_data[,cols] <- norm_data
    
    padj_cutoff <- as.numeric(input$padj)
    foldChange_cutoff <- as.numeric(input$foldChange)
    m <- filt_data
    # Add column which says whether a gene significant or not
    m$Legend <- character(nrow(m))
    m$Size <- character(nrow(m))
    m[, "Size"] <- "40"
    m$Legend <- "NS"
    if (input$dataset == "up" || input$dataset == "up+down" || input$dataset == "selected") 
        m$Legend[m$foldChange >= foldChange_cutoff &
               m$padj <= padj_cutoff] <- "Up"
    if (input$dataset == "down" || input$dataset == "up+down" || input$dataset == "selected")
        m$Legend[m$foldChange <= (1 / foldChange_cutoff) &
               m$padj <= padj_cutoff] <- "Down"
    if (input$dataset == "most-varied" && !is.null(cols)) {
        most_varied <- getMostVariedList(m, cols, input)
        m[rownames(most_varied), c("Legend")] <- "MV"
    }
    if (!is.null(input$genesetarea) && input$genesetarea != ""
        && input$methodtabs == "panel1") {
        genelist <- getGeneSetData(m, c(input$genesetarea))
        m[rownames(genelist), "Legend"] <- "GS"
        m[rownames(genelist), "Size"] <- "100"
        tmp <- m["Legend"=="GS", ]
        tmp1 <- m["Legend"!="GS", ]
        m <- rbind(tmp1, tmp)
    }
    m
}
#' getSelectedDatasetInput
#'
#' Gathers the user selected dataset output to be displayed.
#'
#' @param rdata, filtered dataset
#' @param getSelected, selected data
#' @param getMostVaried, most varied data
#' @param mergedComparison, merged comparison data
#' @param input, input parameters
#' @return data
#' @export
#'
#' @examples
#'     x <- getSelectedDatasetInput()
#'
getSelectedDatasetInput<-function(rdata = NULL, getSelected = NULL, 
    getMostVaried = NULL, mergedComparison = NULL,
    input = NULL) {
    if (is.null(rdata)) return (NULL)
    m <- rdata
    if (input$dataset == "up") {
        m <- getUp(rdata)
    } else if (input$dataset == "down") {
        m <- getDown(rdata)
    } else if (input$dataset == "up+down") {
        m <- getUpDown(rdata)
    } else if (input$dataset == "alldetected") {
        m <- rdata
    } else if (input$dataset == "selected" && !is.null(input$selectedplot)) {
        m <- getSelected
    } else if (input$dataset == "most-varied") {
        m <- rdata[rownames(getMostVaried), ]
    } else if (input$dataset == "comparisons") {
        m <- mergedComparison
    } else if (input$dataset == "searched") {
        m <- getSearchData(rdata, input)
    }
    m
}


#' getMostVariedList
#'
#' Calculates the most varied genes to be used for specific plots
#' within the DEBrowser.
#'
#' @param datavar, loaded dataset
#' @param cols, selected columns
#' @param input, input 
#' @return data
#' @export
#'
#' @examples
#'     x <- getMostVariedList()
#'
getMostVariedList <- function(datavar = NULL, cols = NULL, input = NULL){
    if (is.null(datavar)) return (NULL)
    topn <- as.integer(as.numeric(input$topn))
    filtvar <- datavar[rowSums(datavar[,cols]) >
        as.integer(as.numeric(input$mincount)),]
    cv<-cbind(apply(filtvar, 1, function(x) 
        (sd(x,na.rm=TRUE)/mean(x,na.rm=TRUE))), 1)
    colnames(cv)<-c("coeff", "a")
    cvsort<-cv[order(cv[,1],decreasing=TRUE),]
    topindex<-nrow(cvsort)
    if (topindex > topn) topindex <- topn
    cvsort_top <- head(cvsort, topindex)
    selected_var <- data.frame(datavar[rownames(cvsort_top),])
}


#' getSearchData
#'
#' search the geneset in the tables and return it
#'
#' @param dat, table data
#' @param input, input params
#' @return data
#' @export
#'
#' @examples
#'     x <- getSearchData()
#'
getSearchData <- function(dat = NULL, input = NULL)
{
  if (is.null(dat)) return(NULL)
  if (input$genesetarea != ""){
    dat <- getGeneSetData(dat, c(input$genesetarea))
  }
  dat
}

#' getGeneSetData
#'
#' Gathers the specified gene set list to be used within the
#' DEBrowser.
#'
#' @param data, loaded dataset
#' @param geneset, given gene set
#' @return data
#' @export
#'
#' @examples
#'     x <- getGeneSetData()
#'
getGeneSetData <- function(data = NULL, geneset = NULL) {
    if (is.null(data)) return (NULL)
    
    geneset1 <- unique(unlist(strsplit(geneset, split="[:;, \t\n\t]")))
    geneset2 <- geneset1[geneset1 != ""]
    if(length(geneset2) > 3)
        geneset2 <- paste0("^", geneset2, "$")
    
    dat1 <- as.data.frame(data)
    if(!("ID" %in% names(dat1)))
        dat2 <- addID(dat1)
    else
        dat2 <- dat1

    dat2$ID<-factor(as.character(dat2$ID))

    geneset4 <- unique(as.vector(unlist(lapply(toupper(geneset2), 
        function(x){ sapply(dat2[(grepl(x, toupper(dat2[,"ID"]))), "ID"], 
                            as.character) }))))
    retset <- data.frame(dat2[geneset4, ])
    retset
}

#' getUp
#' get up regulated data
#'
#' @param filt_data, filt_data
#' @return data
#' @export
#'
#' @examples
#'     x <- getUp()
#'
getUp <- function(filt_data = NULL){
    if(is.null(filt_data)) return(NULL)
    filt_data[
        filt_data[, "Legend"] == "Up" | 
        filt_data[, "Legend"] == "GS", ]
}
#' getDown
#' get down regulated data
#'
#' @param filt_data, filt_data
#' @return data
#' @export
#'
#' @examples
#'     x <- getDown()
#'
getDown <- function(filt_data = NULL){
    if(is.null(filt_data)) return(NULL)
    filt_data[
        filt_data[, "Legend"] == "Down"|
        filt_data[, "Legend"] == "GS", ]
}

#' getUpDown
#' get up+down regulated data
#'
#' @param filt_data, filt_data
#' @return data
#' @export
#'
#' @examples
#'     x <- getUpDown()
#'
getUpDown <- function(filt_data = NULL){
    if(is.null(filt_data)) return(NULL)
    filt_data[
        filt_data[, "Legend"] == "Up" | 
        filt_data[, "Legend"] == "Down"|
        filt_data[, "Legend"] == "GS", ]
}

#' getDataForTables
#' get data to fill up tables tab
#'

#' @param input, input parameters
#' @param init_data, initial dataset
#' @param filt_data, filt_data
#' @param selected, selected genes
#' @param getMostVaried, most varied genes
#' @param mergedComp, merged comparison set
#' @param explainedData, pca gene set
#' @return data
#' @export
#'
#' @examples
#'     x <- getDataForTables()
#'
getDataForTables <- function(input = NULL, init_data = NULL,
    filt_data = NULL, selected = NULL,
    getMostVaried = NULL,  mergedComp = NULL,
    explainedData = NULL){
    if (is.null(init_data )) return(NULL)
    if (is.null(filt_data)) filt_data <- init_data
    pastr <- "padj"
    fcstr <- "foldChange"
    dat <- NULL
    if (input$dataset == "alldetected"){
            dat <- getSearchData(filt_data, input)
    }
    else if (input$dataset == "up+down"){
        if (!is.null(filt_data))
            dat <- getSearchData(getUpDown(filt_data), input)
    }
    else if (input$dataset == "up"){
        if (!is.null(filt_data))
            dat <- getSearchData(getUp(filt_data), input)
    }
    else if (input$dataset == "down"){
        if (!is.null(filt_data))
            dat <- getSearchData(getDown(filt_data), input)
    }
    else if (input$dataset == "selected"){
        dat <- getSearchData(selected, input)
    }
    else if (input$dataset == "most-varied"){
        if (!is.null(filt_data)){
            d <- filt_data[rownames(getMostVaried),]
        }else{
            d <- init_data[rownames(getMostVaried),]
        }
        dat <- getSearchData(d, input)
    }
    else if (input$dataset == "comparisons"){
        if (is.null(mergedComp)) return(NULL)
        fcstr<-colnames(mergedComp)[grepl("foldChange", colnames(mergedComp))]
        pastr<-colnames(mergedComp)[grepl("padj", colnames(mergedComp))]
        dat <- getSearchData(mergedComp, input)
    }
    else if (input$dataset == "searched"){
        dat <- getSearchData(init_data, input)
    }
    list(dat, pastr, fcstr)
}


#' getMergedComparison
#'
#' Gathers the merged comparison data to be used within the
#' DEBrowser.
#' @param dc, data container
#' @param nc, the number of comparisons
#' @param input, input params
#' @return data
#' @export
#'
#' @examples
#'     x <- getMergedComparison()
#'
getMergedComparison <- function(dc = NULL, nc = NULL, input = NULL){
    if (is.null(dc)) return (NULL)
    mergeresults <- c()
    mergedata <- c()
    allsamples <- c()
    for ( ni in seq(1:nc)) {
        tmp <- dc[[ni]]$init_data[,c("foldChange", "padj")]

        samples <- dc[[ni]]$cols
        cond_names <- dc[[ni]]$cond_names
        tt <- paste0(cond_names[1],".vs.",cond_names[2])
        #tt <- paste0("C", (2*ni-1),".vs.C",(2*ni))
        fctt <- paste0("foldChange.", tt)
        patt <-  paste0("padj.", tt)
        colnames(tmp) <- c(fctt,  patt)
        if(ni == 1){
            allsamples <- samples
            mergeresults <- tmp
            mergedata <- dc[[ni]]$init_data[,samples]
        }
        else{
            mergeresults[,fctt] <- character(nrow(tmp))
            mergeresults[,patt] <- character(nrow(tmp))
            mergeresults[rownames(tmp),c(fctt, patt)] <- tmp[,c(fctt, patt)]
            mergeresults[rownames(tmp),patt] <- tmp[,patt]
            mergeresults[is.na(mergeresults[,fctt]),fctt] <- 1 
            mergeresults[is.na(mergeresults[,patt]),patt] <- 1 
            remaining_samples <- dc[[ni]]$cols[!(samples %in% colnames(mergedata))]
            allsamples <- unique(c(allsamples, remaining_samples))
            mergedata <- cbind(mergedata,  dc[[ni]]$init_data[,remaining_samples])
            colnames(mergedata) <- allsamples
        }
    }
    mergedata[,allsamples] <- getNormalizedMatrix(mergedata[,allsamples], input$norm_method)
    cbind(mergedata, mergeresults)
}

#' applyFiltersToMergedComparison
#'
#' Gathers the merged comparison data to be used within the
#' DEBrowser.
#'
#' @param dc, all data 
#' @param nc, the number of comparisons
#' @param input, input params
#' @return data
#' @export
#'
#' @examples
#'     x <- applyFiltersToMergedComparison()
#'
applyFiltersToMergedComparison <- function (dc = NULL, 
    nc = NULL, input = NULL)
{
    if (is.null(dc)) return (NULL)
    merged <- getMergedComparison(dc, nc, input)
    padj_cutoff <- as.numeric(input$padj)
    foldChange_cutoff <- as.numeric(input$foldChange)
    if (is.null(merged$Legend)){
        merged$Legend <- character(nrow(merged))
        merged$Legend <- "NS"
    }
    for ( ni in seq(1:nc)) {
        cond_names <- dc[[ni]]$cond_names
        tt <- paste0(cond_names[1],".vs.",cond_names[2])
        #tt <- paste0("C", (2*ni-1),".vs.C",(2*ni))
        merged[which(as.numeric(merged[,c(paste0("foldChange.", tt))]) >= 
            foldChange_cutoff & as.numeric(merged[,c(paste0("padj.", tt))]) <= 
            padj_cutoff), "Legend"] <- "Sig"
        merged[which(as.numeric(merged[,c(paste0("foldChange.", tt))]) <= 
            1/foldChange_cutoff & as.numeric(merged[,c(paste0("padj.", tt))]) <= 
            padj_cutoff), "Legend"] <- "Sig"
    }
    merged 
}

#' removeCols
#'
#' remove unnecessary columns
#'
#' @param cols, columns that are going to be removed from data frame
#' @param dat, data
#' @return data
#' @export
#'
#' @examples
#'     x <- removeCols()
#'
removeCols <- function( cols = NULL, dat = NULL) {
    if (is.null(dat)) return (NULL)
    for (colnum in seq(1:length(cols))){
         if (cols[colnum] %in% colnames(dat) )
              dat[, cols[colnum]]<- NULL
    }
    dat
}

require(shiny)
library(debrowser)
library(testthat)

test_that("demo data can be loaded", {
    load(system.file("extdata", "demo", "demodata.Rda",
        package = "debrowser"))
    expect_true(is.data.frame(demodata))
    expect_equal(demodata[29311, 2], 2)
    expect_equal(demodata[29311, 5], 7.1)
    expect_equal(demodata[29311, 6], 6)
    expect_null(demodata[1, 7])
})

---
title: "DEBrowser user guide"
author: "Alper Kucukural, Onur Yukselen, Manuel Garber"
output:
    rmarkdown::html_document:
    highlight: pygments
    toc: true
fig_caption: true
vignette: >
    %\VignetteIndexEntry{DEBrowser Vignette}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r, echo=FALSE, results="hide", warning=FALSE}
suppressPackageStartupMessages({
    library(gplots)
    library(ggplot2)
    library(shiny)
    library(DT)
    library(clusterProfiler)
    library(DESeq2)
    library(org.Hs.eg.db)
    library(edgeR)
    library(debrowser)
    library(RColorBrewer)
    library(annotate)
})
```

# DEBrowser:
Interactive Differential Expression Analysis Tool

# Introduction

Differential expression (DE) analysis has become an increasingly popular tool
in determining and viewing up and/or down expressed  genes between two sets of
samples.  The goal of differential gene expression analysis is to find genes
or transcripts whose difference in expression, when accounting for the
variance within condition, is higher than expected by chance.
[DESeq2](https://bioconductor.org/packages/release/bioc/html/DESeq2.html) is
an R package available via Bioconductor and is designed to normalize count
data from high-throughput sequencing assays such as RNA-Seq and test for
differential expression (Love et al. 2014).  With multiple parameters such as
padjust values, log fold changes, plot styles, and so on, altering plots
created with your DE data can be a hassle as well as time consuming. The
Differential Expression Browser uses DESeq2 (Love et al., 2014),
[EdgeR](https://bioconductor.org/packages/release/bioc/html/edgeR.html)
(Robinson et al., 2010), and
[Limma](https://bioconductor.org/packages/release/bioc/html/limma.html)
(Ritchie et al., 2015) coupled with
shiny (Chang, W. et al., 2016)  to produce real-time changes within your
plot queries and allows for interactive browsing of your DE results.
In addition to DE analysis, DEBrowser also offers a variety of other plots
and analysis tools to help visualize your data even further.

## DEBrowser

DEBrowser utilizes Shiny, a R based application development tool that creates
a wonderful interactive user interface (UI) combined with all of the
computing prowess of R. After the user has selected the data to analyze and
has used the shiny UI to run DE analysis, the results are then input to DEBrowser.
DEBrowser manipulates your results in a way that allows for interactive
plotting by which changing padj or fold change limits also changes the
displayed graph(s). For more details about these plots and tables, please
visit our quick start guide for some helpful tutorials.

For comparisons against other popular data visualization tools, see the
comparison table below (Figure 40).

# Quick start

Before you start; you will have to install R and/or RStudio. 

```
# Installation instructions:
# 1. Install DEBrowser and its dependencies by running the lines below
# in R or RStudio.

if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("debrowser")

# 2. Load the library

library(debrowser)

# 3. Start DEBrowser

startDEBrowser()
```

Please check *Operating System Dependencies* section, in case your operating system requires packages to be installed.

# Browsing your Data

## Data via TSV file

Once you've made your way to the website, or you have a local instance of DEBrowser running, you will be greeted with data loading section:

![*Figure 1. Data loading.*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/data_load.png "Figure 1. Data loading")
    
To begin the analysis, you need to upload your *count data file* (comma or semicolon separated (CSV), and tab separated (TSV) format) to be analyzed and choose appropriate separator for the file (comma, semicolon or tab).

Gene quantifications table can be obtained running standard software
like HTSeq (Anders,S. et al, 2014) or RSEM (Li and Dewey, 2011). The file
values must contain the gene, transcript(s), and the sample raw count values
you wish to enter into DEBrowser.

If you do not have a dataset to upload, you can use the built in demo data file by clicking on the 'Load Demo (Vernia et al.)!' button.  To view the entire demo data file, you can download
this demo set: https://umms.dolphinnext.com/pub/debrowser/simple_demo.tsv . For another example, try our full dataset (Vernia et. al): https://umms.dolphinnext.com/pub/debrowser/advanced_demo.tsv .

The structure of the count data files are shown below:

| gene     | transcript | exper_rep1 | exper_rep2 | control_rep1 | control_rep2 |
|----------|------------|------------|------------|--------------|--------------|
| DQ714826 | uc007tfl.1 | 0.00       | 0.00       | 0.00         | 0.00         |
| DQ551521 | uc008bml.1 | 0.00       | 0.00       | 0.00         | 0.00         |
| AK028549 | uc011wpi.1 | 2.00       | 1.29       | 0.00         | 0.00         |

Please also note that, DEBrowser reads the gene names from the first column and skips other non numerical columns and starts reading the quantification values from the 3rd column in this case.

In addition to the count data file; you need to upload metadata file to correct for batch effects or any other normalizing conditions you might want to address that might be within your results. To handle for these conditions, simply create a metadata file by using the example table at below or download sample file from following link: <https://umms.dolphinnext.com/pub/debrowser/simple_demo_meta.txt>


| sample       | batch | condition|
|--------------|-------|----------|
| exper_rep1   | 1     | A        |
| exper_rep2   | 2     | A        |
| exper_rep3   | 1     | A        |
| control_rep1 | 2     | B        |
| control_rep2 | 1     | B        |
| control_rep3 | 2     | B        |

Metadata file can be formatted with comma, semicolon or tab separators similar to count data files. These files used to establish different batch effects for multiple conditions.
You can have as many conditions as you may require, as long as all of the samples are present. 

* The example above would result in the first set of conditions as ``exper_rep1``, ``exper_rep2``, ``exper_rep3`` from ``A`` and second set of conditions as ``control_rep1``, ``control_rep2``, ``control_rep3`` from ``B`` as they correspond to those conditions in the ``condition`` column.

* In the same way, 'batch' would have the first set as ``exper_rep1``, ``exper_rep3``, ``control_rep2`` from ``1`` and second set as ``exper_rep2``, ``control_rep1``, ``control_rep3`` from ``2`` as they correspond to those conditions in the ``batch`` column.

Once the count data and metadata files have been loaded in DEBrowser, you can click upload button to visualize your data as shown at below:
    
![*Figure 2. Upload Summary*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/upload_summary.png "Figure 2. Upload Summary")

After loading the gene quantification file, and if specified the metadata file containing your batch correction fields, you then have the option to filter low counts and conduct batch effect correction prior to your analysis. Alternatively, you may skip these steps and directly continue with differential expression analysis or view quality control (QC) information of your dataset.

# Low Count Filtering

In this section, you can simultaneously visualize the changes of your dataset while filtering out the low count genes. Choose your filtration criteria from **Filtering Methods** box which is located just center of the screen. Three methods are available to be used:

* **Max:** Filters out genes where maximum count for each gene across all samples are less than defined threshold. 
* **Mean:** Filters out genes where mean count for each gene are less than defined threshold. 
* **CPM:**	First, counts per million (CPM) is calculated as the raw counts divided by the library sizes and multiplied by one million. Then it filters out genes where at least defined number of samples is less than defined CPM threshold.

After selection of filtering methods and entering threshold value, you can proceed by clicking **Filter** button which is located just bottom part of the **Filtering Methods** box. On the right part of the screen, your filtered dataset will be visualized for comparison as shown at figure below. 

![*Figure 3. Filtering*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/filtering.png "Figure 3. Filtering")

You can easily compare following features, before and after filtering: 

* Number of genes/regions.
* Read counts for each sample.
* Overall histogram of the dataset.
* gene/region vs samples data 


* To investigate the gene/region vs samples data in detail as shown at below, you may click the **Show Data** button, located bottom part of the data tables. Alternatively, you may download all filtered data by clicking **Download** button which located next to **Show Data** button.  

![*Figure 4. Show Data*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/show_data.png "Figure 4. Show Data")

Afterwards, you may continue your analysis with **Batch Effect Correction** or directly jump to differential expression analysis or view quality control (QC) information of your dataset.


# Batch Effect Corrections
If specified metadata file containing your batch correction fields, then you have the option to conduct batch effect correction prior to your analysis. By adjusting parameters of **Options** box, you can investigate your character of your dataset. These parameters of the options box are explained as following:

* **Normalization Method:** DEBrowser allows performing normalization prior the batch effect correction. You may choose your normalization method (among MRN (Median Ratio Normalization), TMM (Trimmed Mean of M-values), RLE (Relative Log Expression) and upperquartile), or skip this step by choosing **none** for this item. For our sample data, we are going to choose MRN normalization.
* **Correction Method:** DEBrowser uses ComBat <https://bioconductor.org/packages/release/bioc/vignettes/sva/inst/doc/sva.pdf> (part of the SVA bioconductor package) or Harman <https://www.bioconductor.org/packages/3.7/bioc/vignettes/Harman/inst/doc/IntroductionToHarman.html> to adjust for possible batch effect or conditional biases. For more information, you can visit following links for documentation: ComBat <https://bioconductor.org/packages/release/bioc/vignettes/sva/inst/doc/sva.pdf>, Harman <https://www.bioconductor.org/packages/3.7/bioc/vignettes/Harman/inst/doc/IntroductionToHarman.html> For our sample data, Combat correction was selected.
* **Treatment:** Please select the column that is specified in metadata file for comparison, such as cancer vs control. It is named *treatment* for our sample metadata.
* **Batch:** Please select the column name in metadata file which differentiate the batches. For example, in our metadata, it is called *batch*.

Upon clicking submit button, comparison tables and plots will be created on the right part of the screen as shown below.

![*Figure 5. Batch Effect Correction - PCA*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/batch_PCA.png "Figure 5. Batch Effect Correction - PCA")

![*Figure 6. Batch Effect Correction - IQR*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/batch_IQR.png "Figure 6. Batch Effect Correction - IQR")

![*Figure 7. Batch Effect Correction - Density*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/batch_density.png "Figure 7. Batch Effect Correction - Density")

You can investigate the changes on the data by comparing following features:

* Read counts for each sample.
* PCA, IQR and Density plot of the dataset.
* Gene/region vs samples data


* You can investigate the gene/region vs samples data in detail by clicking the **Show Data** button, or download all corrected data by clicking **Download** button.

Since we have completed **batch effect correction and normalization** step, we can continue with one of the following options: 'Go to DE Analysis' and, 'Go to QC plots!'. First option takes you to page where differential expression analyses are conducted with DESeq2, EdgeR or Limma. The second option, 'Go to QC plots!', takes you to a page where you can view quality control metrics of your data by PCA, All2All, Heatmap, Density, and IQR plots.

# DE Analysis:

The first option, 'Go to DE Analysis', takes you to the next step where differential expression analyses are conducted.

* **Sample Selection:** In order to run DE analysis, you first need to select the samples which will be compared. To do so, click on "Add New Comparison" button, and choose **Select Meta** box as **treatment** to simplify fill ``Condition 1`` and ``Condition 2`` based on the **treatment** column of the metadata as shown below.

![*Figure 8. DE Selection*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/de_selection.png "Figure 8. DE Selection")


If you need to remove samples from a condition, simply select the sample you wish to remove and hit the delete/backspace key. In case, you need to add a sample to a condition you can click on one of the condition text boxes to bring up a list of samples and then click on the sample you wish to add from the list and it will be added to the textbox for that comparison.


* You can add multiple conditions to compare by clicking on "Add New Comparison" button, and view the results separately after DE analysis.

* **Method Selection:** Three DE methods are available for DEBrowser: DESeq2, EdgeR, and Limma. DESeq2 and EdgeR are designed to normalize count data from high-throughput sequencing assays such as RNA-Seq. On the other hand, Limma is a package to analyse of normalized or transformed data from microarray or RNA-Seq assays. We have selected DESeq2 for our test sample and showed the related results at below.

After clicking on the 'Submit!' button, DESeq2 will analyze your comparisons and store the results into separate data tables. It is important to note that the resulting data produced from DESeq is normalized. Upon finishing the DESeq analysis, a result table will appear which allows you to download the data by clicking "Download" button. To visualize the data with interactive plots please click on "Go to Main Plots!" button.

## The Main Plots of DE Analysis:

Upon finishing the DESeq analysis, please click on **Go to Main Plots!** button which will open **Main Plots** tab where you will be able to view
the interactive plots.

![*Figure 9. Info Tabs*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/info_tabs.png "Figure 9. Info Tabs")


The page will load with **Scatter Plot**. You can switch to **Volcano Plot** and **MA Plot** by using **Plot Type** section at the left side of the menu. Since these plots are interactive, you can click to **zoom** button on the top of the graph and select the area you would like to zoom in by drawing a rectangle. Please see the plots at below:

![*Figure 10. Main Plots*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_main_plots.png "Figure 10. Main Plots")
    

**A.** Scatter plot, **B.** Volcano plot, **C.** MA plot


* Please keep in mind that to increase the performance of the generating graph, by default 10% of non-significant(NS) genes are used to generate plots. You might show all NS genes by please click **Main Options** button and change **Background Data(%)** to 100% on the left sidebar.

![*Figure 11. Background data*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_background_data.png "Figure 11. Background Data")


You can hover over the scatterplot points to display more information about the point selected. A few bargraphs will be generated for the user to view as soon as a scatterplot point is hovered over.

![*Figure 12. main plot hover*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/main_plot_hover.png "Figure 12. main plot hover")


**A.** Hover on Fabp3 gene, **B.** Read Counts vs Samples, **C.** Read Counts vs Conditions

You also have a wide array of options when it comes to fold change cut-off levels, padj cut-off values, which comparison set to use, and dataset of genes to analyze.

![*Figure 13. main plot filters*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/main_plot_filters.png "Figure 13. main plot filters")


* It is important to note that when conducting multiple comparisons, the comparisons are labeled based on the order that they are input. If you don't remember which samples are in your current comparison you can always view the samples in each condition at the top of the main plots.

![*Figure 14. Selected conditions*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/selected_conditions.png "Figure 14. Selected conditions")


After DE analysis, you can always download the results in CSV format by clicking the **Download Data** button located under the **Data Options**. You can also download the plot or graphs by clicking on the **download** button at top of each plot or graph.

# Differential Expression Calculations

## DESeq2

For the details please check the user guide at this location:
<https://www.bioconductor.org/packages/release/bioc/vignettes/DESeq2/inst/doc/DESeq2.pdf>

DESeq2 performs multiple steps in order to analyze the data you’ve provided
for it. The first step is to indicate the condition that each column
(experiment) in the table represent. You can group multiple samples into
one condition column. DESeq2 will compute the probability that a gene is
differentially expressed (DE) for ALL genes in the table. It outputs both a
nominal and a multiple hypothesis corrected p-adjusted (padj) value using a negative
binomial distribution.

## Un-normalized counts
DESeq2 requires count data as input obtained from RNA-Seq or another
high-throughput sequencing experiment in the form of matrix values. Here we
convert un-integer values to integer to be able to run DESeq2. The matrix
values should be un-normalized, since DESeq2 model internally corrects for
library size. So, transformed or normalized values such as counts scaled by
library size should not be used as input. Please use edgeR or limma for
normalized counts.

## Used parameters for DESeq2

* **fitType:**
	Either “parametric”, “local”, or “mean” for the type of fitting of
	dispersions to the mean intensity. See estimate Dispersions for description.

* **betaPrior:**
	Whether or not to put a zero-mean normal prior on the non-intercept
	coefficients See nbinomWaldTest for description of the calculation of the
	beta prior. By default, the beta prior is used only for the Wald test, but
	can also be specified for the likelihood ratio test.

* **testType:**
	Either “Wald” or “LRT”, which will then use either Wald significance tests
	(defined by nbinomWaldTest), or the likelihood ratio test on the difference
	in deviance between a full and reduced model formula (defined by nbinomLRT)

* **rowsum.filter:**
	Regions/Genes/Isoforms with total count (across all samples) below this
	value will be filtered out

## EdgeR

For the details please check the user guide at this location:
<https://www.bioconductor.org/packages/release/bioc/vignettes/edgeR/inst/doc/edgeRUsersGuide.pdf>

## Used parameters for EdgeR

* **Normalization:**
	Calculate normalization factors to scale the raw library sizes. Values
	can be “TMM”, "RLE”, ”upperquartile”, or ”none”.

* **Dispersion:**
	Either a numeric vector of dispersions or a character string indicating
	that dispersions should be taken from the data object.

* **testType:**
	exactTest or glmLRT. exactTest: Computes p-values for differential
	abundance for each gene between two samples, conditioning on the total
	count for each gene. The counts in each group are assumed to follow a
	binomial distribution.  glmLRT: Fits a negative binomial generalized
	log-linear model to the read counts for each gene and conducts
	genewise statistical tests.

* **rowsum.filter:**
	Regions/Genes/Isoforms with total count (across all samples) below this
	value will be filtered out

## Limma

For the details please check the user guide at this location:
<https://bioconductor.org/packages/release/bioc/vignettes/limma/inst/doc/usersguide.pdf>

Limma is a package to analyze  of microarray or RNA-Seq data. If data is
normalized with spike-in or any other scaling, transformation  or
normalization method, Limma can be ideal. In that case, prefer limma rather
than DESeq2 or EdgeR.

## Used parameters for Limma

* **Normalization:**
	Calculate normalization factors to scale the raw library sizes. Values
	can be “TMM”,”RLE”,”upperquartile”, or ”none”.

* **Fit Type:**
	Fitting method: “ls” for least squares or “robust” for robust regression

* **Norm. Bet. Arrays:**
	Normalization Between Arrays; Normalizes expression intensities so that
	the intensities or log-ratios have similar distributions across a set of
	arrays.

* **rowsum.filter:**
	Regions/Genes/Isoforms with total count (across all samples) below this
	value will be filtered out



## The Heatmap of DE Analysis

Once you've selected a specific region on Main Plots (Scatter, Volcano or MA plot), a new heatmap of the selected area will appear just next to your plot. If you want to hide some groups (such as Up, Down or NS based on DE analysis), just click on the group label on the top right part of the figure. In this way, you can select a specific part of the genes by **lasso select** or **box select** tools that includes only **Up** or **Down** Regulated genes. As soon as you completed your selection, heatmap will be created simultaneously. 

![*Figure 15. main plot selection*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/main_plot_selection.png "Figure 15. main plot selection")


**A.** Box Selection, **B.** Lasso Selection, **C.** Created heatmap based on selection

* We strongly recommend normalization before plotting heatmaps. To normalize, please change the parameters that are located under: **Data options -> Normalization Methods** and select the method from the dropdown box.

## Used clustering and linkage methods in heatmap

* **complete:**
	Complete-linkage clustering is one of the linkage method used in hierarchical clustering.
	In each step of clustering, closest cluster pairs are always merged up to a specified distance
	threshold. Distance between clusters for complete link clustering is the maximum of
	the distances between the members of the clusters.

* **ward D2:**
	Ward method aims to find compact and spherical clusters. The distance between two clusters
	is calculated by the sum of squared deviations from points to centroids. "ward.D2" method uses
	criterion (Murtagh and Legendre 2014) to minimize ward clustering method. The only difference
	ward.D2 and ward is the dissimilarities from ward method squared before cluster updating. This
	method tends to be sensitive to the outliers.

* **single:**
	Distance between clusters for single linkage is the minimum of	the distances between
	the members of the clusters.

* **average:**
	Distance between clusters for average linkage is the average of the distances between
	the members of the clusters.

* **mcquitty:**
	mcquitty linkage is when two clusters are joined, the distance of the new cluster
	to any other cluster is calculated by the average of the distances of the soon to be
	joined clusters to that other cluster.

* **median:**
	This is a different averaging method that uses the median instead of the mean.
	It is used to reduce the effect of outliers.

* **centroid:**
	The distance between cluster pairs is defined as the Euclidean distance
	between their centroids or means.

## Used distance methods in heatmap

* **cor:**
	1 - cor(x) are used to define the dissimilarity between samples. It is less
	sensitive to the outliers and scaling.

* **euclidean:**
	It is the most common use of distance. It is sensitive to the outliers and scaling.
	It is defined as the square root of the sum of the square differences between gene counts.

* **maximum:**
	The maximum distance between two samples is the sum of the maximum expression value of the
	corresponding genes.

* **manhattan:**
	The Manhattan distance between two samples is the sum of the differences of their
	corresponding genes.

* **canberra:**
	Canberra distance is similar to the Manhattan distance and it is a special form of
	the Minkowski distance. The difference is that the absolute difference between the
	gene counts of the two genes is divided by the sum of the absolute counts
	prior to summing.

* **minkowsky:**
	It is generalized form of euclidean distance.


## Interactive Heatmap

You can also select to view an interactive version of the heatmap by clicking
on the 'Interactive' checkbox on the left panel under the height and width
options.  Selecting this feature changes the heatmap into an interactive
version with two colors, allowing you to select specific genes to be compared
within the GO term plots.   

Just like in the Main Plots, you can click and drag to create a selection.  To select a specific portion of the heatmap, make sure
to highlight the middle of the heatmap gene box in order to fully select a specific gene.  This selection can be used later within the
GO Term plots for specific queries on your selection.


![*Figure 16. interactive heatmap*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/interactive_heatmap.png "Figure 16. interactive heatmap")


A. Before Selection B. Selection of area with zoom tool C. Zoomed heatmap region which allows better viewing resolution.

**Interactive Feature:** In order to increase the performance of the generating heatmaps, **interactive** option is disabled by default. After deciding plotting/clustering parameters of the heatmap, you might activate this feature to investigate each block in detail.

## The Scale Option of Heatmap

By using **Scale Option** field on the left sidebar menu, it is possible to adjust scaling parameters of DEBrowser. There are four main options:
    
1. **Center**: If it is checked then centering is done by subtracting the column means of data from their corresponding columns. Otherwise no centering is done.(Default value:Checked)
2. **Scale**: The value of scale determines how column scaling is performed (after centering). If scale is checked then scaling is done by dividing the (centered) columns of the data by their standard deviations if center is checked, and the root mean square if center is unchecked. If scale is unchecked, no scaling is done.(Default value:Checked)
3. **Log**: The value of log determines the log2 operation of data matrix (Default value:Checked)
4. **Pseudo-Count**: This value added to each element to prevent getting undefined (logarithm of zero) before calculation of log2(Default value:0.1)
    
![*heatmap scale*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/heatmap_scale.png "heatmap scale")


# GO Term Plots

The next tab, 'GO Term', takes you to the ontology comparison portion of
DEBrowser.  From here you can select the standard dataset options such as
p-adjust value, fold change cut off value, which comparison set to use, and
which dataset to use on the left menu.  In addition to these parameters, you
also can choose from the 4 different ontology plot options: 'enrichGO',
'enrichKEGG', 'Disease', and 'compareCluster'.  Selecting one of these plot
options queries their specific databases with your current DESeq results.

![*Figure 17. GO plots opts*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/go_plots_opts.png "Figure 17. go plots opts")

Your GO plots include:

* enrichGO - use enriched GO terms
* enrichKEGG - use enriched KEGG terms
* Disease - enriched for diseases
* compareClusters - comparison of your clustered data

The types of plots you will be able to generate include:

Summary plot:

![*Figure 18. go summary*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/go_summary.png "Figure 18. go summary")

GOdotplot:

![*Figure 19. go dot plot*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/go_dot_plot.png "Figure 19. go dot plot")

Changing the type of ontology to use will also produce custom parameters for that specific ontology at the bottom of the
left option panel.

Once you have adjusted all of your parameters, you may hit the submit button in the top right and then wait
for the results to show on screen!

# Data Tables

The last tab at the top of the screen displays various different data tables.
These datatables include:

* All Detected
* Up Regulated
* Down Regulated
* Up+down Regulated
* Selected scatterplot points
* Most varied genes
* Comparison differences

![*Figure 20. datatable*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/datatable.png "Figure 20. datatable")


All of the tables tables, except the Comparisons table, contain the following information:

* ID - The specific gene ID
* Sample Names - The names of the samples given and they're corresponding tmm normalized counts
* Conditions - The log averaged values
* padj - padjusted value
* log2FoldChange - The Log2 fold change
* foldChange - The fold change
* log10padj - The log 10 padjusted value

The Comparisons table generates values based on the number of comparisons you have conducted.
For each pairwise comparison, these values will be generated:

* Values for each sample used
* foldChange of comparison A vs B
* pvalue of comparison A vs B
* padj value of comparison A vs B

![*Figure 21. comparisons*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/comparisons.png "Figure 21. comparisons")


You can further customize and filter each specific table a multitude of ways.  For unique table or dataset options, select the type of
table dataset you would like to customize on the left panel under 'Choose a dataset' to view it's additional options.
All of the tables have a built in search function at the top right of the table and you can further sort the table
by column by clicking on the column header you wish to sort by.  The 'Search' box on the left panel allows for multiple searches via
a comma-separated list.  You can additionally use regex terms such as "^al" or "\*lm" for even more advanced searching.
This search will be applied to wherever you are within DEBrowser, including both the plots and the tables.


* If you enter more than three lines of genes, search tool will automatically match the beginning and end of the search phrases. Otherwise it will find matched substrings in the gene list.


----

You can also view specific tables of your input data for each type of dataset available and search for a specific geneset
by inputting a comma-separated list of genes or regex terms to search for in the search box within the left panel.
To view these tables, you must select the tab labeled 'Tables' as well as the dataset from the dropdown menu on the left panel.


* If you ever want to change your parameters, or even add a new set of comparisons, you can always return to the *Data Prep* tab to change and resubmit your data.
    
# Quality Control Plots

Selecting the 'QC Plots' tab will take you to the quality control plots section.  The page opens with a Principal Component Analysis (PCA) plot and users can also view a All2All, heatmap, IQR, and density by choosing **Plot Type** in the left panel. Here the dataset being used in the plots, depends on the parameters you selected in the left panel. Therefore, you are able to adjust the size of the plots under 'width' and 'height' as well as alter a variety of other parameters to adjust the specific plot you're viewing.

Principal Component Analysis (PCA) is excellent method of checking
replicates.  PCA calculates the variance between all of the samples genes
within your current comparison set and creates a two-dimensional
graph to represent the proportion of variance explained in different
components.  Within the PCA plot section you can select the p-adjust
value, fold change cut off value, which comparison set to use, which dataset
to use, the height and width of the corresponding plots, as well as which
principal components to analyze by changing the appropriate values on the
left menu. 

The All2All plot displays the correlation between each sample, Heatmap shows a heatmap representation of your data, IQR displays a barplot displaying the IQR between samples, and Density will display an overlapping density graph for each sample. You also have the ability to select the type of clustering and distance method for the heatmap produced to further customize your quality control measures. Users also have the option to select which type of normalization methods they would like to use for these specific plotting analysis within the left menu.

![*Figure 22. intro sidebar*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/intro_sidebar.png "Figure 22. intro sidebar")

Ploting Options

![*Figure 23. intro qc all2all*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/intro_qc_all2all.png "Figure 23. intro qc all2all")

All2All Plot

![*Figure 24. intro qc heatmap*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/intro_qc_heatmap.png "Figure 24. intro qc heatmap")

Heatmap Options to Normalize All Detected Data and Created Heatmap

![*Figure 25. intro qc PCA*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/intro_qc_pca.png "Figure 25. intro qc PCA")

PCA Plot

![*Figure 26. intro qc pca loads*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/intro_qc_pca_loads.png "Figure 26. intro qc pca loads")

PCA Loadings

![*Figure 27. iqr plot*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/intro_qc_pca_loads.png "Figure 27. iqr plot")

IQR Plot Before Normalization

![*Figure 28. iqr plot norm*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/iqr_plot_norm.png "Figure 28. iqr plot norm")

IQR Plot After Normalization

![*Figure 29. density plot*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/density_plot.png "Figure 29. density plot")

Density Plot Before Normalization

![*Figure 30. density plot norm*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/density_plot_norm.png "Figure 30. density plot norm")

Density Plot After Normalization


* Each QC plot also has options to adjust the plot height and width, as well as a download button for a png output located above each plot.


With that, you've now successfully navigated the DEBrowser and are ready to start inserting your own data files and browsing your own experiments.  Enjoy the DEBrowser!


# Examples

This guide is walkthrough for the preparation of figures which is used in DEBrowser paper. PCA, Heatmap, All2All will be plotted as an example for QC plots. Next, differential expression analysis will be conducted and their results will be visualized with Main plots such as **Scatter**, **Volcano** and **MA**. More detailed analysis will be covered by using simultaneously created Heatmap and KEGG pathway on the selected portion of the data.

## QC plots without Batch Effect Correction

1) **Upload Data:** To begin the analysis, you need to load Demo Data by clicking **Load Demo (Donnard et al)!** button. Then click on **Filter** button to start **Low Count Filtering**. 
2) **Low Count Filtering:** Filtering method is selected as **Max** with cutoff 10 (which filter genes where maximum count for each gene across all samples are less than 10) and activated by clicking **Filter** button which is located at the center of the page. After filtration you can see the distribution of the data as shown at below. Now, you can proceed by clicking **Batch Effect Correction** button.
    
![*Figure 31. example filtering*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_filtering.png "Figure 31. example filtering")

    
3) **Batch Effect Correction and Normalization:**  Following options were selected to normalize the data: 
        
    * **Normalization method:** MRN
    * **Correction Method:** None
        
In order to adjust the appearance, use PCA controls which is located between two PCA plots.
    
* **Text On/Off:** On
* **Select legend:** color
* **Color field:** batch
* **Shape field:** batch
        
![*Figure 32. example pca before batch*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_pca_before_batch.png "Figure 32. example pca before batch")

	
4) **All2All:**  After batch effect correction, you can click 'Go to QC plots!' to view quality control metrics on your data. The page opens with a Principal Component Analysis (PCA) plot. You can select **All2All** option from **Plot type** on the left sidebar menu. In order to get the figure as shown at below, you need to adjust other parameters of **plot options**    on the left sidebar menu.
    
![*Figure 33. example all2all before*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_all2all_before.png "Figure 33. example all2all before")
  
* **All2All - Plot Options:** Following options are selected and their screenshots are shown at below.

    * **Plot Type:** All2All
    * **Data Options:** Choose a dataset: all-detected
    * **QC options - all2all - Size & Margins:** Check the box of the **Plot Size** and adjust width and height as 800 and 800, respectively.
    * **QC options - all2all - Options:**  corr font size: 1.8 (adjust the font size of the text inside the box)

        
![*Figure 34. example all2all menu*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_all2all_menu.png "Figure 34. example all2all menu")
    
5) **Heatmap:**  To visualize heatmap as shown at below, please select **Heatmap** option from **Plot type** on the left sidebar menu and adjust plot options. 

![*Figure 35. example heatmap before*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_heatmap_before.png "Figure 35. example heatmap before")

* **Heatmap - Plot Options:** Similar to All2All plot, we need to adjust plotting options on the left sidebar menu.

    * **Plot Type:** Heatmap
    * **Heatmap Colors:** Check the box of custom colors.
    * **Data Options:** Choose a dataset: most varied, top-n:1000, total min count:100 (to show the top 1000 most varied genes (based on coefficient of variance) whose total counts are higher than 100)
    * **QC options - kmeans:** Check the box of kmeans clustering. Select 7 as # of clusters. You might need to change the order of the clusters and click **change order** button to get gradual changes on heatmap as in the figure.
    * **QC options - heatmap - Size & Margins:** Check the box of the **Plot Size** and adjust width and height to 690 and 1200, respectively.
        
![*Figure 36. example heatmap menu*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_heatmap_menu.png "Figure 36. example heatmap menu")


## QC plots after Batch Effect Correction

Since we finalized out plots without applying batch effect correction, we can return back to batch effect correction step and change the **Correction Method** as **Combat** and continue to create new graphs with the same parameters as we used before. To make it more user friendly, we are going to start explain these steps from the beginning. If you choose to continue from batch effect correction, please skip first two steps and continue reading from 3rd step: **Batch Effect Correction and Normalization**. 

1) **Upload Data:** To begin the analysis, load Demo Data by clicking **Load Demo (Donnard et al)!** button. Then click on **Filter** button to start **Low Count Filtering**. 
2) **Low Count Filtering:** Select **Max** method with cutoff 10 (which filter genes where maximum count for each gene across all samples are less than 10), then click **Filter** button which is located at the center of the page. After filtration, proceed to next step by clicking **Batch Effect Correction** button.
    
3) **Batch Effect Correction and Normalization:**  Following options were selected to apply both normalization and batch effect correction: 
        
    * **Normalization method:** MRN
    * **Correction Method:** Combat
    * **Treatment:** treatment
    * **Batch:** batch
        
* Please adjust PCA controls (which is located between two PCA plots) as listed below:
    
    * **Text On/Off:** On
    * **Select legend:** color
    * **Color field:** batch
    * **Shape field:** batch
        
![*Figure 37. example pca after batch effect correction*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_pca_after_batch.png "Figure 37. example pca after batch effect correction")

	
4) **All2All:**  After batch effect correction, click 'Go to QC plots!' and select **All2All** option from **Plot type** on the left sidebar menu. Please adjust All2All - Plot Options as listed below.
    
![*Figure 38. example all2all after batch effect correction*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_all2all_after.png "Figure 38. example all2all after batch effect correction")

    
* **All2All - Plot Options:**

    * **Plot Type:** All2All
    * **Data Options:** Choose a dataset: all-detected
    * **QC options - all2all - Size & Margins:** Check the box of the **Plot Size** and adjust width and height to 800 and 800, respectively.
    * **QC options - all2all - Options:**  corr font size: 1.8 
        
![*Figure 39. example all2all menu*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_all2all_menu.png "Figure 39. example all2all menu")
    
5) **Heatmap:**  Please select **Heatmap** option from **Plot type** on the left sidebar menu and adjust plot options according to the list below.

![*Figure 40. example heatmap after batch effect correction*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_heatmap_after.png "Figure 40. example all2all after batch effect correction")


* **Heatmap - Plot Options:** 

    * **Plot Type:** Heatmap
    * **Heatmap Colors:** Check the box of custom colors.
    * **Data Options:** Choose a dataset: most varied, top-n:1000, total min count:100 (to show the top 1000 most varied genes (based on coefficient of variance) whose total counts are higher than 100)
    * **QC options - kmeans:** Check the box of kmeans clustering. Select 7 as # of clusters. You might need to change the order of the clusters and click **change order** button to get gradual changes on heatmap as in the figure.
    * **QC options - heatmap - Size & Margins:** Check the box of the **Plot Size** and adjust width and height to 690 and 1200, respectively.
        
![*Figure 41. example heatmap menu*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_heatmap_menu.png "Figure 41. example heatmap menu")



## The Differential Expression Plots

1) **Upload Data:** To begin the analysis, load Count Data by clicking **Load Demo (Vernia et. al)!** button. Then click on **Filter** button to start **Low Count Filtering**. 
2) **Low Count Filtering:** Select **Max** method with cutoff 10 (which filter genes where maximum count for each gene across all samples are less than 10), then click **Filter** button which is located at the center of the page. Proceed to next step by clicking **Batch Effect Correction** button.
3) **Batch Effect Correction and Normalization:**  We are going to skip both normalization and batch effect correction by selecting following options: 
        
    * **Normalization method:** None
    * **Correction Method:** None
        
4) **DE Analysis:**  After batch effect correction, click 'Go to DE Analysis'. In this page, we will add groups for comparison. Click on **Add New Comparison** button and select **Select Meta** as **treatment**. It will automatically separate experiment and control data into two groups. You can leave other parameters as default as listed below and click "Submit" button.
    
    * **DE method:** DESeq2
    * **Fit Type:** parametric
    * **betaPrior:** FALSE
    * **Test Type:** Wald
    
![*Figure 42. example DE form*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_DE_form.png "Figure 42. example DE form")

5) **Main Plots Analysis:** Upon finishing the DESeq analysis, you will see DE Results in table format. Please click on **Go to Main Plots!** button which will open **Scatter Plot**. You can switch to **Volcano Plot** and **MA Plot** by using **Plot Type** section at the left side of the menu. Since these plots are interactive, you can click to **zoom** button on the top of the graph and select the area you would like to zoom in by drawing a rectangle. Please see the plots at below:

![*Figure 43. example main plots*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_main_plots.png "Figure 43. example main plots")
    
Please keep in mind that to increase the performance of the generating graph, by default 10% of non-significant(NS) genes are used to generate plots. We used all of the NS genes in our plots that showed above, therefore please click **Main Options** button and change Background Data(%) to 100% on the left sidebar.
    
![*Figure 44. example background data*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_background_data.png "Figure 44. example background data")

6) **Read count plots:** Lets return back to **Scatter Plot** by using **Plot Type** section. You can hover on each point on the graph to see their read counts as a bar graph as shown at below. In this example FABP3 is selected to show the high variance of this gene across samples.

![*Figure 45. example scatter hover*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_scatter_hover.png "Figure 45. example scatter hover")
       
If you want to mark FABP3 gene on the plot, click on Data Options and enter **FABP3** in to the **search field** as showed below. You will see green mark on the plot that shows FABP3.
    
![*Figure 46. example scatter search*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_scatter_search.png "Figure 46. example scatter search")


7) **Lasso selection:** DEBrowser can draw heatmaps of any selected region of any main plot. Selection can be made in a rectangular form or as a free-form using plotly’s lasso select. To do so, first click **NS** label at the upper right side of the figure, and hide non-significant genes. Then click on lasso select button at the top of the plot and select the genes you're interested as shown at below. Heatmap will appear just next to scatter plot. Additionally, you can activate interactive mapping option for heatmap by clicking **Interactive** button under **Heatmap Options** on the left sidebar menu. Now, you can hover on each block of heatmap to see gene name and its value.
    
    
**Interactive Feature:** In order to increase the performance of the generating heatmaps, **interactive** option is disabled by default. After deciding plotting/clustering parameters of the heatmap, you might activate this feature to investigate each block in detail.
    
![*Figure 47. example scatter lasso*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_scatter_lasso.png "Figure 47. example scatter lasso")
       
![*Figure 48. example scatter lasso heatmap*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_scatter_lasso_heatmap.png "Figure 48. example scatter lasso heatmap")
    
8) **Scatter plot of the genes enriched in insulin signaling pathway:** In this example, we will highlight genes enriched in insulin signaling pathway. If you already hid NS genes, you can show them by clicking on the **NS** label at the upper right side of the figure. Click on the **Data Options** and enter following genes in to the **search field**: 
    
        Cbl
        Sos1
        Irs2
        Insr
        Ptprf
        Tsc1
        Crkl
        Prkar2a
        Acaca
        Fasn
        Mapk8
        Ppp1r3b
        Ppp1r3c
        Srebf1
        Pklr
        Pik3r1
        Pygl
        Pik3r3
        Socs4
        Socs2
        Eif4ebp1
        
    
* If you enter more than three lines of genes, search tool will automatically match the beginning and end of the search phrases. Otherwise it will find matched substrings in the gene list.
        
        
Now, you will see green marks on the **searched genes** as shown below:
    
![*Figure 49. example scatter insulin A*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_scatter_insulin_A.png "Figure 49. example scatter insulin A")

    
Lets, hide all the genes other then **searched genes** by clicking **NS**, **Up** and **Down** labels at the upper right side of the figure. Since only the selected genes are left on the graph, we can select these genes by clicking on **Select Box** icon and drawing a rectangle which covers all of these genes.  

![*Figure 50. example scatter insulin selection*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_scatter_insulin_select.png "Figure 50. example scatter insulin selection")

    
Here as shown below, heatmap will be simultaneously created just next to scatter plot. You might need to change plot margins as following: 
    
* **Heatmap options -> heatmap - Size & Margins:** Please check the box of the **Plot Size** and adjust width and height to 580 and 500, respectively. 
    
![*Figure 51. example scatter insulin before normalization*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_scatter_insulin_before_norm.png "Figure 51. example scatter insulin before normalization")

    
Since the data is not normalized, data of exper_rep3 looks like it belongs to control group. We strongly recommend normalization before plotting subset of genes. To normalize, please change the parameters as described below and see the updated figure at below:
    
* **Data options -> Normalization Methods:** Please select **MRN** from the dropdown box.
    
![*Figure 52. example scatter insulin B*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_scatter_insulin_B.png "Figure 52. example scatter insulin B")
    

Activating **Interactive** feature transforms the heatmap into an interactive version with two colors, allowing you to select specific genes to be compared
within the GO term plots.  

## Log2 fold change comparison for PPARα pathway
    
1) **Upload Data:** To begin the analysis, download full dataset (Vernia et. al) <https://umms.dolphinnext.com/pub/debrowser/advanced_demo.tsv> and full metadata <https://umms.dolphinnext.com/pub/debrowser/advanced_meta.tsv> on your computer. Then click **browse** button, and select downloaded files from your computer. Please keep **Separator** as **Tab** while this processes. Finally click **upload** button to see **Upload Summary**. Now you can click on **Filter** button to start **Low Count Filtering**.

2) **Low Count Filtering:** Select **Max** method with cutoff 10 (which filter genes where maximum count for each gene across all samples are less than 10), then click **Filter** button which is located at the center of the page. We are going to skip normalization and batch effect correction step by clicking 'Go to DE Analysis' button.
        
3) **DE Analysis:**  In this page, we will add multiple groups for comparison. Click on **Add New Comparison** button and select **Select Meta** as **Cond1**. Repeat this step for **Cond2** and **Cond3** and add two more comparisons. It will automatically separate experiment and control data into two groups. You can leave other parameters as default as listed below and click "Submit" button.
    
    * **DE method:** DESeq2
    * **Fit Type:** parametric
    * **betaPrior:** FALSE
    * **Test Type:** Wald
    
![*Figure 53. example multiple condition*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_multi_cond.png "Figure 53. example multiple condition")
    
4) **Downloading fold2Change data of selected genes**: Upon finishing the DE analysis, you will see DE Results in table format. Please click on **Go to Main Plots!** button which will open **Scatter Plot**. On the left sidebar menu, click **Data options* tab and enter following genes regarding to PPARα pathway:
    
        Cyp4a12b
        Cyp4a14
        Ehhadh
        Cyp8b1
        Cpt1b
        Cyp7b1
        Slc27a1
        Apoa5
        Pdpk1
        Apoa1
        Acadl
        Fads2
        Fabp4
        Acadm
        Apoa2
        Apoc3
        Fgf21
        Fabp5
        Fabp3
        Lpl
        Dbi
        Nr1h3
        Fabp7
        Ppara
        Ucp1
        Sdc1
        Sdc3
        Sdc2
        Fabp2
       
Afterwards, select **comparison** option for the **Choose a dataset** field. This option will add fold change columns to to our data.
    
Now, we need to disable filtration to get all searched genes in our dataset. To do so, enter following parameters into **Filter** field on the left sidebar menu.
       
    * **padj:** 1
    * **foldChange:** 1
       
To confirm you can check all adjusted parameters at image below. 

![*Figure 54. example fold selection*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_fold_selection.png "Figure 54. example fold selection")

It is time to download our dataset by clicking **Download Data** button on the **Data Options** field. You can open downloaded tsv file in Excel or similar programs. Once you open the file, you will see columns of count data, padj and fold2Change for all comparisons. Since we are only interested in fold2Change columns, you can delete the rest. Final data file should look like image on the left at below. 
    
We will rename column names as follows and add new column called **chow.wt** which compares chow.wildtype with itself therefore it is filled with 1. 
    
    * **foldChange.C1.vs.C2** to chow.dbl
    * **foldChange.C3.vs.C4** to hfd.wt
    * **foldChange.C5.vs.C6** to hfd.dbl
        
To confirm you can also download the final version of the fold2data from this link: <https://umms.dolphinnext.com/pub/debrowser/comparisons.tsv>
    
![*Figure 55. example table conversion*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_table_conversion.png "Figure 55. example table conversion")
    
5) **Creating Heatmap for fold2change data**: To create heatmap for fold change data, you have two options: A. Using startHeatmap() function or B. Use DEBrowser Heatmap module.
    
    * A. Open new R session and run following command in R or R Studio to run Heatmap module in web browser:
            
        startHeatmap()
            
    Similar to DEBrowser, you can click **browse** button, and select prepared log2change file from your computer. Please keep **Separator** as **Tab**. Finally click **upload** button to see **Upload Summary**. 
    
    * B. Open new R session and run following command in R or R Studio to load dataset as data frame (comparisons)::
    
        comparisons <- read.delim("~/Downloads/comparisons.tsv", row.names=1)
        
    You may need to change the path of the file according to your folder structure. Now, in order to open heatmap module, you need to run following script:
        
        library(debrowser)
        library(DESeq2)
        library(heatmaply)
        library(RColorBrewer)
        library(gplots)
                
        options(warn=-1)
        header <- dashboardHeader(title = "DEBrowser Heatmap")
        sidebar <- dashboardSidebar(  getJSLine(), sidebarMenu(id="DataAssessment",
            menuItem("Heatmap", tabName = "Heatmap"),
            plotSizeMarginsUI("heatmap"),
            heatmapControlsUI("heatmap")))
        body <- dashboardBody(
            tabItems(
            tabItem(tabName="Heatmap",  getHeatmapUI("heatmap"),
                column(4, verbatimTextOutput("heatmap_hover"), verbatimTextOutput("heatmap_selected")
                )
            )
        ))
            
        ui <- dashboardPage(header, sidebar, body, skin = "blue")
            
        server <- function(input, output, session) {
        selected <- reactiveVal()
        observe({
            withProgress(message = 'Creating plot', style = "notification", value = 0.1, {
            selected(callModule(debrowserheatmap, "heatmap", comparisons))
            })
        })
        output$heatmap_hover <- renderPrint({
            if (!is.null(selected()) && !is.null(selected()$shgClicked()) && selected()$shgClicked() != "")
                return(paste0("Clicked: ",selected()$shgClicked()))
            else
                return(paste0("Hovered:", selected()$shg()))
        })
        output$heatmap_selected <- renderPrint({
            if (!is.null(selected()))
                selected()$selGenes()
        })
        }
            
        shinyApp(ui, server)
        
        
Shiny will launch a web browser which is ready to use as a heatmap module. You need to specify following parameters to create log2fold change graph:
        
    * **Interactive:** Checked
    * **Custom Colors:** Checked
    * **Custom Colors -> Choose min colour:** #33FF00
    * **Custom Colors -> Choose median colour:** #000000
    * **Custom Colors -> Choose max colour:** #FF0000
    * **Heatmap Dendrogram -> Type:** none
    * **Scale Options -> Scale:** Checked
    * **Scale Options -> Center:** Unchecked
    * **Scale Options -> Log:** Checked
    * **Scale Options -> Pseudo Count:** 0
            
Once you specify these parameters, your heatmap will be seen as image at below.

![*Figure 56. example log heatmap*](https://umms.dolphinnext.com/pub/debrowser/debrowser_pics2/example_log_heatmap.png "Figure 56. example log heatmap")


# Case Study

Taking a look at the case study (Vernia S. et al 2014), Multiple heatmaps were
created to display findings within the research.  The heatmaps generated
for the study were customized to a high level of specificity.  However,
using a sample dataset generated from this study, it is possible to
recreate similar heatmaps (Figure 31-32) displayed within the studies findings.

![*Figure 31. All detected genes heatmap using case study data.*](https://umms.dolphinnext.com/pub/debrowser/imgs/figure_31.png "Figure 31. All detected genes heatmap using case study data")

![*Figure 32. Most varied genes heatmap using case study data.*](https://umms.dolphinnext.com/pub/debrowser/imgs/figure_32.png "Figure 32. Most varied genes heatmap using case study data")

The main difference between the heatmaps created within DEBrowser
and the heatmaps created within the research paper is that the clustering
method used within the paper was a k-means method with k equaling 6.

**The JNK2 Knock-out versus JNK1 Knock-out:**

High fat diet JNK1 knock-out and JNK2 knock-out samples compared against high fat
diet wild type samples showed a stronger effect from JNK2 KO.  
From the figures below, JNK2 KO has a
stronger effect than JNK1 KO samples. There are 177 genes (Figure 33) that have
padj < 0.01 and |log2 foldchange| > 1 in the JNK2 KO comparison while there are
only 17 genes (Figure 34) detected in the JNK1 KO comparison with the same cutoffs.

![*Figure 33. High fat diet JNK2 vs. High fat diet wild type.*](https://umms.dolphinnext.com/pub/debrowser/imgs/figure_33.png "Figure 33. High fat diet JNK2 vs. High fat diet wild type.")

![*Figure 34. High fat diet JNK1 vs. High fat diet wild type.*](https://umms.dolphinnext.com/pub/debrowser/imgs/figure_34.png "Figure 34. High fat diet JNK1 vs. High fat diet wild type.")

**JNK1 and JNK2 serve partially redundant functions:**

High fat diet JNK1 and High fat diet JNK2 double KO has 1018 significantly different genes.
When we compare HFD JNK1 KO only (177 Genes) and HFD JNK2 KO only (17 genes)  with
HFD wild type side-by-side, most of the up and down regulated genes are not
overlapping.  Up regulated genes (Figure 35.) and down regulated (Figure 36.) in
JNK1 KO was easy to analyze for these gene comparisons.
There is only 1 gene overlapping out of the 17 that were
significantly different in JNK1 KO comparisons with padj < 0.01 and
|log2foldchange| > 1 cutoffs.  It shows that both individual KO might have
individual functions in addition to their redundant functions.
When we looked at the genes in JNK1 KO in the KEGG database,  they are enriched
in "Fatty acid elongation”. JNK2 KO are enriched in "PPAR signaling pathway”
and "Biosynthesis of unsaturated fatty acids”. DEBrowser’s powerful comparison
function makes different condition comparisons and running GO Term analysis
on selected genes much easier.

![*Figure 35. Upregulated genes in hfd JNK1 KO (C1) vs. hfd wt (C2) DE comparison shows 4 upregulated genes (padj <0.01 and |log2foldchange| > 1).*](https://umms.dolphinnext.com/pub/debrowser/imgs/figure_35.png "Figure 35. Upregulated genes in hfd JNK1 KO (C1) vs. hfd wt (C2) DE comparison shows 4 upregulated genes (padj <0.01 and |log2foldchange| > 1).")

![*Figure 36. Downregulated genes in hfd JNK1 KO (C1) vs. hfd wt (C2) DE comparison shows 13 downregulated genes (padj <0.01 and |log2foldchange| > 1). Only one of them is in JNK2 KO (C3) vs. hfd wt (C4) DE comparison.*](https://umms.dolphinnext.com/pub/debrowser/imgs/figure_36.png "Figure 36. Downregulated genes in hfd JNK1 KO (C1) vs. hfd wt (C2) DE comparison shows 13 downregulated genes (padj <0.01 and |log2foldchange| > 1). Only one of them is in JNK2 KO (C3) vs. hfd wt (C4) DE comparison.")

Comparing the HFD wild type and the normal chow wild type also shows significant differences between regulated genes (Figure 37).
Expanding on the analysis further, the upregulated genes analyzed are then compared to KEGG and Disease ontologies
to show a variety of metabolism related correlations (Figures 38-39).

Using the 'advanced demo' dataset we mentioned earlier, you too can
recreate these tables using the same data.  Browsing, changing parameters,
and creating unique plots to view and analyze data can be a creative way
to recreate the same analytical results produced.  DEBrowser can be used in
multiple ways to check the reproducibility of research results in a highly
interactive format!

**DEBrowser vs other Differential Expression analysis software:**

The comparison table (Figure 40) displays multiple comparisons between debrowser and other various methods of viewing Differential Expression Analysis results.

Some of the comparisons can be viewed either within the tool itself or within some of the figures provided. A multiple tool comparison
can be observed within figure 34, an interactive visualization of gene highlighting can be observed for figures 12-14, and an interactive
visualization of biological variation or condition comparisons can also be observed for figure 34.

For more information on MeV (Howe et al., 2011) please visit this link: [MeV](http://www.tm4.org/mev.html)

For more information about Chipster (Kallio et al., 2011), please visit this link: [Chipster](http://chipster.csc.fi/manual/)

For more information about Galaxy (Giardine et al., 2005), please visit this link: [Galaxy](https://usegalaxy.org/)

For more information about CummeRBund (Trapnell et al., 2012), please visit this link: [CummeRbund](http://compbio.mit.edu/cummeRbund/manual_2_0.html)

![*Figure 40. Comparison table of DEBrowser, MeV, Chipster, Galaxy, and CummeRBund*](https://umms.dolphinnext.com/pub/debrowser/imgs/figure_40.png "Figure 40. Comparison table of DEBrowser, MeV, Chipster, Galaxy, and CummeRBund*")

## Operating System Dependencies 

On Fedora/Red Hat/CentOS, these packages have to be installed:
    
    openssl-devel, libxml2-devel, libcurl-devel, libpng-devel

On Ubuntu 18.04 LTS, you can install required packages by following command:

    sudo apt-get install libcurl4-openssl-dev libssl-dev libv8-3.14-dev udunits-bin libudunits2-* libxml2-dev 

## Autoload Data via Hyperlink

DEBrowser also accepts JSON files via hyperlink.

    1. jsonobject=https://umms.dolphinnext.com/pub/debrowser/advanced_demo_org.json
	

Now this link can be used in debrowser as::

	https://debrowser.umassmed.edu/?jsonobject=https://umms.dolphinnext.com/pub/debrowser/advanced_demo_org.json


If you'd like to add metadata::

    2. meta=https://umms.dolphinnext.com/pub/debrowser/advanced_meta.json

	https://debrowser.umassmed.edu/?jsonobject=https://umms.dolphinnext.com/pub/debrowser/advanced_demo_org.json&meta=https://umms.dolphinnext.com/pub/debrowser/advanced_meta.json

Inputting this URL into your browser will automatically load in that tsv to be analyzed by DEBrowser!


#       References

1. Anders,S. et al. (2014) HTSeq - A Python framework to work with high-throughput sequencing data.

2. Chang,W. et al. (2016) shiny: Web Application Framework for R.

3. Giardine,B. et al. (2005) Galaxy: a platform for interactive large-scale genome analysis. Genome Res., 15, 1451–1455.

4. Howe,E.A. et al. (2011) RNA-Seq analysis in MeV. Bioinformatics, 27, 3209–3210.

5. Kallio,M.A. et al. (2011) Chipster: user-friendly analysis software for microarray and other high-throughput data. BMC Genomics, 12, 507.

6. Li,B. and Dewey,C.N. (2011) RSEM: accurate transcript quantification from RNA-Seq data with or without a reference genome. BMC Bioinformatics, 12, 323.

7. Love,M.I. et al. (2014) Moderated estimation of fold change and dispersion for RNA-seq data with DESeq2. Genome Biol., 15, 550.

8. Reese,S.E. et al. (2013) A new statistic for identifying batch effects in high-throughput genomic data that uses guided principal component analysis. Bioinformatics, 29, 2877–2883.

9. Reich,M. et al. (2006) GenePattern 2.0. Nat. Genet., 38, 500–501.

10. Risso,D. et al. (2014) Normalization of RNA-seq data using factor analysis of control genes or samples. Nat. Biotechnol., 32, 896–902.

11. Ritchie,M.E. et al. (2015) limma powers differential expression analyses for RNA-sequencing and microarray studies. Nucleic Acids Res., 43, e47–e47.

12. Trapnell,C. et al. (2012) Differential gene and transcript expression analysis of RNA-seq experiments with TopHat and Cufflinks. Nat. Protoc., 7, 562–578.

13. Vernia,S. et al. (2014) The PPAR$\alpha$-FGF21 hormone axis contributes to metabolic regulation by the hepatic JNK signaling pathway. Cell Metab., 20, 512–525.

14. Murtagh, Fionn and Legendre, Pierre (2014). Ward's hierarchical agglomerative clustering method: which algorithms implement Ward's criterion? Journal of Classification 31 (forthcoming).

15. Johnson et al. (2007) Adjusting batch effects in microarray expression data using empirical Bayes methods.  Biostatistics, 8, 118-127.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/all2all.R
\name{debrowserall2all}
\alias{debrowserall2all}
\title{debrowserall2all}
\usage{
debrowserall2all(input, output, session, data = NULL, cex = 2)
}
\arguments{
\item{input, }{input variables}

\item{output, }{output objects}

\item{session, }{session}

\item{data, }{a matrix that includes expression values}

\item{cex, }{the size of the dots}
}
\value{
all2all plot
}
\description{
Module for a bar plot that can be used in data prep, main plots 
low count removal modules or any desired module
}
\examples{
    x <- debrowserall2all()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{getSampleDetails}
\alias{getSampleDetails}
\title{getSampleDetails}
\usage{
getSampleDetails(output = NULL, summary = NULL, details = NULL, data = NULL)
}
\arguments{
\item{output, }{output}

\item{summary, }{summary output name}

\item{details, }{details ouput name}

\item{data, }{data}
}
\value{
panel
}
\description{
get sample details
}
\examples{
    x <- getSampleDetails()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{push}
\alias{push}
\title{push}
\usage{
push(l, ...)
}
\arguments{
\item{l, }{that are going to push to the list}

\item{..., }{list object}
}
\value{
combined list
}
\description{
Push an object to the list.
}
\examples{
    mylist <- list()
    newlist <- push ( 1, mylist )
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mainScatter.R
\name{plotData}
\alias{plotData}
\title{plotData}
\usage{
plotData(pdata = NULL, input = NULL)
}
\arguments{
\item{pdata, }{data}

\item{input, }{input}
}
\value{
prepdata
}
\description{
prepare plot data for mainplots
}
\note{
\code{plotData}
}
\examples{
    x <- plotData()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{getLegendSelect}
\alias{getLegendSelect}
\title{getLegendSelect}
\usage{
getLegendSelect(id = "pca")
}
\arguments{
\item{id, }{namespace id}
}
\description{
select legend
}
\note{
\code{getLegendSelect}
}
\examples{
    x <- getLegendSelect("pca")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{prepHeatData}
\alias{prepHeatData}
\title{prepHeatData}
\usage{
prepHeatData(expdata = NULL, input = NULL)
}
\arguments{
\item{expdata, }{a matrixthat includes expression values}

\item{input, }{input variables}
}
\value{
heatdata
}
\description{
scales the data
}
\examples{
    x <- prepHeatData()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataLoad.R
\name{fileTypes}
\alias{fileTypes}
\title{fileTypes}
\usage{
fileTypes()
}
\value{
file types
}
\description{
Returns fileTypes that are going to be used in creating fileUpload UI
}
\note{
\code{fileTypes}
}
\examples{
    x <- fileTypes()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{getCutOffSelection}
\alias{getCutOffSelection}
\title{getCutOffSelection}
\usage{
getCutOffSelection(nc = 1)
}
\arguments{
\item{nc, }{total number of comparisons}
}
\value{
returns the left menu according to the selected tab;
}
\description{
Gathers the cut off selection for DE analysis
}
\note{
\code{getCutOffSelection}
}
\examples{
    x <- getCutOffSelection()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOterm.R
\name{getEnrichKEGG}
\alias{getEnrichKEGG}
\title{getEnrichKEGG}
\usage{
getEnrichKEGG(genelist = NULL, pvalueCutoff = 0.01, org = "org.Hs.eg.db")
}
\arguments{
\item{genelist, }{gene list}

\item{pvalueCutoff, }{the p value cutoff}

\item{org, }{the organism used}
}
\value{
Enriched KEGG
}
\description{
Gathers the Enriched KEGG analysis data to be used within the
GO Term plots.
}
\note{
\code{getEnrichKEGG}
}
\examples{
    x <- getEnrichKEGG()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOterm.R
\name{getGeneList}
\alias{getGeneList}
\title{getGeneList}
\usage{
getGeneList(
  genes = NULL,
  org = "org.Hs.eg.db",
  fromType = "SYMBOL",
  toType = c("ENTREZID")
)
}
\arguments{
\item{genes, }{gene list}

\item{org, }{orgranism for gene symbol entrez ID conversion}

\item{fromType, }{from Type}

\item{toType, }{to Type}
}
\value{
ENTREZ ID list
}
\description{
Gathers the gene list to use for GOTerm analysis.
}
\note{
\code{GOTerm}

\code{getGeneList}
symobol to ENTREZ ID conversion
}
\examples{
    x <- getGeneList(c('OCLN', 'ABCC2'))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepdata.R
\name{getGeneSetData}
\alias{getGeneSetData}
\title{getGeneSetData}
\usage{
getGeneSetData(data = NULL, geneset = NULL)
}
\arguments{
\item{data, }{loaded dataset}

\item{geneset, }{given gene set}
}
\value{
data
}
\description{
Gathers the specified gene set list to be used within the
DEBrowser.
}
\examples{
    x <- getGeneSetData()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotSize.R
\name{plotSizeUI}
\alias{plotSizeUI}
\title{plotSizeUI}
\usage{
plotSizeUI(id, w = 800, h = 600)
}
\arguments{
\item{id, }{id}

\item{w, }{width}

\item{h, }{height}
}
\value{
size and margins controls
}
\description{
Size module for plotly plots
}
\note{
\code{plotSizeUI}
}
\examples{
    x <- plotSizeUI("heatmap")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprogs.R
\name{runEdgeR}
\alias{runEdgeR}
\title{runEdgeR}
\usage{
runEdgeR(
  data = NULL,
  metadata = NULL,
  columns = NULL,
  conds = NULL,
  params = NULL
)
}
\arguments{
\item{data, }{A matrix that includes all the expression raw counts,
rownames has to be the gene, isoform or region names/IDs}

\item{metadata, }{metadata of the matrix of expression raw counts}

\item{columns, }{is a vector that includes the columns that are going
to be analyzed. These columns has to match with the given data.}

\item{conds, }{experimental conditions. The order has to match
with the column order}

\item{params, }{normfact: Calculate normalization factors to scale the raw 
    library sizes. Values can be "TMM","RLE","upperquartile","none".
dispersion: either a numeric vector of dispersions or a character 
    string indicating that dispersions should be taken from the data 
    object. If a numeric vector, then can be either of length one or 
    of length equal to the number of genes. Allowable character 
    values are "common", "trended", "tagwise" or "auto". 
    Default behavior ("auto" is to use most complex dispersions 
    found in data object.
testType: exactTest or glmLRT. exactTest: Computes p-values for differential 
    abundance for each gene between two digital libraries, conditioning 
    on the total count for each gene. The counts in each group as a 
    proportion of the whole are assumed to follow a binomial distribution. 
    glmLRT: Fit a negative binomial generalized log-linear model to the read 
    counts for each gene. Conduct genewise statistical tests for a given 
    coefficient or coefficient contrast.}
}
\value{
edgeR results
}
\description{
Run EdgeR algorithm on the selected conditions.  Output is
to be used for the interactive display.
}
\examples{
    x <- runEdgeR()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batcheffect.R
\name{debrowserbatcheffect}
\alias{debrowserbatcheffect}
\title{debrowserbatcheffect}
\usage{
debrowserbatcheffect(input, output, session, ldata = NULL)
}
\arguments{
\item{input, }{input variables}

\item{output, }{output objects}

\item{session, }{session}

\item{ldata, }{loaded data}
}
\value{
main plot

panel
}
\description{
Module to correct batch effect
}
\examples{
    x <- debrowserbatcheffect()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{getColors}
\alias{getColors}
\title{getColors}
\usage{
getColors(domains = NULL)
}
\arguments{
\item{domains, }{domains to be colored}
}
\value{
colors
}
\description{
get colors for the domains
}
\examples{
    x<-getColors()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/help.R
\name{getIntroText}
\alias{getIntroText}
\title{getIntroText
Intro text}
\usage{
getIntroText()
}
\value{
the JS for tab updates
}
\description{
getIntroText
Intro text
}
\examples{
    x<- getIntroText()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{getStartPlotsMsg}
\alias{getStartPlotsMsg}
\title{getStartPlotsMsg}
\usage{
getStartPlotsMsg()
}
\value{
return start plot msg
}
\description{
Generates and displays the starting messgae to be shown once
the user has first seen the main plots page within DEBrowser.
}
\note{
\code{getStartPlotsMsg}
}
\examples{
    x <- getStartPlotsMsg()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{getSelectedCols}
\alias{getSelectedCols}
\title{getSelectedCols}
\usage{
getSelectedCols(data = NULL, datasetInput = NULL, input = NULL)
}
\arguments{
\item{data, }{all loaded data}

\item{datasetInput, }{selected dataset}

\item{input, }{user input params}
}
\description{
gets selected columns
}
\examples{
    getSelectedCols()


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOterm.R
\name{getGSEA}
\alias{getGSEA}
\title{getGSEA}
\usage{
getGSEA(
  dataset = NULL,
  pvalueCutoff = 0.01,
  org = "org.Hs.eg.db",
  sortfield = "log2FoldChange"
)
}
\arguments{
\item{dataset, }{dataset}

\item{pvalueCutoff, }{the p value cutoff}

\item{org, }{the organism used}

\item{sortfield, }{sort field for GSEA}
}
\value{
GSEA
}
\description{
Gathers the Enriched KEGG analysis data to be used within the
GO Term plots.
}
\note{
\code{getGSEA}
}
\examples{
    x <- getGSEA()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{heatmapControlsUI}
\alias{heatmapControlsUI}
\title{heatmapControlsUI}
\usage{
heatmapControlsUI(id)
}
\arguments{
\item{id, }{module ID}
}
\value{
HeatmapControls
}
\description{
Generates the left menu to be used for heatmap plots
}
\note{
\code{heatmapControlsUI}
}
\examples{
    x <- heatmapControlsUI("heatmap")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{getMainPanel}
\alias{getMainPanel}
\title{getMainPanel}
\usage{
getMainPanel()
}
\value{
the panel for main plots;
}
\description{
main panel for volcano, scatter and maplot.  
Barplot and box plots are in this page as well.
}
\note{
\code{getMainPanel}
}
\examples{
    x <- getMainPanel()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{getSelHeat}
\alias{getSelHeat}
\title{getSelHeat}
\usage{
getSelHeat(expdata = NULL, input = NULL)
}
\arguments{
\item{expdata, }{selected genes}

\item{input, }{input params}
}
\value{
plot
}
\description{
heatmap selection functionality
}
\examples{
    x <- getSelHeat()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/installpack.R
\name{installpack}
\alias{installpack}
\title{installpack}
\usage{
installpack(package_name = NULL)
}
\arguments{
\item{package_name, }{package name to be installed}
}
\description{
install packages if they don't exist
display.
}
\note{
\code{installpack}
}
\examples{
    x <- installpack()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batcheffect.R
\name{normalizationMethods}
\alias{normalizationMethods}
\title{normalizationMethods}
\usage{
normalizationMethods(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
radio control
}
\description{
Select box to select normalization method prior to batch effect correction
}
\note{
\code{normalizationMethods}
}
\examples{
   
    x <- normalizationMethods("batch")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IQR.R
\name{getIQRPlotUI}
\alias{getIQRPlotUI}
\title{getIQRPlotUI}
\usage{
getIQRPlotUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
the panel for IQR plots;
}
\description{
IQR plot UI.
}
\note{
\code{getIQRPlotUI}
}
\examples{
    x <- getIQRPlotUI("IQR")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{heatmapUI}
\alias{heatmapUI}
\title{heatmapUI}
\usage{
heatmapUI(input, output, session)
}
\arguments{
\item{input, }{input variables}

\item{output, }{output objects}

\item{session, }{session}
}
\value{
the panel for heatmapUI;
}
\description{
Creates a shinyUI to be able to run DEBrowser interactively.
}
\note{
\code{heatmapUI}
}
\examples{
    x<-heatmapUI()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{getPCselection}
\alias{getPCselection}
\title{getPCselection}
\usage{
getPCselection(id, num = 1, xy = "x")
}
\arguments{
\item{id, }{namespace id}

\item{num, }{PC selection number}

\item{xy, }{x or y coordinate}
}
\value{
PC selection for PCA analysis
}
\description{
Generates the PC selection number to be used within DEBrowser.
}
\note{
\code{getPCselection}
}
\examples{
    x <- getPCselection("pca")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotSize.R
\name{plotTypeUI}
\alias{plotTypeUI}
\title{plotTypeUI}
\usage{
plotTypeUI(id)
}
\arguments{
\item{id, }{id}
}
\value{
size and margins controls
}
\description{
Plot download type
}
\note{
\code{plotTypeUI}
}
\examples{
    x <- plotTypeUI("heatmap")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mainScatter.R
\name{getMainPlotUI}
\alias{getMainPlotUI}
\title{getMainPlotUI}
\usage{
getMainPlotUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
the panel for main plots;
}
\description{
main plot for volcano, scatter and maplot.
}
\note{
\code{getMainPlotUI}
}
\examples{
    x <- getMainPlotUI("main")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batcheffect.R
\name{correctHarman}
\alias{correctHarman}
\title{Correct Batch Effect using Harman}
\usage{
correctHarman(input = NULL, idata = NULL, metadata = NULL)
}
\arguments{
\item{input, }{input values}

\item{idata, }{data}

\item{metadata, }{metadata}
}
\value{
data
}
\description{
Batch effect correction
}
\examples{
    x<-correctHarman ()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{selectGroupInfo}
\alias{selectGroupInfo}
\title{selectGroupInfo}
\usage{
selectGroupInfo(
  metadata = NULL,
  input = NULL,
  selectname = "groupselect",
  label = "Group info"
)
}
\arguments{
\item{metadata, }{metadata}

\item{input, }{input values}

\item{selectname, }{name of the select box}

\item{label, }{label of the select box}
}
\description{
Group info column selection. This can be used in batch effect
or coloring the groups in the plots.
}
\note{
\code{selectGroupInfo}
}
\examples{
    x <- selectGroupInfo()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepdata.R
\name{getDataForTables}
\alias{getDataForTables}
\title{getDataForTables
get data to fill up tables tab}
\usage{
getDataForTables(
  input = NULL,
  init_data = NULL,
  filt_data = NULL,
  selected = NULL,
  getMostVaried = NULL,
  mergedComp = NULL,
  explainedData = NULL
)
}
\arguments{
\item{input, }{input parameters}

\item{init_data, }{initial dataset}

\item{filt_data, }{filt_data}

\item{selected, }{selected genes}

\item{getMostVaried, }{most varied genes}

\item{mergedComp, }{merged comparison set}

\item{explainedData, }{pca gene set}
}
\value{
data
}
\description{
getDataForTables
get data to fill up tables tab
}
\examples{
    x <- getDataForTables()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{plot_pca}
\alias{plot_pca}
\title{plot_pca}
\usage{
plot_pca(
  dat = NULL,
  pcx = 1,
  pcy = 2,
  metadata = NULL,
  color = NULL,
  shape = NULL,
  size = NULL,
  textonoff = "On",
  legendSelect = "samples",
  input = NULL
)
}
\arguments{
\item{dat, }{data}

\item{pcx, }{x axis label}

\item{pcy, }{y axis label}

\item{metadata, }{additional data}

\item{color, }{color for plot}

\item{shape, }{shape for plot}

\item{size, }{size of the plot}

\item{textonoff, }{text on off}

\item{legendSelect, }{select legend}

\item{input, }{input param}
}
\value{
pca list
}
\description{
Plots the PCA results for the selected dataset.
}
\examples{
    load(system.file("extdata", "demo", "demodata.Rda",
            package="debrowser"))
    metadata<-cbind(colnames(demodata[,1:6]), 
            colnames(demodata[,1:6]),
            c(rep("Cond1",3), rep("Cond2",3)))
    colnames(metadata)<-c("samples", "color", "shape")
    
    a <- plot_pca(getNormalizedMatrix(
            demodata[rowSums(demodata[,1:6])>10,1:6]),
            metadata = metadata, color = "samples",
            size = 5, shape = "shape")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/condSelect.R
\name{prepDataContainer}
\alias{prepDataContainer}
\title{prepDataContainer}
\usage{
prepDataContainer(data = NULL, counter = NULL, input = NULL, meta = NULL)
}
\arguments{
\item{data, }{loaded dataset}

\item{counter, }{the number of comparisons}

\item{input, }{input parameters}

\item{meta, }{loaded metadata}
}
\value{
data
}
\description{
Prepares the data container that stores values used within DESeq.
}
\examples{
    x <- prepDataContainer()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gopanel.R
\name{getOrganism}
\alias{getOrganism}
\title{getOrganism}
\usage{
getOrganism(org)
}
\arguments{
\item{org, }{organism}
}
\value{
organism name for keg
}
\description{
getOrganism
}
\note{
\code{getOrganism}
}
\examples{
    x <- getOrganism()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mainScatter.R
\name{getLegendColors}
\alias{getLegendColors}
\title{getLegendColors}
\usage{
getLegendColors(Legend = c("up", "down", "NS"))
}
\arguments{
\item{Legend, }{unique Legends}
}
\value{
mainPlotControls
}
\description{
Generates colors according to the data
}
\note{
\code{getLegendColors}
}
\examples{
    x <- getLegendColors(c("up", "down", "GS", "NS"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepdata.R
\name{getMostVariedList}
\alias{getMostVariedList}
\title{getMostVariedList}
\usage{
getMostVariedList(datavar = NULL, cols = NULL, input = NULL)
}
\arguments{
\item{datavar, }{loaded dataset}

\item{cols, }{selected columns}

\item{input, }{input}
}
\value{
data
}
\description{
Calculates the most varied genes to be used for specific plots
within the DEBrowser.
}
\examples{
    x <- getMostVariedList()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOterm.R
\name{getEnrichGO}
\alias{getEnrichGO}
\title{getEnrichGO}
\usage{
getEnrichGO(
  genelist = NULL,
  pvalueCutoff = 0.01,
  org = "org.Hs.eg.db",
  ont = "CC"
)
}
\arguments{
\item{genelist, }{gene list}

\item{pvalueCutoff, }{p value cutoff}

\item{org, }{the organism used}

\item{ont, }{the ontology used}
}
\value{
Enriched GO
}
\description{
Gathers the Enriched GO Term analysis data to be used within the
GO Term plots.
}
\note{
\code{getEnrichGO}
}
\examples{
    x <- getEnrichGO()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprogs.R
\name{applyFiltersNew}
\alias{applyFiltersNew}
\title{applyFiltersNew}
\usage{
applyFiltersNew(data = NULL, input = NULL)
}
\arguments{
\item{data, }{loaded dataset}

\item{input, }{input parameters}
}
\value{
data
}
\description{
Apply filters based on foldChange cutoff and padj value.
This function adds a "Legend" column with "Up", "Down" or
"NS" values for visualization.
}
\examples{
    x <- applyFiltersNew()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataLoad.R
\name{sepRadio}
\alias{sepRadio}
\title{sepRadio}
\usage{
sepRadio(id, name)
}
\arguments{
\item{id, }{module id}

\item{name, }{name}
}
\value{
radio control
}
\description{
Radio button for separators
}
\note{
\code{sepRadio}
}
\examples{
   
    x <- sepRadio("meta", "metadata")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batcheffect.R
\name{batchEffectUI}
\alias{batchEffectUI}
\title{batchEffectUI
Creates a panel to coorect batch effect}
\usage{
batchEffectUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
panel
}
\description{
batchEffectUI
Creates a panel to coorect batch effect
}
\examples{
    x <- batchEffectUI("batcheffect")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{getVariationData}
\alias{getVariationData}
\title{getVariationData}
\usage{
getVariationData(inputdata = NULL, cols = NULL, conds = NULL, key = NULL)
}
\arguments{
\item{inputdata, }{dataset}

\item{cols, }{columns}

\item{conds, }{conditions}

\item{key, }{gene or region name}
}
\value{
plotdata
}
\description{
Adds an id to the data frame being used.
}
\examples{
    x <- getVariationData()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepdata.R
\name{applyFiltersToMergedComparison}
\alias{applyFiltersToMergedComparison}
\title{applyFiltersToMergedComparison}
\usage{
applyFiltersToMergedComparison(dc = NULL, nc = NULL, input = NULL)
}
\arguments{
\item{dc, }{all data}

\item{nc, }{the number of comparisons}

\item{input, }{input params}
}
\value{
data
}
\description{
Gathers the merged comparison data to be used within the
DEBrowser.
}
\examples{
    x <- applyFiltersToMergedComparison()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mainScatter.R
\name{generateTestData}
\alias{generateTestData}
\title{generateTestData}
\usage{
generateTestData(dat = NULL)
}
\arguments{
\item{dat, }{DESeq results will be generated for loaded data}
}
\value{
testData
}
\description{
This generates a test data that is suitable to main plots in debrowser
}
\examples{
    x <- generateTestData()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R, R/uifuncs.R
\name{getKEGGModal}
\alias{getKEGGModal}
\title{getKEGGModal
prepares a modal for KEGG plots}
\usage{
getKEGGModal()

getKEGGModal()
}
\value{
the info button

the info button
}
\description{
getKEGGModal
prepares a modal for KEGG plots

getKEGGModal
prepares a helpbutton for to go to a specific site in the documentation
}
\examples{
    x<- getKEGGModal()

    x<- getKEGGModal()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{getProgramTitle}
\alias{getProgramTitle}
\title{getProgramTitle}
\usage{
getProgramTitle(session = NULL)
}
\arguments{
\item{session, }{session var}
}
\value{
program title
}
\description{
Generates the title of the program to be displayed within DEBrowser.
If it is called in a program, the program title will be hidden
}
\note{
\code{getProgramTitle}
}
\examples{
    title<-getProgramTitle()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/help.R
\name{getDataAssesmentText}
\alias{getDataAssesmentText}
\title{getDataAssesmentText
DataAssesment text}
\usage{
getDataAssesmentText()
}
\value{
help text for data assesment
}
\description{
getDataAssesmentText
DataAssesment text
}
\examples{
    x<- getDataAssesmentText()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepdata.R
\name{getSearchData}
\alias{getSearchData}
\title{getSearchData}
\usage{
getSearchData(dat = NULL, input = NULL)
}
\arguments{
\item{dat, }{table data}

\item{input, }{input params}
}
\value{
data
}
\description{
search the geneset in the tables and return it
}
\examples{
    x <- getSearchData()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{hideObj}
\alias{hideObj}
\title{hideObj}
\usage{
hideObj(btns = NULL)
}
\arguments{
\item{btns, }{hide group of objects with shinyjs}
}
\description{
Hides a shiny object.
}
\examples{
    x <- hideObj()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/startShiny.R
\name{startDEBrowser}
\alias{startDEBrowser}
\title{startDEBrowser}
\usage{
startDEBrowser()
}
\value{
the app
}
\description{
Starts the DEBrowser to be able to run interactively.
}
\note{
\code{startDEBrowser}
}
\examples{
    startDEBrowser()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprogs.R
\name{addDataCols}
\alias{addDataCols}
\title{addDataCols}
\usage{
addDataCols(data = NULL, de_res = NULL, cols = NULL, conds = NULL)
}
\arguments{
\item{data, }{loaded dataset}

\item{de_res, }{de results}

\item{cols, }{columns}

\item{conds, }{inputconds}
}
\value{
data
}
\description{
add aditional data columns to de results
}
\examples{
    x <- addDataCols()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/installpack.R
\name{loadpack}
\alias{loadpack}
\title{loadpack}
\usage{
loadpack(package_name = NULL)
}
\arguments{
\item{package_name, }{package name to be loaded}
}
\description{
load packages
}
\note{
\code{loadpack}
}
\examples{
    x <- loadpack()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{distFunParamsUI}
\alias{distFunParamsUI}
\title{distFunParamsUI}
\usage{
distFunParamsUI()
}
\value{
funParams
}
\description{
get distance metric parameters
}
\note{
\code{distFunParamsUI}
}
\examples{
    x <- distFunParamsUI()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lowcountfilter.R
\name{debrowserlowcountfilter}
\alias{debrowserlowcountfilter}
\title{debrowserlowcountfilter}
\usage{
debrowserlowcountfilter(
  input = NULL,
  output = NULL,
  session = NULL,
  ldata = NULL
)
}
\arguments{
\item{input, }{input variables}

\item{output, }{output objects}

\item{session, }{session}

\item{ldata, }{loaded data}
}
\value{
main plot

panel
}
\description{
Module to filter low count genes/regions
}
\examples{
    x <- debrowserlowcountfilter()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{getDownloadSection}
\alias{getDownloadSection}
\title{getDownloadSection}
\usage{
getDownloadSection(choices = NULL)
}
\arguments{
\item{choices, }{main vs. QC section}
}
\value{
the panel for download section in the menu;
}
\description{
download section button and dataset selection box in the
menu for user to download selected data.
}
\note{
\code{getDownloadSection}
}
\examples{
    x<- getDownloadSection()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{run_pca}
\alias{run_pca}
\title{run_pca}
\usage{
run_pca(x = NULL, retx = TRUE, center = TRUE, scale = TRUE)
}
\arguments{
\item{x, }{dataframe with experiment data}

\item{retx, }{specifies if the data should be returned}

\item{center, }{center the PCA (Boolean)}

\item{scale, }{scale the PCA (Boolean)}
}
\value{
pca list
}
\description{
Runs PCA on the selected dataset.
}
\examples{
    load(system.file("extdata", "demo", "demodata.Rda", 
        package="debrowser"))
    pca_data<-run_pca(getNormalizedMatrix(
        demodata[rowSums(demodata[,1:6])>10,1:6]))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{drawPCAExplained}
\alias{drawPCAExplained}
\title{Creates a more detailed plot using the PCA results from
the selected dataset.}
\usage{
drawPCAExplained(explainedData = NULL)
}
\arguments{
\item{explainedData, }{selected data}
}
\value{
explained plot
}
\description{
Creates a more detailed plot using the PCA results from
the selected dataset.
}
\examples{
    x <- drawPCAExplained()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{actionButtonDE}
\alias{actionButtonDE}
\title{Buttons including Action Buttons and Event Buttons}
\usage{
actionButtonDE(
  inputId,
  label,
  styleclass = "",
  size = "",
  block = FALSE,
  icon = NULL,
  css.class = "",
  ...
)
}
\arguments{
\item{inputId}{Specifies the input slot that will be used to access the
value.}

\item{label}{The contents of the button--usually a text label, but you could
also use any other HTML, like an image.}

\item{styleclass}{The Bootstrap styling class of the button--options are
primary, info, success, warning, danger, inverse, link or blank}

\item{size}{The size of the button--options are large, small, mini}

\item{block}{Whehter the button should fill the block}

\item{icon}{Display an icon for the button}

\item{css.class}{Any additional CSS class one wishes to add to the action
button}

\item{...}{Other argument to feed into shiny::actionButton}
}
\description{
Creates an action button whose value is initially zero, and increments by one
each time it is pressed.
}
\examples{
    actionButtonDE("goDE", "Go to DE Analysis")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOterm.R
\name{compareClust}
\alias{compareClust}
\title{compareClust}
\usage{
compareClust(
  dat = NULL,
  ont = "CC",
  org = "org.Hs.eg.db",
  fun = "enrichGO",
  title = "Ontology Distribution Comparison",
  pvalueCutoff = 0.01
)
}
\arguments{
\item{dat, }{data to compare clusters}

\item{ont, }{the ontology to use}

\item{org, }{the organism used}

\item{fun, }{fun}

\item{title, }{title of the comparison}

\item{pvalueCutoff, }{pvalueCutoff}
}
\value{
compared cluster
}
\description{
Compares the clustered data to be displayed within the GO Term
plots.
}
\note{
\code{compareClust}
}
\examples{
    x <- compareClust()
  
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IQR.R
\name{getIQRPlot}
\alias{getIQRPlot}
\title{getIQRPlot}
\usage{
getIQRPlot(data = NULL, input = NULL, title = "")
}
\arguments{
\item{data, }{count or normalized data}

\item{input, }{input}

\item{title, }{title}
}
\description{
Makes IQR boxplot plot
}
\examples{
    getIQRPlot()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{palUI}
\alias{palUI}
\title{palUI}
\usage{
palUI(id)
}
\arguments{
\item{id, }{namespace ID}
}
\value{
pals
}
\description{
get pallete
}
\note{
\code{palUI}
}
\examples{
    x <- palUI("heatmap")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{getCompSelection}
\alias{getCompSelection}
\title{getCompSelection}
\usage{
getCompSelection(name = NULL, count = NULL)
}
\arguments{
\item{name, }{the name of the selectInput}

\item{count, }{comparison count}
}
\description{
Gathers the user selected comparison set to be used within the
DEBrowser.
}
\note{
\code{getCompSelection}
}
\examples{
    x <- getCompSelection(name="comp", count = 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{customColorsUI}
\alias{customColorsUI}
\title{customColorsUI}
\usage{
customColorsUI(id)
}
\arguments{
\item{id, }{namespace ID}
}
\value{
color range
}
\description{
get Custom Color controls
}
\note{
\code{getColRng}
}
\examples{
    x <- customColorsUI("heatmap")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/density.R
\name{densityPlotControlsUI}
\alias{densityPlotControlsUI}
\title{densityPlotControlsUI}
\usage{
densityPlotControlsUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
returns the left menu
}
\description{
Generates the controls in the left menu for a densityPlot
}
\note{
\code{densityPlotControlsUI}
}
\examples{
    x <- densityPlotControlsUI("density")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{getLoadingMsg}
\alias{getLoadingMsg}
\title{getLoadingMsg}
\usage{
getLoadingMsg(output = NULL)
}
\arguments{
\item{output, }{output message}
}
\value{
loading msg
}
\description{
Creates and displays the loading message/gif to be displayed
within the DEBrowser.
}
\note{
\code{getLoadingMsg}
}
\examples{
    x <- getLoadingMsg()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{debrowserpcaplot}
\alias{debrowserpcaplot}
\title{debrowserpcaplot}
\usage{
debrowserpcaplot(
  input = NULL,
  output = NULL,
  session = NULL,
  pcadata = NULL,
  metadata = NULL
)
}
\arguments{
\item{input, }{input variables}

\item{output, }{output objects}

\item{session, }{session}

\item{pcadata, }{a matrix that includes expression values}

\item{metadata, }{metadata to color the plots}
}
\value{
main plot

panel
}
\description{
Module for a pca plot with its loadings 
as a mainplot in debrowser
}
\examples{
    x <- debrowserpcaplot()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/help.R
\name{getDEAnalysisText}
\alias{getDEAnalysisText}
\title{getDEAnalysisText
DEAnalysis text}
\usage{
getDEAnalysisText()
}
\value{
help text for DE Analysis
}
\description{
getDEAnalysisText
DEAnalysis text
}
\examples{
    x<- getDEAnalysisText()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{getCondMsg}
\alias{getCondMsg}
\title{getCondMsg}
\usage{
getCondMsg(dc = NULL, input = NULL, cols = NULL, conds = NULL)
}
\arguments{
\item{dc, }{columns}

\item{input, }{selected comparison}

\item{cols, }{columns}

\item{conds, }{selected conditions}
}
\value{
return conditions
}
\description{
Generates and displays the current conditions and their samples
within the DEBrowser.
}
\note{
\code{getCondMsg}
}
\examples{
    x <- getCondMsg()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOterm.R
\name{clusterData}
\alias{clusterData}
\title{clusterData}
\usage{
clusterData(dat = NULL)
}
\arguments{
\item{dat, }{the data to cluster}
}
\value{
clustered data
}
\description{
Gathers the Cluster analysis data to be used within the
GO Term plots.
}
\note{
\code{clusterData}
}
\examples{
    mycluster <- clusterData()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/density.R
\name{getDensityPlot}
\alias{getDensityPlot}
\title{getDensityPlot}
\usage{
getDensityPlot(data = NULL, input = NULL, title = "")
}
\arguments{
\item{data, }{count or normalized data}

\item{input, }{input}

\item{title, }{title}
}
\description{
Makes Density plots
}
\examples{
    getDensityPlot()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{prepPCADat}
\alias{prepPCADat}
\title{prepPCADat}
\usage{
prepPCADat(pca_data = NULL, metadata = NULL, input = NULL, pcx = 1, pcy = 2)
}
\arguments{
\item{pca_data, }{pca run results}

\item{metadata, }{additional meta data}

\item{input, }{input}

\item{pcx, }{x axis label}

\item{pcy, }{y axis label}
}
\value{
Color and shape from selection boxes or defaults
}
\description{
prepares pca data with metadata. If metadata doesn't exists
it puts all the sampels into a signlge group; "Conds".
}
\examples{
    x <- prepPCADat()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lowcountfilter.R
\name{lcfMetRadio}
\alias{lcfMetRadio}
\title{lcfMetRadio}
\usage{
lcfMetRadio(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
radio control
}
\description{
Radio buttons for low count removal methods
}
\note{
\code{lcfMetRadio}
}
\examples{
   
    x <- lcfMetRadio("lcf")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprogs.R
\name{prepGroup}
\alias{prepGroup}
\title{prepGroup}
\usage{
prepGroup(conds = NULL, cols = NULL, metadata = NULL, covariates = NULL)
}
\arguments{
\item{conds, }{inputconds}

\item{cols, }{columns}

\item{metadata, }{metadata}

\item{covariates, }{covariates}
}
\value{
data
}
\description{
prepare group table
}
\examples{
    x <- prepGroup()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gopanel.R
\name{getOrganismPathway}
\alias{getOrganismPathway}
\title{getOrganismPathway}
\usage{
getOrganismPathway(org)
}
\arguments{
\item{org, }{organism}
}
\value{
organism name for pathway
}
\description{
getOrganismPathway
}
\note{
\code{getOrganismPathway}
}
\examples{
    x <- getOrganismPathway()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/condSelect.R
\name{getCovariateDetails}
\alias{getCovariateDetails}
\title{getCovariateDetails}
\usage{
getCovariateDetails(num = NULL, input = NULL, metadata = NULL)
}
\arguments{
\item{num, }{panel that is going to be shown}

\item{input, }{user input}

\item{metadata, }{metadata}
}
\description{
get the covariate detail box after DE method selected
}
\examples{
    x <- getCovariateDetails()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/all2all.R
\name{all2allControlsUI}
\alias{all2allControlsUI}
\title{all2allControlsUI}
\usage{
all2allControlsUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
returns the controls for left menu
}
\description{
Generates the controls in the left menu for an all2all plot
}
\note{
\code{all2allControlsUI}
}
\examples{
    x <- all2allControlsUI("bar")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IQR.R
\name{debrowserIQRplot}
\alias{debrowserIQRplot}
\title{debrowserIQRplot}
\usage{
debrowserIQRplot(input = NULL, output = NULL, session = NULL, data = NULL)
}
\arguments{
\item{input, }{input variables}

\item{output, }{output objects}

\item{session, }{session}

\item{data, }{a matrix that includes expression values}
}
\value{
IQR
}
\description{
Module for an IQR plot that can be used in data prep and 
low count removal modules
}
\examples{
    x <- debrowserIQRplot()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/help.R
\name{getQAText}
\alias{getQAText}
\title{getQAText
Some questions and answers}
\usage{
getQAText()
}
\value{
help text for QA
}
\description{
getQAText
Some questions and answers
}
\examples{
    x<- getQAText()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{getTableDetails}
\alias{getTableDetails}
\title{getTableDetails}
\usage{
getTableDetails(
  output = NULL,
  session = NULL,
  tablename = NULL,
  data = NULL,
  modal = NULL
)
}
\arguments{
\item{output, }{output}

\item{session, }{session}

\item{tablename, }{table name}

\item{data, }{matrix data}

\item{modal, }{if it is true, the matrix is going to be in a modal}
}
\value{
panel
}
\description{
get table details
To be able to put a table into two lines are necessary;
into the server part;
getTableDetails(output, session, "dataname", data, modal=TRUE)
into the ui part;
uiOutput(ns("dataname"))
}
\examples{
    x <- getTableDetails()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{heatmapJScode}
\alias{heatmapJScode}
\title{heatmapJScode}
\usage{
heatmapJScode()
}
\value{
JS Code
}
\description{
heatmap JS code for selection functionality
}
\examples{
    x <- heatmapJScode()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/server.R
\name{deServer}
\alias{deServer}
\title{deServer}
\usage{
deServer(input, output, session)
}
\arguments{
\item{input, }{input params from UI}

\item{output, }{output params to UI}

\item{session, }{session variable}
}
\value{
the panel for main plots;
}
\description{
Sets up shinyServer to be able to run DEBrowser interactively.
}
\note{
\code{deServer}
}
\examples{
    deServer

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{getPCAexplained}
\alias{getPCAexplained}
\title{getPCAexplained}
\usage{
getPCAexplained(datasetInput = NULL, pca_data = NULL, input = NULL)
}
\arguments{
\item{datasetInput, }{selected data}

\item{pca_data, }{from user}

\item{input, }{input params}
}
\value{
explained plot
}
\description{
Creates a more detailed plot using the PCA results from
the selected dataset.
}
\examples{
load(system.file("extdata", "demo", "demodata.Rda", package="debrowser"))
input<-c()
input$qcplot<-"pca"
input$col_list<-colnames(demodata[,1:6])
dat <- getNormalizedMatrix(demodata[,1:6])
pca_data <- run_pca(dat)
x <- getPCAexplained(dat, pca_data, input)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{getPCAPlotUI}
\alias{getPCAPlotUI}
\title{getPCAPlotUI}
\usage{
getPCAPlotUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
the panel for PCA plots;
}
\description{
PCA plots UI.
}
\note{
\code{getPCAPlotUI}
}
\examples{
    x <- getPCAPlotUI("pca")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IQR.R
\name{IQRPlotControlsUI}
\alias{IQRPlotControlsUI}
\title{IQRPlotControlsUI}
\usage{
IQRPlotControlsUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
returns the left menu
}
\description{
Generates the controls in the left menu for an IQR plot#'
}
\note{
\code{IQRPlotControlsUI}
}
\examples{
    x <- IQRPlotControlsUI("IQR")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/help.R
\name{getDataPreparationText}
\alias{getDataPreparationText}
\title{getDataPreparationText
DataPreparation text}
\usage{
getDataPreparationText()
}
\value{
help text for data preparation
}
\description{
getDataPreparationText
DataPreparation text
}
\examples{
    x<- getDataPreparationText()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{pcaPlotControlsUI}
\alias{pcaPlotControlsUI}
\title{pcaPlotControlsUI}
\usage{
pcaPlotControlsUI(id = "pca")
}
\arguments{
\item{id, }{namespace id}
}
\value{
returns the left menu according to the selected tab;
}
\description{
Generates the PCA PLots Left menu to be displayed within the DEBrowser.
}
\note{
\code{pcaPlotControlsUI}
}
\examples{
    x <- pcaPlotControlsUI("pca")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{textareaInput}
\alias{textareaInput}
\title{textareaInput}
\usage{
textareaInput(id, label, value, rows = 20, cols = 35, class = "form-control")
}
\arguments{
\item{id, }{id of the control}

\item{label, }{label of the control}

\item{value, }{initial value}

\item{rows, }{the # of rows}

\item{cols, }{the # of  cols}

\item{class, }{css class}
}
\description{
Generates a text area input to be used for gene selection within
the DEBrowser.
}
\examples{
    x <- textareaInput("genesetarea", "Gene Set",
        "Fgf21", rows = 5, cols = 35)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{getPCAcontolUpdatesJS}
\alias{getPCAcontolUpdatesJS}
\title{getPCAcontolUpdatesJS
in the prep menu we have two PCA plots to show how batch effect correction worked. 
One set of PCA input controls updates two PCA plots with this JS.}
\usage{
getPCAcontolUpdatesJS()
}
\value{
the JS for tab updates
}
\description{
getPCAcontolUpdatesJS
in the prep menu we have two PCA plots to show how batch effect correction worked. 
One set of PCA input controls updates two PCA plots with this JS.
}
\examples{
    x<- getTabUpdateJS()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataLoad.R
\name{fileUploadBox}
\alias{fileUploadBox}
\title{fileUploadBox}
\usage{
fileUploadBox(id = NULL, inputId = NULL, label = NULL)
}
\arguments{
\item{id, }{namespace id}

\item{inputId, }{input file ID}

\item{label, }{label}
}
\value{
radio control
}
\description{
File upload module
}
\note{
\code{fileUploadBox}
}
\examples{
   
    x <- fileUploadBox("meta", "metadata", "Metadata")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/barmain.R
\name{debrowserbarmainplot}
\alias{debrowserbarmainplot}
\title{debrowserbarmainplot}
\usage{
debrowserbarmainplot(
  input,
  output,
  session,
  data = NULL,
  cols = NULL,
  conds = NULL,
  cond_names = NULL,
  key = NULL
)
}
\arguments{
\item{input, }{input variables}

\item{output, }{output objects}

\item{session, }{session}

\item{data, }{a matrix that includes expression values}

\item{cols, }{columns}

\item{conds, }{conditions}

\item{cond_names, }{condition names}

\item{key, }{the gene or region name}
}
\value{
density plot
}
\description{
Module for a bar plot that can be used in data prep, main plots 
low count removal modules or any desired module
}
\examples{
    x <- debrowserbarmainplot()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{round_vals}
\alias{round_vals}
\title{round_vals}
\usage{
round_vals(l)
}
\arguments{
\item{l, }{the value}
}
\value{
round value
}
\description{
Plot PCA results.
}
\examples{
    x<-round_vals(5.1323223)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/condSelect.R
\name{get_conditions_given_selection}
\alias{get_conditions_given_selection}
\title{get_conditions_given_selection}
\usage{
get_conditions_given_selection(metadata = NULL, selection = NULL)
}
\arguments{
\item{metadata, }{meta data table}

\item{selection, }{selection}
}
\value{
meta select box
}
\description{
Return the two set of conditions given the selection of meta select box
}
\examples{
    x<-get_conditions_given_selection()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/all2all.R
\name{getAll2AllPlotUI}
\alias{getAll2AllPlotUI}
\title{getAll2AllPlotUI}
\usage{
getAll2AllPlotUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
the panel for all2all plots;
}
\description{
all2all plots UI.
}
\note{
\code{getAll2AllPlotUI}
}
\examples{
    x <- getAll2AllPlotUI("bar")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprogs.R
\name{runDESeq2}
\alias{runDESeq2}
\title{runDESeq2}
\usage{
runDESeq2(
  data = NULL,
  metadata = NULL,
  columns = NULL,
  conds = NULL,
  params = NULL
)
}
\arguments{
\item{data, }{A matrix that includes all the expression raw counts,
rownames has to be the gene, isoform or region names/IDs}

\item{metadata, }{metadata of the matrix of expression raw counts}

\item{columns, }{is a vector that includes the columns that are going
to be analyzed. These columns has to match with the given data.}

\item{conds, }{experimental conditions. The order has to match
with the column order}

\item{params, }{fitType: either "parametric", "local", or "mean" for the type 
    of fitting of dispersions to the mean intensity. 
    See estimateDispersions for description.
 betaPrior: whether or not to put a zero-mean normal prior
    on the non-intercept coefficients See nbinomWaldTest for 
    description of the calculation of the beta prior. By default, 
    the beta prior is used only for the Wald test, but can also be 
    specified for the likelihood ratio test.
testType: either "Wald" or "LRT", which will then use either 
    Wald significance tests (defined by nbinomWaldTest), or the 
    likelihood ratio test on the difference in deviance between a 
    full and reduced model formula (defined by nbinomLRT)
shrinkage: Adds shrunken log2 fold changes (LFC) and SE to a results 
    table from DESeq run without LFC shrinkage. For consistency with 
    results, the column name lfcSE is used here although what is 
    returned is a posterior SD. Three shrinkage estimators for 
    LFC are available via type (see the vignette for more details 
    on the estimators). The apeglm publication demonstrates that 
    'apeglm' and 'ashr' outperform the original 'normal' shrinkage 
    estimator.}
}
\value{
deseq2 results
}
\description{
Run DESeq2 algorithm on the selected conditions.  Output is
to be used for the interactive display.
}
\examples{
    x <- runDESeq2()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/condSelect.R
\name{condSelectUI}
\alias{condSelectUI}
\title{condSelectUI
Creates a panel to select samples for each condition}
\usage{
condSelectUI()
}
\value{
panel
}
\description{
condSelectUI
Creates a panel to select samples for each condition
}
\examples{
    x <- condSelectUI()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/condSelect.R
\name{getSelectInputBox}
\alias{getSelectInputBox}
\title{getSelectInputBox}
\usage{
getSelectInputBox(
  id = NULL,
  name = NULL,
  num = 0,
  choices = NULL,
  selected = NULL,
  cw = 2,
  multiple = FALSE
)
}
\arguments{
\item{id, }{input id}

\item{name, }{label of the box}

\item{num, }{panel that is going to be shown}

\item{choices, }{sample list}

\item{selected, }{selected smaple list}

\item{cw, }{column width}

\item{multiple, }{if multiple choices are available}
}
\description{
Selects user input conditions to run in DESeq.
}
\examples{
    x <- getSelectInputBox()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/histogram.R
\name{getHistogramUI}
\alias{getHistogramUI}
\title{getHistogramUI}
\usage{
getHistogramUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
the panel for PCA plots;
}
\description{
Histogram plots UI.
}
\note{
\code{getHistogramUI}
}
\examples{
    x <- getHistogramUI("histogram")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/barmain.R
\name{barMainPlotControlsUI}
\alias{barMainPlotControlsUI}
\title{barMainPlotControlsUI}
\usage{
barMainPlotControlsUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
returns the controls for left menu
}
\description{
Generates the controls in the left menu for a bar main plot
}
\note{
\code{barMainPlotControlsUI}
}
\examples{
    x <- barMainPlotControlsUI("bar")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepdata.R
\name{applyFilters}
\alias{applyFilters}
\title{applyFilters}
\usage{
applyFilters(filt_data = NULL, cols = NULL, conds = NULL, input = NULL)
}
\arguments{
\item{filt_data, }{loaded dataset}

\item{cols, }{selected samples}

\item{conds, }{seleced conditions}

\item{input, }{input parameters}
}
\value{
data
}
\description{
Applies filters based on user selected parameters to be
displayed within the DEBrowser.
}
\examples{
    x <- applyFilters()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/condSelect.R
\name{getConditionSelectorFromMeta}
\alias{getConditionSelectorFromMeta}
\title{getConditionSelectorFromMeta}
\usage{
getConditionSelectorFromMeta(
  metadata = NULL,
  input = NULL,
  index = 1,
  num = 0,
  choices = NULL,
  selected = NULL
)
}
\arguments{
\item{metadata, }{meta data table}

\item{input, }{input}

\item{index, }{index}

\item{num, }{num}

\item{choices, }{choices}

\item{selected, }{selected}
}
\description{
Selects user input conditions to run in DESeq from metadata
}
\examples{
    x <- getConditionSelectorFromMeta()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxmain.R
\name{debrowserboxmainplot}
\alias{debrowserboxmainplot}
\title{debrowserboxmainplot}
\usage{
debrowserboxmainplot(
  input = NULL,
  output = NULL,
  session = NULL,
  data = NULL,
  cols = NULL,
  conds = NULL,
  cond_names = NULL,
  key = NULL
)
}
\arguments{
\item{input, }{input variables}

\item{output, }{output objects}

\item{session, }{session}

\item{data, }{a matrix that includes expression values}

\item{cols, }{columns}

\item{conds, }{conditions}

\item{cond_names, }{condition names}

\item{key, }{the gene or region name}
}
\value{
density plot
}
\description{
Module for a box plot that can be used in DEanalysis main part and 
used heatmaps
}
\examples{
    x <- debrowserboxmainplot()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/condSelect.R
\name{getMetaSelector}
\alias{getMetaSelector}
\title{getMetaSelector}
\usage{
getMetaSelector(metadata = NULL, input = NULL, n = 0)
}
\arguments{
\item{metadata, }{meta data table}

\item{input, }{input params}

\item{n, }{the box number}
}
\value{
meta select box
}
\description{
Return the sample selection box using meta data table
}
\examples{
    x<-getMetaSelector()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/all2all.R
\name{panel.cor}
\alias{panel.cor}
\title{panel.cor}
\usage{
panel.cor(x, y, prefix = "rho=", cex.cor = 2, ...)
}
\arguments{
\item{x, }{numeric vector x}

\item{y, }{numeric vector y}

\item{prefix, }{prefix for the text}

\item{cex.cor, }{correlation font size}

\item{..., }{additional parameters}
}
\value{
all2all correlation plots
}
\description{
Prepares the correlations for the all2all plot.
}
\examples{
    panel.cor(c(1,2,3), c(4,5,6))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{getGOLeftMenu}
\alias{getGOLeftMenu}
\title{getGOLeftMenu}
\usage{
getGOLeftMenu()
}
\value{
returns the left menu according to the selected tab;
}
\description{
Generates the GO Left menu to be displayed within the DEBrowser.
}
\note{
\code{getGOLeftMenu}
}
\examples{
    x <- getGOLeftMenu()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{getShapeColor}
\alias{getShapeColor}
\title{getShapeColor}
\usage{
getShapeColor(input = NULL)
}
\arguments{
\item{input, }{input values}
}
\value{
Color and shape from selection boxes or defaults
}
\description{
Generates the fill and shape selection boxes for PCA plots.
metadata file has to be loaded in this case
}
\examples{
    x <- getShapeColor()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batcheffect.R
\name{correctCombat}
\alias{correctCombat}
\title{Correct Batch Effect using Combat in sva package}
\usage{
correctCombat(input = NULL, idata = NULL, metadata = NULL, method = NULL)
}
\arguments{
\item{input, }{input values}

\item{idata, }{data}

\item{metadata, }{metadata}

\item{method, }{method: either Combat or CombatSeq}
}
\value{
data
}
\description{
Batch effect correction
}
\examples{
    x<-correctCombat ()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprogs.R
\name{runLimma}
\alias{runLimma}
\title{runLimma}
\usage{
runLimma(
  data = NULL,
  metadata = NULL,
  columns = NULL,
  conds = NULL,
  params = NULL
)
}
\arguments{
\item{data, }{A matrix that includes all the expression raw counts,
rownames has to be the gene, isoform or region names/IDs}

\item{metadata, }{metadata of the matrix of expression raw counts}

\item{columns, }{is a vector that includes the columns that are going
to be analyzed. These columns has to match with the given data.}

\item{conds, }{experimental conditions. The order has to match
with the column order}

\item{params, }{normfact: Calculate normalization factors to scale the raw 
    library sizes. Values can be "TMM","RLE","upperquartile","none".
fitType, fitting method; "ls" for least squares or "robust" 
    for robust regression
normBet: Normalizes expression intensities so that the 
    intensities or log-ratios have similar distributions across a set of arrays.}
}
\value{
Limma results
}
\description{
Run Limma algorithm on the selected conditions.  Output is
to be used for the interactive display.
}
\examples{
    x <- runLimma()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{getAfterLoadMsg}
\alias{getAfterLoadMsg}
\title{getAfterLoadMsg}
\usage{
getAfterLoadMsg()
}
\value{
return After Load Msg
}
\description{
Generates and displays the message to be shown after loading data
within the DEBrowser.
}
\note{
\code{getAfterLoadMsg}
}
\examples{
    x <- getAfterLoadMsg()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/startShiny.R
\name{startHeatmap}
\alias{startHeatmap}
\title{startHeatmap}
\usage{
startHeatmap()
}
\value{
the app
}
\description{
Starts the DEBrowser heatmap
}
\note{
\code{startHeatmap}
}
\examples{
    startHeatmap()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataLoad.R
\name{checkCountData}
\alias{checkCountData}
\title{checkCountData}
\usage{
checkCountData(input = NULL)
}
\arguments{
\item{input, }{inputs}
}
\value{
error if there is a problem about the loaded data
}
\description{
Returns if there is a problem in the count data.
}
\note{
\code{checkCountData}
}
\examples{
    x <- checkCountData()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{getLeftMenu}
\alias{getLeftMenu}
\title{getLeftMenu}
\usage{
getLeftMenu(input = NULL)
}
\arguments{
\item{input, }{input values}
}
\value{
returns the left menu according to the selected tab;
}
\description{
Generates the left menu for for plots within the DEBrowser.
}
\note{
\code{getLeftMenu}
}
\examples{
    x <- getLeftMenu()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOterm.R
\name{getEntrezTable}
\alias{getEntrezTable}
\title{getEntrezTable}
\usage{
getEntrezTable(genes = NULL, dat = NULL, org = "org.Hs.eg.db")
}
\arguments{
\item{genes, }{gene list}

\item{dat, }{data matrix}

\item{org, }{orgranism for gene symbol entrez ID conversion}
}
\value{
table with the entrez IDs in the rownames
}
\description{
Gathers the entrezIds of the genes in given list and their data
}
\note{
\code{GOTerm}

\code{getEntrezTable}
symobol to ENTREZ ID conversion
}
\examples{
    x <- getEntrezTable()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{niceKmeans}
\alias{niceKmeans}
\title{niceKmeans}
\usage{
niceKmeans(df = NULL, input = NULL, iter.max = 1000, nstart = 100)
}
\arguments{
\item{df, }{data}

\item{input, }{user inputs}

\item{iter.max, }{max iteration for kmeans clustering}

\item{nstart, }{n for kmeans clustering}
}
\value{
heatmap plot area
}
\description{
Generates hierarchially clustered K-means clusters
}
\note{
\code{niceKmeans}
}
\examples{
    x <- niceKmeans()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepdata.R
\name{getMergedComparison}
\alias{getMergedComparison}
\title{getMergedComparison}
\usage{
getMergedComparison(dc = NULL, nc = NULL, input = NULL)
}
\arguments{
\item{dc, }{data container}

\item{nc, }{the number of comparisons}

\item{input, }{input params}
}
\value{
data
}
\description{
Gathers the merged comparison data to be used within the
DEBrowser.
}
\examples{
    x <- getMergedComparison()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprogs.R
\name{getDEResultsUI}
\alias{getDEResultsUI}
\title{getDEResultsUI
Creates a panel to visualize DE results}
\usage{
getDEResultsUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
panel
}
\description{
getDEResultsUI
Creates a panel to visualize DE results
}
\examples{
    x <- getDEResultsUI("batcheffect")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{getNormalizedMatrix}
\alias{getNormalizedMatrix}
\title{getNormalizedMatrix}
\usage{
getNormalizedMatrix(M = NULL, method = "TMM")
}
\arguments{
\item{M, }{numeric matrix}

\item{method, }{normalization method for edgeR. default is TMM}
}
\value{
normalized matrix
}
\description{
Normalizes the matrix passed to be used within various methods
within DEBrowser.  Requires edgeR package
}
\note{
\code{getNormalizedMatrix}
}
\examples{
    x <- getNormalizedMatrix(mtcars)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{getHeatmapUI}
\alias{getHeatmapUI}
\title{getHeatmapUI}
\usage{
getHeatmapUI(id)
}
\arguments{
\item{id, }{module ID}
}
\value{
heatmap plot area
}
\description{
Generates the left menu to be used for heatmap plots
}
\note{
\code{getHeatmapUI}
}
\examples{
    x <- getHeatmapUI("heatmap")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{dendControlsUI}
\alias{dendControlsUI}
\title{dendControlsUI}
\usage{
dendControlsUI(id, dendtype = "Row")
}
\arguments{
\item{id, }{module ID}

\item{dendtype, }{Row or Col}
}
\value{
controls
}
\description{
get distance metric parameters
}
\note{
\code{dendControlsUI}
}
\examples{
    x <- dendControlsUI("heatmap")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepdata.R
\name{removeCols}
\alias{removeCols}
\title{removeCols}
\usage{
removeCols(cols = NULL, dat = NULL)
}
\arguments{
\item{cols, }{columns that are going to be removed from data frame}

\item{dat, }{data}
}
\value{
data
}
\description{
remove unnecessary columns
}
\examples{
    x <- removeCols()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/condSelect.R
\name{getConditionSelector}
\alias{getConditionSelector}
\title{getConditionSelector}
\usage{
getConditionSelector(num = NULL, choices = NULL, selected = NULL)
}
\arguments{
\item{num, }{panel that is going to be shown}

\item{choices, }{sample list}

\item{selected, }{selected smaple list}
}
\description{
Selects user input conditions to run in DESeq.
}
\examples{
    x <- getConditionSelector()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/histogram.R
\name{debrowserhistogram}
\alias{debrowserhistogram}
\title{debrowserhistogram}
\usage{
debrowserhistogram(input = NULL, output = NULL, session = NULL, data = NULL)
}
\arguments{
\item{input, }{input variables}

\item{output, }{output objects}

\item{session, }{session}

\item{data, }{a matrix that includes expression values}
}
\value{
histogram
}
\description{
Module for a histogram that can be used in data prep and 
low count removal modules
}
\examples{
    x <- debrowserhistogram()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/condSelect.R
\name{getMethodDetails}
\alias{getMethodDetails}
\title{get the detail boxes after DE method selected}
\usage{
getMethodDetails(num = NULL, input = NULL)
}
\arguments{
\item{num, }{panel that is going to be shown}

\item{input, }{user input}
}
\description{
get the detail boxes after DE method selected
}
\examples{
    x <- getMethodDetails()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{getPlotArea}
\alias{getPlotArea}
\title{getPlotArea}
\usage{
getPlotArea(input = NULL, session = NULL)
}
\arguments{
\item{input, }{input variables}

\item{session, }{session}
}
\value{
heatmapply/heatmap.2 plot area
}
\description{
returns plot area either for heatmaply or heatmap.2
}
\examples{
    x <- getPlotArea()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{getQCPanel}
\alias{getQCPanel}
\title{getQCPanel}
\usage{
getQCPanel(input = NULL)
}
\arguments{
\item{input, }{user input}
}
\value{
the panel for QC plots
}
\description{
Gathers the conditional panel for QC plots
}
\note{
\code{getQCSection}
}
\examples{
    x <- getQCPanel()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/barmain.R
\name{getBarMainPlotUI}
\alias{getBarMainPlotUI}
\title{getBarMainPlotUI}
\usage{
getBarMainPlotUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
the panel for Density plots;
}
\description{
main bar plots UI.
}
\note{
\code{getBarMainPlotUI}
}
\examples{
    x <- getBarMainPlotUI("bar")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataLoad.R
\name{debrowserdataload}
\alias{debrowserdataload}
\title{debrowserdataload}
\usage{
debrowserdataload(
  input = NULL,
  output = NULL,
  session = NULL,
  nextpagebutton = NULL
)
}
\arguments{
\item{input, }{input variables}

\item{output, }{output objects}

\item{session, }{session}

\item{nextpagebutton, }{the name of the next page button after loading the data}
}
\value{
main plot

panel
}
\description{
Module to load count data and metadata
}
\examples{
    x <- debrowserdataload()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{kmeansControlsUI}
\alias{kmeansControlsUI}
\title{kmeansControlsUI}
\usage{
kmeansControlsUI(id)
}
\arguments{
\item{id, }{module ID}
}
\value{
controls
}
\description{
get kmeans controls
}
\note{
\code{kmeansControlsUI}
}
\examples{
    x <- kmeansControlsUI("heatmap")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/condSelect.R
\name{selectConditions}
\alias{selectConditions}
\title{selectConditions}
\usage{
selectConditions(
  Dataset = NULL,
  metadata = NULL,
  choicecounter = NULL,
  session = NULL,
  input = NULL
)
}
\arguments{
\item{Dataset, }{used dataset}

\item{metadata, }{metadatatable to select from metadata}

\item{choicecounter, }{choicecounter to add multiple comparisons}

\item{session, }{session}

\item{input, }{input params}
}
\value{
the panel for go plots;
}
\description{
Selects user input conditions, multiple if present, to be
used in DESeq.
}
\note{
\code{selectConditions}
}
\examples{
    x<- selectConditions()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxmain.R
\name{getBoxMainPlotUI}
\alias{getBoxMainPlotUI}
\title{getBoxMainPlotUI}
\usage{
getBoxMainPlotUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
the panel for Density plots;
}
\description{
main Box plots UI.
}
\note{
\code{getBoxMainPlotUI}
}
\examples{
    x <- getBoxMainPlotUI("box")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprogs.R
\name{getLegendRadio}
\alias{getLegendRadio}
\title{getLegendRadio}
\usage{
getLegendRadio(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
radio control
}
\description{
Radio buttons for the types in the legend
}
\note{
\code{getLegendRadio}
}
\examples{
   
    x <- getLegendRadio("deprog")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ui.R
\name{deUI}
\alias{deUI}
\title{deUI}
\usage{
deUI()
}
\value{
the panel for main plots;
}
\description{
Creates a shinyUI to be able to run DEBrowser interactively.
}
\note{
\code{deUI}
}
\examples{
    x<-deUI()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprogs.R
\name{cutOffSelectionUI}
\alias{cutOffSelectionUI}
\title{cutOffSelectionUI}
\usage{
cutOffSelectionUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
returns the left menu according to the selected tab;
}
\description{
Gathers the cut off selection for DE analysis
}
\note{
\code{cutOffSelectionUI}
}
\examples{
    x <- cutOffSelectionUI("cutoff")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/condSelect.R
\name{getSampleNames}
\alias{getSampleNames}
\title{getSampleNames}
\usage{
getSampleNames(cnames = NULL, part = 1)
}
\arguments{
\item{cnames, }{sample names in the header of a dataset}

\item{part, }{c(1,2). 1=first half and 2= second half}
}
\value{
sample names.
}
\description{
Prepares initial samples to fill condition boxes.  
it reads the sample names from the data and splits into two.
}
\examples{
    x<-getSampleNames()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{heatmapServer}
\alias{heatmapServer}
\title{heatmapServer}
\usage{
heatmapServer(input, output, session)
}
\arguments{
\item{input, }{input params from UI}

\item{output, }{output params to UI}

\item{session, }{session variable}
}
\value{
the panel for main plots;
}
\description{
Sets up shinyServer to be able to run heatmapServer interactively.
}
\note{
\code{heatmapServer}
}
\examples{
    heatmapServer

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{getDomains}
\alias{getDomains}
\title{getDomains}
\usage{
getDomains(filt_data = NULL)
}
\arguments{
\item{filt_data, }{data to get the domains}
}
\value{
domains
}
\description{
Get domains for the main plots.
}
\examples{
    x<-getDomains()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/density.R
\name{getDensityPlotUI}
\alias{getDensityPlotUI}
\title{getDensityPlotUI}
\usage{
getDensityPlotUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
the panel for Density plots;
}
\description{
Density plot UI.
}
\note{
\code{getDensityPlotUI}
}
\examples{
    x <- getDensityPlotUI("density")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/all2all.R
\name{all2all}
\alias{all2all}
\title{all2all}
\usage{
all2all(data, cex = 2)
}
\arguments{
\item{data, }{data that have the sample names in the header.}

\item{cex}{text size}
}
\value{
all2all scatter plots
}
\description{
Prepares all2all scatter plots for given datasets.
}
\examples{
    plot<-all2all(mtcars)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataLoad.R
\name{checkMetaData}
\alias{checkMetaData}
\title{checkMetaData}
\usage{
checkMetaData(input = NULL, counttable = NULL)
}
\arguments{
\item{input, }{input}

\item{counttable, }{counttable}
}
\value{
error if there is a problem about the loaded data
}
\description{
Returns if there is a problem in the count data.
}
\note{
\code{checkMetaData}
}
\examples{
    x <- checkMetaData()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lowcountfilter.R
\name{dataLCFUI}
\alias{dataLCFUI}
\title{dataLCFUI
Creates a panel to filter low count genes and regions}
\usage{
dataLCFUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
panel
}
\description{
dataLCFUI
Creates a panel to filter low count genes and regions
}
\examples{
    x <- dataLCFUI("lcf")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mainScatter.R
\name{debrowsermainplot}
\alias{debrowsermainplot}
\title{debrowsermainplot}
\usage{
debrowsermainplot(
  input = NULL,
  output = NULL,
  session = NULL,
  data = NULL,
  cond_names = NULL
)
}
\arguments{
\item{input, }{input variables}

\item{output, }{output objects}

\item{session, }{session}

\item{data, }{a matrix that includes expression values}

\item{cond_names, }{condition names}
}
\value{
main plot

panel
}
\description{
Module for a scatter, volcano and ma plots that are going to be used 
as a mainplot in debrowser
}
\examples{
    x <- debrowsermainplot()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{getHideLegendOnOff}
\alias{getHideLegendOnOff}
\title{getHideLegendOnOff}
\usage{
getHideLegendOnOff(id = "pca")
}
\arguments{
\item{id, }{namespace id}
}
\description{
hide legend
}
\examples{
    x <- getHideLegendOnOff("pca")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{runHeatmap}
\alias{runHeatmap}
\title{runHeatmap}
\usage{
runHeatmap(input = NULL, session = NULL, expdata = NULL)
}
\arguments{
\item{input, }{input variables}

\item{session, }{session}

\item{expdata, }{a matrix that includes expression values}
}
\value{
heatmapply plot
}
\description{
Creates a heatmap based on the user selected parameters within shiny
}
\examples{
    x <- runHeatmap()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{clustFunParamsUI}
\alias{clustFunParamsUI}
\title{clustFunParamsUI}
\usage{
clustFunParamsUI()
}
\value{
cluster params
}
\description{
get cluster function parameter control
}
\note{
\code{clustFunParamsUI}
}
\examples{
    x <- clustFunParamsUI()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepdata.R
\name{getUp}
\alias{getUp}
\title{getUp
get up regulated data}
\usage{
getUp(filt_data = NULL)
}
\arguments{
\item{filt_data, }{filt_data}
}
\value{
data
}
\description{
getUp
get up regulated data
}
\examples{
    x <- getUp()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotSize.R
\name{plotSizeMarginsUI}
\alias{plotSizeMarginsUI}
\title{plotSizeMarginsUI}
\usage{
plotSizeMarginsUI(id, w = 800, h = 640, t = 20, b = 100, l = 100, r = 20)
}
\arguments{
\item{id, }{id}

\item{w, }{width}

\item{h, }{height}

\item{t, }{top margin}

\item{b, }{bottom margin}

\item{l, }{left margin}

\item{r, }{right margin}
}
\value{
size and margins controls
}
\description{
Size and margins module for plotly plots
}
\note{
\code{plotSizeMarginsUI}
}
\examples{
    x <- plotSizeMarginsUI("heatmap")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{removeExtraCols}
\alias{removeExtraCols}
\title{removeExtraCols}
\usage{
removeExtraCols(dat = NULL)
}
\arguments{
\item{dat, }{selected data}
}
\description{
remove extra columns for QC plots
}
\examples{
    removeExtraCols()


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprogs.R
\name{debrowserdeanalysis}
\alias{debrowserdeanalysis}
\title{debrowserdeanalysis}
\usage{
debrowserdeanalysis(
  input = NULL,
  output = NULL,
  session = NULL,
  data = NULL,
  metadata = NULL,
  columns = NULL,
  conds = NULL,
  params = NULL
)
}
\arguments{
\item{input, }{input variables}

\item{output, }{output objects}

\item{session, }{session}

\item{data, }{a matrix that includes expression values}

\item{metadata, }{metadata}

\item{columns, }{columns}

\item{conds, }{conditions}

\item{params, }{de parameters}
}
\value{
DE panel
}
\description{
Module to perform and visualize DE results.
}
\examples{
    x <- debrowserdeanalysis()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{addID}
\alias{addID}
\title{addID}
\usage{
addID(data = NULL)
}
\arguments{
\item{data, }{loaded dataset}
}
\value{
data
}
\description{
Adds an id to the data frame being used.
}
\examples{
    x <- addID()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{showObj}
\alias{showObj}
\title{showObj}
\usage{
showObj(btns = NULL)
}
\arguments{
\item{btns, }{show group of objects with shinyjs}
}
\description{
Displays a shiny object.
}
\examples{
    x <- showObj()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/histogram.R
\name{histogramControlsUI}
\alias{histogramControlsUI}
\title{histogramControlsUI}
\usage{
histogramControlsUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
returns the left menu
}
\description{
Generates the controls in the left menu for a histogram
}
\note{
\code{histogramControlsUI}
}
\examples{
    x <- histogramControlsUI("histogram")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/density.R
\name{debrowserdensityplot}
\alias{debrowserdensityplot}
\title{debrowserdensityplot}
\usage{
debrowserdensityplot(input = NULL, output = NULL, session = NULL, data = NULL)
}
\arguments{
\item{input, }{input variables}

\item{output, }{output objects}

\item{session, }{session}

\item{data, }{a matrix that includes expression values}
}
\value{
density plot
}
\description{
Module for a density plot that can be used in data prep and 
low count removal modules
}
\examples{
    x <- debrowserdensityplot()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataLoad.R
\name{dataLoadUI}
\alias{dataLoadUI}
\title{dataLoadUI}
\usage{
dataLoadUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
panel
}
\description{
Creates a panel to upload the data
}
\examples{
    x <- dataLoadUI("load")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{getStartupMsg}
\alias{getStartupMsg}
\title{getStartupMsg}
\usage{
getStartupMsg()
}
\value{
return startup msg
}
\description{
Generates and displays the starting message within DEBrowser.
}
\note{
\code{getStartupMsg}
}
\examples{
    x <- getStartupMsg()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{getHelpButton}
\alias{getHelpButton}
\title{getHelpButton
prepares a helpbutton for to go to a specific site in the documentation}
\usage{
getHelpButton(name = NULL, link = NULL)
}
\arguments{
\item{name, }{name that are going to come after info}

\item{link, }{link of the help}
}
\value{
the info button
}
\description{
getHelpButton
prepares a helpbutton for to go to a specific site in the documentation
}
\examples{
    x<- getHelpButton()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOterm.R
\name{drawKEGG}
\alias{drawKEGG}
\title{drawKEGG}
\usage{
drawKEGG(input = NULL, dat = NULL, pid = NULL)
}
\arguments{
\item{input, }{input}

\item{dat, }{expression matrix}

\item{pid, }{pathway id}
}
\value{
enriched DO
}
\description{
draw KEGG patwhay with expression values
}
\note{
\code{drawKEGG}
}
\examples{
    x <- drawKEGG()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boxmain.R
\name{BoxMainPlotControlsUI}
\alias{BoxMainPlotControlsUI}
\title{BoxMainPlotControlsUI}
\usage{
BoxMainPlotControlsUI(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
returns the controls for left menu
}
\description{
Generates the controls in the left menu for a Box main plot
}
\note{
\code{BoxMainPlotControlsUI}
}
\examples{
    x <- BoxMainPlotControlsUI("box")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOterm.R
\name{getEntrezIds}
\alias{getEntrezIds}
\title{getEntrezIds}
\usage{
getEntrezIds(genes = NULL, org = "org.Hs.eg.db")
}
\arguments{
\item{genes, }{gene list with fold changes}

\item{org, }{orgranism for gene symbol entrez ID conversion}
}
\value{
ENTREZ ID list
}
\description{
Gathers the gene list to use for GOTerm analysis.
}
\note{
\code{GOTerm}

\code{getEntrezIds}
symobol to ENTREZ ID conversion
}
\examples{
    x <- getEntrezIds()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gopanel.R
\name{getGoPanel}
\alias{getGoPanel}
\title{getGoPanel}
\usage{
getGoPanel()
}
\value{
the panel for go term analysis;
}
\description{
Creates go term analysis panel within the shiny
display.
}
\note{
\code{getGoPanel}
}
\examples{
 
    x <- getGoPanel()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mainScatter.R
\name{mainScatterNew}
\alias{mainScatterNew}
\title{mainScatterNew}
\usage{
mainScatterNew(input = NULL, data = NULL, cond_names = NULL, source = NULL)
}
\arguments{
\item{input, }{input params}

\item{data, }{dataframe that has log2FoldChange and log10padj values}

\item{cond_names}{condition names}

\item{source, }{for event triggering to select genes}
}
\value{
scatter, volcano or MA plot
}
\description{
Creates the main scatter, volcano or MA plot to be displayed within the main
panel.
}
\examples{
    
    x <- mainScatterNew()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/barmain.R
\name{getBarMainPlot}
\alias{getBarMainPlot}
\title{getBarMainPlot}
\usage{
getBarMainPlot(
  data = NULL,
  cols = NULL,
  conds = NULL,
  cond_names = NULL,
  key = NULL,
  title = "",
  input = NULL
)
}
\arguments{
\item{data, }{count or normalized data}

\item{cols, }{cols}

\item{conds, }{conds}

\item{cond_names, }{condition names}

\item{key, }{key}

\item{title, }{title}

\item{input, }{input}
}
\description{
Makes Density plots
}
\examples{
    getBarMainPlot()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batcheffect.R
\name{batchMethod}
\alias{batchMethod}
\title{batchMethod}
\usage{
batchMethod(id)
}
\arguments{
\item{id, }{namespace id}
}
\value{
radio control
}
\description{
select batch effect method
}
\note{
\code{batchMethod}
}
\examples{
   
    x <- batchMethod("batch")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gopanel.R
\name{getGOPlots}
\alias{getGOPlots}
\title{getGOPlots}
\usage{
getGOPlots(dataset = NULL, GSEARes = NULL, input = NULL)
}
\arguments{
\item{dataset, }{the dataset used}

\item{GSEARes, }{GSEA results}

\item{input, }{input params}
}
\value{
the panel for go plots;
}
\description{
Go term analysis panel.  Generates appropriate GO plot
based on user selection.
}
\note{
\code{getGOPlots}
}
\examples{
    x<- getGOPlots()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepdata.R
\name{getUpDown}
\alias{getUpDown}
\title{getUpDown
get up+down regulated data}
\usage{
getUpDown(filt_data = NULL)
}
\arguments{
\item{filt_data, }{filt_data}
}
\value{
data
}
\description{
getUpDown
get up+down regulated data
}
\examples{
    x <- getUpDown()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pca.R
\name{getColorShapeSelection}
\alias{getColorShapeSelection}
\title{getColorShapeSelection}
\usage{
getColorShapeSelection(metadata = NULL, input = NULL, session = NULL)
}
\arguments{
\item{metadata, }{metadata table}

\item{input, }{input}

\item{session, }{session}
}
\value{
Color and shape selection boxes
}
\description{
Generates the fill and shape selection boxes for PCA plots.
metadata file has to be loaded in this case
}
\examples{
    x <- getColorShapeSelection()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mainScatter.R
\name{getLevelOrder}
\alias{getLevelOrder}
\title{getLevelOrder}
\usage{
getLevelOrder(Level = c("up", "down", "NS"))
}
\arguments{
\item{Level, }{factor levels shown in the legend}
}
\value{
order
}
\description{
Generates the order of the overlapping points
}
\note{
\code{getLevelOrder}
}
\examples{
    x <- getLevelOrder(c("up", "down", "GS", "NS"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepdata.R
\name{getDown}
\alias{getDown}
\title{getDown
get down regulated data}
\usage{
getDown(filt_data = NULL)
}
\arguments{
\item{filt_data, }{filt_data}
}
\value{
data
}
\description{
getDown
get down regulated data
}
\examples{
    x <- getDown()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{debrowserheatmap}
\alias{debrowserheatmap}
\title{debrowserheatmap}
\usage{
debrowserheatmap(input, output, session, expdata = NULL)
}
\arguments{
\item{input, }{input variables}

\item{output, }{output objects}

\item{session, }{session}

\item{expdata, }{a matrix that includes expression values}
}
\value{
heatmapply plot
}
\description{
Heatmap module to create interactive heatmaps and get selected list from
a heatmap
}
\examples{
    x <- debrowserheatmap()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{changeClusterOrder}
\alias{changeClusterOrder}
\title{changeClusterOrder}
\usage{
changeClusterOrder(order = NULL, cld = NULL)
}
\arguments{
\item{order, }{order}

\item{cld, }{data}
}
\value{
heatmap plot area
}
\description{
change order of K-means clusters
}
\note{
\code{changeClusterOrder}
}
\examples{
    x <- changeClusterOrder()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/heatmap.R
\name{getJSLine}
\alias{getJSLine}
\title{getJSLine}
\usage{
getJSLine()
}
\value{
JS Code
}
\description{
heatmap JS code for selection functionality
}
\examples{
    x <- getJSLine()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{getQCLeftMenu}
\alias{getQCLeftMenu}
\title{getQCLeftMenu}
\usage{
getQCLeftMenu(input = NULL)
}
\arguments{
\item{input, }{input values}
}
\value{
QC left menu
}
\description{
Generates the left menu to be used for QC plots within the
DEBrowser.
}
\note{
\code{getQCLeftMenu}
}
\examples{
    x <- getQCLeftMenu()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/condSelect.R
\name{getGroupSelector}
\alias{getGroupSelector}
\title{getGroupSelector
Return the groups}
\usage{
getGroupSelector(metadata = NULL, input = NULL, index = 1, num = 0)
}
\arguments{
\item{metadata, }{meta data table}

\item{input, }{input params}

\item{index, }{index}

\item{num, }{num}
}
\value{
meta select box
}
\description{
getGroupSelector
Return the groups
}
\examples{
    x<-getGroupSelector()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/condSelect.R
\name{debrowsercondselect}
\alias{debrowsercondselect}
\title{debrowsercondselect}
\usage{
debrowsercondselect(
  input = NULL,
  output = NULL,
  session = NULL,
  data = NULL,
  metadata = NULL
)
}
\arguments{
\item{input, }{input variables}

\item{output, }{output objects}

\item{session, }{session}

\item{data, }{count data}

\item{metadata, }{metadata}
}
\value{
main plot

panel
}
\description{
Condition selection
This is not a module. Module construction didn't used here, just use it 
as functions not in a module.
}
\examples{
    x <- debrowsercondselect()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{getTabUpdateJS}
\alias{getTabUpdateJS}
\title{getTabUpdateJS
prepmenu tab and discovery menu tab updates}
\usage{
getTabUpdateJS()
}
\value{
the JS for tab updates
}
\description{
getTabUpdateJS
prepmenu tab and discovery menu tab updates
}
\examples{
    x<- getTabUpdateJS()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{togglePanels}
\alias{togglePanels}
\title{togglePanels}
\usage{
togglePanels(num = NULL, nums = NULL, session = NULL)
}
\arguments{
\item{num, }{selected panel}

\item{nums, }{all panels}

\item{session, }{session info}
}
\description{
User defined toggle to display which panels are to be shown within
DEBrowser.
}
\note{
\code{togglePanels}
}
\examples{
    x <- togglePanels()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOterm.R
\name{getEnrichDO}
\alias{getEnrichDO}
\title{getEnrichDO}
\usage{
getEnrichDO(genelist = NULL, pvalueCutoff = 0.01)
}
\arguments{
\item{genelist, }{gene list}

\item{pvalueCutoff, }{the p value cutoff}
}
\value{
enriched DO
}
\description{
Gathers the Enriched DO Term analysis data to be used within the
GO Term plots.
}
\note{
\code{getEnrichDO}
}
\examples{
    x <- getEnrichDO()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mainScatter.R
\name{mainPlotControlsUI}
\alias{mainPlotControlsUI}
\title{mainPlotControlsUI}
\usage{
mainPlotControlsUI(id)
}
\arguments{
\item{id, }{module ID}
}
\value{
mainPlotControls
}
\description{
Generates the left menu to be used for main plots
}
\note{
\code{mainPlotControlsUI}
}
\examples{
    x <- mainPlotControlsUI("main")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{getTableModal}
\alias{getTableModal}
\title{getTableModal
prepares table modal for KEGG}
\usage{
getTableModal()
}
\value{
the info button
}
\description{
getTableModal
prepares table modal for KEGG
}
\examples{
    x<- getTableModal()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{getMainPlotsLeftMenu}
\alias{getMainPlotsLeftMenu}
\title{getMainPlotsLeftMenu}
\usage{
getMainPlotsLeftMenu()
}
\value{
returns the left menu according to the selected tab;
}
\description{
Generates the Main PLots Left menu to be displayed within the DEBrowser.
}
\note{
\code{getMainPlotsLeftMenu}
}
\examples{
    x <- getMainPlotsLeftMenu()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prepdata.R
\name{getSelectedDatasetInput}
\alias{getSelectedDatasetInput}
\title{getSelectedDatasetInput}
\usage{
getSelectedDatasetInput(
  rdata = NULL,
  getSelected = NULL,
  getMostVaried = NULL,
  mergedComparison = NULL,
  input = NULL
)
}
\arguments{
\item{rdata, }{filtered dataset}

\item{getSelected, }{selected data}

\item{getMostVaried, }{most varied data}

\item{mergedComparison, }{merged comparison data}

\item{input, }{input parameters}
}
\value{
data
}
\description{
Gathers the user selected dataset output to be displayed.
}
\examples{
    x <- getSelectedDatasetInput()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/all2all.R
\name{panel.hist}
\alias{panel.hist}
\title{panel.hist}
\usage{
panel.hist(x, ...)
}
\arguments{
\item{x, }{a vector of values for which the histogram is desired}

\item{..., }{any additional params}
}
\value{
all2all histogram plots
}
\description{
Prepares the historgram for the all2all plot.
}
\examples{
    panel.hist(1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotSize.R
\name{plotMarginsUI}
\alias{plotMarginsUI}
\title{plotMarginsUI}
\usage{
plotMarginsUI(id, t = 20, b = 100, l = 100, r = 20)
}
\arguments{
\item{id, }{id}

\item{t, }{top margin}

\item{b, }{bottom margin}

\item{l, }{left margin}

\item{r, }{right margin}
}
\value{
size and margins controls
}
\description{
Margins module for plotly plots
}
\note{
\code{plotMarginsUI}
}
\examples{
    x <- plotMarginsUI("heatmap")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprogs.R
\name{runDE}
\alias{runDE}
\title{runDE}
\usage{
runDE(
  data = NULL,
  metadata = NULL,
  columns = NULL,
  conds = NULL,
  params = NULL
)
}
\arguments{
\item{data, }{A matrix that includes all the expression raw counts,
rownames has to be the gene, isoform or region names/IDs}

\item{metadata, }{metadata of the matrix of expression raw counts}

\item{columns, }{is a vector that includes the columns that are going
to be analyzed. These columns has to match with the given data.}

\item{conds, }{experimental conditions. The order has to match
with the column order}

\item{params, }{all params for the DE methods}
}
\value{
de results
}
\description{
Run DE algorithms on the selected parameters.  Output is
to be used for the interactive display.
}
\examples{
    x <- runDE()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprogs.R
\name{getMean}
\alias{getMean}
\title{getMean}
\usage{
getMean(data = NULL, selcols = NULL)
}
\arguments{
\item{data, }{dataset}

\item{selcols, }{input cols}
}
\value{
data
}
\description{
Gathers the mean for selected condition.
}
\examples{
    x <- getMean()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{setBatch}
\alias{setBatch}
\title{setBatch
to skip batch effect correction batch variable set with the filter results}
\usage{
setBatch(fd = NULL)
}
\arguments{
\item{fd, }{filtered data}
}
\value{
fd data
}
\description{
setBatch
to skip batch effect correction batch variable set with the filter results
}
\examples{
   
    x <- setBatch()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{getLogo}
\alias{getLogo}
\title{getLogo}
\usage{
getLogo()
}
\value{
return logo
}
\description{
Generates and displays the logo to be shown within DEBrowser.
}
\note{
\code{getLogo}
}
\examples{
    x <- getLogo()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uifuncs.R
\name{getTableStyle}
\alias{getTableStyle}
\title{getTableStyle}
\usage{
getTableStyle(
  dat = NULL,
  input = NULL,
  padj = c("padj"),
  foldChange = c("foldChange"),
  DEsection = TRUE
)
}
\arguments{
\item{dat, }{dataset}

\item{input, }{input params}

\item{padj, }{the name of the padj value column in the dataset}

\item{foldChange, }{the name of the foldChange column in the dataset}

\item{DEsection, }{if it is in DESection or not}
}
\description{
User defined selection that selects the style of table to display
within the DEBrowser.
}
\note{
\code{getTableStyle}
}
\examples{
    x <- getTableStyle()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/condSelect.R
\name{selectedInput}
\alias{selectedInput}
\title{selectedInput}
\usage{
selectedInput(id = NULL, num = 0, default = NULL, input = NULL)
}
\arguments{
\item{id, }{input id}

\item{num, }{panel that is going to be shown}

\item{default, }{default text}

\item{input, }{input params}
}
\description{
Selects user input conditions to run in DESeq.
}
\examples{
    x <- selectedInput()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gopanel.R
\name{getOrganismBox}
\alias{getOrganismBox}
\title{getOrganismBox}
\usage{
getOrganismBox()
}
\value{
selectInput
}
\description{
Get the organism Box.
}
\note{
\code{getOrganismBox}

\code{getOrganismBox}
makes the organism box
}
\examples{
    x <- getOrganismBox()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funcs.R
\name{getBSTableUI}
\alias{getBSTableUI}
\title{getBSTableUI
prepares a Modal to put a table}
\usage{
getBSTableUI(
  name = NULL,
  label = NULL,
  trigger = NULL,
  size = "large",
  modal = NULL
)
}
\arguments{
\item{name, }{name}

\item{label, }{label}

\item{trigger, }{trigger button for the modal}

\item{size, }{size of the modal}

\item{modal, }{modal yes/no}
}
\value{
the modal
}
\description{
getBSTableUI
prepares a Modal to put a table
}
\examples{
    x<- getBSTableUI()

}
